#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################
##############################################################################
# $Id: balloon.tcl,v 1.3 1997/04/18 02:00:38 stewart Exp $
#
# balloon.tcl - procedures used by balloon help
#
# Copyright (C) 1996-1997 Stewart Allen
# 
# This is part of vtcl source code Adapted for 
# general purpose by Daniel Roche <dan@lectra.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##############################################################################
proc BindBulle {} {

    global Bulle

    bind Bulle <Enter> {
	set Bulle(set) 0
	set Bulle(first) 1
	set Bulle(id) [after 500 {balloon %W $Bulle(%W) %X %Y}]
    }
    
    bind Bulle <Button> {
	set Bulle(first) 0
	kill_balloon
    }
    
    bind Bulle <Leave> {
	set Bulle(first) 0
	kill_balloon
    }
    
    bind Bulle <Motion> {
	if {$Bulle(set) == 0} {
	    after cancel $Bulle(id)
        set Bulle(id) [after 500 {balloon %W $Bulle(%W) %X %Y}]
	}
    }
}

proc set_balloon {target message} {
    global Bulle

    if {! [info exists Bulle]} {BindBulle}

    set Bulle($target) $message
    bindtags $target "[bindtags $target] Bulle"
}

proc kill_balloon {} {
    global Bulle
    after cancel $Bulle(id)
    if {[winfo exists .balloon] == 1} {
        destroy .balloon
    }
    set Bulle(set) 0
}

proc OLD_balloon {target message {cx 0} {cy 0} } {
    global Bulle
    if {$Bulle(first) == 1 } {
        set Bulle(first) 2
	if { $cx == 0 && $cy == 0 } {
	    set x [expr [winfo rootx $target] + ([winfo width $target]/2)]
	    set y [expr [winfo rooty $target] + [winfo height $target] + 4]
	} else {
	    set x [expr $cx + 4]
	    set y [expr $cy + 4]
	}

        toplevel .balloon -bg black
        wm overrideredirect .balloon 1
        label .balloon.l \
            -text $message -relief flat \
            -bg #ffffaa -fg black -padx 2 -pady 0 -anchor w
        pack .balloon.l -side left -padx 1 -pady 1

        wm geometry .balloon +${x}+${y}
        set Bulle(set) 1
    }
}

proc balloon {target message {cx 0} {cy 0} } {
    global Bulle
    if {$Bulle(first) == 1 } {
        set Bulle(first) 2
	if { $cx == 0 && $cy == 0 } {
	    set X [winfo rootx $target]
	    set Y [winfo rooty $target]
	} else {
	    set X $cx
	    set Y $cy
	}

	set Xmin [winfo rootx $target]
	set Xmax [expr {$Xmin + [winfo width  $target]}]
	set Ymin [winfo rooty $target]
	set Ymax [expr {$Ymin + [winfo height $target]}]

        toplevel .balloon -bg black
        wm overrideredirect .balloon 1
        label .balloon.l \
            -text $message -relief flat \
            -bg #ffffaa -fg black -padx 2 -pady 0 -anchor w
        pack .balloon.l -side left -padx 1 -pady 1

	wm withdraw .balloon
	update
	set MessageLargeur [winfo width  .balloon]
	set MessageHauteur [winfo height .balloon]
	if {$MessageLargeur=="1" && $MessageHauteur=="1"} {
	    set MessageLargeur [winfo reqwidth  .balloon]
	    set MessageHauteur [winfo reqheight .balloon] 
	}
	
	#Ajustement des positions de l InfoBulle
	#Sur Beaufort le withdraw empeche d avoir la taille de la fenetre voir mise a jour
	#Mais le reqwidth et le reqheight contourne le pb
	set OnForceADroite 1
	set OnForceEnBas   0
 	if {$MessageHauteur==1} {set OnForceADroite 0}
	if {$MessageLargeur==1} {set OnForceEnBas   1}
		
	set EcranHauteur [winfo screenheight .]
	set EcranLargeur [winfo screenwidth  .]
	if {$OnForceADroite && $X> [expr {$EcranLargeur/2}]} {
	    #On affiche a gauche
	    incr X -10
	    #while {$X>$Xmin} {incr X -1}
	    set PosX [expr {$X - $MessageLargeur}]
	} else {
	    #On affiche a droite
	    incr X  10
	    #while {$X<$Xmax} {incr X 1}
	    set PosX $X
	}
	if {$OnForceEnBas || $Y< 50} {
	    #On affiche en bas
	    incr Y 5
	    #set PosY $Y
	    set PosY $Ymax
	} else {
	    #On affiche en haut
	    incr Y -5 
	    set PosY [expr {$Ymin -$MessageHauteur}]
	}
	
	set PosX [format "%.0f" $PosX]
	set PosY [format "%.0f" $PosY]
	#Ajustement final des positions de l InfoBulle et positionnement
	if {$PosX<0} {
	    #On sort de l ecran par la gauche
	    set PosX 0
	} elseif {[expr {$PosX + $MessageLargeur}]> $EcranLargeur} {
	    #On sort de l ecran par le droite
	    set PosX [expr {$EcranLargeur - $MessageLargeur}]
	}
	if {$PosY<0} {
	    #On sort de l ecran par le dessus
	    set PosY 0
	} elseif {[expr {$PosY + $MessageHauteur}]> $EcranHauteur} {
	    #On sort de l ecran par le dessous
	    set PosY [expr {$EcranHauteur - $MessageHauteur}]
	}
	
	wm geometry  .balloon +${PosX}+${PosY}
	#wm geometry .balloon +${x}+${y}
        set Bulle(set) 1
	#5 Apres l avoir oublie on se souvient
	wm deiconify .balloon
    }
}
