#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

#############################################################################
#   0.Proc pour les chemins et home dir (ATTENTION aussi dans CADO4MI.tcl)  #
#############################################################################
proc CADO4MI_Version {} {

    #Version de CADO4MI

    global CADO4MI_Version

    if {[info exists CADO4MI_Version]} {return $CADO4MI_Version}	
    set     CADO4MI_Version "1.6"

    return $CADO4MI_Version
}

proc HOMEDIR {} {

    #Home directory du user en cours

    global HOMEDIR
    global env

    if {[info exists HOMEDIR] && [file exists $HOMEDIR]} {return $HOMEDIR}	
    set     HOMEDIR ""
    if {[info exists env(HOME)]} {
	set HOMEDIR [file join [set env(HOME)]]
    }
    if {   $HOMEDIR == ""} {
	set HOMEDIR [file join [RepertoireDuGenome]]
    }
    return $HOMEDIR
}

proc TEMPDIR {} {

    #Temp directory du user en cours voir avec RepertoireDeTravail

    global TEMPDIR
    global env

    if {[info exists TEMPDIR] && [file exists $TEMPDIR]} {return $TEMPDIR}	
    set     TEMPDIR ""
    if {      [info exists env(TEMP)]} {
	set TEMPDIR [file join [set env(TEMP)]]
    } elseif {[info exists env(TMP)]} {
	set TEMPDIR [file join [set env(TMP)]]
    }
    if {   $TEMPDIR == ""} {
	set TEMPDIR [file join [HOMEDIR] junkdir]
    }

    return $TEMPDIR
}

proc BLASTDBDIR {} {

    #BLAST directory

    global BLASTDBDIR
    global env

    if {[info exists BLASTDBDIR] && [file exists $BLASTDBDIR]} {return $BLASTDBDIR}	
	set     BLASTDBDIR ""
	if {[info exists env(BLASTDB)]} {
	    set BLASTDBDIR [file join [set env(BLASTDB)]]
	}
	if {   $BLASTDBDIR == ""} {
	    set BLASTDBDIR [HOMEDIR]
	}
    return $BLASTDBDIR
}

proc BROWSEDDIR {{Quoi ""}} {

    #Browsed directory du user en cours

    global BROWSEDDIR

    if {$Quoi != ""} {
	if {$Quoi=="ResetBROWSEDDIR"} {set Quoi "[HOMEDIR]"}
	set     BROWSEDDIR $Quoi
	
	return $BROWSEDDIR
    }

    if {[info exists BROWSEDDIR] && [file exists $BROWSEDDIR] && $Quoi == ""} {return $BROWSEDDIR}	
    set     BROWSEDDIR ""

    return $BROWSEDDIR
}

proc CADO4MI_PATH {What} {

    #Permet de chercher les chemins des programmes ou des banques importants pour CADO4MI
    #Inclut un test si l executable existe vraiment meme si il specifie dans le fichier

    global CADO4MI_PATH

    set PathFile "[file join "[ProgSourceDir]" "CADO4MI.path"]"

    if {[info exists CADO4MI_PATH($What)]}  {return [set CADO4MI_PATH($What)]}
    #if {[info exists CADO4MI_PATH(Loaded)]} {return "$What"}

    if {! [info exists CADO4MI_PATH(Loaded)] && [file exists $PathFile]} {
	set CADO4MI_PATH(Loaded) 1
	set F [open "$PathFile"]
	while {[gets $F Ligne]>=0} {
	    if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	    if {$Ligne == ""}                          {continue}
	    if {[regexp {^\#} $Ligne]}                 {continue}
	    
	    set  Ligne [split $Ligne "\t"]
	    if { [info exists W]} {unset W; unset Where}
	    set W     ""
	    set Where ""
	    set W     [lindex $Ligne 0]
	    set Where [lindex $Ligne 1]
	    if {![info exists W]} {continue}
	    
	    #1- Serie de tests pour verifier que les repertoires existent ou que les programmes s executent
	    set W_Exists 0
	    set W_Exec   0
	    set Commande ""
	    set Commande $Where

	    #1-1 Test si c un executable
	    if {[catch {eval exec $Commande} Message]} {
		if {[regexp -nocase "invalid command name" $Message] || [regexp -nocase "no such file or directory" $Message]} {
		    set W_Exec 0
		} else { 
		    set W_Exec 1
		}
	    } else {
		set     W_Exec 1
	    }

	    #1-2 Test si c un repertoire ou un fichier existant
	    if {[file exists $Commande]} {
		set W_Exists 1
	    } else {
		set W_Exists 0
	    } 
	    
	    #1-3 Decision Finale
	    if {$W_Exec || $W_Exists} {
		set CADO4MI_PATH($W) $Where  
	    } else {
		#set CADO4MI_PATH($W) ""
	    }
	}
	close $F
    }
    
    #2- On test egalement notre executable ou dir (What) qui peut ne pas etre dans la liste chargee
    set W_Exists 0
    set W_Exec   0
    if {      [info exists CADO4MI_PATH($What)]} {
	set Commande  [set CADO4MI_PATH($What)]
    } elseif {[info exists CADO4MI_PATH(Loaded)]} {
	set Commande  $What
    } else {
	set Commande  $What
    }

    #2-1 Test si c un executable
    if {[catch {eval exec $Commande} Message]} {
	if {[regexp -nocase "invalid command name" $Message] || [regexp -nocase "no such file or directory" $Message]} {
	    set W_Exec 0
	} else { 
	    set W_Exec 1
	}
    } else {
	set     W_Exec 1
    }

    #2-2 Test si c un repertoire ou un fichier existant
    if {[file exists $Commande]} {
	set W_Exists 1
    } else {
	set W_Exists 0
    }

    #2-3 Decision Finale
    if {$W_Exec || $W_Exists} {
	set CADO4MI_PATH($What) $Commande
    } else {
	set CADO4MI_PATH($What) ""
    }
    
    return [CADO4MI_PATH $What]
}

#####################################################################
#   1.Panel de demarrage du programme de design d oligonucleotide   #
#####################################################################
proc CADO4MI_Panel {{w ""}} {
    
    Wup "Interface graphique de CADO4MI"
    Wup "Permet de lancer un design avec des criteres restrictifs"
    Wup "Permet de visualiser les resultats du design"
    Wup "Permet de visualiser certains des criteres de choix"

    Wup "MAJ Ajout de l interface vers IllustreOligo"
    Wup "MAJ Ajout advanced parameters (step and blastn parameters, Batch mode)"
    Wup "MAJ Ajout File management (delete files et save to excel file)"
    Wup "MAJ Ajout des bulles infos, restriction pour les entrees"
    Wup "MAJ Ajout de la gestion des zones de design specifiques (liste debut et fin)"
    Wup "MAJ Ajout batch parameters (Batch mode, each steps can be done independently plus X-Selection and Save results in tab file)"
    Wup "MAJ Ajout de la gestion des zones de design globales (distance du 3prime)"
    Wup "MAJ Ajout Best GC et Tm calculation"
    Wup "MAJ Ajout de Font management (gestion des tailles et autres fontes)"
    Wup "MAJ Ajout de la version de CADO4MI"
    Wup "MAJ Ajout de la advanced parameters pour Tm (salt, form et DNA concentration)"
    Wup "MAJ Ajout sauvegarde des parametres de design (Design.parameters)"
    Wup "MAJ Ajout sauvegarde des parametres de design et recharge"
    Wup "MAJ Refonte majeur de l interface avec ajout d un menu pour simplifier"
    Wup "MAJ 17/05/2006"

    #On gere la premiere fenetre en . si il y deja autre chose d utiliser dans .
    #ATTENTION on doit modifier . en "" pour pouvoir conserver la hierarchie des fenetres
    if {[winfo children .] != ""} {
	set       w ".startCADO4MI"
	if {[winfo exists $w]} {return ""}
	toplevel $w
    } else {
	set w "."
	#$w config -bd 3 -relief flat
    }
    wm geometry  $w +300+100
    wm title     $w "CADO4MI: Oligonucleotide Design"
    wm resizable $w false false
    #wm resizable $w true true
    wm protocol  $w WM_DELETE_WINDOW "exit"
    if {$w == "."} {set w ""}

    #0 Le titre
    set  Titre  "CADO4MI [CADO4MI_Version]"
    set FTitre  "$w.ftitre"
    set  LTitre "$FTitre.ltitre"
    frame $FTitre -borderwidth "3" -relief "groove"
    label $LTitre -borderwidth "2" -relief "ridge"  -background "orange" -text "$Titre" -width "45" -pady "3"
    bind  $LTitre <Enter> "$LTitre configure -relief sunken -background red    -text \"----------- EXIT -----------\""
    bind  $LTitre <Leave> "$LTitre configure -relief ridge  -background orange -text \"CADO4MI [CADO4MI_Version]\""
    if {$w ==  ""} {set w "."}
    bind  $LTitre <1> "eval \[wm protocol $w WM_DELETE_WINDOW\]"
    if {$w == "."} {set w ""}

    #Les 2 frames majeures
    #1 Predesigned Frame pour dessiner les resultats (IllustreOligo Panel)
    #2 Design      Frame pour effectuer un design (Parameters Panel)
    #3 Batch Mode  Frame Batch mode
    set    FPreDesigned "$w.fpredesigned"
    set    FDesign      "$w.fdesign"
    set    FBatch       "$w.fbatchmode"
    frame $FPreDesigned -borderwidth "3" -relief "groove"
    frame $FDesign      -borderwidth "3" -relief "groove"
    frame $FBatch       -borderwidth "3" -relief "groove"

    #0.1 Le Menu
    set   Menu "$w.menu"
    menu $Menu -tearoff 0
      
    set modifier "Control"

    #0.1.1 Menu Project
    set   m "$Menu.project"
    menu $m -tearoff 0
    $Menu add cascade -label "Project" -menu $m -underline 0
    $m    add command -label "Parameters"
    $m    add command -label "--Load..."    -accelerator $modifier+L -command {MenuChargeLesParametres;MontreLesParametresDuDesign}
    $m    add command -label "--Save As..." -accelerator $modifier+S -command {MenuSauveLesParametres}                             
    $m    add separator
    $m    add command -label "Manage files"                          -command {ManageFiles}
    $m    add separator

    if {$w ==  ""} {set w "."}
    bind $w <$modifier-l> {MenuChargeLesParametres;MontreLesParametresDuDesign}
    bind $w <$modifier-s> {MenuSauveLesParametres}
    $m    add command -label "Exit"                                  -command "eval \[wm protocol $w WM_DELETE_WINDOW\]"
    if {$w == "."} {set w ""}

    #0.1.2 Menu Parameters
    set   m "$Menu.parameters"
    menu $m -tearoff 0
    $Menu add cascade -label "Parameters" -menu $m -underline 0
    $m    add command -label "Define" -accelerator $modifier+D -command "HideGridedFrame $FDesign"
    $m    add command -label "Check"  -accelerator $modifier+H -command {MontreLesParametresDuDesign}
    $m    add command -label "Reset"  -accelerator $modifier+R -command {
	LesParametresDuDesign UnsetAll
	set Warning "All designing parameters set to null and default restored."
	AjouteAuLog "$Warning"
	LesParametresParDefaut
    }  
    $m    add separator
    $m    add command -label "Font setting" -command {Font_Panel}

    if {$w ==  ""} {set w "."}
    bind $w <$modifier-d> "HideGridedFrame $FDesign"
    bind $w <$modifier-h> {MontreLesParametresDuDesign}
    bind $w <$modifier-r> {
	LesParametresDuDesign UnsetAll
	set Warning "All designing parameters set to null and default restored."
	AjouteAuLog "$Warning"
	LesParametresParDefaut
    }  
    if {$w == "."} {set w ""}

    #0.1.3 Menu Run
    set   m "$Menu.run"
    menu $m -tearoff 0
    $Menu add cascade -label "Run"   -menu $m -underline 0 
    $m    add command -label "Design" -command {
	if {[BonNombreDeParametres "AvecVerif"]} {
	    MontreLesParametresDuDesign
	    if {[OuiOuNon "Are you ready to launch the design ?"]} {	
		set Fichier "[file join [LesParametresDuDesign WorkingDir "LaValeurMerci"] "Design.parameters"]"
		#SauveLesParametresDuDesign $Fichier
		MenuSauveLesParametres $Fichier
		
		OligoLauncher
	    } else {
		AjouteAuLog "I wait ..."
	    }
	} else {
	    AjouteAuLog "Missing parameters."
	    AjouteAuLog "You must define at least Length, Tm and Tm range, Database and saving directory."
	}
    }
    $m    add command -label "Batch Mode" -command "HideGridedFrame $FBatch"

    #0.1.4 Menu Display
    set   m "$Menu.display"
    menu $m -tearoff 0
    $Menu add cascade -label "Display" -menu $m -underline 0
    $m    add command -label "Results" -command "HideGridedFrame $FPreDesigned"

    #0.1.5 Menu Help
    set   m "$Menu.help"
    menu $m -tearoff 0
    $Menu add cascade -label "?" -menu $m -underline 0
    $m    add command -label "Help" 
    $m    add separator
    $m    add command -label "Copyright" -command {CADO4MICopyright}

    if {$w ==  ""} {set w "."}
    $w configure -menu $Menu
    if {$w == "."} {set w ""}

    #1.1 IllustreOligo panel
    global E1Illustre2
    global E2Illustre2
    set FIllustre1   "$FPreDesigned.fillustre1"
    set  LIllustre1  "$FIllustre1.lillustre1"
    set  BIllustre1  "$FIllustre1.billustre1"
    set FIllustre2   "$FPreDesigned.illustre2"
    set  L1Illustre2 "$FIllustre2.l1illustre2"
    set  E1Illustre2 "$FIllustre2.e1illustre2"
    set  B2Illustre2 "$FIllustre2.b2illustre2"
    set  L2Illustre2 "$FIllustre2.l2illustre2"
    set  E2Illustre2 "$FIllustre2.e2illustre2"
    set  B4Illustre2 "$FIllustre2.b4illustre2"
    set  B5Illustre2 "$FIllustre2.b5illustre2"

    frame  $FIllustre1  -borderwidth "1"          -relief "ridge" 
    label  $LIllustre1  -background  "#808FFF"    -relief "groove" -text "Design Result Menu"
    button $BIllustre1  -background  "OliveDrab1" -relief "ridge"  -text "Show / Hide" -padx "2" -command "HideGridedFrame $FIllustre2"
    frame  $FIllustre2  -background  "#808080"    -relief "groove" -borderwidth "0"
    label  $L1Illustre2 -background  "#808FFF"    -relief "groove" -text "File:"       -padx "3" -width "10" -anchor "w"
    entry  $E1Illustre2 -textvariable PAB		      
    button $B2Illustre2 -background  "yellow"     -padx "2"        -text "Browse" -command {	
	set types {
	    {{CADO4MI files} {.masked}}
	    {{Fasta files}   {.tfa}}
	    {{Fasta files}   {.fa}}
	    {{Fasta files}   {.fasta}}
	    {{All files}      *}
	}
	if {[BROWSEDDIR] != ""} {
	    set BrowsedPAB [tk_getOpenFile -multiple "y" -title "Choose 1 or more sequence(s)." -initialdir "[BROWSEDDIR]" -filetypes $types]
	} else {
	    set BrowsedPAB [tk_getOpenFile -multiple "y" -title "Choose 1 or more sequence(s)." -initialdir "[HOMEDIR]"    -filetypes $types]
	}
	if {$BrowsedPAB == "" } {
	    return ""
	} else {
	    set BrowsedDir [file dirname [lindex $BrowsedPAB end]]
	    BROWSEDDIR $BrowsedDir
	}
	$E1Illustre2 delete 0 end
	$E1Illustre2 insert   end $BrowsedPAB
    }
    label  $L2Illustre2 -background  "#808FFF"    -relief "groove" -text "Directory:"  -padx "3" -width "10" -anchor "w"
    entry  $E2Illustre2 -textvariable ProbesDir
    button $B4Illustre2 -background  "yellow"     -padx   "2"      -text "Browse" -command {
	if {[BROWSEDDIR] != ""} {
	    set BrowsedDir [tk_chooseDirectory -title "Choose one directory." -initialdir "[BROWSEDDIR]"]
	} else {
	    set BrowsedDir [tk_chooseDirectory -title "Choose one directory." -initialdir "[HOMEDIR]"]
	}
	if {$BrowsedDir == "" } {
	    #BROWSEDDIR ResetBROWSEDDIR
	    return ""
	} else {
	    BROWSEDDIR $BrowsedDir
	}

	if {[info exists ProbesDir] && $ProbesDir != ""} {
	    set BrowsedDir "${ProbesDir} ${BrowsedDir}"
	}

	$E2Illustre2 delete 0 end
	$E2Illustre2 insert   end $BrowsedDir
    }
    button $B5Illustre2 -background  "yellow"     -padx   "2"      -text "OK"     -command {
	#Rq: Il est necessaire de positionner LesSeqs et LesDirs pour IllustreOligo
	if {[info exists TabDir]}     {array unset TabDir     "*"}
	if {[info exists TabDir_Tmp]} {array unset TabDir_Tmp "*"}

	set LesSeqs ""
	set LesDirs ""

	if {$PAB=="" && $ProbesDir==""} {
	    #1- Acc et Directory sont vides
	    #   On rend la main et on ne fait rien
	    set Warning "You must choose either a file or a directory containing designed oligos or both."
	    AjouteAuLog $Warning
	    return ""
	} elseif {$PAB!="" && $ProbesDir==""} {
	    #2- Acc existe et Directory est vide
	    #   On cherche le directory certainement contenu dans le nom du fichier ou alors un repertoire par defaut
	    set LesSeqs $PAB
	    if {[regexp {\\} $PAB] || [regexp {/} $PAB]} {
		#Cas particulier: Multiple PAB avec Multiple directory differents
		#On doit stocker uniquement le directory du PAB et pour lui seul
		foreach Q $PAB {
		    set Dir ""
		    set Dir [file dirname $Q]
		    if {[file exists $Dir]} {
			set TabDir($Q) $Dir
		    }
		}
	    } else {
		set     RepOligo "[CADO4MI_PATH Probes_DIR]"
		if {   $RepOligo=="" || ! [file exists $RepOligo]} {
		    set RepOligo "[file join [RepertoireDuGenome] Probes]"
		}
		if {[file exists $RepOligo]} {
		    set LaListe ""
		    foreach File [glob -nocomplain -directory $RepOligo -types d "*"] {
			lappend LaListe [file tail $File]
		    }
		    if {$LaListe==""} {AjouteAuLog "No Directory found.";return ""}
		    set  LaListe [EntreDeux [lsort $LaListe]]
		    if {$LaListe==""} {AjouteAuLog "No Directory selected.";return ""}
		    foreach Dir $LaListe {
			lappend LesDirs "[file join $RepOligo $Dir]"
		    }
		} else {AjouteAuLog "No Directory specified.";return ""}
	    }
	} elseif {$PAB=="" && $ProbesDir!=""} {
	    #3- Acc est vide et Directory existe
	    #   On cherche les access contenus dans les directory
	    #   On utilise que le premier directory pour les access

	    #3-1 Recuperation de tout les access dans tout les repertoires	
	    set LaListe ""
	    set Dir     ""
	    foreach Dir $ProbesDir {
		if {![file exists $Dir]} {
		    set Warning "$Dir does not exists.\nDirectory should exist and contain previously designed oligo(s)."
		    AjouteAuLog $Warning
		    continue
		}  
		lappend LesDirs $Dir
		
		set File ""
		foreach File [glob -nocomplain -directory $Dir "*.masked"] {
		    regsub {.masked$} $File "" File
		    set File [file tail $File]
		    lappend LaListe $File
		    
		    if {! [info exists TabDir_Tmp($File)]} {
			set TabDir_Tmp($File) ""
		    }
		    lappend TabDir_Tmp($File) $Dir
		}
	    }
	    set LaListe [lsort -unique $LaListe]
	    
	    #3-2 Formatage de la liste des Acc
	    set Acc         "" 
	    set LaListe_Tmp ""
	    set NbDir       [llength $LesDirs]
	    foreach Acc $LaListe {
		if {$NbDir=="1"} {
		    lappend LaListe_Tmp "$Acc"
		} elseif {[llength [set TabDir_Tmp($Acc)]] == $NbDir} {
		    lappend LaListe_Tmp "$Acc (All)"
		} else {
		    set DirsDeAcc ""
		    foreach D [set TabDir_Tmp($Acc)] {
			lappend DirsDeAcc [file tail $D]
		    }
		    lappend LaListe_Tmp "$Acc ([string trim [join $DirsDeAcc " "]])"
		}
	    }
	    set LaListe $LaListe_Tmp

	    #3-3 Creation de liste choisie par l utilisateur
	    if {$LaListe==""} {AjouteAuLog "No sequence found.";return ""}
	    set  LaListe [EntreDeux [lsort $LaListe]]
	    if {$LaListe==""} {AjouteAuLog "No sequence selected.";return ""}
	    foreach Acc $LaListe {
		set Acc_S [split  $Acc " "]
		set Acc   [lindex $Acc_S 0] 
		
		#Par defaut on donne le chemin du 1er repertoire si il en existe plusieurs pour le meme
		set RepOligo [lindex [set TabDir_Tmp($Acc)] 0]
		set   UnPAB "[file join $RepOligo "${Acc}.masked"]"
		set  TabDir($UnPAB) [set TabDir_Tmp($Acc)] 
		lappend LesSeqs $UnPAB
	    }
	    if {[info exists TabDir_Tmp]} {array unset TabDir_Tmp "*"}
	} elseif {$PAB!="" && $ProbesDir!=""} {
	    #4- Acc et Directory existent
	    set LesSeqs $PAB
	    foreach Dir $ProbesDir {
		if {![file exists $Dir]} {
		    set Warning "$Dir does not exists.\nDirectory should exist and contain previously designed oligo(s)."
		    AjouteAuLog $Warning
		    continue
		}  
		lappend LesDirs $Dir
	    }
	}

	if {$LesSeqs=="" && $LesDirs==""} {
	    if {[info exists TabDir]}     {array unset TabDir     "*"}
	    if {[info exists TabDir_Tmp]} {array unset TabDir_Tmp "*"}

	    set Warning "Neither Acc nor directory could have been selected."
	    AjouteAuLog $Warning
	    return ""
	}

	FaireAttendre .waitapply "Waiting" "Please wait ...\nDisplaying design results ..."
	foreach Seq $LesSeqs {
	    if {[info exists TabDir($Seq)]} {
		IllustreOligo $Seq [set TabDir($Seq)]
	    } else {
		IllustreOligo $Seq $LesDirs
	    }
	}
	FaireLaSuite  .waitapply
	if {[info exists TabDir]}     {array unset TabDir     "*"}
	if {[info exists TabDir_Tmp]} {array unset TabDir_Tmp "*"}
    }
    set_balloon $LIllustre1  "To display the design of oligonucleotides"
    set_balloon $L1Illustre2 "One or several file name with existing designed oligonucleotide.\nYou can use a combination of File and Directory."
    set_balloon $L2Illustre2 "One or several directories containing design results.\nYou can use a combination of File and Directory."

    #Les 2 frames des parametres
    #2.1 Menu du design
    #2.2 Frame de tous les parametres
    set FParamMenu         "$FDesign.fparammenu"
    set  LParamMenu        "$FParamMenu.lparammenu"
    set  BParamMenu        "$FParamMenu.bparammenu"
    set FTousLesParameters "$FDesign.fparameters"

    frame  $FParamMenu -borderwidth "1"          -relief "ridge"  
    label  $LParamMenu -background  "#808FFF"    -relief "groove" -text "Oligo Design Menu"
    button $BParamMenu -background  "OliveDrab1" -relief "ridge"  -text "Show / Hide" -padx "2" -command "HideGridedFrame $FTousLesParameters"
    frame  $FTousLesParameters -borderwidth "0"  -relief "groove" 
    set_balloon $LParamMenu "To start the design of oligonucleotides"

    #Les petites frames, parametres, Tm methode, menu, log
    #2.2.1 Frame des parametres query, edge, GC ...
    #2.2.3 Frame Advanced Parameters step, specificity et BLAST
    set FParameters1 "$FTousLesParameters.fparameters1"
    set FParameters3 "$FTousLesParameters.fparameters3"

    frame $FParameters1 -borderwidth "0" -relief "groove"
    frame $FParameters3 -borderwidth "0" -relief "groove"

    #2.2.1.1 Query panel
    global EQuery
    global BQuery
    set FQuery   "$FParameters1.fquery"
    set  LQuery  "$FQuery.lquery"
    set  EQuery  "$FQuery.equery"
    set  BQuery  "$FQuery.bquery"
    set  B2Query "$FQuery.b2query"
    set  B3Query "$FQuery.b3query"

    frame  $FQuery  -background "#808080" -relief "groove"
    label  $LQuery  -background "#808FFF" -relief "groove" -text "Query" -padx "3" -width "10"      -anchor "w"
    entry  $EQuery  -textvariable QueryVar
    button $B3Query -background  "orange" -relief "groove" -text "Info"  -padx "3" -borderwidth "2" -command {
	if {[LesParametresDuDesign Query "LaValeurMerci"]!=""} {
	    set LesQuerys [LesParametresDuDesign Query "LaValeurMerci"]
	    foreach Qy $LesQuerys {
		AfficheLesInfosDeMaSequence $Qy
	    }
	} else {
	    AjouteAuLog "Query must exist and also being validated"
	}
    }
    button $B2Query -background "yellow"  -padx "2"        -text "Browse" -command {
	set types {
	    {{Fasta files}   {.tfa}}
	    {{Fasta files}   {.fa}}
	    {{Fasta files}   {.fasta}}
	    {{CADO4MI files} {.masked}}
	    {{All files}      *}
	}
	if {[BROWSEDDIR] != ""} {
	    set BrowsedQuery [tk_getOpenFile -multiple "y" -title "Choose at least one sequence file." -initialdir "[BROWSEDDIR]" -filetypes $types]
	} else {
	    set BrowsedQuery [tk_getOpenFile -multiple "y" -title "Choose at least one sequence file." -initialdir "[HOMEDIR]"    -filetypes $types]
	}
	if {$BrowsedQuery == "" } {
	    return ""
	} else {
	    set BrowsedDir [file dirname [lindex $BrowsedQuery end]]
	    BROWSEDDIR $BrowsedDir
	}

	$EQuery delete 0 end
	$EQuery insert   end $BrowsedQuery
	$BQuery invoke
    }
    button $BQuery  -background "yellow"  -padx "2"        -text "OK"     -command {
	if {$QueryVar!=""} {
	    if {$QueryVar=="ActiChip1.0" || [regexp -nocase "actichip1" $QueryVar]} {set QueryVar [ActiChip1_0]}
	    if {$QueryVar=="ActiChip2.0" || [regexp -nocase "actichip2" $QueryVar]} {set QueryVar [ActiChip2_0]}
	    if {$QueryVar=="ActiChip3.0" || [regexp -nocase "actichip3" $QueryVar]} {set QueryVar [ActiChip3_0]}
	    if {$QueryVar==""} {
		LesParametresDuDesign Query "Reset"
		set Warning "Query must be a regular sequence file."
		AjouteAuLog $Warning
	    } else {
		if {[llength $QueryVar]=="1"} {
		    #On a un seul fichier
		    #Test si on un single ou multiple fasta file
		    set  NbTFA  "0"
		    set  NbTFA  [EstCeUnTFAOuUnTFAS $QueryVar]
		    if {$NbTFA=="0"} {
			AjouteAuLog "Query: $QueryVar is not a sequence in fasta format."
		    } elseif {$NbTFA=="1"} {
			LesParametresDuDesign Query $QueryVar "Positionne"
			AjouteAuLog "Query: $QueryVar"
		    } else {
			set  Return [CheckTFAFile $QueryVar]
			if {$Return!=""} {
			    #On a un multi fasta
			    LesParametresDuDesign Query           "Reset"
			    LesParametresDuDesign Query "$Return" "Positionne"
			    set QueryVar [LesParametresDuDesign Query "LaValeurMerci"]
			    AjouteAuLog "Query: $QueryVar"
			}
		    }
		} else {
		    #On a plusieurs fichiers
		    set LesMauvais ""
		    foreach Q $QueryVar {
			set  NbTFA  "0"
			set  NbTFA  [EstCeUnTFAOuUnTFAS $Q]
			if {$NbTFA=="0"} {
			    lappend LesMauvais $Q
			}
		    }
		    if {$LesMauvais != ""} {
			AjouteAuLog "Query: Be careful $LesMauvais seems not to be in single fasta format."
		    }
		    LesParametresDuDesign Query $QueryVar "Positionne"
		    AjouteAuLog "Query: $QueryVar"
		}
		#AjouteAuLog "Query: $QueryVar"
	    }
	} else {
	    LesParametresDuDesign Query "Reset"
	    set Warning "Query must be a regular sequence file."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $LQuery  "Single or multiple fasta sequence file(s)."
    set_balloon $B3Query "Display sequence analysis (Tm and GC profile)\nand allows direct selection of designing areas."

    #2.2.1.2 Edge panel
    set FEdge   "$FParameters1.fedge"
    set  LEdge  "$FEdge.ledge"
    set  L2Edge "$FEdge.l2edge"
    set  EEdge  "$FEdge.eedge"
    set  BEdge  "$FEdge.bedge"

    frame  $FEdge  -background "#808080"           -relief "groove"
    label  $LEdge  -background "#808FFF" -padx "3" -width "10" -relief "groove" -text "Distance"          -anchor "w"
    label  $L2Edge -background "#808FFF" -padx "3" -width  "5" -relief "groove" -text "nt(s) from 3' end" -anchor "w" 
    entry  $EEdge                                  -width "10" -textvariable EdgeBegin
    button $BEdge  -background "yellow"  -padx "2"                              -text "OK"                -command {
	if {$EdgeBegin!=""} {
	    if {$EdgeBegin<="0"} {set EdgeBegin 10000000}
	  	    
	    set EdgeEnd "End"

	    LesParametresDuDesign Areas                            "Reset"
	    LesParametresDuDesign Areas [list $EdgeBegin $EdgeEnd] "Ajoute"
	    AjouteAuLog "Design from $EdgeBegin to the $EdgeEnd"
	} else {
	    set Warning "Distance from 3' end is an option."
	    AjouteAuLog $Warning
	}
    } 
    set_balloon $LEdge  "Global designing areas defined for all queries.\nWill be used if no specific area are defined for the query."

    #2.2.1.3 Taille panel
    set FTaille   "$FParameters1.ftaille"
    set  LTaille  "$FTaille.ltaille"
    set  L2Taille "$FTaille.l2taille"
    set  ETaille  "$FTaille.etaille"
    set  BTaille  "$FTaille.btaille"

    frame  $FTaille  -background "#808080"           -relief "groove"
    label  $LTaille  -background "#808FFF" -padx "3" -relief "groove" -text "Length" -width "10" -anchor "w"
    label  $L2Taille -background "#808FFF" -padx "3" -relief "groove" -text "nucleotide(s)"      -anchor "w"
    entry  $ETaille  -justify    "center"  -textvariable LengthVar                   -width  "5"    
    button $BTaille  -background "yellow"  -padx "2"                  -text   "OK"   -command {
	if {$LengthVar!=""} {
	    if {$LengthVar>100 | $LengthVar<=0} {
		set Warning "Please enter a regular value for Length\nLength: 0<Length<=100"
		AjouteAuLog $Warning
	    } else {
		    LesParametresDuDesign LengthOligo $LengthVar "Positionne"
		AjouteAuLog "Length: $LengthVar nt(s)"	
	    }
	} else { 
	    LesParametresDuDesign LengthOligo "Reset"
	    set Warning "Choose one value for Length."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $LTaille "Length of Oligo (between 1 and 100 nt)"

    #2.2.1.4 NombreOligo panel
    set FNbOligo   "$FParameters1.fnboligo"
    set  LNbOligo  "$FNbOligo.lnboligo"
    set  L2NbOligo "$FNbOligo.l2nboligo"
    set  ENbOligo  "$FNbOligo.enboligo"
    set  BNbOligo  "$FNbOligo.bnboligo"

    frame  $FNbOligo  -background "#808080"           -relief "groove"
    label  $LNbOligo  -background "#808FFF" -padx "3" -relief "groove" -text "Number" -width "10" -anchor "w"
    label  $L2NbOligo -background "#808FFF" -padx "3" -relief "groove" -text "oligonucleotide(s)" -anchor "w"
    entry  $ENbOligo  -justify    "center"  -textvariable NbOligoVar                  -width  "5" 
    button $BNbOligo  -background "yellow"  -padx "2"                  -text "OK"     -command {
	if {$NbOligoVar!=""} {
	    if {$NbOligoVar>1000000 | $NbOligoVar<=0} {
		set Warning "Please enter a regular value for the Number of Oligo to design \nNbOligo: 0<NbOligo<=1000000"
		AjouteAuLog $Warning
	    } else {
		LesParametresDuDesign NbOligo $NbOligoVar "Positionne"
		AjouteAuLog "NbOligo: $NbOligoVar oligo(s)"	
	    }
	} else {
	    LesParametresDuDesign NbOligo "Reset"
	    set Warning "Choose one value for NbOligo."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $LNbOligo "Number of desired Oligo to keep (between 1 and 1000000, default set to 100)"

    #2.2.1.5 Tm panel
    set FTm   "$FParameters1.ftm"
    set  LTm  "$FTm.ltm"
    set  L2Tm "$FTm.l2tm"
    set  L3Tm "$FTm.l3tm"
    set  ETm  "$FTm.etm"
    set  E2Tm "$FTm.e2tm"
    set  BTm  "$FTm.btm"
    set  B2Tm "$FTm.b2tm"

    frame  $FTm  -background "#808080"                       -relief  "groove"
    label  $LTm  -background "#808FFF" -padx "3" -width "10" -relief  "groove" -text "Tm"  -anchor "w"
    label  $L2Tm -background "#808FFF" -padx "3" -width  "3" -relief  "groove" -text "+/-" -anchor "center"
    label  $L3Tm -background "#808FFF" -padx "3"             -relief  "groove" -text "C"  -anchor "w"
    entry  $ETm  -textvariable TmVar             -width  "5" -justify "center"
    entry  $E2Tm -textvariable TmRange           -width  "5" -justify "center"
    button $BTm  -background "yellow"  -padx "2"                               -text "OK"  -command {
	if {$TmVar!="" && $TmRange!=""} {	   
	    if {   $TmVar == "."} {
		set TmVar "0.0"
	    } elseif {! [regexp {\.[0-9]+} $TmVar] || ! [regexp {[0-9]+\.} $TmVar]} {
		set TmVar [format "%.1f" $TmVar]
	    }
	    if {   $TmRange == "."} {
		set TmRange "0.0"
	    } elseif {! [regexp {\.[0-9]+} $TmRange] || ! [regexp {[0-9]+\.} $TmRange]} {
		set TmRange [format "%.1f" $TmRange]
	    }

	    if {$TmVar>120.0 | $TmVar<=0.0 | $TmRange>=$TmVar | $TmRange<0.0} {
		set Warning "Please enter a regular value for Tm and Range\nTm: 0.0<Tm<=120.0\nRange: 0.0<=Range<TmVar"
		AjouteAuLog $Warning
	    } else {
		LesParametresDuDesign TmLimit $TmVar   "Positionne"
		LesParametresDuDesign TmRange $TmRange "Positionne"
		AjouteAuLog "Tm: $TmVar \C +/- $TmRange \C"
	    }
	} else {
	    LesParametresDuDesign TmLimit "Reset"
	    LesParametresDuDesign TmRange "Reset"
	    set Warning "Choose one value for both Tm and Tm Range."
	    AjouteAuLog $Warning
	}
    }
    button $B2Tm -background "orange"  -padx "3"             -relief "groove" -text "Best" -borderwidth "2" -command {
	if {[LesParametresDuDesign Query "LaValeurMerci"]!=""} {
	    FaireAttendre .waitapply "Waiting" "Please wait ...\nComputing Best Tm analysis ..."
	    BestTmAnalysis
	    FaireLaSuite  .waitapply
	} else {
	    AjouteAuLog "Query must exist and also being validated to calculate Best Tm analysis"
	}
    }
    set_balloon $LTm  "Melting temperature (Tm) in degree (between 0 and 120 C)"
    set_balloon $L2Tm "Tm Range in degree < Tm"
    set_balloon $B2Tm "Calculate and display best Tm analysis"

    #2.2.1.6 GC content panel
    set FGCContent   "$FParameters1.fgccontent"
    set  LGCContent  "$FGCContent.lgccontent"
    set  L2GCContent "$FGCContent.l2gccontent"
    set  L3GCContent "$FGCContent.l3gccontent"
    set  EGCContent  "$FGCContent.egccontent"
    set  E2GCContent "$FGCContent.e2gccontent"
    set  BGCContent  "$FGCContent.bgccontent"
    set  B2GCContent "$FGCContent.b2gccontent"

    frame  $FGCContent  -background "#808080"                       -relief  "groove"
    label  $LGCContent  -background "#808FFF" -padx "3" -width "10" -relief  "groove" -text "GC % from" -anchor "w"
    label  $L2GCContent -background "#808FFF" -padx "3" -width  "3" -relief  "groove" -text "to"        -anchor "center"
    label  $L3GCContent -background "#808FFF" -padx "3" -width  "3" -relief  "groove" -text "%"         -anchor "w"
    entry  $EGCContent  -textvariable GCContentMinusVar -width  "5" -justify "center" 
    entry  $E2GCContent -textvariable GCContentPlusVar  -width  "5" -justify "center" 
    button $BGCContent  -background "yellow"  -padx "2"                               -text "OK"        -command {
	if {$GCContentMinusVar!="" && $GCContentPlusVar!=""} {
	    if {   $GCContentMinusVar == "."} {
		set GCContentMinusVar "0.0"
	    } elseif {! [regexp {\.[0-9]+} $GCContentMinusVar] || ! [regexp {[0-9]+\.} $GCContentMinusVar]} {
		set GCContentMinusVar [format "%.1f" $GCContentMinusVar]
	    }
	    if {   $GCContentPlusVar == "."} {
		set GCContentPlusVar "0.0"
	    } elseif {! [regexp {\.[0-9]+} $GCContentPlusVar] || ! [regexp {[0-9]+\.} $GCContentPlusVar]} {
		set GCContentPlusVar [format "%.1f" $GCContentPlusVar]
	    }
	    	    
	    if {$GCContentMinusVar>100.0 | $GCContentMinusVar<0.0 | $GCContentPlusVar>100.0 | $GCContentPlusVar<0.0 | $GCContentMinusVar>$GCContentPlusVar} {
		set Warning "Please enter a regular value for the GC Content:\n0<\%GC<=100\nFirst<Second"
		AjouteAuLog $Warning
	    } else {
		LesParametresDuDesign GCContentMinus $GCContentMinusVar "Positionne"
		LesParametresDuDesign GCContentPlus  $GCContentPlusVar  "Positionne"
		AjouteAuLog "% GC lower: $GCContentMinusVar \%"
		AjouteAuLog "% GC upper: $GCContentPlusVar \%" 
	    }
	} else {
	    LesParametresDuDesign GCContentMinus "Reset"
	    LesParametresDuDesign GCContentPlus  "Reset"
	    set Warning "GC Content is an option."
	    AjouteAuLog $Warning
	}
    }

    button $B2GCContent -background "orange"  -padx "3"             -relief  "groove" -text "Best" -borderwidth "2" -command {
	if {[LesParametresDuDesign Query "LaValeurMerci"]!=""} {
	    FaireAttendre .waitapply "Waiting" "Please wait ...\nComputing Best GC content analysis ..."
	    BestGCAnalysis
	    FaireLaSuite  .waitapply
	} else {
	    AjouteAuLog "Query must exist and also being validated to calculate Best GC analysis"
	}
    }
    set_balloon $LGCContent  "GC content to avoid strong or weak interaction (0.0-100.0 %)"
    set_balloon $B2GCContent "Calculate and display best GC analysis"

    #2.2.1.7 Prohibited panel
    set FProhib   "$FParameters1.fprohib"
    set  LProhib  "$FProhib.lprohib"
    set  EProhib  "$FProhib.eprohib"
    set  BProhib  "$FProhib.bprohib"
    set  B2Prohib "$FProhib.bprohib2"
    set  B3Prohib "$FProhib.bprohib3"

    frame  $FProhib  -background "#808080"           -relief "groove"
    label  $LProhib  -background "#808FFF" -padx "3" -relief "groove" -text "Prohibited" -width "10" -anchor "w"
    entry  $EProhib  -textvariable ProhibVar
    button $BProhib  -background "yellow"  -padx "2"                  -text "Add"        -command {
	if {$ProhibVar!=""} {
	    set LesProhibExistant [LesParametresDuDesign Prohibited "LaValeurMerci"]
	    LesParametresDuDesign Prohibited $ProhibVar "Ajoute"
	    AjouteAuLog "Prohibited Sequence $ProhibVar added to previous: $LesProhibExistant"
	} else {
	    set Warning "Prohibited Sequence is an option."
	    AjouteAuLog $Warning
	}
    }
    button $B2Prohib -background "yellow"  -padx "2"                  -text "Reset"      -command {
	LesParametresDuDesign Prohibited "Reset"
	set Warning "Prohibited Sequence reset to null."
	AjouteAuLog $Warning
    }
    button $B3Prohib -background "yellow"  -padx "2"                  -text "OK"         -command {
	if {$ProhibVar!=""} {
	    LesParametresDuDesign Prohibited $ProhibVar "Positionne"
	    AjouteAuLog "Prohibited Sequence: $ProhibVar"
	} else {
	    set Warning "Prohibited Sequence is an option."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $LProhib "Sequence you want to avoid from the oligo(s) (only A,T,G, or C)\nex: AAA TTT GCCC (blank separated)"
    
    #2.2.1.8 Database panel
    global EDataBase 
    global BDataBase  
    set FDataBase   "$FParameters1.fdatabase"
    set  LDataBase  "$FDataBase.ldatabase"
    set  EDataBase  "$FDataBase.edatabase"
    set  BDataBase  "$FDataBase.bdatabase"
    set  B2DataBase "$FDataBase.b2database"

    frame  $FDataBase  -background "#808080"           -relief "groove"
    label  $LDataBase  -background "#808FFF" -padx "3" -relief "groove" -text "DataBase" -width "10" -anchor "w" 
    entry  $EDataBase  -textvariable DataBaseVar              
    button $B2DataBase -background "yellow"  -padx "2"                  -text "Browse"   -command {
	#Les types de fichier autorises
	set types {
	    {{Nt blast files} {.nsq}}
	    {{Nt blast files} {.nal}}
	    {{All files}       *}
	}
	set BrowsedDB [tk_getOpenFile -title "Choose one database." -initialdir "[BLASTDBDIR]" -filetypes $types]

	$EDataBase delete 0 end
	$EDataBase insert   end $BrowsedDB
	$BDataBase invoke
    }
    button $BDataBase  -background "yellow"  -padx "2"                  -text "OK"       -command {
	if {$DataBaseVar!=""} {
	    if {$DataBaseVar=="none"} {
		set Warning "Database is none, that means that you already have blastn files."
		AjouteAuLog $Warning
		LesParametresDuDesign DataBase $DataBaseVar "Positionne"
	    } else {
		if {![file exists $DataBaseVar]} {
		    set Warning "$DataBaseVar does not exists, try another one."
		    AjouteAuLog $Warning
		} else {
		    LesParametresDuDesign DataBase $DataBaseVar "Positionne"
		    AjouteAuLog "DataBase Sequence: $DataBaseVar"
		}
	    }
	} else {
	    LesParametresDuDesign DataBase "Reset"	    
	    set Warning "You must define a DataBase for Specificity analysis."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $LDataBase "Blast formatted database (ex: .nsq, .nal) for sequence database searches.\nUse \"none\" to reuse previous blastn file(s) (ex: in a redesign process)."
       
    #2.2.1.9 WorkingDir panel
    global EWorkingDir   
    global BWorkingDir   
    set FWorkingDir   "$FParameters1.fworkingdir"
    set  LWorkingDir  "$FWorkingDir.lworkingdir"
    set  EWorkingDir  "$FWorkingDir.eworkingdir"
    set  BWorkingDir  "$FWorkingDir.bworkingdir"
    set  B2WorkingDir "$FWorkingDir.b2workingdir"

    frame  $FWorkingDir  -background "#808080"                                    -relief "groove"
    label  $LWorkingDir  -background "#808FFF" -padx "3" -text "Saving Directory" -relief "groove" -width "15" -anchor "w"
    entry  $EWorkingDir  -textvariable WorkingDirVar              
    button $B2WorkingDir -background "yellow"  -padx "2" -text "Browse" -command {
	if {[BROWSEDDIR] != ""} {
	    set BrowsedWDir [tk_chooseDirectory -title "Choose the saving directory." -initialdir "[BROWSEDDIR]"]
	} else {
	    set BrowsedWDir [tk_chooseDirectory -title "Choose the saving directory." -initialdir "[HOMEDIR]"]
	}
	if {$BrowsedWDir == "" } {
	    return ""
	} else {
	    BROWSEDDIR $BrowsedWDir
	}
	$EWorkingDir delete   0 end
	$EWorkingDir insert end $BrowsedWDir
	$BWorkingDir invoke
    }
    button $BWorkingDir  -background "yellow"  -padx "2" -text "OK"     -command {
	if {$WorkingDirVar!=""} {
	    if {![file exists $WorkingDirVar]} {
		set Warning "$WorkingDirVar does not exists, try another one."
		AjouteAuLog $Warning
	    } elseif {![file isdirectory $WorkingDirVar]} {
		set Warning "$WorkingDirVar is not a directory, try another one."
		AjouteAuLog $Warning
	    } else {
		LesParametresDuDesign WorkingDir $WorkingDirVar "Positionne"
		AjouteAuLog "WorkingDir : $WorkingDirVar"
	    }
	} else {
	    LesParametresDuDesign WorkingDir "Reset"	
	    set Warning "You must define a WorkingDir."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $LWorkingDir "Directory where to save your results"

    #2.2.1.10 Tm Method panel
    global TabMethodTm
    global LMethodTm2
    #Positionnement des Defauts pour les methodes de Tm (Formules, abreges, Textes explicatifs)
    set TabMethodTm(Methods)  [list "Nearest Neighbor" "Wallace" "DNA-DNA" "DNA-RNA" "RNA-RNA"]	
    set TabMethodTm(Variable) [list "NearestNeighbor"  "Wallace" "DNADNA"  "DNARNA"  "RNARNA"]	
    set TabMethodTm(Formule)  [list "1000*DH/(DS+R*ln(\[ADN\]/4)-273.15)" "4*(G+C)+2*(A+T)" "81.5+16.6*log M+41(%GC)-500/L-0.62F" "79.8+18.5*log M+58.4(%GC)+11.8(%GC)^2-820/L-0.35F" "79.8+18.5*log M+58.4(%GC)+11.8(%GC)^2-820/L-0.50F"]
    set TabMethodTm(Text)     [list "Thermodynamic method (<70 nts)" "Empiric method for short oligos (<20 nts)" "Method for DNA-DNA duplex (50<L<100 nts)" "Method for DNA-RNA duplex (50<L<100 nts)" "Method for RNA-RNA duplex (50<L<100 nts)"]
    foreach M [set TabMethodTm(Methods)] V [set TabMethodTm(Variable)] F [set TabMethodTm(Formule)] T [set TabMethodTm(Text)] {
	set TabMethodTm($M,Variable) $V
	set TabMethodTm($M,Formule)  $F
	set TabMethodTm($M,Text)     $T
    }

    set FMethodTm   "$FParameters1.fmethodtm"
    set  LMethodTm  "$FMethodTm.lmethodtm"
    set  SbMethodTm "$FMethodTm.sbmethodtm"
    set  LMethodTm2 "$FMethodTm.lmethodtm2"
    set  LMethodTm3 "$FMethodTm.lmethodtm3"

    frame   $FMethodTm  -background "#808080"           -relief "groove"
    label   $LMethodTm  -background "#808FFF" -padx "3" -relief "groove" -text "Tm Method"     -width "15" -anchor "w"
    spinbox $SbMethodTm -values [set TabMethodTm(Methods)] -textvariable TabMethodTm(Selected) -width "20" -state readonly -command {
	#Recuperation de la valeur utilisee
	set TabMethodTm(Selected) %s
	
	#Positionnement du label et de la formule
	$LMethodTm2 configure -text [set TabMethodTm([set TabMethodTm(Selected)],Formule)]
	set_balloon $LMethodTm2     [set TabMethodTm([set TabMethodTm(Selected)],Text)]
	
	LesParametresDuDesign TmMethod [set TabMethodTm([set TabMethodTm(Selected)],Variable)] "Positionne"
	AjouteAuLog "Tm method: [set TabMethodTm(Selected)]"
    }
    label   $LMethodTm3 -background "#808FFF" -padx "3" -relief "groove"
    label   $LMethodTm2 -background "#808FFF" -padx "3" -relief "groove" -anchor "center" -width "45" 
    set_balloon $LMethodTm "How to calculate the melting temperature (Tm)"

    #2.2.3.1 Advanced Parameters
    set FAdvanced1  "$FParameters3.fadvanced1"
    set  LAdvanced1 "$FAdvanced1.ladvanced1"
    set  BAdvanced1 "$FAdvanced1.badvanced1"
    set FAdvanced2  "$FParameters3.fadvanced2"

    frame $FAdvanced1                       -relief "ridge"  -borderwidth "1"
    label $LAdvanced1 -background "#808FFF" -relief "groove" -text "Advanced Parameters"
    bind  $LAdvanced1 <1> "HideGridedFrame $FAdvanced2"
    frame $FAdvanced2 -background "#808080" -relief "groove" -borderwidth "0"
    set_balloon $LAdvanced1 "Parameters for advanced users (Blast, Specificity threshold ...)"

    #2.2.3.2 Query Validation
    set FCheckQuery    "$FAdvanced2.fcheckquery"
    set  L1CheckQuery  "$FCheckQuery.lcquery1"
    set  L2CheckQuery  "$FCheckQuery.lcquery2"
    set  RB1CheckQuery "$FCheckQuery.rbcquery1"
    set  RB2CheckQuery "$FCheckQuery.rbcquery2"

    frame       $FCheckQuery   -background "#808080"                       -relief "groove"
    label       $L1CheckQuery  -background "#808FFF" -padx "3" -width "10" -relief "groove" -text "Query"               -anchor "w"
    label       $L2CheckQuery  -background "#808FFF" -padx "3" -width  "3" -relief "groove" -text "Database Validation" -anchor "w"
    radiobutton $RB1CheckQuery -variable CheckQueryVar         -value  "1" -relief "groove" -text "Yes"                 -anchor "w" -command {
	LesParametresDuDesign QueryCheck $CheckQueryVar "Positionne"	     
	AjouteAuLog "Query: database validation"				     
    }									     
    radiobutton $RB2CheckQuery -variable CheckQueryVar         -value  "0" -relief "groove" -text "No"                  -anchor "w" -command {
	LesParametresDuDesign QueryCheck $CheckQueryVar "Positionne"		     
	AjouteAuLog "Query: no database validation"				     
    }									     
    set_balloon $L1CheckQuery  "Additional query parameters"
    set_balloon $L2CheckQuery  "Allow to perform query sequence validation using % ID and % Cover in database."
    set_balloon $RB1CheckQuery "Yes for query validation in database."
    set_balloon $RB2CheckQuery "No for no query validation in database."

    #2.2.3.2 Step panel
    set FStep   "$FAdvanced2.fstep"
    set  LStep  "$FStep.lstep"
    set  L2Step "$FStep.l2step"
    set  EStep  "$FStep.estep"
    set  BStep  "$FStep.bstep"

    frame  $FStep  -background "#808080"           -relief "groove"
    label  $LStep  -background "#808FFF" -padx "3" -relief "groove" -text "Step"  -width "10" -anchor "w"
    label  $L2Step -background "#808FFF" -padx "3" -relief "groove" -text "nt(s)"             -anchor "w"
    entry  $EStep  -justify    "center"  -textvariable StepVar                    -width  "5"  
    button $BStep  -background "yellow"  -padx "2"                  -text "OK"    -command {
	if {$StepVar!=""} {
	    if {$StepVar<=0} {
		set Warning "Please enter a regular value for the Number of Oligo to design \nStep: 0<Step"
		AjouteAuLog $Warning
	    } else {
		LesParametresDuDesign Step $StepVar "Positionne"
		AjouteAuLog "Step: $StepVar nt(s)"	
	    }
	} else {
	    LesParametresDuDesign Step "Reset"
	    set Warning "Choose one value for Step."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $LStep "Number of nt(s) to skip until next oligo analysis"

    #2.2.3.3 Specificity panel
    set FSpecificity   "$FAdvanced2.fspecificity"
    set  LSpecificity  "$FSpecificity.lspecificity"
    set  L2Specificity "$FSpecificity.l2specificity"
    set  L3Specificity "$FSpecificity.l3specificity"
    set  ESpecificity  "$FSpecificity.especificity"
    set  E2Specificity "$FSpecificity.e2specificity"
    set  BSpecificity  "$FSpecificity.bspecificity"

    frame  $FSpecificity  -background "#808080" -relief "groove"
    label  $LSpecificity  -background "#808FFF" -relief "groove" -padx "3" -width "10" -text "Specificity"      -anchor "w"
    label  $L2Specificity -background "#808FFF" -relief "groove" -padx "3" -width  "3" -text "%"                -anchor "center"
    label  $L3Specificity -background "#808FFF" -relief "groove" -padx "3" -width "10" -text "contiguous bases" -anchor "w"
    entry  $ESpecificity  -textvariable SpecificityVar                     -width  "5" -justify "center"
    entry  $E2Specificity -textvariable NbContiguous                       -width  "4" -justify "center"
    button $BSpecificity  -background "yellow"  -text   "OK"     -padx "2" -command {
	if {$SpecificityVar!="" && $NbContiguous!=""} {
	    if {   $SpecificityVar == "."} {
		set SpecificityVar "0.0"
	    } elseif {! [regexp {\.[0-9]+} $SpecificityVar] || ! [regexp {[0-9]+\.} $SpecificityVar]} {
		set SpecificityVar [format "%.1f" $SpecificityVar]
	    }
	    
	    if {$SpecificityVar>100.0 | $SpecificityVar<=0.0 | $NbContiguous<0} {
		set Warning "Please enter a regular value for Specificity and MaxContiguous\nSpecificity: 0<Specificity<=100\nRange: 0<=NbContiguous"
		AjouteAuLog $Warning
	    } else {
		LesParametresDuDesign IdLimit      $SpecificityVar "Positionne"
		LesParametresDuDesign NbContiguous $NbContiguous   "Positionne"
		AjouteAuLog "Specificity: $SpecificityVar \%"
		AjouteAuLog "Max contiguous: $NbContiguous nt(s)"
	    }
	} else {
	    set Warning "Choose one value for both Specificity and Contiguous."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $LSpecificity  "Define specificity threshold to avoid X-Hybridization."
    set_balloon $L2Specificity "Threshold for % Identity."
    set_balloon $L3Specificity "Threshold for the maximum identical contiguous bases."

    #2.2.3.4 TmParam panel
    set FTmParam   "$FAdvanced2.ftmparam"
    set  L1TmParam "$FTmParam.l1tmparam"
    set  L2TmParam "$FTmParam.l2tmparam"
    set  L3TmParam "$FTmParam.l3tmparam"
    set  L4TmParam "$FTmParam.l4tmparam"
    set  E1TmParam "$FTmParam.e1tmparam"
    set  E2TmParam "$FTmParam.e2tmparam"
    set  E3TmParam "$FTmParam.e3tmparam"
    set  BTmParam  "$FTmParam.btmparam"

    frame  $FTmParam  -background "#808080" -relief "groove"
    label  $L1TmParam -background "#808FFF" -relief "groove" -padx "3" -width "10" -text "Tm"  -anchor "w"
    label  $L2TmParam -background "#808FFF" -relief "groove" -padx "3" -width  "3" -text "Na+" -anchor "center"
    label  $L3TmParam -background "#808FFF" -relief "groove" -padx "3" -width  "3" -text "F"   -anchor "center"
    label  $L4TmParam -background "#808FFF" -relief "groove" -padx "3"             -text "DNA" -anchor "center"
    entry  $E1TmParam -textvariable TmSaltVar                          -width  "5" -justify "center"
    entry  $E2TmParam -textvariable TmFormVar                          -width  "4" -justify "center"
    entry  $E3TmParam -textvariable TmConcVar                          -width  "5" -justify "center"

    button $BTmParam  -background "yellow"  -text   "OK"     -padx "2" -command {		
	if {$TmSaltVar != ""} {
	    if {   $TmSaltVar == "."} {
		set TmSaltVar "0.0"
	    } elseif {! [regexp {\.[0-9]+} $TmSaltVar] || ! [regexp {[0-9]+\.} $TmSaltVar]} {
		set TmSaltVar [format "%.1f" $TmSaltVar]
	    } 
	}
	if {$TmFormVar != ""} {
	    if {   $TmFormVar == "."} {
		set TmFormVar "0.0"
	    } elseif {! [regexp {\.[0-9]+} $TmFormVar] || ! [regexp {[0-9]+\.} $TmFormVar]} {
		set TmFormVar [format "%.1f" $TmFormVar]
	    } 
	}
	if {$TmConcVar != ""} {
	    if {   $TmConcVar == "."} {
		set TmConcVar "0.0"
	    } elseif {! [regexp {\.[0-9]+} $TmConcVar] || ! [regexp {[0-9]+\.} $TmConcVar]} {
		set TmConcVar [format "%.1f" $TmConcVar]
	    } 
	}
	LesParametresDuDesign TmSalt           $TmSaltVar "Positionne"
	LesParametresDuDesign TmFormamide      $TmFormVar "Positionne"
	LesParametresDuDesign TmConcentration  $TmConcVar "Positionne"

	AjouteAuLog "Tm paramameters: Salt $TmSaltVar M, Formamide $TmFormVar \% and \[DNA\] $TmConcVar M"
    }
    set_balloon $L1TmParam "Define Tm advanced parameters."
    set_balloon $L2TmParam "Salt concentration."
    set_balloon $L3TmParam "Formamide concentration."
    set_balloon $L4TmParam "Target sequence concentration."

    #2.2.3.5 Blast Panel
    set FBlast   "$FAdvanced2.fblast"
    set  L1Blast "$FBlast.l1blast"
    set  L2Blast "$FBlast.l2blast"
    set  L3Blast "$FBlast.l3blast"
    set  L4Blast "$FBlast.l4blast"
    set  E2Blast "$FBlast.e2blast"
    set  E3Blast "$FBlast.e3blast"
    set  E4Blast "$FBlast.e4blast"
    set  B1Blast "$FBlast.b1blast"

    frame  $FBlast  -background "#808080" -relief "groove"
    label  $L1Blast -background "#808FFF" -relief "groove" -padx "3" -width "10" -text    "BLAST"    -anchor "w"
    label  $L2Blast -background "#808FFF" -relief "groove" -padx "3" -width  "3" -text    "E"        -anchor "center"
    label  $L3Blast -background "#808FFF" -relief "groove" -padx "3" -width  "3" -text    "W"        -anchor "center"
    label  $L4Blast -background "#808FFF" -relief "groove" -padx "3"             -text    "To Align" -anchor "w"
    entry  $E2Blast -textvariable ExpectVar                          -width  "5" -justify "center"
    entry  $E3Blast -textvariable WordVar                            -width  "4" -justify "center"
    entry  $E4Blast -textvariable NbAlignedVar                       -width  "5" -justify "center"
    button $B1Blast -background "yellow"  -text   "OK"     -padx "2" -command {
	if {$ExpectVar!=""} {	
	    if {   $ExpectVar == "."} {
		set ExpectVar "0.0"
	    } elseif {! [regexp {\.[0-9]+} $ExpectVar] || ! [regexp {[0-9]+\.} $ExpectVar]} {
		set ExpectVar [format "%.1f" $ExpectVar]
	    } 
		    
	    LesParametresDuDesign BlastExpect $ExpectVar "Positionne"
	    set Warning "E-value: $ExpectVar"	
	    AjouteAuLog $Warning
	} else {
	    LesParametresDuDesign BlastExpect "Reset"
	    set Warning "Expect is an option."
	    AjouteAuLog $Warning
	}

	if {$WordVar!=""} {
	    if {$WordVar<=0} {
		set Warning "Please enter a regular value for the length of word \nWord: 0<Word"
		AjouteAuLog $Warning
	    } else {
		LesParametresDuDesign BlastWord $WordVar "Positionne"
		set Warning "Word: $WordVar nt(s)"
		AjouteAuLog $Warning	
	    }
	} else {
	    LesParametresDuDesign BlastWord "Reset"
	    set Warning "Word is an option."
	    AjouteAuLog $Warning
	}

	if {$NbAlignedVar!=""} {
	    if {$NbAlignedVar<=0} {
		set Warning "Please enter a regular value for max subject to align \nTo align: 0<Nb Align"
		AjouteAuLog $Warning
	    } else {
		LesParametresDuDesign BlastNbAligned $NbAlignedVar "Positionne"
	        set Warning "To align: $NbAlignedVar"	
		AjouteAuLog $Warning
	    }
	} else {
	    LesParametresDuDesign BlastNbAligned "Reset"
	    set Warning "To align is an option."
	    AjouteAuLog $Warning
	}
    }
    set_balloon $L1Blast "BLAST parameters"
    set_balloon $L2Blast "E value\nex: 10 ou 10e-3 (equivalent blastall option -e)"
    set_balloon $L3Blast "Length of words used in BLAST algorithm (equivalent blastall option -W)"
    set_balloon $L4Blast "Maximum number of sequence to be aligned (equivalent blastall options -v and -b)"

    #2.2.3.6 Blast Panel 3
    set FBlast3    "$FAdvanced2.fblast3"
    set  L1Blast3  "$FBlast3.l1blast"
    set  L2Blast3  "$FBlast3.l2blast"
    set  RB1Blast3 "$FBlast3.rbblast1"
    set  RB2Blast3 "$FBlast3.rbblast2"

    frame       $FBlast3   -background "#808080" -relief "groove"
    label       $L1Blast3  -background "#808FFF" -relief "groove" -padx "3" -width "10" -text "BLAST"  -anchor "w"
    label       $L2Blast3  -background "#808FFF" -relief "groove" -padx "3" -width  "3" -text "Filter" -anchor "w"
    radiobutton $RB1Blast3 -variable FilterVar   -relief "groove" -value "T"            -text "Yes"    -anchor "w" -command {
	LesParametresDuDesign BlastFilter $FilterVar "Positionne"	     
	AjouteAuLog "BLAST: Filtering"				     
    }									     
    radiobutton $RB2Blast3 -variable FilterVar   -relief "groove" -value "F"            -text "No"     -anchor "w" -command {
	LesParametresDuDesign BlastFilter $FilterVar "Positionne"		     
	AjouteAuLog "BLAST: no Filtering"				     
    }									     
    set_balloon $L1Blast3  "BLAST parameters"
    set_balloon $L2Blast3  "BLAST filtering (equivalent blastall option -F)"
    set_balloon $RB1Blast3 "Yes for active filtering."
    set_balloon $RB2Blast3 "No for non active filtering."

    #2.2.3.7 Blast Panel 2
    set FBlast2    "$FAdvanced2.fblast2"
    set  L1Blast2  "$FBlast2.l1blast"
    set  L2Blast2  "$FBlast2.l2blast"
    set  E2Blast2  "$FBlast2.e2blast"
    set  B1Blast2  "$FBlast2.b1blast"
    set  RB1Blast2 "$FBlast2.rb1blast"
    set  RB2Blast2 "$FBlast2.rb2blast"
    set  RB3Blast2 "$FBlast2.rb3blast"
    #-S  Query strands to search against database (for blast[nx], and tblastx)
    #       3 is both, 1 is top, 2 is bottom [Integer]
    frame       $FBlast2   -background "#808080" -relief "groove"
    label       $L1Blast2  -background "#808FFF" -relief "groove" -padx  "3" -width "10" -text "BLAST"  -anchor "w"
    label       $L2Blast2  -background "#808FFF" -relief "groove" -padx  "3" -width  "3" -text "Strand" -anchor "w"
    radiobutton $RB1Blast2 -variable StrandVar   -relief "groove" -value "1"             -text "Fwd"    -anchor "w" -command {
	LesParametresDuDesign BlastStrand $StrandVar "Positionne"	  
	AjouteAuLog "BLAST Strand: Forward"				  
    }									  
    radiobutton $RB2Blast2 -variable StrandVar   -relief "groove" -value "2"             -text "Rev"    -anchor "w" -command {
	LesParametresDuDesign BlastStrand $StrandVar "Positionne"	  	     
	AjouteAuLog "BLAST Strand: Reverse"			     
    }									  
    radiobutton $RB3Blast2 -variable StrandVar   -relief "groove" -value "3"             -text "Both"   -anchor "w" -command {
	LesParametresDuDesign BlastStrand $StrandVar "Positionne"	       
	AjouteAuLog "BLAST Strand: Both"			       
    }								       
    set_balloon $L1Blast2  "BLAST parameters"
    set_balloon $RB1Blast2 "Foward permits to search on top strand (equivalent blastall option -S 1)"
    set_balloon $RB2Blast2 "Reverse permits to search on bottom strand (equivalent blastall option -S 2)"
    set_balloon $RB3Blast2 "Both permits to search on top strand (equivalent blastall option -S 3)"
    set_balloon $L2Blast2  "Depending on your searches (mRNA or Genome) it is important to check both strands or not."

    #2.2.4.1 Batch Parameters
    set FBatchPanel1  "$FBatch.fbatch1"
    #set FBatchPanel1  "$FParameters4.fbatch1"
    set  LBatchPanel1 "$FBatchPanel1.lbatch1"
    set  BBatchPanel1 "$FBatchPanel1.bbatch1"
    #set FBatchPanel2  "$FParameters4.fbatch2"
    set FBatchPanel2  "$FBatch.fbatch2"

    frame $FBatchPanel1                       -relief "ridge"  -borderwidth "1"
    label $LBatchPanel1 -background "#808FFF" -relief "groove" -text "Batch Mode"
    bind  $LBatchPanel1 <1> "HideGridedFrame $FBatchPanel2"
    frame $FBatchPanel2 -background "#808080" -relief "groove" -borderwidth "0"
    set    Texte "CADO4MI - Batch mode to run each step separately.\n"
    append Texte "Step1 - Masking poly A tail\n"
    append Texte "Step2 - BLASTN\n"
    append Texte "Step3 - Sequence Analysis\n"
    append Texte "Step4 - Oligo Selection\n"
    append Texte "Other - X-Selection\n"
    append Texte "Other - Saving results in tab files"
    set_balloon $LBatchPanel1 "$Texte"
 
    #3.1 Batch Mode panel
    set FBatch1          "$FBatchPanel2.fbatch1"
    set  LBatch1         "$FBatch1.lbatch1"
    set  BBatch1         "$FBatch1.bbatch1"
    set  LBlastthemall   "$FBatch1.lblastthemall"
    set  BBlastthemall   "$FBatch1.bblastthemall"
    set  LAnalysethemall "$FBatch1.lanalysethemall"
    set  BAnalysethemall "$FBatch1.banalysethemall"

    frame  $FBatch1         -background "#808080"           -relief "groove"
    label  $LBatch1         -background "#808FFF" -padx "3" -relief "groove" -text "Masking"    -anchor "center" -width "10" 
    button $BBatch1         -background "yellow"  -padx "2"                  -text "OK"         -command { 
	#Test si les parametres necessaires sont disponibles
	set OK 1
	foreach  Parametre [list Query WorkingDir] {
	    set $Parametre [LesParametresDuDesign $Parametre "LaValeurMerci"]
	    if {[set $Parametre]==""} {set OK 0;break}
	}
	if {$OK!="1"} {
	    AjouteAuLog "Missing parameters.\nMasking Step requires the following parameters: Query and Saving Directory."
	    return ""
	}

	FaireAttendre .waitapply "Waiting" "Please wait ...\nMasking all Querys for poly A ..."
	Oligo_MaskingStep_PourTous
	FaireLaSuite  .waitapply
    }
    label  $LBlastthemall   -background "#808FFF" -padx "3" -relief "groove" -text "BLAST"      -anchor "center" -width "10" 
    button $BBlastthemall   -background "yellow"  -padx "2"                  -text "OK"         -command {  
	#Test si les parametres necessaires sont disponibles
	set OK 1

	foreach  Parametre [list Query DataBase WorkingDir] {
	    set $Parametre [LesParametresDuDesign $Parametre "LaValeurMerci"]
	    if {[set $Parametre]==""} {set OK 0;break}
	}
	if {$OK!="1"} {
	    AjouteAuLog "Missing parameters.\nBlast Step requires the following parameters: Query, DataBase and Saving Directory."	
	    return ""
	}

	FaireAttendre .waitapply "Waiting" "Please wait ...\nBlasting all Querys against selected database ..."
	Oligo_BlastNStep_PourTous
	FaireLaSuite  .waitapply
    }
    label  $LAnalysethemall -background "#808FFF" -padx "3" -relief "groove" -text "Analysis"   -anchor "center" -width "10" 
    button $BAnalysethemall -background "yellow"  -padx "2"                  -text "OK"         -command { 
	#Test si les parametres necessaires sont disponibles
	set OK 1
	foreach  Parametre [list Query LengthOligo TmLimit TmRange WorkingDir] {
	    set $Parametre [LesParametresDuDesign $Parametre "LaValeurMerci"]
	    if {[set $Parametre]==""} {set OK 0;break}
	}
	if {$OK!="1"} {
	    AjouteAuLog "Missing parameters.\nAnalysis Step requires the following parameters: Query, Oligo Length, Tm and Saving Directory."
	    return ""
	}
	LesParametresDuDesign BonNombreDeParametre 1 "Positionne"

	FaireAttendre .waitapply "Waiting" "Please wait ...\nAnalysing all Querys ..."
	Oligo_AnalysisStep_PourTous
	FaireLaSuite  .waitapply

	LesParametresDuDesign BonNombreDeParametre "Reset"
    }
    set_balloon $LBatch1         "Step1 - Masking for poly A tail.\nIt will create all .masked files."
    set_balloon $LBlastthemall   "Step2 - BLAST for all query sequences against Database.\nIt will create all .blastn files."
    set_balloon $LAnalysethemall "Step3 - Analysis Step for all query sequences.\nIt will create all .log and .oligo files."

    #3.1.1 Batch Mode panel 2
    set FBatch2          "$FBatchPanel2.fbatch2"
    set  LSelectthemall  "$FBatch2.lselectthemall"
    set  BSelectthemall  "$FBatch2.bselectthemall"
    set  LXselectthemall "$FBatch2.lxselectthemall"
    set  BXselectthemall "$FBatch2.bxselectthemall"
    set  LSavethemall    "$FBatch2.lsavethemall"
    set  BSavethemall    "$FBatch2.bsavethemall"

    frame  $FBatch2         -background "#808080"           -relief "groove"
    label  $LSelectthemall  -background "#808FFF" -padx "3" -relief "groove" -text "Selection"    -anchor "center" -width "10" 
    button $BSelectthemall  -background "yellow"  -padx "2"                  -text "OK"           -command {SelectionDesOligosPourTous}
    label  $LXselectthemall -background "#808FFF" -padx "3" -relief "groove" -text "X-Selection"  -anchor "center" -width "10" 
    button $BXselectthemall -background "yellow"  -padx "2"                  -text "OK"           -command {SelectionCroiseeDesOligosPourTous}
    label  $LSavethemall    -background "#808FFF" -padx "3" -relief "groove" -text "Save Results" -anchor "center" -width "10" 
    button $BSavethemall    -background "yellow"  -padx "2"                  -text "OK"           -command {LesOligosPourExcel_GUI}
    set_balloon $LSelectthemall  "Step4 - Selection of the best oligonucelotide.\nIt will create all .selection files."
    set_balloon $LXselectthemall "Other - Selection of the best oligonucelotide between 2 designs.\nIt will create all .croisee files."
    set_balloon $LSavethemall    "Other - To save your results under a tab delimited file."

    #On positionne toute l interface
    ################################
    grid  $FTitre                              -sticky ewns
    grid  $LTitre                              -sticky ewns
    grid  columnconfigure $FTitre 0       -weight 1

    #Les 2 frames majeures
    #1 Predesigned IllustreOligo Panel
    #2 Design      Tous les parametres du design
    #3 Batch mode  
    grid  $FPreDesigned                        -sticky ewns
    grid  $FDesign                             -sticky ewns
    grid  $FBatch                              -sticky ewns

    grid  columnconfigure $FPreDesigned 0 -weight 1
    grid  columnconfigure $FDesign      0 -weight 1
    grid  columnconfigure $FBatch       0 -weight 1

    #1.1 IllustreOligo panel
    grid $FIllustre1    -row 0     -column 0   -sticky ewns
    grid  $LIllustre1   -row 0     -column 0   -sticky ewns
    grid  $BIllustre1   -row 0     -column 1   -sticky ewns
    grid  columnconfigure $FIllustre1 0   -weight 1
    grid  columnconfigure $FIllustre1 1   -weight 0
    grid $FIllustre2    -row 1     -column 0   -sticky ewns
    grid  columnconfigure $FIllustre2 0   -weight 0
    grid  columnconfigure $FIllustre2 1   -weight 1
    grid  columnconfigure $FIllustre2 2   -weight 0
    grid  columnconfigure $FIllustre2 3   -weight 0
    grid  $L1Illustre2  -row 0     -column 0   -sticky ewns
    grid  $E1Illustre2  -row 0     -column 1   -sticky ewns
    grid  $B2Illustre2  -row 0     -column 2   -sticky ewns
    grid  $B5Illustre2  -row 0     -column 3   -sticky ewns -rowspan 2
    grid  $L2Illustre2  -row 1     -column 0   -sticky ewns
    grid  $E2Illustre2  -row 1     -column 1   -sticky ewns
    grid  $B4Illustre2  -row 1     -column 2   -sticky ewns
     
    #Les petites frames, parametres, Tm methode, menu, log
    #2.2.1 Frame des parametres query, edge, GC ...
    #2.2.3 Frame Advanced Parameters step, specificity et BLAST
    grid $FParamMenu    -row 0     -column 0   -sticky ewns
    grid $LParamMenu    -row 0     -column 0   -sticky ewns
    grid $BParamMenu    -row 0     -column 1   -sticky ewns
    grid columnconfigure $FParamMenu    0 -weight 1
    grid $FTousLesParameters                   -sticky ewns

    grid  $FParameters1                        -sticky ewns

    grid  $FParameters3                        -sticky ewns
    grid  columnconfigure $FParameters3 0 -weight 1

    #2.2.1.1 Query panel
    pack $FQuery        -fill x    -side top
    pack $LQuery        -fill both -side left
    pack $EQuery        -fill both -side left  -expand true
    pack $BQuery        -fill both -side right
    pack $B2Query       -fill both -side right
    pack $B3Query       -fill both -side right
    
    #2.2.1.2 Edge panel
    pack $FEdge         -fill x    -side top
    pack $LEdge         -fill both -side left
    pack $EEdge         -fill both -side left
    pack $L2Edge        -fill both -side left  -expand true
    pack $BEdge         -fill both -side right 

    #2.2.1.3 Taille panel
    pack $FTaille       -fill x    -side top
    pack $LTaille       -fill both -side left
    pack $ETaille       -fill both -side left
    pack $L2Taille      -fill both -side left  -expand true
    pack $BTaille       -fill both -side right

    #2.2.1.4 NombreOligo panel
    pack $FNbOligo      -fill x    -side top
    pack $LNbOligo      -fill both -side left
    pack $ENbOligo      -fill both -side left
    pack $L2NbOligo     -fill both -side left  -expand true
    pack $BNbOligo      -fill both -side right

    #2.2.1.5 Tm panel
    pack $FTm           -fill x    -side top
    pack $LTm           -fill both -side left
    pack $ETm           -fill both -side left
    pack $L2Tm          -fill both -side left
    pack $E2Tm          -fill both -side left
    pack $L3Tm          -fill both -side left  -expand true
    pack $BTm           -fill both -side right 
    pack $B2Tm          -fill both -side right 

    #2.2.1.6 GC content panel
    pack $FGCContent    -fill x    -side top
    pack $LGCContent    -fill both -side left
    pack $EGCContent    -fill both -side left
    pack $L2GCContent   -fill both -side left
    pack $E2GCContent   -fill both -side left
    pack $L3GCContent   -fill both -side left  -expand true
    pack $BGCContent    -fill both -side right 
    pack $B2GCContent   -fill both -side right 
 
    #2.2.1.7 Prohibited panel
    pack $FProhib       -fill x    -side top
    pack $LProhib       -fill both -side left
    pack $EProhib       -fill both -side left  -expand true
    pack $BProhib       -fill both -side left
    pack $B2Prohib      -fill both -side left
    pack $B3Prohib      -fill both -side right    
    
    #2.2.1.8 Database panel
    pack $FDataBase     -fill x    -side top
    pack $LDataBase     -fill both -side left
    pack $EDataBase     -fill both -side left  -expand true
    pack $BDataBase     -fill both -side right
    pack $B2DataBase    -fill both -side right
    
    #2.2.1.9 WorkingDir panel
    pack $FWorkingDir   -fill x    -side top
    pack $LWorkingDir   -fill both -side left
    pack $EWorkingDir   -fill both -side left  -expand true
    pack $BWorkingDir   -fill both -side right
    pack $B2WorkingDir  -fill both -side right

    #2.2.1.10 Tm Method Panel
    pack $FMethodTm     -fill both -side top  
    grid $LMethodTm     -row 0     -column 0   -sticky ns
    grid $SbMethodTm    -row 0     -column 1   -sticky ns
    grid $LMethodTm3    -row 0     -column 2   -sticky ewns
    grid $LMethodTm2    -row 1     -column 0   -sticky ewns -columnspan 3  
    grid columnconfigure $FMethodTm 0  -weight 0
    grid columnconfigure $FMethodTm 1  -weight 0
    grid columnconfigure $FMethodTm 2  -weight 1

    #2.2.3.1 Advanced Parameters
    grid $FAdvanced1    -row 0     -column 0   -sticky ewns
    grid  $LAdvanced1   -row 0     -column 0   -sticky ewns
    grid  columnconfigure $FAdvanced1 0   -weight 1
    grid  columnconfigure $FAdvanced1 1   -weight 0
 
    grid $FAdvanced2    -row 1     -column 0   -sticky ewns
    grid  columnconfigure $FAdvanced2 0   -weight 1

    #2.2.3.2 Query Validation
    pack   $FCheckQuery     -fill x    -side top
    pack    $L1CheckQuery   -fill both -side left
    pack    $RB1CheckQuery  -fill both -side left
    pack    $RB2CheckQuery  -fill both -side left 
    pack    $L2CheckQuery   -fill both -side left  -expand true
 
    #2.2.3.2 Step panel
    pack   $FStep           -fill x    -side top
    pack    $LStep          -fill both -side left
    pack    $EStep          -fill both -side left
    pack    $L2Step         -fill both -side left  -expand true
    pack    $BStep          -fill both -side right
    
    #2.2.3.3 Specificity panel
    pack   $FSpecificity    -fill x    -side top
    pack    $LSpecificity   -fill both -side left
    pack    $ESpecificity   -fill both -side left
    pack    $L2Specificity  -fill both -side left
    pack    $E2Specificity  -fill both -side left
    pack    $L3Specificity  -fill both -side left  -expand true
    pack    $BSpecificity   -fill both -side right 
    
    #2.2.3.4 Tm parameters panel
    pack   $FTmParam        -fill x    -side top
    pack    $L1TmParam      -fill both -side left
    pack    $E1TmParam      -fill both -side left
    pack    $L2TmParam      -fill both -side left
    pack    $E2TmParam      -fill both -side left
    pack    $L3TmParam      -fill both -side left
    pack    $E3TmParam      -fill both -side left  
    pack    $L4TmParam      -fill both -side left  -expand true
    pack    $BTmParam       -fill both -side right 

    #2.2.3.5 Blast Panel
    pack   $FBlast          -fill x    -side top
    pack    $L1Blast        -fill both -side left
    pack    $E2Blast        -fill both -side left
    pack    $L2Blast        -fill both -side left
    pack    $E3Blast        -fill both -side left
    pack    $L3Blast        -fill both -side left
    pack    $E4Blast        -fill both -side left
    pack    $L4Blast        -fill both -side left  -expand true
    pack    $B1Blast        -fill both -side left

    #2.2.3.5 Query Validation
    pack   $FBlast3         -fill x    -side top
    pack    $L1Blast3       -fill both -side left
    pack    $RB1Blast3      -fill both -side left
    pack    $RB2Blast3      -fill both -side left 
    pack    $L2Blast3       -fill both -side left  -expand true

    #2.2.3.7 Blast Panel 2
    pack   $FBlast2         -fill x    -side top
    pack    $L1Blast2       -fill both -side left
    pack    $RB1Blast2      -fill both -side left
    pack    $RB2Blast2      -fill both -side left
    pack    $RB3Blast2      -fill both -side left
    pack    $L2Blast2       -fill both -side left  -expand true

    #3.1 BatchPanel Parameters
    grid $FBatchPanel1    -row 0     -column 0   -sticky ewns
    grid  $LBatchPanel1   -row 0     -column 0   -sticky ewns
    grid  columnconfigure $FBatchPanel1 0   -weight 1
    grid  columnconfigure $FBatchPanel1 1   -weight 0
 
    grid $FBatchPanel2    -row 1     -column 0   -sticky ewns
    grid  columnconfigure $FBatchPanel2 0   -weight 1
    grid  columnconfigure $FBatchPanel2 1   -weight 0

    #3.1.1 Batch Mode panel
    pack   $FBatch1         -fill x    -side top   -expand true
    pack   $LBatch1         -fill both -side left  -expand true
    pack   $BBatch1         -fill both -side left 
    pack   $LBlastthemall   -fill both -side left  -expand true
    pack   $BBlastthemall   -fill both -side left 
    pack   $LAnalysethemall -fill both -side left  -expand true
    pack   $BAnalysethemall -fill both -side left 

    #3.1.2 Batch Mode panel 2
    pack   $FBatch2         -fill x    -side top   -expand true
    pack   $LSelectthemall  -fill both -side left  -expand true
    pack   $BSelectthemall  -fill both -side left 
    pack   $LXselectthemall -fill both -side left  -expand true
    pack   $BXselectthemall -fill both -side left 
    pack   $LSavethemall    -fill both -side left  -expand true
    pack   $BSavethemall    -fill both -side left 

    if {$w ==  ""} {set w "."}
    LogPanel $w
    if {$w == "."} {set w ""}
    
    #On positionne les espions sur les variables
    SpyOnEntryVariable

    #Par defaut on cache les menus
    #HideGridedFrame $FIllustre2
    #HideGridedFrame $FTousLesParameters    
    HideGridedFrame $FAdvanced2 
    HideGridedFrame $FBatch
    #HideGridedFrame $FBatchPanel2
    HideGridedFrame $FPreDesigned
    HideGridedFrame $FDesign

    #Parametres par defaut
    ######################
    #Pour la methode de calcul du Tm NearestNeighbor    
    $SbMethodTm invoke buttondown
    #Pour les blasts strands Forward (-S 1)
    $RB1Blast2     invoke
    #Pour verifier la query dans les banques
    $RB1CheckQuery invoke
    #Pour le filtre dans le blast (-F T)
    $RB1Blast3     invoke
    EffaceLeLog 

    LesParametresParDefaut 
    #Pour le nombre de sequences
    $ENbOligo      delete 0 end			       
    $ENbOligo      insert   end [LesParametresDuDesign NbOligo         "LaValeurMerci"]
    #Pour les parametres avances de Tm
    $E1TmParam     delete 0 end
    $E1TmParam     insert   end [LesParametresDuDesign TmSalt          "LaValeurMerci"]
    $E2TmParam     delete 0 end
    $E2TmParam     insert   end [LesParametresDuDesign TmFormamide     "LaValeurMerci"]
    $E3TmParam     delete 0 end
    $E3TmParam     insert   end [LesParametresDuDesign TmConcentration "LaValeurMerci"]
    #Pour la Specificite
    $ESpecificity  delete 0 end
    $ESpecificity  insert   end [LesParametresDuDesign IdLimit         "LaValeurMerci"]
    $E2Specificity delete 0 end
    $E2Specificity insert   end [LesParametresDuDesign NbContiguous    "LaValeurMerci"]
    #Pour le pas
    $EStep         delete 0 end
    $EStep         insert   end [LesParametresDuDesign Step            "LaValeurMerci"]
    #Pour le Blast expect limite
    $E2Blast       delete 0 end
    $E2Blast       insert   end [LesParametresDuDesign BlastExpect     "LaValeurMerci"]
    #Pour le Blast taille des mots		       
    $E3Blast       delete 0 end			       
    $E3Blast       insert   end [LesParametresDuDesign BlastWord       "LaValeurMerci"]
    #Pour le Blast max a aligne			       
    $E4Blast       delete 0 end			       
    $E4Blast       insert   end [LesParametresDuDesign BlastNbAligned  "LaValeurMerci"]

    CADO4MICopyright

    if {$w == ""} {set w "."}
    
    #wm deiconify $w

    return ""
}

proc CADO4MICopyright {} {

    AjouteAuLog "CADO4MI version [CADO4MI_Version]"
    AjouteAuLog ""
    AjouteAuLog "Copyright (C) 2006 Jean Muller"
    AjouteAuLog ""
    AjouteAuLog "Please cite:"
    AjouteAuLog "Muller et al (2007) BMC Genomics Aug 29;8(1):294."
    AjouteAuLog ""
    AjouteAuLog "A program for oligonucleotide design."
    AjouteAuLog "Information is available through documentation or bubble help."
    AjouteAuLog "Please feel free to contact us for any suggestions or bug reports."
    AjouteAuLog "email: jean.muller@igbmc.u-strasbg.fr"
    AjouteAuLog ""

    return ""
}

proc SpyOnEntryVariable {} {
    
    Wup "Positionne des espions et des procedures pour empecher l entree de donnees non valides"

    set          LesVars [list EdgeBegin]
    foreach Var $LesVars {
	global $Var
	trace add variable $Var write {forceLen 8}
	trace add variable $Var write forceInt
	set $Var ""
    }
    set          LesVars [list StepVar LengthVar NbContiguous WordVar]
    foreach Var $LesVars {
	global $Var
	trace add variable $Var write {forceLen 3}
	trace add variable $Var write forceInt
	set $Var ""
    }	
    set          LesVars [list TmVar TmRange TmSaltVar TmFormVar SpecificityVar GCContentMinusVar GCContentPlusVar]
    foreach Var $LesVars {
	global $Var
	trace add variable $Var write {forceLen 5}
	trace add variable $Var write forceReal
	set $Var ""
    }
    set          LesVars [list TmConcVar ExpectVar]
    foreach Var $LesVars {
	global $Var
	trace add variable $Var write {forceLen 10}
	trace add variable $Var write forceReal
	set $Var ""
    }
    set          LesVars [list NbAlignedVar]
    foreach Var $LesVars {
	global $Var
	trace add variable $Var write {forceLen 5}
	trace add variable $Var write forceInt
	set $Var ""
    }
    set          LesVars [list NbOligoVar]
    foreach Var $LesVars {
	global $Var
	trace add variable $Var write {forceLen 7}
	trace add variable $Var write forceInt
	set $Var ""
    }
    set          LesVars [list ProhibVar]
    foreach Var $LesVars {
	global $Var
	trace add variable $Var write {forceRegexp {^[ ATGC]*$}}
	set $Var ""
    }

    return ""
}

proc LogPanel {{w ""}} {

    Wup "Log Panel de CADO4MI"
    Wup "24/05/2006"

    #On gere la premiere fenetre en . si il y deja autre chose d utiliser dans .
    #ATTENTION on doit modifier . en "" pour pouvoir conserver la hierarchie des fenetres
    if {$w == ""} {
	if {[winfo children .] != ""} {
	    set       w ".logcado4mi"
	    if {[winfo exists $w]} {return ""}
	    toplevel $w
	} else {
	    set w "."
	    #$w config -bd 3 -relief flat
	}
    } else {
	if {$w == "."} {
	    set       w ".logcado4mi"
	} else {
	    set       w "$w.logcado4mi"
	}
	if {[winfo exists $w]} {return ""}
	toplevel $w
    }
    
    #set PosXLog [expr [winfo screenwidth .] - 0.10]
    set PosYLog [format "%.0f" [expr {[winfo screenheight .] * 0.50}]]
    set PosXLog "100"     

    wm geometry  $w "+$PosXLog+$PosYLog"
    #wm geometry  $w +500+100
    wm title     $w "CADO4MI Log Panel"
    wm resizable $w false false
    #wm resizable $w true true
    wm protocol  $w WM_DELETE_WINDOW "exit"
    if {$w == "."} {set w ""}

    set FVisible "$w.fvisible"
    frame  $FVisible    -borderwidth "2"          -relief "ridge" 

    #6.1 Log Panel
    global LBLog    
    set FLog    "$FVisible.flog"
    set  LBLog  "$FLog.lblog"
    set  SBVLog "$FLog.yscroll"
    #set  SBHLog "$FLog.xscroll"
    set FLog2   "$FVisible.log2"
    set  BLog   "$FLog2.blog"
    set  BLog2  "$FLog2.blog2"
    set  LLog1  "$FLog2.llog1"
    set  LLog2  "$FLog2.llog2"
 
    #scrollbar   $SBHLog -command    "$LBLog xview" -orient "horizontal"
    frame     $FLog   -background "#808080"  -relief  "groove"
    scrollbar $SBVLog -orient     "vertical" -command "$LBLog yview"
    text      $LBLog  -state      "disabled" -wrap    "word"   -width "50" -height "20" -yscrollcommand "$SBVLog set"
    frame     $FLog2  -background "#808080"  -relief  "groove"
    button    $BLog   -background "red"      -relief  "ridge"  -padx "3"   -text "Clear" -command "EffaceLeLog"
    #button    $BLog2  -background "green"    -relief  "ridge"  -padx "3"   -text "C"     -command "CADO4MICopyright"
    #button    $BLog   -background "green"    -relief  "ridge"  -padx "3"   -text "Reset Log" -command "EffaceLeLog $LBLog"
    label     $LLog1  -background "#808FFF"  -relief  "groove"
    label     $LLog2  -background "#808FFF"  -relief  "groove"

    #6.1 Log Panel
    grid $FVisible -sticky ewns   
    pack  $FLog    -fill x    -side top
    pack   $SBVLog -fill y    -side right 
    pack   $LBLog  -fill both
    #pack $SBHLog  -fill x    -side bottom 
    pack  $FLog2   -fill x    -side top
    pack   $LLog1  -fill both -side left -expand true
    pack   $BLog   -fill both -side left
    #pack   $BLog2  -fill both -side left
    pack   $LLog2  -fill both -side left -expand true
    grid columnconfigure $FVisible 0 -weight 1

    return ""
}

proc AjouteAuLog {Log} {
  
    Wup "Permet d ecrire dans le log du panel general (GUI) ou par defaut d ecrire sur la sortie terminal"

    global LBLog

    #if {! [info exists LBLog] || $Log==""} {Espionne "$Log";return ""}
    if {! [info exists LBLog]} {Espionne "$Log";return ""}

    $LBLog configure -state normal
    $LBLog insert    "end"  "$Log\n"
    $LBLog configure -state disabled
    #Pour se positionner a la fin du texte
    $LBLog see end
    return ""
}

proc EffaceLeLog {} {

    Wup "Permet d effacer le log du panel general"

    global LBLog
    
    if {$LBLog==""} {return ""}
    
    $LBLog configure -state normal
    $LBLog delete    0.0    end
    $LBLog configure -state disabled
    return ""
}

proc OLDEffaceLeLog {LBLog} {

    Wup "Permet d effacer  le log du panel general"

    global $LBLog
    
    if {$LBLog==""} {return ""}
    
    $LBLog configure -state normal
    $LBLog delete    0.0    end
    $LBLog configure -state disabled
    return ""
}

proc HideGridedFrame {Frame} {

    Wup "Permet d oublier une frame griddee et de s en rappeller le moment venu"

    global PosFramePerdue

    if {[info exists PosFramePerdue] && [info exists PosFramePerdue($Frame)]} {
	grid $Frame
	unset PosFramePerdue($Frame)
	return ""
    }
    set PosFramePerdue($Frame) 1
    grid remove $Frame
    return ""
}

proc ManageFiles {} {

    Wup "Manageur des fichiers a partir du CADO4MI_Panel"
    Wup "Permet de supprimer des fichiers crees"
    Wup "Permet de sauver les resultats dans un fichier delimite par des tabulations"
    
    set w ".filemanagement"
    if {[winfo exists $w]} {wm deiconify $w;raise $w;return ""} else {toplevel $w}
    
    wm geometry  $w +300+100
    wm title     $w "Menu"
    wm resizable $w false false
    wm protocol  $w WM_DELETE_WINDOW "destroy $w"

    #Frame Generale contenant les autres frame et le bouton dismiss
    set     FirstFrame    "$w.framehome"
    set     ButtonDismiss "$FirstFrame.bdismiss"
    frame  $FirstFrame    -borderwidth "3"  -relief "groove"
    button $ButtonDismiss -background "red" -relief "ridge" -text "Dismiss" -padx "2" -command "eval \[wm protocol $w WM_DELETE_WINDOW\]"

    #Frame pour supprimer les types de fichier
    set     FInfo       "$FirstFrame.finfo"
    set     ButtonErase "$FInfo.berase"
    frame  $FInfo       -borderwidth "3"          -relief "groove"
    button $ButtonErase -background  "OliveDrab1" -relief  "ridge" -text "Delete File" -padx "2" -command {
	set  LesExtensions [EntreDeux [list ".masked" ".blastn" ".oligo" ".log" ".selection" ".croisee" ".working"]]
	if {$LesExtensions == ""} {return ""}

	set  BrowsedDir [tk_chooseDirectory -title "Choose one directory." -initialdir "[HOMEDIR]"]
	if {$BrowsedDir == ""} {return ""}

	set ListeAcc ""
	foreach Acc [glob -nocomplain -directory $BrowsedDir "*.masked"] {
	    regsub ".masked" $Acc "" Acc
	    lappend ListeAcc [file tail $Acc]
	}
	set  LesQuels [EntreDeux $ListeAcc]
	if {$LesQuels == ""} {return ""}

	if {[OuiOuNon "Are you sure to erase the $LesExtensions file from $BrowsedDir"]} {
	    foreach PAB $LesQuels {
		foreach Extension $LesExtensions {
		    set Fichier "[file join ${BrowsedDir} "${PAB}${Extension}"]"
		    if {[file exists $Fichier]} {file delete -force $Fichier}
		}
	    }
	}
    }
    set Texte [format "%-10s" "Allow to erase the different types of files generated."]
    set_balloon $ButtonErase "$Texte"

    #Frame pour sauver les resultats dans un fichier separes par des tabulations
    #set    FExcel         "$FirstFrame.frameexcel"
    #set    ButtonToExcel  "$FExcel.bsave"
    #frame  $FExcel        -borderwidth "3"          -relief "groove"
    #button $ButtonToExcel -background  "OliveDrab1" -relief  "ridge" -text "Save Results" -padx "2" -command "LesOligosPourExcel_GUI"
    #set Texte [format "%-10s" "Allow to save the results in a tab-delimited file."]
    #set_balloon $ButtonToExcel "$Texte"

    #Frame pour sauver les resultats dans un fichier separes par des tabulations
    #set    FFont         "$FirstFrame.framefont"
    #set    ButtonToFont  "$FFont.bsave"
    #frame  $FFont        -borderwidth "3"          -relief "groove"
    #button $ButtonToFont -background  "OliveDrab1" -relief  "ridge" -text "Font Panel" -padx "2" -command "Font_Panel"
    #set Texte [format "%-10s" "Setup:\nAllows you to change Font parameters"]
    #set_balloon $ButtonToFont "$Texte"

    pack $FirstFrame -fill both -expand true

    pack  $FInfo          -side top -fill both -expand true
    pack  $ButtonErase    -side top -fill both -expand true
    #pack  $FExcel         -side top -fill both -expand true
    #pack  $ButtonToExcel  -side top -fill both -expand true
    #pack  $FFont          -side top -fill both -expand true
    #pack  $ButtonToFont   -side top -fill both -expand true

    pack $ButtonDismiss   -side top -fill both -expand true
    return ""
}


#########################################
#   3.Gestion des parametres du design  #
#########################################
proc LesParametresDuDesign args {
    
    Wup "Permet de gerer et stocker les parametres du design et de les donner a toutes les procedures qui en ont besoin"

    #Utilisation   Qui ValeurOuNon Quoi 

    #Valeur pour Qui (ex):
    #Areas
    #NbOligo ...

    #Valeur pour Quoi:
    #Positionne    Positionne le parametre a la valeur
    #Ajoute        Ajoute ou positionne le parametre si il n existe pas
    #Existe        Retourne 1 si la valeur existe ou 0 sinon
    #LaValeurMerci Retourne la valeur du parametre ou vide sinon
    #Reset         Positionne la valeur a vide
    #UnsetAll      Efface tout

    #Particulier:
    #Qui=all                  Retourne toutes les valeurs du tableau
    #Qui=UnsetAll             Vide le tableau en entier
    #Quoi=Reset et Valeur=all Supprime toutes les entrees de Qui (Ex:Areas*)
    #Qui=Areas peut etre seul ou associe avec LaSequence (voir les join args par ",")

    global ParametresDuDesign

    set  Qui [lindex $args 0]
    if {      $Qui==""} {
	return      ""
    } elseif {$Qui=="all"} {
	if {[info exists ParametresDuDesign]} {return [array get ParametresDuDesign]}  
    } elseif {$Qui=="UnsetAll"}               {
	array unset ParametresDuDesign "*"
	return      ""
    }
    
    set NbArgs [llength $args]
    #Valeur si il y en a une ok sinon c est aussi $Qui
    set Valeur [lindex $args end-1]
    set Quoi   [lindex $args end]

    if {$Quoi==""} {set Quoi "LaValeurMerci"}
    if {$Quoi=="Positionne"} {
	if {$Qui=="Areas" && [expr $NbArgs>3]} {
	    set Qui [join [lrange $args 0 end-2] ","]
	    if {![info exists ParametresDuDesign(Areas,LesQuery)]} {
		set ParametresDuDesign(Areas,LesQuery) ""
	    }
	    lappend ParametresDuDesign(Areas,LesQuery) [lrange $args 1 end-2]
	    set     ParametresDuDesign(Areas,LesQuery) [lsort -unique [set ParametresDuDesign(Areas,LesQuery)]]
	}
	set ParametresDuDesign($Qui) $Valeur
	return ""
    } elseif {$Quoi=="Reset"} {  
	if {$Valeur=="all"} {
	    array unset ParametresDuDesign "$Qui*"
	} else {
	    if {$Qui=="Areas" && [expr $NbArgs>2]} {
		set Qui [join [lrange $args 0 end-1] ","]
	    }
	    if {[info exists ParametresDuDesign($Qui)]} {unset ParametresDuDesign($Qui)}
	}
    } elseif {$Quoi=="LaValeurMerci"} {
	if {$Qui=="Areas" && [expr $NbArgs>2]} {
	    set Qui [join [lrange $args 0 end-1] ","]
	}
	if {[info exists ParametresDuDesign($Qui)]} {return [set ParametresDuDesign($Qui)]} else {return ""}
    } elseif {$Quoi=="Ajoute"} {
	if {$Qui=="Areas" && [expr $NbArgs>3]} {
	    set Qui [join [lrange $args 0 end-2] ","]
	    if {![info exists ParametresDuDesign(Areas,LesQuery)]} {
		set ParametresDuDesign(Areas,LesQuery) ""
	    }
	    lappend ParametresDuDesign(Areas,LesQuery) [lrange $args 1 end-2]
	    set     ParametresDuDesign(Areas,LesQuery) [lsort -unique [set ParametresDuDesign(Areas,LesQuery)]]
	}
	lappend ParametresDuDesign($Qui) $Valeur
    } elseif {$Quoi=="Existe"} {
	if {$Qui=="Areas" && [expr $NbArgs>2]} {
	    set Qui [join [lrange $args 0 end-1] ","]
	}
	if {[info exists ParametresDuDesign($Qui)]} {return 1} else {return 0}
    }
        
    return ""
}

proc LesParametresDuDesign_OLD {Qui {Valeur ""} {Quoi ""}} {
    
    Wup "Permet de gerer et stocker les parametres du design et de les donner a toutes les procedures qui en ont besoin"

    #Positionne    Positionne le parametre a la valeur
    #Ajoute        Ajoute ou positionne le parametre si il n existe pas
    #Existe        Retourne 1 si la valeur existe ou 0 sinon
    #LaValeurMerci Retourne la valeur du parametre ou vide sinon
    #Reset         Positionne la valeur a vide
    #UnsetAll      Efface tout

    global ParametresDuDesign

    #Espionne "Param $Qui $Valeur $Quoi"
    if {$Qui==""} {
	return ""
    } elseif {$Qui=="all"} {
	if {[info exists ParametresDuDesign]} {return [array get ParametresDuDesign]}  
    } elseif {$Qui=="UnsetAll"}               {
	array unset ParametresDuDesign "*"
	return ""
    }

    if {$Quoi==""} {set Quoi "LaValeurMerci"}
    if {$Quoi=="Positionne"} {
	set ParametresDuDesign($Qui) $Valeur
	return ""
    } elseif {$Quoi=="Reset"} {
	if {[info exists ParametresDuDesign($Qui)]} {unset       ParametresDuDesign($Qui)}
    } elseif {$Quoi=="LaValeurMerci"} {
	if {[info exists ParametresDuDesign($Qui)]} {return [set ParametresDuDesign($Qui)]} else {return ""}
    } elseif {$Quoi=="Ajoute"} {
	lappend          ParametresDuDesign($Qui) $Valeur
    } elseif {$Quoi=="Existe"} {
	if {[info exists ParametresDuDesign($Qui)]} {return 1} else {return 0}
    }
    return ""
}

proc LesParametresObligatoires {Param} {

    Wup "Permet de savoir quels sont les parametres obligatoires"

    global CADO4MI_Obligatoire
    
    if {[info exists CADO4MI_Obligatoire] && [info exists CADO4MI_Obligatoire(Query)]} {
	if {[info exists CADO4MI_Obligatoire($Param)]} {return "1"} else {return "0"}
    } else {
	#Les parametres obligatoires
	set LesParamsObligatoires [list \
				       Query \
				       TmLimit \
				       TmRange \
				       LengthOligo \
				       WorkingDir \
				       DataBase]
	
	foreach P $LesParamsObligatoires {
	    set CADO4MI_Obligatoire($P) 1
	}
    }
    
    return "[LesParametresObligatoires $Param]"
}

proc LesParametresParDefaut {} {

    Wup "Positione les parametres du design par defaut"

    #Les parametres par defauts
    #Pour le nombre d oligo         100
    #Pour la methode de calcul du Tm NearestNeighbor
    #ATTENTION MODIFIER AUSSI DANS CADO4MI_TmEtGC.tcl
    #Pour la concentration sel (Tm)   1.0
    #Pour la concentration formamide 12.3 
    #Pour la concentration DNA        0.000001 
    #Pour la Specificite 70 percent et 15 bases contigues
    #Pour le Pas on positionne a     10
    #Pour le Blast expect limite    100.0
    #Pour le Blast taille des mots   15
    #Pour le Blast max a aligne    2000
    #Pour le Blast quel brin (Fwd)    1
    #Pour le Blast filtre             F
    #Pour le valider la query         1

    set LesParamsParDefaut [list \
	    NbOligo            "100" \
	    TmMethod "NearestNeighbor" \
	    TmSalt               "1.0" \
	    TmFormamide         "12.3" \
	    TmConcentration      "0.000001" \
	    IdLimit             "70.0" \
	    NbContiguous        "15" \
	    Step                "10" \
	    BlastExpect        "100.0" \
	    BlastWord           "15" \
	    BlastNbAligned    "2000"\
	    BlastStrand          "1"\
	    BlastFilter          "T"\
	    QueryCheck           "1"]
    foreach {Param Valeur} $LesParamsParDefaut {
	if {! [LesParametresDuDesign $Param         "Existe"]} {
	    LesParametresDuDesign    $Param $Valeur "Positionne"
	} else {continue}
    }
    return ""
}

proc BonNombreDeParametres {{Comment "1"}} {

    Wup "Verifie le bon nombre de parametre pour le design"

    if {$Comment=="AvecVerif"} {set SansVerif "0"} else {set SansVerif "1"}
    if {$SansVerif && [LesParametresDuDesign BonNombreDeParametre "Existe"]} {return "[LesParametresDuDesign BonNombreDeParametre "LaValeurMerci"]"}
    if {    [LesParametresDuDesign LengthOligo    Existe] &&\
	    [LesParametresDuDesign Query          Existe] &&\
	    [LesParametresDuDesign TmLimit        Existe] &&\
	    [LesParametresDuDesign Step           Existe] &&\
	    [LesParametresDuDesign IdLimit        Existe] &&\
	    [LesParametresDuDesign NbContiguous   Existe] &&\
	    [LesParametresDuDesign WorkingDir     Existe] &&\
	    [LesParametresDuDesign BlastExpect    Existe] &&\
	    [LesParametresDuDesign BlastWord      Existe] &&\
	    [LesParametresDuDesign BlastNbAligned Existe] &&\
	    [LesParametresDuDesign BlastFilter    Existe] &&\
	    [LesParametresDuDesign DataBase       Existe]} {
	LesParametresDuDesign BonNombreDeParametre 1 "Positionne"
	return "1"
    } else {
	LesParametresDuDesign BonNombreDeParametre 0 "Positionne"
	return "0"
    } 
}

proc MontreLesParametresDuDesign {} {
    
    Wup "Permet de montrer les parametres du design quelque soit la situation (GUI ou non)"
    
    AjouteAuLog "Your parameters for the design:"

    set LesParams [list \
	    Query Query \
	    Length LengthOligo \
	    Tm TmLimit \
	    "Tm Range" TmRange \
	    DataBase DataBase \
	    "Saving Dir" WorkingDir \
	    "Nb Oligo" NbOligo \
	    Areas Areas \
	    GCUpper GCContentPlus \
	    GCLower GCContentMinus \
	    Prohibited Prohibited \
	    Step Step \
	    Specificity IdLimit \
	    Contiguous NbContiguous \
	    "Tm Method" TmMethod \
	    "Tm Salt" TmSalt \
	    "Tm Form." TmFormamide \
	    "Tm \[DNA\]" TmConcentration \
	    BlastExpect BlastExpect \
	    BlastWord BlastWord \
	    BlastNbAligned BlastNbAligned \
	    BlastStrand BlastStrand \
	    BlastFilter BlastFilter \
	    "Check Query" QueryCheck]    

    #LesParametresObligatoires
    foreach {Nom Param} $LesParams {
	if {![LesParametresDuDesign $Param "Existe"]} {
	    if {[LesParametresObligatoires $Param]} {
		AjouteAuLog [format "%-15s %-s" $Nom "Missing"]
	    } else {
		AjouteAuLog [format "%-15s %-s" $Nom "Not used"]
	    }
	} else { 
	    if {$Param=="Areas"} {
		#On doit traiter 2 cas differents
		#1 parametres globaux - ok regle par la suite
		#2 parametres specifiaues a chaque query
		if {[LesParametresDuDesign $Param LesQuery "LaValeurMerci"] != ""} {
		    foreach Q [LesParametresDuDesign $Param LesQuery "LaValeurMerci"] {
			AjouteAuLog [format "%-15s %-s" "$Nom ($Q)" [join [LesParametresDuDesign $Param $Q "LaValeurMerci"] " "]]
		    }
		}
		AjouteAuLog [format "%-15s %-s" $Nom [join [LesParametresDuDesign $Param "LaValeurMerci"] " "]]
		continue
	    }
	    AjouteAuLog     [format "%-15s %-s" $Nom       [LesParametresDuDesign $Param "LaValeurMerci"]]
	}
    }
    return ""
}

proc SauveLesParametresDuDesign {Fichier} {
    
    Wup "Permet de sauver les parametres du design quelque soit la situation (GUI ou non)"
    
    set LesParams [list \
	    Query Query \
	    Length LengthOligo \
	    Tm TmLimit \
	    "Tm Range" TmRange \
	    DataBase DataBase \
	    "Saving Dir" WorkingDir \
	    "Nb Oligo" NbOligo \
	    Areas Areas \
	    GCUpper GCContentPlus \
	    GCLower GCContentMinus \
	    Prohibited Prohibited \
	    Step Step \
	    Specificity IdLimit \
	    Contiguous NbContiguous \
	    "Tm Method" TmMethod \
	    "Tm Salt" TmSalt \
	    "Tm Form." TmFormamide \
	    "Tm \[DNA\]" TmConcentration \
	    BlastExpect BlastExpect \
	    BlastWord BlastWord \
	    BlastNbAligned BlastNbAligned \
	    BlastStrand BlastStrand \
	    BlastFilter BlastFilter \
	    "Check Query" QueryCheck]    

    set LesLignes ""
    #LesParametresObligatoires
    foreach {Nom Param} $LesParams {
	if {![LesParametresDuDesign $Param "Existe"]} {
	    if {[LesParametresObligatoires $Param]} {
		lappend LesLignes [join [list $Param "Missing"] "\t"]
	    } else {
		lappend LesLignes [join [list $Param "Not used"] "\t"]
	    }
	} else { 
	    if {$Param=="Areas"} {
		lappend LesLignes  [join [list $Param [join [LesParametresDuDesign $Param "LaValeurMerci"] " "]] "\t"]
		continue
	    }
	    lappend LesLignes [join [list $Param [join [LesParametresDuDesign $Param "LaValeurMerci"] " "]] "\t"]
	}
    }
    if {[file exists $Fichier]} {
	file delete -force $Fichier
    }

    AppendAuFichier $Fichier [join $LesLignes "\n"]

    return ""
}

proc MenuSauveLesParametres {{FichierSave ""}} {
    
    Wup "Permet de sauver les parametres du design quelque soit la situation (GUI ou non)"
        
    if {$FichierSave == ""} {
	while {1} {
	    #set Texte "Choose a file name to save your parameters."
	    #FaireLire "$Texte"	
	    set types {
		{{CADO4MI files} {.parameters}}
		{{All files}     *}
	    }
	    set IniFile "Design.parameters"
	    if {[BROWSEDDIR] != ""} {
		set FichierSave [tk_getSaveFile -title "Save parameters - Choose a file name." -initialdir "[BROWSEDDIR]" -initialfile "$IniFile" -filetypes $types]
	    } else {
		set FichierSave [tk_getSaveFile -title "Save parameters - Choose a file name." -initialdir "[HOMEDIR]"    -initialfile "$IniFile" -filetypes $types]
	    }
	    if {$FichierSave == ""} {
		break
	    } else {
		set BrowsedDir [file dirname [lindex $FichierSave end]]
		BROWSEDDIR $BrowsedDir
		if {[file exists $FichierSave]} {
		    file delete -force $FichierSave
		} 
		break
	    }
	}
    } else {
	if {[file exists $FichierSave]} {
	    file delete -force $FichierSave
	}
    }

    set LesParams [list \
	    Query \
	    LengthOligo \
	    TmLimit \
	    TmRange \
	    DataBase \
	    WorkingDir \
	    NbOligo \
	    Areas \
	    GCContentPlus \
	    GCContentMinus \
	    Prohibited \
	    Step \
	    IdLimit \
	    NbContiguous \
	    TmMethod \
	    TmSalt \
	    TmFormamide \
	    TmConcentration \
	    BlastExpect \
	    BlastWord \
	    BlastNbAligned \
	    BlastStrand \
	    BlastFilter \
	    QueryCheck]    
    
    set LesParametres ""
    foreach Param $LesParams {
	if {![LesParametresDuDesign $Param "Existe"]} {
	    if {[LesParametresObligatoires $Param]} {
		lappend LesParametres [join [list $Param ""] "\t"]
	    } else {
		lappend LesParametres [join [list $Param ""] "\t"]
	    }
	} else { 
	    if {$Param == "Areas"} {
		#On doit traiter 2 cas differents
		#1 parametres globaux - ok regle par la suite
		#2 parametres specifiaues a chaque query
		if {[LesParametresDuDesign $Param LesQuery "LaValeurMerci"] != ""} {
		    foreach Q [LesParametresDuDesign $Param LesQuery "LaValeurMerci"] {
			lappend LesParametres [join [list "$Param specific to $Q" [LesParametresDuDesign $Param $Q "LaValeurMerci"]] "\t"]
		    }
		}
	    }
	    lappend LesParametres [join [list $Param [LesParametresDuDesign $Param "LaValeurMerci"]] "\t"]
	}
    }
    
    if {$FichierSave != ""} {
	AppendAuFichier $FichierSave [join $LesParametres "\n"]
	AjouteAuLog "\nParameters saved in $FichierSave"
    }
    
    return ""
}

proc MenuChargeLesParametres {} {

    Wup "Permet de relire et de charger les parametres"
    
    set types {
	{{CADO4MI files} {.parameters}}
	{{All files}     *}
    }
    set IniFile "Design.parameters"
    if {[BROWSEDDIR] != ""} {
	set FichierLoad [tk_getOpenFile -title "Load parameters - Choose 1 project file." -initialdir "[BROWSEDDIR]" -filetypes $types]
    } else {
	set FichierLoad [tk_getOpenFile -title "Load parameters - Choose 1 project file." -initialdir "[HOMEDIR]"    -filetypes $types]
    }
    if {$FichierLoad == ""} {
	return ""
    } else {
	set BrowsedDir [file dirname [lindex $FichierLoad end]]
	BROWSEDDIR $BrowsedDir
    }

    set F [open "$FichierLoad"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	
	set LigneSplitee [split $Ligne "\t"]
	set Param [lindex $LigneSplitee 0]

	if {[regexp "specific to" $Param]} {
	    #"$Param specific to $Q" 
	    set Value [lindex $LigneSplitee 1]
	    set Query ""
	    regsub "Areas specific to " $Param  "" Query
	    
	    LesParametresDuDesign Areas $Query $Value "Positionne"
	} else {
	    set Value [lindex $LigneSplitee 1]
	    LesParametresDuDesign $Param $Value "Positionne"
	}
    }
    close $F
    
    AjouteAuLog "\nParameters loaded from $FichierLoad"
    
    return ""
}

proc InterrogeDesignParameter args {

    Wup "Permet de questionner les infos des Parametres de design (cf MenuSauveLesParametres et MenuChargeLesParametres)"
    Wup "Voir aussi decharge et charge"
    
    global InfoDesignP
    
    #Arguments:
    ###########
    #$FileParam $Param     - Retourne la valeur du parametre stocke
    #$FileParam Parameters - Liste des parametres stockes

    if {$args == "unset"} {
	if {[info exists InfoDesignP]}    {array unset InfoDesignP}
	return ""
    } 
    
    set Quoi [join $args ","]
    if {[info exists InfoDesignP($Quoi)]} {return [set InfoDesignP($Quoi)]}
    
    set FichierACharger [lindex $args 0]
    if {! [info exists InfoDesignP($FichierACharger,EstCharge)]} {set InfoDesignP($FichierACharger,EstCharge) 0}
    if {$Quoi=="$FichierACharger,Decharge"} {
	#Espionne "Decharge $FichierACharger"
	DechargeDesignParameter $FichierACharger
	return ""
    }
    if {  [info exists InfoDesignP($FichierACharger,EstVide)]}   {return ""}
    if {  [info exists InfoDesignP($Quoi)]}                      {return [set InfoDesignP($Quoi)]}

    ###################CHARGE le Fichier avec les Oligos#########################
    if {! [info exists InfoDesignP($FichierACharger,EstCharge)] || [set InfoDesignP($FichierACharger,EstCharge)]=="0"} {
	set InfoDesignP($FichierACharger,EstCharge) 1
	#Espionne "On charge $FichierACharger"

	set  ResultatCharge [ChargeDesignParameter InfoDesignP $FichierACharger]
	if {$ResultatCharge != ""} {
	    set InfoDesignP($FichierACharger,EstVide) 1
	    return ""
	}
    }
    if {![info exists InfoDesignP($Quoi)]} {return ""}
    return [set InfoDesignP($Quoi)] 
}

proc ChargeDesignParameter {aInfoDesignP FileParam} {

    Wup "Permet de charger les infos du design sauve dans le repertoire de design"

    upvar $aInfoDesignP InfoDesignP

    if {! [file exists $FileParam]} {set InfoDesignP($FileParam,NotDone) 1;return "-2"}
    
    set F [open "$FileParam"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}

	set  Ligne [split $Ligne "\t"]
	set Param  [lindex $Ligne 0]
	set Valeur [lindex $Ligne 1]

	if {[regexp "specific to" $Param]} {
	    set Query ""
	    regsub "Areas specific to " $Param  "" Query
	    
	    set InfoDesignP($FileParam,Areas,$Query) $Valeur
	    if {! [info exists InfoDesignP($FileParam,Parameters)]} {
		set InfoDesignP($FileParam,Parameters) ""
	    }
	    lappend InfoDesignP($FileParam,Parameters) Areas

	} else {
	    set InfoDesignP($FileParam,$Param) $Valeur
	    if {! [info exists InfoDesignP($FileParam,Parameters)]} {
		set InfoDesignP($FileParam,Parameters) ""
	    }
	    lappend InfoDesignP($FileParam,Parameters) $Param
	}
	
    }
    close $F

    set InfoDesignP($FileParam,Parameters) [lsort -unique [set InfoDesignP($FileParam,Parameters)]]


    return ""
}

proc DechargeDesignParameter {FileParam} {
    
    Wup "Decharge les infos chargees avec ChargeDesignParameter"

    global InfoDesignP

    array unset InfoDesignP "$FileParam*"
    
    return ""
}