#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

#############################################################################
#   1.Prise en charge du fichier au format fasta avec plusieurs sequences   # 
#   2.Proc pour gerer les informations des banques                          #
#   3.Proc pour generer les informations de Unigene                         #
#############################################################################

###########################################################################
#   Prise en charge du fichier au format fasta avec plusieurs sequences   # 
###########################################################################
proc CheckTFAFile {Fichier} {

    Wup "Permet de tester et de decider pour un fichier fasta multiple"
    Wup "Retourne vide si rien n est fait et le nom des fichiers si des fichiers sont crees"

    if {! [file exists $Fichier]} {return ""}

    set LesNewFiles ""
    if {[EstCeUnTFAOuUnTFAS $Fichier]>1} {
	if {[OuiOuNon "$Fichier seems to be a multiple fasta sequence file.\nI need one file per sequence.\nWould you like to split your sequences ?"]} {
	    set Dir         [tk_chooseDirectory -title "Choose a saving directory." -initialdir "[RepertoireDuGenome]"]
	    set LesNewFiles [SplitLeTFAS $Fichier $Dir]
	}
    }

    return "$LesNewFiles"
}

proc EstCeUnTFAOuUnTFAS {{Fichier ""}} {

    Wup "Test si le fichier est un tfa multiple ou pas"
    Wup "Retourne le nombre de chevrons par fichier"

    if {$Fichier == "" || ! [file exists $Fichier]} {return "0"}

    set nbChevrons 0
    set nbLigne    0
    set F [open "$Fichier"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	incr nbLigne
	if {  [string range $Ligne 0 0] == ">"} {
	    incr nbChevrons
	}
	#Attention on doit trouver au moins un chevron dans les 3 premieres lignes
	if {$nbChevrons=="0" && $nbLigne > 3} {break}
    }
    close $F

    return "$nbChevrons"
}

proc SplitLeTFAS {{Fichier ""} {Destination ""}} {

    Wup "Split le fichier fasta multiple en fichier separes"
    Wup "Retourne les noms de fichiers crees"

    if {   $Destination == ""} {
	set Destination [tk_chooseDirectory -title "Choose a saving directory." -initialdir "[RepertoireDuGenome]"]
    }
    if {   $Destination != ""} {set Destination "$Destination/"}

    set PABOk 0
    if {1} {
	if {[OuiOuNon "The name of the files will be the word next to the >\nex: >NM_001100 will be saved in NM_001100\nDo you want to use this (Yes) or choose yourself the file name (No) ?"]} {
	    set PABOk 1
	} else {set PABOk 0}
    }
    set PABOk 1

    set i           0
    set FileToSave  ""
    set LesNewFiles ""
    set NewLines    ""
    set F [open "$Fichier"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {  [string range $Ligne 0 0] == ">"} {
	    incr i

	    if {$NewLines == ""} {
 		lappend NewLines $Ligne
	    } else {
		if {$FileToSave != "" && [file exists $FileToSave]} {
		    if {[OuiOuNon "$FileToSave already exists !\nWould you like to save either and lost previous?"]} {
			file delete -force $FileToSave
			#Espionne "[join $NewLines "\n"]"
			AppendAuFichier $FileToSave [join $NewLines "\n"]
		    } else {
			set FileToSave ""
		    }
		} else {
		    #Espionne "[join $NewLines "\n"]"
		    AppendAuFichier $FileToSave [join $NewLines "\n"]
		}
	    }
	    #Espionne "$FileToSave"
	    #Espionne "$NewLines"

	    set     NewLines ""
	    lappend NewLines $Ligne

	    if {$PABOk} {
		scan $Ligne "%s" PAB
		regsub {\>} $PAB "" PAB
		if {[regexp "RSHUM:" $PAB]} {
		    regsub  "RSHUM:" $PAB "" PAB
		}
		set FileToSave "${Destination}${PAB}"
	    } else {
		set FileToSave ""
		while {$FileToSave==""} {
		    set FileToSave [tk_getSaveFile -title "Choose a file name to save sequence N${i}" -initialdir "[RepertoireDuGenome]"]
		}
	    }

	    if {$FileToSave != ""} {
		lappend LesNewFiles $FileToSave
	    }
	    continue
	}
	lappend NewLines $Ligne
    }
    close $F

    if {$NewLines != ""} {
	if {[file exists $FileToSave]} {
	    if {[OuiOuNon "$FileToSave already exists !\nWould you like to save either and lost previous?"]} {
		file delete -force $FileToSave
		AppendAuFichier $FileToSave [join $NewLines "\n"]
	    } else {
		set FileToSave ""
	    }
	} else {
	    AppendAuFichier $FileToSave [join $NewLines "\n"]
	}
    }
    if {$FileToSave != ""} {
	lappend LesNewFiles $FileToSave
    }
    
    return "[lsort -unique $LesNewFiles]"
}

####################################################
#   Proc pour gerer les informations des banques   #
#        de sequences (ex: unigene et uniseq)      #
####################################################
proc DefDeAcc {AccessOuId Banques {Option "EntreeComplete"} {Link "NoLink"}} {

    Wup "Pour pouvoir recuperer la definition la taille par getz ..."

    set option "-e"

    if {$Option == "EntreeComplete"} {
	set option "-e"
    } elseif {$Option == "IdDef"} {
	set option "-f \"id des\""
    } elseif {$Option == "AccDef"} {
	set option "-vf \"acc des\""
    } elseif {$Option == "Def"} {
	set option "-vf \"des\""
    } elseif {$Option == "Id"} {
	set option "-vf \"id\""
    } elseif {$Option == "Taille"} {
	set option "-vf \"sl\""
    } elseif {$Option == "Acc"} {
	set option "-f \"acc\""
    }
    
    set LeType [list "acc" "id" "prd" "gid"] 
    foreach Type $LeType {
	if {$Link   == "NoLink"} {
	    set Commande "[CADO4MI_PATH getz] $option \"\\\[$Banques-$Type: $AccessOuId\\\]\""
	} else {
	    set Commande "[CADO4MI_PATH getz] $option \"\\\[$Banques-$Type: $AccessOuId\\\] \\\> $Link \""
	}
	catch {set GetzOut [eval exec $Commande]} Message
	if {![info exists GetzOut] || $GetzOut == ""} {continue}
	return $GetzOut
    }
    return ""
}

proc AccessDuTFADumRNA {PAB} {

    Wup "Rend le banque Id du TFA"

    if {! [file exists $PAB]} {return ""}

    set F [open "$PAB"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {  [string range $Ligne 0 0] != ">"}           {continue}

	regsub "^>" $Ligne "" Ligne
	set AccessLu ""
	set BanqueId ""
	set bidon    ""
	scan $Ligne "%s %s %s" bidon BanqueId AccessLu
    }
    if {  ($BanqueId==":" || $BanqueId=="mRNA") && $AccessLu!=""} {set BanqueId $AccessLu} 
    return $BanqueId
}

proc AccessDuTFADeLaProteine {PAB} {

    Wup "Rend le banque Id du TFA"

    if {! [file exists $PAB]} {return ""}

    set F [open "$PAB"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {  [string range $Ligne 0 0] != ">"}           {continue}
	regsub "^>" $Ligne "" Ligne
	set AccessLu ""
	set BanqueId ""
	set bidon    ""
	scan $Ligne "%s %s %s" bidon BanqueId AccessLu
    }
    if {  ($BanqueId==":" || $BanqueId=="Protein" || $BanqueId=="PROTEIN") && $AccessLu!=""} {set BanqueId $AccessLu} 
    return $BanqueId
}

proc AccessDeLaBoite {PAB {Quoi ""} {DefOuNon ""}} {

    Wup "Retourne l Access de la boite Prot mrna ou tfa originel ou boite"
    Wup "Attention que pour Homo sapiens"

    #Prot puis mrna puis nuctfa puis PAB
    set Access ""
    set OnATrouve 0
  
    if {$Quoi=="protein"} {set OnATrouve 1}

    if {! $OnATrouve || $Quoi=="protein" || [string equal -nocase $Access ""]} {
	set Access [InterrogeBestProteinAndmRNAForeachOrga $PAB Organisme "Homo sapiens" Protein Access]
    }
    if {! $OnATrouve || $Quoi=="mRNA"    || [string equal -nocase $Access ""]} {
	set Access [InterrogeBestProteinAndmRNAForeachOrga $PAB Organisme "Homo sapiens" mRNA Access]
	if {$Quoi=="mRNA"} {set OnATrouve 1}
    }
    if {! $OnATrouve && $Access == ""} {
	set FichierNuctfa "[file join [RepertoireDuGenome] nuctfa $PAB]"
	if {[file exists $FichierNuctfa]} {
	    set Access [EnteteDuFichierTFA $FichierNuctfa "access"]
	}
    }
    if {$Access == "" } {set Access $PAB}
    if {$DefOuNon=="Definition"} {
	set Access [InterrogeBestProteinAndmRNAForeachOrga Access $Access Definition]
    }
    return "$Access" 
}

proc FastaCmd {LesAcc {Complet ""} {AvecAffichage "1"} {Banque ""}} {

    Wup "Pour les access unigene on recupere la definition par fastacmd"
    
    #Il faudrait mettre un defaut commme genbank par exemple
    
    set Results ""
    
    if {[CADO4MI_PATH fastacmd]!=""} {
	foreach Acc $LesAcc {
	    if {$Banque != ""} {
		if {[regexp {\.[a-z]{3}$} $Banque]} {regsub {\.[a-z]{3}$} $Banque "" Banque}
		set CommandeFastaCmd "[CADO4MI_PATH fastacmd] -d $Banque                -s $Acc"
	    } elseif {      [regexp "UG:Hs"  $Acc]} {
		set CommandeFastaCmd "[CADO4MI_PATH fastacmd] -d [CADO4MI_PATH Unigene] -s $Acc"
	    } elseif {[regexp "RSHUM:" $Acc]} {
		set CommandeFastaCmd "[CADO4MI_PATH fastacmd] -d [CADO4MI_PATH rshuman] -s $Acc"
	    } elseif {[regexp -nocase {[a-z]{2}_[0-9]{1,}} $Acc]} {
		set CommandeFastaCmd "[CADO4MI_PATH fastacmd] -d [CADO4MI_PATH RefSeq]  -s $Acc"
	    } else {
		set CommandeFastaCmd ""
		#Espionne "Default fastacmd pour $Acc"
		continue
	    }
	    if {$CommandeFastaCmd==""} {continue}
	    
	    #AjouteAuLog "$CommandeFastaCmd"
	    
	    set             FastaCmdOut ""
	    if {[catch {set FastaCmdOut [eval exec $CommandeFastaCmd]} Message]} {
		AjouteAuLog "$CommandeFastaCmd"
		AjouteAuLog "$Message"
	    }
	    
	    if {![info exists FastaCmdOut]} {continue}
	    if {$FastaCmdOut == ""}         {set FastaCmdOut "No Result for $Acc"}
	    if {$Complet     == ""} {
		regsub -all {>lcl\|} $FastaCmdOut "" FastaCmdOut
		lappend Results [lindex [split $FastaCmdOut "\n"] 0]
	    } else {lappend Results $FastaCmdOut}
	} 
    } else {
	set Results "No Result due to absence of fastacmd"
    }

    if {$AvecAffichage} {
	return "[AfficheVariable [join $Results "\n"] "AvecFormateAvecBlaste" "FastaCmd search result"]"
    } else {
	return "[join $Results "\n"]"
    }
}

proc GetzUniseq_GUI {K Access {Banques "uniseq"} {Link "genbank"} {Option "AccessDef"}} {

    Wup "Affiche grace a getz les liens dans Link pour les Access dans Banques"
    Wup "Option EntreeComplete == sortie getz de toute l entree"
    Wup "Option AccessDef      == sortie getz avec juste Id et Def"

    set  GetzOut [LeGetzUniseq $Access $Banques $Link $Option]
    if {$GetzOut==""} {set GetzOut "No link from $Banques to $Link available for [join $Access " "]"}
 
    set Fen     [AfficheVariable $GetzOut "AvecFormateAvecBlaste" "$Banques to $Link"] 
    if {$Link=="genbank"} {
	set FenB "$Fen.buttons"
	TouchePour $FenB NouvelleGamme
	TouchePour $FenB NouvelleCouleur "white"
	set B [TouchePour $FenB "Link GenBank To" ""]
	TouchePour $FenB NouvelleCouleur "green"
	TouchePour $FenB "Protein Sel" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\] genbank protein All\]}"
	TouchePour <2>    "/Id" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]        genbank protein\]}"
	TouchePour <3>   "/All" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[set PagePropre($Fen)\] genbank protein\]}"
	
	set    Texte [format   "%-10s" "To Link Genbank access to Protein access"]
	append Texte [format "\n%-10s" "Allows to retrieve access only (Id and all) or complete entry (Sel.)"]
	set_balloon $B "$Texte"
	$B configure -state disabled -disabledforeground "black"
    }

    return $Fen
}

proc LeGetzUniseq {Access {Banques "uniseq"} {Link "genbank"} {Option "AccessDef"}} {

    Wup "Recupere grace a getz les liens dans Link pour les Access dans Banques"
    Wup "Option EntreeComplete == sortie getz de toute l entree"
    Wup "Option AccessDef      == sortie getz avec juste Id et Def"
    
    set LesAccess ""
    set UnAccess  ""
    set bidon     ""
    set Access    [split $Access "\n"]

    if {$Banques == "uniseq"} {    
	foreach AC [lsort -unique $Access] {
	    set UnAccess [lindex $AC 0]
	    if {[regexp -nocase "UG:" $AC]} {regsub -all "UG:" $UnAccess ""    UnAccess}
	    if {[regexp -nocase "Hs#" $AC]} {regsub -all "Hs#" $UnAccess "Hs_" UnAccess}
	    lappend LesAccess $UnAccess
	}
	set Access [join $LesAccess "|"]
    } elseif {$Banques == "genbank"} {
	foreach AC [lsort -unique $Access] { 
	    if {! [regexp -nocase {[a-z0-9]+} $AC]}                             {continue}
	    if {  [regexp {^ACT[0-9]{4}$} $AC]}                                 {continue}
	    if {! [regexp "^ACCESSION" $AC] && ! [regexp -nocase "^LOCUS" $AC]} {continue}
	    scan $AC "%s %s" bidon UnAccess
	    lappend LesAccess $UnAccess
	}
	set Access [join [lsort -unique $LesAccess] "|"]
    }

    if {$Access == ""}                                {return ""}
    if {$Option == "EntreeComplete"|| $Option=="All"} {set option "-e"} elseif {$Option == "AccessDef"} {set option "-f \"id des\""}
    set Typ1 id
    set Typ2 acc
    if {$Link   == "NoLink"} {
	#set Commande "[CADO4MI_PATH getz] $option \"\\\[$Banques-id: $Access\\\]\""
	set Commande "[CADO4MI_PATH getz] $option \"(\\\[$Banques-$Typ1: $Access\\\] | \\\[$Banques-$Typ2: $Access\\\])\""
    } else {	
	#set Commande "[CADO4MI_PATH getz] $option \"\\\[$Banques-id: $Access\\\] \\\> $Link \""
	set Commande "[CADO4MI_PATH getz] $option \"(\\\[$Banques-$Typ1: $Access\\\] | \\\[$Banques-$Typ2: $Access\\\]) \\\> $Link \""
    }
    #http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?[uniseq-all:$Access]>protein+-vn+2+-f+id%20des
    #http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?[uniseq-all:$Access]>protein+-vn+2+-e


    catch {unset GetzOut}
    catch {set   GetzOut [eval exec $Commande]} Message

    if {![info exists GetzOut]} {return ""}
    if {$GetzOut=={}}           {return ""}
    return $GetzOut
}
proc testgetzonweb {molid {url ""}} {

    set molid "Hs_S1730160"

    set SiteIGBMC "http://igbmc.u-strasbg.fr"
    set WGetz     "srsbin/cgi-bin/wgetz"

    set SwTr   "swissprot_SP_sptrembl"
    set SwTrNw "swissprot_SP_sptrembl_SP_sptnew"

    #set Defauts(UrlPdbHome)  "$SiteIGBMC/PipeAlign/get_pdb_file.cgi?XXXX"
    #set Defauts(UrlSRSHome)  "$SiteIGBMC/$WGetz?\[uniseq-all:XXXX\]>protein+-vn+2+-e"
    set Defauts(UrlSRSHome)  "$SiteIGBMC/$WGetz?\[uniseq-all:XXXX\]>protein+-vn+2+-vf+\{id_SP_des\}"
    set Defauts(UrlSRSWorld) "$SiteIGBMC/$WGetz?\[uniseq-all:XXXX\]>protein+-vn+2+-vf+id%20des"
    #set Defauts(UrlSRSWorld) "$SiteIGBMC/$WGetz?-e+\[\{$SwTr\}-acc:XXXX\]|\[swissprot-id:XXXX\]+-vn+2"
    #set Defauts(UrlPdbWorld) "$SiteRSCB/pdb/cgi/export.cgi/XXXX.pdb?format=PDB&pdbId=XXXX&compression=None"
    
    if {$url == ""} {
	set site "SRS"
	set url [set Defauts(UrlSRSHome)]
    }
    Espionne "URL: $url"
    if {[regexp "\:" $molid]} { set molid [StringApres ":" dans $molid] }

    regsub -all "XXXX" $url $molid url
    HttpCopy $url LesLignes

    Espionne "URL: $url"

    if {$site == "SRS"} {
	set TexteHTML [join $LesLignes "\n"]
	set Texte [ValeurDeLaBalise "pre" TexteHTML]
	set LesLignes [split $Texte "\n"]
    }
    #Espionne "$Texte"
    #return ""

    #set head "[string range [lindex $LesLignes 0] 0 1]"
    #Espionne "hrad $head"

    set LesBonnesLignes {}
    foreach Ligne $LesLignes {
	if {[regexp -nocase "<A HREF" $Ligne]} {
	    set V [ValeurDeLaBalise A Ligne "NePasRogner"]
	    regsub -nocase {<A HREF.+</A>} $Ligne $V Ligne
	}
	lappend LesBonnesLignes  $Ligne
    }    
    return $LesBonnesLignes
}

proc LeUnigeneDeUniseq_GUI {LesAcc} {
    
    Wup "Permet de recuperer l acc unigene (Hs.524782) a partir de l acc uniseq (Hs_S1000253 ou Hs#S1000253)" 
    
    if {$LesAcc==""} {return ""}
    
    set     Resultat ""
    set     Resultat [LeUGIdUN_Getz     $LesAcc uniseq unigene 1]
    if {   $Resultat==""} {
        set Resultat [LeUGIdUN_Fastacmd $LesAcc]
    }
    if {   $Resultat==""} {
	set Resultat "No Unigene Id or link available for [join $LesAcc " "]"
    } else {
	set Resultat [join $Resultat "\n"]
    }
	
    set     Fen [AfficheVariable $Resultat "AvecFormateAvecBlaste" "Unigene Id search"] 
    return $Fen
}

proc LeUnigeneDeUniseq {LesAcc {Database ""}} {
    
    Wup "Permet de recuperer l acc unigene (Hs.524782) a partir de l acc uniseq (Hs_S1000253 ou Hs#S1000253)" 
    
    if {$LesAcc==""} {return ""}
    
    set     Resultat ""
    set     Resultat [LeUGIdUN_Getz     $LesAcc uniseq unigene 0]
    if {   $Resultat==""} {
	
        set Resultat [LeUGIdUN_Fastacmd $LesAcc $Database UGIdOnly]
    }
    set     Resultat [join $Resultat ""]
    return $Resultat
}

proc LeUGIdUN_Fastacmd {LesAccess {Database ""} {UGIdOnly ""}} {
    
    Wup "Recupere grace a fastacmd le numero unigene"
    
    #####lcl|UG:Hs#S1730160 
    #Homo sapiens actin, alpha 1, skeletal muscle (ACTA1), /cds=(104,1237) /gb=NM_001100 /gi=5016087 /ug=Hs.1288 /len=1374
    set LesResultats ""
    foreach Acc $LesAccess {
	set EnteteFCmd [FastaCmd "$Acc" "" 0 $Database]
	set tmp  ""
	set tmp2 ""
	set Res [regexp -nocase {/ug=([a-z][a-z].[0-9]+)} $EnteteFCmd tmp tmp2]
	if {$UGIdOnly==""} {
	    lappend LesResultats "$Acc $tmp2"
	} else { 
	    lappend LesResultats "$tmp2"
	}
    }
    return $LesResultats
}

proc LeUGIdUN_Getz {Access {Banques "" } {Link ""} {FullOut "0"}} {

    Wup "Permet de recuperer l acc unigene (Hs.524782) a partir de l acc uniseq (Hs_S1000253 ou Hs#S1000253)" 
    
    if {$Banques==""} {set Banques "uniseq"}
    if {$Link   ==""} {set Link    "unigene"}

    set LesAccess ""
    set UnAccess  ""
    if {$Banques == "uniseq"} {    
	foreach AC [lsort -unique $Access] {
	    set UnAccess [lindex $AC 0]
	    if {[regexp -nocase "UG:" $AC]} {regsub -all "UG:" $UnAccess ""    UnAccess}
	    if {[regexp -nocase "Hs#" $AC]} {regsub -all "Hs#" $UnAccess "Hs_" UnAccess}
	    if {[regexp -nocase "Mm#" $AC]} {regsub -all "Mm#" $UnAccess "Mm_" UnAccess}
	    lappend LesAccess $UnAccess
	}
	set Access [join $LesAccess "|"]
    }
    if {$Access == ""} {return ""}
    
    set option   "-vf \"ugi\" -cs \";\""
    #On utilise le champs ugi dans Uniseq pour obtenir l Id Unigene
    set Commande "[CADO4MI_PATH getz] $option \"\\\[$Banques-all: $Access\\\]\""
    #UNISEQ:Hs_S1000253;Hs.524782
    #On utilise le champs ugi dans Uniseq pour obtenir l Id Unigene
    #set Commande "[CADO4MI_PATH getz] $option \"\\\[$Banques-all: $Access\\\] \\\> $Link \""
    #UNIGENE:Hs_S1000253;Hs.524782
    
    catch {unset GetzOut}
    catch {set   GetzOut [eval exec $Commande]} Message

    set UGId ""
    if {![info exists GetzOut]} {return ""}
    if {       $GetzOut == ""}  {return ""}
    foreach ID $GetzOut {
	if {$FullOut} {lappend UGId [split $ID ";"]} else {lappend UGId [lindex [split $ID ";"] 1]}
    }
    #set UGId [lindex [split $GetzOut ";"] 1]
    return "$UGId"
}

