#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

proc InterrogeBestProteinAndmRNAForeachOrga args {
    global TableauBestProteinAndmRNAForeachOrga

    ### $Quoi peut etre egale a Protein ou mRNA####
    #LesNoms
    #$Nom    LesOrganismes
    #$Nom    $Quoi         LesAccess
    #$Nom    Organisme     $Orga         $Quoi    Access
    #$Nom    Access        $Access       Identity
    #LesOrganismes
    #Access  $Access       LesNoms
    #Access  $Access       LesOrganismes
    #Access  $Access       Definition
    #Access  $Access       Taille
    #Access  $Access       ProteinOumRNA

    if {$args == "unset"} {
	if {[info exists TableauBestProteinAndmRNAForeachOrga]} {unset TableauBestProteinAndmRNAForeachOrga}
	return ""
    }
    set Quoi [join $args ","]
    if {[info exists TableauBestProteinAndmRNAForeachOrga($Quoi)]} {
	return [set TableauBestProteinAndmRNAForeachOrga($Quoi)] 
    }
    set FichierBestProtAndmRNA "[file join [RepertoireDuGenome] fiches BestProteinAndmRNAForeachOrga]"

    ###################CHARGE BestProtAndmRNA#########################
    if {     [file exists $FichierBestProtAndmRNA] && \
	    ![info exists TableauBestProteinAndmRNAForeachOrga($FichierBestProtAndmRNA,EstCharge)]} {
	set TableauBestProteinAndmRNAForeachOrga($FichierBestProtAndmRNA,EstCharge) 1
	ChargeBestProteinAndmRNAForeachOrga TableauBestProteinAndmRNAForeachOrga $FichierBestProtAndmRNA
    }
    if {![info exists TableauBestProteinAndmRNAForeachOrga($Quoi)]} {return}
    return [set TableauBestProteinAndmRNAForeachOrga($Quoi)] 
}

proc ChargeBestProteinAndmRNAForeachOrga {aTab {FichierContigAndProt ""}} {
    upvar $aTab Tab

    if {$FichierContigAndProt == ""} {set FichierContigAndProt "[file join [RepertoireDuGenome] fiches ContigAndProt]"}
    if {![file exists $FichierContigAndProt]} {return}

    set Tab(LesNoms)       {}
    set Tab(LesOrganismes) {}

    set F [open $FichierContigAndProt]
    while {[gets $F Ligne]>=0} {
	if {$Ligne == ""} {continue}
	set LaLigne [split $Ligne "\t"]

	set Nom    [lindex $LaLigne 0]
	set Orga   [lindex $LaLigne 1]

	if {$Orga == "Nothing"} {continue}

	set Quoi   [lindex $LaLigne 2]
	set Access [lindex $LaLigne 3]
	set Taille [lindex $LaLigne 4]
	set Id     [lindex $LaLigne 5]
	set Def    [lindex $LaLigne 6]

	if {![info exists TabTmp($Orga)]} {
	    set     TabTmp($Orga)                             1
	    lappend Tab(LesOrganismes)                        $Orga
	}
	if {![info exists Tab($Nom,$Quoi,LesAccess)]} {
	    set     Tab($Nom,$Quoi,LesAccess)                 {}
	    set     Tab($Nom,LesOrganismes)                   {}
	    lappend Tab(LesNoms)                              $Nom
	}
	if {![info exists TabTmp($Nom,Organisme,$Orga)]} {
	    set     TabTmp($Nom,Organisme,$Orga)              1
	    lappend Tab($Nom,LesOrganismes)                   $Orga
	}
	if {![info exists Tab(Access,$Access,Definition)]} {
	    set     Tab(Access,$Access,Definition)            $Def
	    set     Tab(Access,$Access,Taille)                $Taille
	    set     Tab(Access,$Access,ProteinOumRNA)         $Quoi
	    set     Tab(Access,$Access,LesNoms)               {}
	    set     Tab(Access,$Access,LesOrganismes)         {}
	}
	if {![info exists  Tab($Nom,Access,$Access,Identity)]} {
	    set     Tab($Nom,Access,$Access,Identity)         $Id
	    lappend Tab($Nom,$Quoi,LesAccess)                 $Access
	    lappend Tab(Access,$Access,LesNoms)               $Nom
	}
	if {![info exists  TabTmp($Orga,$Access)]} {
	    set     TabTmp($Orga,$Access)                     1
	    lappend Tab(Access,$Access,LesOrganismes)         $Orga
	}
	set         Tab($Nom,Organisme,$Orga,$Quoi,Access)    $Access
    }
    if {[info exists TabTmp]} {unset TabTmp}
    return $aTab
}

proc MasqueLesRegionsPourTous {LesRegions {Rep ""} {LesNoms ""}} {
    global RepertoireDuGenome
    
    foreach Nom $LesNoms {
	set F "[file join $RepertoireDuGenome $Rep $Nom]"
	if {![file exists $F]} {continue}
	MasqueLesRegionsDuFichierTFA $F $F $LesRegions
    }
    return
}
#Modification pour mettre les chaines en majuscules
proc MasqueLesRegionsDuFichierTFA {FichierTFA FichierMasked LesRegions {Mask "N"}} {
    set Entete [PremiereLigneDuFichier $FichierTFA]
    set Seq    [MasqueLesRegions [string toupper [QueLaSequenceDuTFA $FichierTFA]] $LesRegions $Mask]
    set SeqTFA [SequenceFormatTFA $Seq $Entete]
    Sauve      $SeqTFA dans $FichierMasked
    return     $SeqTFA
}
proc MasqueLaRegionDuFichierTFA {FichierTFA FichierMasked Debut Fin {Mask "N"}} {
    set Entete [PremiereLigneDuFichier $FichierTFA]
    set Seq    [MasqueLaRegion [QueLaSequenceDuTFA $FichierTFA] $Debut $Fin $Mask]
    set SeqTFA [SequenceFormatTFA $Seq $Entete]
    Sauve      $SeqTFA dans $FichierMasked
    return     $SeqTFA
}
proc MasqueLesRegions {Seq LesRegions {Mask "N"}} {
    foreach LaRegion $LesRegions {
	set Debut [lindex $LaRegion 0]
	set Fin   [lindex $LaRegion 1]
        set Seq [MasqueLaRegion $Seq $Debut $Fin $Mask]
    }
    return $Seq
}
proc MasqueLaRegion {Seq Debut Fin {Mask "N"}} {
    return [string replace $Seq  [expr {[Mini $Debut $Fin]-1}] [expr {[Maxi $Debut $Fin]-1}] [string repeat "$Mask" [expr {$Fin - $Debut + 1}]]]
}

proc LesInfosDesRegionsDansLeBlast {FileBlast LesRegions {LesBanqueId "All"} {SeuilId "None"} {QuEstCeQueCest ""}} {
    if {$LesBanqueId != "All"} {
	foreach BId $LesBanqueId {set TabBIdOK($BId) 1}
    }

    set NbSubject [AskBlast $FileBlast NbSubject]
    if {$NbSubject == "" || $NbSubject == 0} {return}

    set TypeDeBlast [AskBlast $FileBlast TypeDeBlast]
    set IndiceQ 1
    if {$TypeDeBlast == "BLASTX" || $TypeDeBlast == "TBLASTX"} {set IndiceQ 3}
    set IndiceS 1
    if {$TypeDeBlast == "TBLASTN" || $TypeDeBlast == "TBLASTX"} {set IndiceS 3}

    if {    $TypeDeBlast == "BLASTX"  || $TypeDeBlast == "BLASTP" || \
	    $TypeDeBlast == "TBLASTN" || $TypeDeBlast == "TBLASTX"} {
	set BaseFiltree "X"
    } elseif {$TypeDeBlast == "BLASTN"} {
	set BaseFiltree "N"
    }
    set LesRegions [lsort -integer -index 0 $LesRegions]

    set NbSubject [AskBlast $FileBlast NbSubject]
    for {set i 1} {$i <= $NbSubject} {incr i} {
	set BId [AskBlast $FileBlast Subject $i]
	if {$LesBanqueId != "All" && ![info exists TabBIdOK($BId)]} {continue}
	if {[info exist TabTmp($BId)]} {continue}
	set TabTmp($BId) 1
	
	set NbSegment [AskBlast $FileBlast BanqueId $BId NbSegment]
	for {set j 1} {$j <= $NbSegment} {incr j} {
	    if {$SeuilId != "None"} {
		set Identities [AskBlast $FileBlast BanqueId $BId Segment $j Identities]
		scan $Identities "%d/%d" nmRNA nmRNATot
		set  Id        [expr {$nmRNA*1.0/$nmRNATot}]
		if {$Id < $SeuilId} {continue}
	    }
	    set DQ [AskBlast $FileBlast BanqueId $BId Segment $j DQ]
	    set FQ [AskBlast $FileBlast BanqueId $BId Segment $j FQ]
	    set DS [AskBlast $FileBlast BanqueId $BId Segment $j DS]
	    set FS [AskBlast $FileBlast BanqueId $BId Segment $j FS]

	    if {$FQ < $DQ} {set SensQ     "-"} else {set SensQ    "+"}
	    if {$FQ < $DQ} {set FactSensQ -1 } else {set FactSensQ 1 }
	    if {$FS < $DS} {set SensS     "-"} else {set SensS    "+"}
	    if {$FS < $DS} {set FactSensS -1 } else {set FactSensS 1 }

	    set Orientation [AskBlast $FileBlast BanqueId $BId Segment $j Orientation]
	    set Sens        [SensDeLaRegion $Orientation]

	    set Query [lindex [split [AskBlast $FileBlast BanqueId $BId Segment $j AlignLineaire] "\n"] 0]
	    scan $Query "%s %d %s %d" Entete D AliQuery F
	    set LAliQuery [split $AliQuery ""]
	    
	    set Sbjct [lindex [split [AskBlast $FileBlast BanqueId $BId Segment $j AlignLineaire] "\n"] 2]
	    scan $Sbjct "%s %d %s %d" Entete D AliSbjct F
	    set LAliSbjct [split $AliSbjct ""]

	    foreach UneRegion $LesRegions {
		set D [lindex $UneRegion 0]
		set F [lindex $UneRegion 1]
		
		if {![info exists TabTmp($D,$F,LesBId)]} {
		    set           TabTmp($D,$F,LesBId)                                 {}
		}
		if {![CesRegionsSontEllesOverlappantes $D $F [Mini $DQ $FQ] [Maxi $DQ $FQ]]} {continue}
		if {![info exists TabTmp($D,$F,$BId,LesSegments)]} {
		    set           TabTmp($D,$F,$BId,LesSegments)                       {}
		}
		if {![info exists TabTmp($D,$F,$BId,$Sens,NbArea)]} {
		    set           TabTmp($D,$F,$BId,$Sens,NbArea)                      0
		}
		
		if {![info exists TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NAli)]} {
		    set           TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NAli)        0
		    set           TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NAliTot)     0
		}
		lappend           TabTmp($D,$F,$BId,LesSegments) [list $DQ $FQ $DS $FS $Orientation $Sens]

		set NContMax 0
		set NCont    0
		set PosIniQ  $DQ
		set PosIniS  $DS

		set LesBornesSurSubject {}
 		foreach AQ $LAliQuery AS $LAliSbjct {
		    if {[string equal -nocase "-" $AQ]} {
			set PosIniS [expr {$PosIniS + $IndiceS * $FactSensS}]
			if {    [Maxi $D [Mini $DQ $FQ]] <= $PosIniQ                    && \
				$PosIniQ                 <= [Mini $F [Maxi $DQ $FQ]]} {
			    if {$NContMax < $NCont} {set NContMax $NCont}
			    set NCont 0
			}
			continue
		    }
		    if {[string equal -nocase "-" $AS]} {
			set PosIniQ [expr {$PosIniQ + $IndiceQ * $FactSensQ}]
			if {    [Maxi $D [Mini $DQ $FQ]] <= $PosIniQ                    && \
				$PosIniQ                 <= [Mini $F [Maxi $DQ $FQ]]} {
			    if {$NContMax < $NCont} {set NContMax $NCont}
			    set NCont 0
			}
			continue
		    }
		    for {set k 0} {$k < $IndiceQ} {incr k} {
			set PosQ [expr {$PosIniQ + $k * $FactSensQ}]

			for {set l 0} {$l < $IndiceS} {incr l} {
			    set PosS [expr {$PosIniS + $l * $FactSensS}]
			    if {    [Maxi $D [Mini $DQ $FQ]] <= $PosQ                  && \
				    $PosQ                    <= [Mini $F [Maxi $DQ $FQ]]} {
				lappend LesBornesSurSubject $PosS
			    }
			}
			if {    [Maxi $D [Mini $DQ $FQ]] <= $PosQ                    && \
				$PosQ                     <= [Mini $F [Maxi $DQ $FQ]]} {
			    if {[string equal -nocase $BaseFiltree $AQ]} {
				if {$NContMax < $NCont} {set NContMax $NCont}
				set NCont 0
				continue
			    }

			    if {$AQ == $AS} {
				incr NCont
				incr TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NAli)
			    } else {
				if {$NContMax < $NCont} {set NContMax $NCont}
				set NCont 0
			    }

			    incr     TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NAliTot)
			}
		    }
		    set PosIniQ [expr {$PosIniQ + $IndiceQ * $FactSensQ}]
		    set PosIniS [expr {$PosIniS + $IndiceS * $FactSensS}]
		}
		if {$LesBornesSurSubject == {}} {continue}
		set LesBornesSurSubject [lsort -integer $LesBornesSurSubject]
		lappend TabTmp($D,$F,LesBId)                         $BId
		incr TabTmp($D,$F,$BId,$Sens,NbArea)

		lappend TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,D)           [lindex $LesBornesSurSubject 0  ]
		lappend TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,F)           [lindex $LesBornesSurSubject end]
		if {$NContMax < $NCont} {set NContMax $NCont}
		set     TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NCont)       $NContMax     
	    }
	}
    }

    set LesRegionsEtLeurFrequence {}
    foreach UneRegion $LesRegions {
	set D [lindex $UneRegion 0]
	set F [lindex $UneRegion 1]
	set LesBId  [set TabTmp($D,$F,LesBId)]

	set NTot [llength $LesBId]
	set LesBIdDeLaRegion {}
	foreach BId [lsort -unique $LesBId] {
	    foreach UnSegment [set TabTmp($D,$F,$BId,LesSegments)] {
		set DQ     [lindex $UnSegment 0]
		set FQ     [lindex $UnSegment 1]
		set DS     [lindex $UnSegment 2]
		set FS     [lindex $UnSegment 3]
		set Sens   [lindex $UnSegment 5]

		if {![info exists TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NAli)]} {continue}

		set NAli   [set   TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NAli)   ]
		if {$NAli == 0} {continue}

		set Orient [lindex $UnSegment 4]
		
		set NAliTot  [set TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NAliTot)]
		set NCont    [set TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,NCont)]
		set DsurS    [set TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,D)]
		set FsurS    [set TabTmp($D,$F,$BId,$DQ,$FQ,$DS,$FS,$Sens,F)]

		set N   [set TabTmp($D,$F,$BId,$Sens,NbArea)]
		
		lappend LesRegionsEtLeurFrequence [list \
			$D $F \
			$NTot $N $BId $Orient $Sens $DsurS $FsurS \
			$NAli $NAliTot $NCont]
	    }
	}
    }
    if {[info exists TabTmp]} {unset TabTmp}
    if {$QuEstCeQueCest == "Contig"} {
	set TailleContig 100000

	set LesRegionsEtLeurFrequenceTmp {}
	foreach UneRegion $LesRegionsEtLeurFrequence {
	    set BId [lindex $UneRegion 4]
	    if {[regexp {_[0-9]*} $BId]} {
		regexp -nocase {([a-z0-9]+):([0-9a-z]+)_([0-9]+)} $BId tmp Chromosome Contig Fragment
	    } else {
		regexp -nocase {([a-z0-9]+):([0-9a-z]+)} $BId tmp Chromosome Contig
		set Fragment ""
	    }
	    if {$Fragment != ""} {set NbFragment [Base10 $Fragment]} else {set NbFragment 0}

	    set DS    [expr {[lindex $UneRegion 7] + $TailleContig * $NbFragment}]
	    set FS    [expr {[lindex $UneRegion 8] + $TailleContig * $NbFragment}]

	    set BId [list $Chromosome $Contig [string length $Fragment]]

	    set D    [lindex $UneRegion 0]
	    set F    [lindex $UneRegion 1]
	    set Sens [lindex $UneRegion 6]

	    if {[info exists TabTmp($D,$F,$Contig,$DS,$FS,$Sens)]} {continue}
	    set TabTmp($D,$F,$Contig,$DS,$FS,$Sens) 1

	    if {![info exists TabTmp($D,$F,$Contig,$Sens,NbArea)]} {set TabTmp($D,$F,$Contig,$Sens,NbArea) 0}
	    incr TabTmp($D,$F,$Contig,$Sens,NbArea)

	    set UneRegion  [lreplace $UneRegion 4 4 $BId]
	    set UneRegion  [lreplace $UneRegion 7 7 $DS ]
	    set UneRegion  [lreplace $UneRegion 8 8 $FS ]
	    lappend LesRegionsEtLeurFrequenceTmp $UneRegion
	}
	set LesRegionsEtLeurFrequence    {}
	foreach UneRegion $LesRegionsEtLeurFrequenceTmp {
	    set Contig [lindex [lindex $UneRegion 4] 1]
	    set Sens   [lindex         $UneRegion    6]
	    set D      [lindex         $UneRegion    0]
	    set F      [lindex         $UneRegion    1]
	    set UneRegion [lreplace $UneRegion 3 3 [set TabTmp($D,$F,$Contig,$Sens,NbArea)]]

	    if {$Sens == "+"} {
		set DS   [Mini [lindex $UneRegion 7] [lindex $UneRegion 8]]
		set FS   [Maxi [lindex $UneRegion 7] [lindex $UneRegion 8]]
	    } else {
		set DS   [Maxi [lindex $UneRegion 7] [lindex $UneRegion 8]]
		set FS   [Mini [lindex $UneRegion 7] [lindex $UneRegion 8]]
	    }
	    set UneRegion [lreplace $UneRegion 0 0 [Mini $D $F]]
	    set UneRegion [lreplace $UneRegion 1 1 [Maxi $D $F]]
	    set UneRegion [lreplace $UneRegion 7 7 $DS]
	    set UneRegion [lreplace $UneRegion 8 8 $FS]
	    lappend LesRegionsEtLeurFrequence $UneRegion
	}
    }
    if {[info exists TabTmp)]} {unset TabTmp}

    return [lsort -index 0 -integer $LesRegionsEtLeurFrequence]
}

proc LesRegionsAbsentesDesPetitesRegions {LesPetitesRegions {TailleMin ""} {TailleMax ""}} {
    if {$TailleMax == ""} {
	set Max1 [lindex [lindex [lsort -integer -index 1 $LesPetitesRegions] end] 1]
	set Max2 [lindex [lindex [lsort -integer -index 0 $LesPetitesRegions] end] 0]
	set TailleMax [Maxi $Max1 $Max2]
    }
    if {$TailleMin == ""} {
	set Min1 [lindex [lindex [lsort -integer -index 1 $LesPetitesRegions] 0] 1]
	set Min2 [lindex [lindex [lsort -integer -index 0 $LesPetitesRegions] 0] 0]
	set TailleMin [Mini $Max1 $Max2]
    }
    set Debut 1
    set LesRegionsAbsentes {}
    foreach UneRegion [lsort -integer -index 0 [lsort -integer -index 1 $LesPetitesRegions]] {
	set D [Mini [lindex $UneRegion 0] [lindex $UneRegion 1]]
	set F [Maxi [lindex $UneRegion 0] [lindex $UneRegion 1]]
	if {$TailleMax < $F} {set TailleMax $F}

	if {$Debut < $D} {lappend LesRegionsAbsentes [list $Debut        [expr {$D - 1}]]}
	set Debut [expr {$F + 1}]
    }
    if {$F < $TailleMax} {lappend LesRegionsAbsentes [list [expr {$F + 1}] $TailleMax   ]}
    return $LesRegionsAbsentes
}

proc LesPetitesRegionsDesGrandesRegions {LesGrandesRegions} {
    set LesBornes         {}
    set NbRegions [llength $LesGrandesRegions]
    foreach UneGrandeRegion $LesGrandesRegions {
	set D [Mini [lindex $UneGrandeRegion 0] [lindex $UneGrandeRegion 1]]
	set F [Maxi [lindex $UneGrandeRegion 0] [lindex $UneGrandeRegion 1]]
	if {![info exists TabDebut($D)]} {set TabDebut($D) 1}
	if {![info exists TabFin($F)  ]} {set TabFin($F)   1}
	for {set i $D} {$i <= $F} {incr i} {lappend LesBornes $i}
    }
    set     i                 0
    set     LesPetitesRegions {}
    set     LesBornes         [lsort -unique -integer $LesBornes]
    set     DebutDeRef        [lindex $LesBornes 0]
    foreach Borne             $LesBornes {
	incr i
	set EstUnDebut         0
	set EstUneFin          0
	set EstDebutDeRef      0
	set ProchainEstUnDebut 0
	set ProchainEstUneFin  0
	if {[info exists TabDebut($Borne)         ]} {set EstUnDebut         1}
	if {[info exists   TabFin($Borne)         ]} {set EstUneFin          1}
	if {[info exists TabDebut([expr {$Borne+1}])]} {set ProchainEstUnDebut 1}
	if {[info exists   TabFin([expr {$Borne+1}])]} {set ProchainEstUneFin  1}
	if {$Borne == $DebutDeRef         }          {set EstDebutDeRef      1}

	if {$EstUnDebut && $EstUneFin} {
	    if {!$EstDebutDeRef} {
		lappend LesPetitesRegions [list $DebutDeRef [lindex $LesBornes [expr {$i-2}]]]
	    }
	    lappend LesPetitesRegions [list $Borne $Borne]
	    set DebutDeRef [lindex $LesBornes $i]
	    continue
	}
	if {$EstUneFin} {
	    if {!$EstDebutDeRef} {
		lappend LesPetitesRegions [list $DebutDeRef $Borne]
	    } else {
		lappend LesPetitesRegions [list $Borne $Borne]
	    }
	    set DebutDeRef [lindex $LesBornes $i]
	    continue
	}
	if {$EstUnDebut} {
	    if {!$EstDebutDeRef} {
		lappend LesPetitesRegions [list $DebutDeRef [lindex $LesBornes [expr {$i-2}]]]
	    }
	    if {$ProchainEstUnDebut} {
		lappend LesPetitesRegions [list $Borne $Borne]
		set DebutDeRef [lindex $LesBornes $i]
	    } else {
		set DebutDeRef $Borne
	    }
	    continue
	}
    }
    return [lsort -integer -index 0 $LesPetitesRegions]
}

proc Canva {LargeurMaxi HauteurMaxi {LargeurVoulue 768} {HauteurVoulue 512} {GonfleAussiY "NoY"} {Titre ""} {AvecMainLevee ""} {w ""}} {

    global ScrollRectangleOriginal IdScrollRectangle
    global CouleurDuFondDeUnCanva

    if { ! [info exists CouleurDuFondDeUnCanva]} {
	set CouleurDuFondDeUnCanva "white"
    }

    if {[regexp "MainLevee" $AvecMainLevee]} { set AvecMainLevee 1 } else { set AvecMainLevee 0 } 

    if {$GonfleAussiY=="NoY"} {
	set GonfleY 1.0
	set DegonfleY 1.0
    } else {
	set GonfleY "gonfle"
	set DegonfleY "degonfle"
    }
    set JAppartientAUneAutreFenetre 1
    if {$w == ""} {
	set JAppartientAUneAutreFenetre 0
	set w [NomDe canva]
	if {$Titre==""} { set Titre $w } 
	catch {destroy $w}
	toplevel $w
	wm title $w $Titre
	wm iconname $w $Titre
    } else {
	set w "$w.framecanva"
	if {![winfo exists $w]} {
	    frame $w
	    pack $w -fill both -expand 1
	} else {
	    $w.canvas delete all
	}
    }

    set CanvaScroMinX 0
    set CanvaScroMinY 0
    set CanvaScroMaxX $LargeurMaxi
    set CanvaScroMaxY $HauteurMaxi

    set K       $w.canvas
    set ScroHor $w.hscroll
    set ScroVer $w.vscroll
    set KShSv   $w.grid
    set Boutons $w.buttons

    if {![winfo exists $K]} {
	frame $KShSv
	scrollbar $ScroHor -orient horiz -command "$K xview"
	scrollbar $ScroVer               -command "$K yview"
    
	canvas $K

	grid rowconfig    $KShSv 0 -weight 1 -minsize 0
	grid columnconfig $KShSv 0 -weight 1 -minsize 0
	
	grid $K       -in $KShSv -padx 1 -pady 1  -row 0 -column 0  -rowspan 1 -columnspan 1  -sticky nsew
	grid $ScroVer -in $KShSv -padx 1 -pady 1  -row 0 -column 1  -rowspan 1 -columnspan 1  -sticky nsew
	grid $ScroHor -in $KShSv -padx 1 -pady 1  -row 1 -column 0  -rowspan 1 -columnspan 1  -sticky nsew
	pack $KShSv -expand 1 -fill both
    }

    set IdScrollRectangle($K) [$K create rect $CanvaScroMinX $CanvaScroMinY $CanvaScroMaxX $CanvaScroMaxY -tags [list "Cadre"] -outline $CouleurDuFondDeUnCanva]
    set ScrollRectangleOriginal($K) "[$K coords [set IdScrollRectangle($K)]]"

    $K configure \
	-width $LargeurVoulue \
        -height $HauteurVoulue \
	-scrollregion [list $CanvaScroMinX $CanvaScroMinY $CanvaScroMaxX $CanvaScroMaxY] \
	-xscrollcommand "$ScroHor set" \
	-yscrollcommand "$ScroVer set"
    
    if {![winfo exists $Boutons]} {
	#########################
	frame $Boutons
	pack  $Boutons -side bottom -fill x
	if { ! [string equal -nocase "NoZoom" $GonfleAussiY]} {
	    button $Boutons.postscript -text "Zoom/UnZoom/Reset"
	    bind $Boutons.postscript <1>       "Gonfle $w all gonfle $GonfleY"
	    bind $Boutons.postscript <2>       "Gonfle $w all degonfle $DegonfleY"
	    bind $Boutons.postscript <3>       "Gonfle $w all reset reset"
	    pack $Boutons.postscript -fill x -side top -expand 1
	}
	if {!$JAppartientAUneAutreFenetre} {
	    button $Boutons.dismiss -text "CLOSE" -command "destroy $w"
	    pack   $Boutons.dismiss -fill x -side top -expand 1
	}
	##########################################################
    }
    if {$AvecMainLevee} { MainLeveeSurUnCanva $K }

    return $K
}

proc ProgPathway {Program} {
    global TabProgPathway
    if {![info exists TabProgPathway]} {
	set TabProgPathway(LesProgrammes) {}
	set FileProgPathway "[file join [ProgSourceDir] program.path]"
	if {![file exists $FileProgPathway]} {set FileProgPathway "[file join [ProgSourceDir] etc gscope_program.path]"}
	if {![file exists $FileProgPathway]} {return $Program}

	set F [open $FileProgPathway]
	while {[gets $F Ligne]>=0} {
	    set Prog    ""
	    set Pathway ""
	    scan $Ligne "%s %s" Prog Pathway
	    if {$Prog == "" || $Pathway == ""} {continue}
	    lappend TabProgPathway(LesProgrammes) $Prog

	    set     TabProgPathway($Prog)         $Pathway
	    set     Prog                          [string toupper $Prog]
	    set     TabProgPathway($Prog)         $Pathway
	    set     Prog                          [string tolower $Prog]
	    set     TabProgPathway($Prog)         $Pathway
	}
	close $F
    }
    if {![info exists TabProgPathway($Program)]} {return $Program}
    return [set TabProgPathway($Program)]
}

proc DrawChooseFile {{Title ""} {Directory ""}} {
    global CurrentDirectory
    if {![info exists CurrentDirectory]} {
	set CurrentDirectory [ProgSourceDir]
    }
    if {$Directory != ""} {set CurrentDirectory $Directory}

    set File [tk_getOpenFile -title $Title  -initialdir $CurrentDirectory]
    set CurrentDirectory [file dirname $File]
    return $File
}

proc DrawInfo args {
    if {$args == ""} {return}

    set Quoi [lindex $args 0]

    set w ""
    if {[string equal -nocase $Quoi "Access"]} {
	set Qui [lindex $args 1]
	set PageInfo [PageInfoLaTotale $Qui protein]
	set w   [lindex $args 2]
    }
    if {[string equal -nocase $Quoi "Interpro"]} {
	set Qui [lindex $args 1]
	set PageInfo [PageInfoLaTotale $Qui interpro]
	set w   [lindex $args 2]
    }
    if {[string equal -nocase $Quoi "Box"]} {
	set FichierGO [lindex $args 1]
	set Nom       [lindex $args 2]
	set w         [lindex $args 3]
	set FichierHTML "[file dirname $FichierGO]/../BilanHTML/$Nom.html"
	if {![file exists $FichierHTML]} {return}
	set PageInfo [ContenuDuFichier $FichierHTML]
    }
    if {[string equal -nocase $Quoi "Brut"]} {
	set PageInfo [lindex $args 1]
	set w        [lindex $args 2]
    }

    set EstCeQueJeFaisPartiDUneAutreFenetre 1
    if {$w == "" || ![winfo exists $w]} {
	set w [NomDe fenetre]
	toplevel $w
	set EstCeQueJeFaisPartiDUneAutreFenetre 0
    }

    set FrameInfo "$w.frameinfo"
    set ListBox  "$FrameInfo.list"
    if {![winfo exists $FrameInfo]} {
	frame $FrameInfo
	pack  $FrameInfo -side top -expand yes -fill both
	scrollbar $FrameInfo.yscroll -command "$ListBox yview"
	scrollbar $FrameInfo.xscroll -command "$ListBox xview" -orient horizontal 
	set Largeur 82
	set Hauteur 35
	listbox $ListBox -width $Largeur -height $Hauteur -setgrid 1 \
	    -yscroll "$FrameInfo.yscroll set" \
	    -xscroll "$FrameInfo.xscroll set" \
	    -selectmode extended \
	    -background "LightGrey" \
	    -foreground "Black" \
	    -selectbackground "LightYellow" \
	    -selectforeground "Black" \
	    -font "courier 10" 
	
	grid $FrameInfo.list    -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
	grid $FrameInfo.yscroll -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky nsew
	grid $FrameInfo.xscroll -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
	grid rowconfig    $FrameInfo 0 -weight 1 -minsize 0
	grid columnconfig $FrameInfo 0 -weight 1 -minsize 0
    }
    if {[winfo exists $ListBox]} {$ListBox delete 0 end}

    set nLignes 0
    foreach Ligne [split $PageInfo "\n"] {	
	if {[string equal -nocase $Quoi "Box"] || [string equal -nocase $Quoi "interpro"] } {
	    regsub -all "\t" $Ligne " " Ligne
	    regsub -all "&nbsp|&nbsp;" $Ligne " " Ligne
	    regsub -all {<[^>]+>} $Ligne "" Ligne
	    if {[regexp -nocase "<BR>" $Ligne]} {continue}
	}
	if {![string equal -nocase $Quoi "Brut"] && [string trim $Ligne] == ""} {continue}
	$FrameInfo.list insert end $Ligne
    }

    if {!$EstCeQueJeFaisPartiDUneAutreFenetre} {
	set BoutonExit "$w.buttonexit"
	button       $BoutonExit  -borderwidth 1 -text "CLOSE"
	pack         $BoutonExit   -fill x    -side bottom -expand 0
	$BoutonExit configure -command "destroy $w"
    }
    return $ListBox
}

proc LaListeSplittee {ListeIni NbElementsParListe} {
    set LesListes {}
    set Liste     $ListeIni
    while {0 < [llength $Liste]} {
	set LengthListe [llength $Liste]
	
	if {$NbElementsParListe < $LengthListe} {
	    lappend LesListes [lrange $Liste 0                   [expr {$NbElementsParListe - 1}]]
	    set     Liste     [lrange $Liste $NbElementsParListe end                           ]
	} else {
	    lappend LesListes [lrange $Liste 0                   [expr {$LengthListe        - 1}]]
	    set     Liste     {}
	}
    }
    return $LesListes
}

proc GammeDeCouleurBlancVersRouVouB {RouVouB NbCouleurs} {
    set pi 3.141592

    if {$NbCouleurs <=1} {
	if {$RouVouB == "R"} {return [list red  ]}
	if {$RouVouB == "B"} {return [list blue ]}
	if {$RouVouB == "G"} {return [list green]}
    }     
    set BHexa [Deci2Hexa 255]
    set VHexa [Deci2Hexa 255]
    set RHexa [Deci2Hexa 255]

    set LesCouleurs {}
    for {set i $NbCouleurs} {0 <= $i} {incr i -1} {

	set Couleur     [expr round(255.0/$NbCouleurs * $i)]
	set HexaCouleur [Deci2Hexa $Couleur]

	if {[string length $HexaCouleur] == 1} {set HexaCouleur "0$HexaCouleur"}

	if {$RouVouB == "R"} {set VHexa $HexaCouleur;set BHexa $HexaCouleur}
	if {$RouVouB == "V"} {set BHexa $HexaCouleur;set RHexa $HexaCouleur}
	if {$RouVouB == "B"} {set VHexa $HexaCouleur;set RHexa $HexaCouleur}
	lappend LesCouleurs "\#$RHexa$VHexa$BHexa"
    }
    return $LesCouleurs
}

proc SensDeLaRegion {TypeRegion} {
    if {$TypeRegion == "Plus / Plus"} {
	set Sens "+"
    } elseif {$TypeRegion == "Plus / Minus"} {
	set Sens "-"
    } elseif {$TypeRegion == "N"} {
	set Sens "0"
    } elseif {$TypeRegion == "RAS"} {
	set Sens "0"
    } elseif {$TypeRegion == "+1"} {
	set Sens "+"
    } elseif {$TypeRegion == "+2"} {
	set Sens "+"
    } elseif {$TypeRegion == "+3"} {
	set Sens "+"
    } elseif {$TypeRegion == "-1"} {
	set Sens "-"
    } elseif {$TypeRegion == "-2"} {
	set Sens "-"
    } elseif {$TypeRegion == "-3"} {
	set Sens "-"
    } elseif {$TypeRegion == "Prot / Prot"} {
	set Sens "+"
    } elseif {[regexp "\[1-3\+]{2} / \[1-3\+]{2}" $TypeRegion] || \
	    [regexp "\[1-3\-]{2} / \[1-3\-]{2}" $TypeRegion]} {
	set Sens "+"
    } elseif {[regexp "\[1-3\+]{2} / \[1-3\-]{2}" $TypeRegion] || \
	    [regexp "\[1-3\-]{2} / \[1-3\+]{2}" $TypeRegion]} {
	set Sens "-"
    } else {
	set Sens "+"
    }
    return $Sens
}

proc CesRegionsSontEllesOverlappantes {D1 F1 D2 F2} {
    if {$F1 < $D1} {set tmp $F1;set F1 $D1;set D1 $tmp}
    if {$F2 < $D2} {set tmp $F2;set F2 $D2;set D2 $tmp}
    if {    ($D1 <= $F2 && $F2 <= $F1) || \
	    ($D1 <= $D2 && $D2 <= $F1) || \
	    ($D1 <= $D2 && $F2 <= $F1) || \
	    ($D2 <= $D1 && $F1 <= $F2) } {
	return 1
    }
    return 0
}

proc Deci2Hexa {Deci} {
    return [format "%X" $Deci]
}

proc Hexa2Deci {Hexa} {
    return [format "%x" $Hexa]
}