#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

###############################################################################
#   1.Interface graphique pour la visualisation du design d oligonucleotide   #
#   2.Fiche info de la sequence et PAB                                        #
#     Selection des zones de design preferentielles                           #
#   3.Les Profils sur la sequence                                             #
#     Calcul du Tm Content          et graphe                                 #
#     Calcul du GC Content          et graphe                                 #
#     Calcul du Id Content du blast et graphe                                 #
#     Calcul du Nb Seq     du blast et graphe                                 #
#   4.Petits outils pour le GUI                                               # 
###############################################################################

#############################################################################
#                          ---  IllustreOligo ---                           #
#   Interface graphique pour la visualisation du design d oligonucleotide   #
#############################################################################
proc IllustreOligo {{PAB ""} {Directories ""}} {

    Wup "Illustre une sortie de design d Oligo "
    
    #Version pour visualiser plusieurs oligos de plusieurs banques en simultanee.
    #Refonte pour la visualisation de plusieurs oligos sur la meme ligne.
    #Addon GC content, Help, Color, Color Help, Id content, Nb Seq, Tm content.
    #Selection des oligos, sauvegarde et affichage des selectionnes et des X-selectionnes
    #MAJ Modif algo de visualisation avec InterrogeInfoOligo
    #MAJ Modif pour vider la memoire et gerer plusieurs repertoires simultanement
    #MAJ Modif pour chercher dans les infos de l oligo 
    #MAJ Modif pour l absence des infos et valeurs par defaut
    #MAJ Modif pour ajout des coordonnees du CDS et de la carte exonique
    #MAJ Modif pour detruire toutes les infos en memoire
    #15/02/2005
    
    set OnADejaFaitLesSpike  0

    #1 Verification des differents arguments
    if {$PAB==""} {return "Correct syntax is IllustreOligo PAB ProbesDir\nfor ex: IllustreOligo ACT0001\n"}

    if {$Directories==""} {
	set     RepOligo "[CADO4MI_PATH Probes_DIR]"
	if {   $RepOligo == "" || ! [file exists $RepOligo]} {
	    set RepOligo "[file join [RepertoireDuGenome] Probes]"
	}
	
	if {[file exists $RepOligo]} {
	    set LesDirs ""
	    set LaListe ""
	    foreach File [glob -nocomplain -directory $RepOligo -types d "*"] {
		lappend LaListe [file tail $File]
	    }
	    set  LaListe [EntreDeux [lsort $LaListe]]
	    if {$LaListe==""} {AjouteAuLog "No Directories found";return ""}
	    foreach Dir $LaListe {
		lappend LesDirs "[file join $RepOligo $Dir]"
	    }
	} else {AjouteAuLog "No Directories specified";return ""}
    } else {set LesDirs $Directories} 

    #2 La query doit etre une fichier de sequence et on le recherche
    #  Attention il faut une query, sinon pas de taille ....
    set RepmRNA      "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set Repnuctfa    "[file join [RepertoireDuGenome] nuctfa]"
    set RepRefSeq    "[file join [RepertoireDuGenome] Probes RefSeq_TFA]"
    set RepDirectory "[lindex $LesDirs 0]"

    set FichierQuery $PAB
    if {[regexp {/} $PAB]} {set PAB [file tail $PAB]}
    if {[file exists $FichierQuery]} {
	set FichierQuery $FichierQuery
    } elseif {[file exists "[file join $RepmRNA $PAB]"]} {
	set FichierQuery   "[file join $RepmRNA $PAB]"
	set OnAPeutEtreExonicMap 1
    } elseif {[file exists "[file join $Repnuctfa $PAB]"]} {
	set FichierQuery   "[file join $Repnuctfa $PAB]"
    } elseif {[regexp {^NM[0-9]{4,}} $PAB]} {
	set FichierQuery   "[file join $RepRefSeq $PAB]" 
    } elseif {[file exists "[file join $RepDirectory "${PAB}.masked"]"]} {
	set FichierQuery   "[file join $RepDirectory "${PAB}.masked"]"
    } else {AjouteAuLog "No Input File for the Query ($PAB)";return ""}
    if {! [file exists "$FichierQuery"]} {AjouteAuLog "$FichierQuery does not exists ($PAB)";return ""}

    if {[regexp {\.masked$} $PAB]} {regsub -all {\.masked$} $PAB "" PAB}

#    #Pour le design particulier de la banque RefSeq totale
#    if {0} {
#	if {[regexp {^NM[0-9]{4,}} $PAB]} {
#	    set Dir     "[RepertoireDuGenome]/Probes/RefSeq_Design"
#	    if {[regexp     "RefSeq_Design" $LesDirs]} {
#		set NMDIR [LeRepertoireDeRefSeq $PAB] 
#		regsub -all "RefSeq_Design" $LesDirs "RefSeq_Design/$NMDIR" LesDirs
#	    }
#	}
#    }

    #3 Parametres par defaut Police, dimensions, bordures, couleurs ...
    set Police         "Courier 12"
    set Police2         {-family "arial" -size 12 -weight bold}
    set CouleurOutline "black"
    set LargeurMax     [expr {[winfo screenwidth  .] - 200}]
    set HauteurMax     [expr {[winfo screenheight .] * 1/2}]
    set HauteurMax     [expr {[winfo screenheight .] * 0.75}]
    set LargeurFenetre [expr {[winfo screenwidth  .] - 200}]
    set HauteurFenetre [expr {[winfo screenheight .] * 1/2}]
    set HauteurFenetre [expr {[winfo screenheight .] * 0.75}]
    set XMilieu        [expr {$LargeurMax/2}]
    set BordureX       50.0
    set BordureY       50.0
    set Intervalle     10.0
    set EpaisseurBoite 15.0

    #3-1 Position du Graph content
    set YDebutGraph    [expr {0 + $BordureY}]
    set YFinGraph      [expr {$YDebutGraph + 100}]

    #3-2 Position de la query
    set XDebutQuery    [expr {0            + $BordureX}]
    #set YDebutQuery    [expr $YFinGraph   + 10]
    set YDebutQuery    [expr {$YFinGraph   + 20}]
    set XFinQuery      [expr {$LargeurMax  - $BordureX}] 
    set YFinQuery      [expr {$YDebutQuery + $EpaisseurBoite}]

    #3-3 Besoin de la taille de la query pour conversion et affichage
    set AccessQuery    [AccessDuTFADumRNA         $FichierQuery]
    set  SeqQuery      [QueLaSequenceDuFichierTFA $FichierQuery]
    if {$SeqQuery==""} {AjouteAuLog "$FichierQuery no sequence available";return ""}
    set TailleQueryNt  [string length $SeqQuery]
    
    #3-4 Rapport du nombre de pixel pour un nt
    set OneNtPixel     [expr {($XFinQuery-$XDebutQuery)/($TailleQueryNt-1)}] 
    #Attention on commence au premier nt

    #3-5 Debut des Oligos
    set YDebutDeOligo  [expr {$YFinQuery + $Intervalle}]
    
    #4 Partie Graphique
    #4-1 Fenetre principale (canva)
    set K [UnCanvaDe $LargeurMax $HauteurMax $LargeurFenetre $HauteurFenetre "CADO4MI - Design for $PAB"]
    $K configure -background "grey" 

    #4-2 Modification du comportement de la fenetre principale lors de sa destruction 
    #    Ceci pour tuer toutes les autres fenetres crees
    set          w [winfo toplevel $K]
    wm geometry $w +020+020
    wm iconify  $w
    wm protocol $w WM_DELETE_WINDOW "DetruitLesInfosDe $K;after 10 destroy $w"

    #4-3 Rectangle de masque pour tout les graphes (Tm, GC, Id. ...)
    $K create rectangle [expr {$XDebutQuery-40}] [expr {$YDebutGraph-4}] [expr {$XFinQuery+5}] [expr {$YFinGraph+4}] \
	    -fill    "gray97" \
	    -outline "gray97" \
	    -tags    [list "" "Masque" "Entete"]

    #4-4 Affiche la query
    $K create rectangle $XDebutQuery $YDebutQuery $XFinQuery $YFinQuery \
	    -outline "$CouleurOutline" \
	    -fill    "red" \
	    -tags    [list "" "Query" "Entete"]
    $K create text [expr {$XDebutQuery-3.0}] [expr {$YDebutQuery + ($EpaisseurBoite/2.0)}] \
	    -text    "1" \
	    -tags    [list "" "Entete"] \
	    -font    $Police2 \
	    -anchor  "e"
    $K create text [expr {$XFinQuery+3.0}]   [expr {$YDebutQuery + ($EpaisseurBoite/2.0)}] \
	    -text    "$TailleQueryNt" \
	    -tags    [list "" "Entete"] \
	    -font    $Police2 \
	    -anchor  "w"
    $K create text $XMilieu [expr {1*$BordureY/3}] \
	    -text    "Design for $PAB" \
	    -tags    [list "" "Entete"] \
	    -font    $Police2 \
	    -anchor  "center" \
	    -justify "center"

    #4-4-1 Coordonnees du CDS (si dispo)
    set  CDS    ""
    #set  CDS [LesCoordonneesDuCDSDumRNA $AccessQuery]
    if {$CDS != ""} {
	set DCDS [expr {$XDebutQuery   + $OneNtPixel * [lindex $CDS 0] - $OneNtPixel}]
	set FCDS [expr {$XDebutQuery   + $OneNtPixel * [lindex $CDS 1] - $OneNtPixel}]
	
	$K create line $DCDS [expr {$YDebutQuery-10}] $DCDS $YFinQuery \
		-width  "3" \
		-fill   "black" \
		-tags   [list "" "CDS\nBegin [lindex $CDS 0]" "Affichable" "Entete"]
	$K create line $FCDS [expr {$YDebutQuery-10}] $FCDS $YFinQuery \
		-width  "3" \
		-fill   "black" \
		-tags   [list "" "CDS\nEnd [lindex $CDS 1]" "Affichable" "Entete"]
	$K create text  [expr {$FCDS - (($FCDS-$DCDS)/2)}] [expr {$YDebutQuery-7}] \
		-text   "CDS" \
		-tags   [list "" "Entete"] \
		-anchor "center" \
		-font   "$Police2"
    }

    #4-4-2 Carte exonique (si dispo)
    #####WARNING
    set  OnAPeutEtreExonicMap 1
    set  OnAPeutEtreExonicMap 0
    if {$OnAPeutEtreExonicMap} {
	set RepExonicMap "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens_exonicmap]"
	if {[file exists "[file join $RepExonicMap $PAB]"]} {
	    set LesCoordonnees [LesCoordonneesExonicMap "[file join $RepExonicMap $PAB]"]
	    
	    set i 0
	    foreach DF $LesCoordonnees {
		incr i
		set DQ [expr {$XDebutQuery + $OneNtPixel * [lindex $DF 0] - $OneNtPixel}]
		set FQ [expr {$XDebutQuery + $OneNtPixel * [lindex $DF 1] - $OneNtPixel}]

		$K create rectangle $DQ [expr {$YDebutQuery +5}] $FQ [expr {$YFinQuery -5}] \
			-fill yellow\
			-tags [list "" "Exon $i\nD [lindex $DF 0] F [lindex $DF 1]" "Affichable" "Entete"]
	    }
	}
    }
    
    #4-4-3 Affiche la petite aide
    $K create oval [expr {$XDebutQuery-30}] [expr {$YDebutGraph-30}] [expr {$XDebutQuery-10}] [expr {$YDebutGraph-10}] \
	    -fill    "green" \
	    -tags    [list "" "Entete"] \
	    -outline "white"
    $K create text [expr {$XDebutQuery-20}] [expr {$YDebutGraph-20}] \
	    -text    "?" \
	    -tags    [list "" "[AfficheHelp]" "Affichable" "Entete"] \
	    -anchor  "center" 
    
    #4-3-1 Affichage du Tm content (methode NearestNeighbor)
    #      On recupere la methode de Tm choisie et on la remet apres
    set TmDefault [LesParametresDuDesign TmMethod "LaValeurMerci"]
    LesParametresDuDesign TmMethod "NearestNeighbor" "Positionne"

    set  TmResults [PosEtTmDeLaRegion $SeqQuery $TailleQueryNt] 
    if {$TmResults !=""} {
	AfficheLeSpectreTmDUneListe $K $TmResults \
		$XDebutQuery $XFinQuery \
		$YDebutGraph $YFinGraph $OneNtPixel "orange" "TmNN"
    }
    LesParametresDuDesign TmMethod $TmDefault "Positionne"

    #4-3-2 Affichage du GC content
    set  GCResults ""
    set  GCResults [PosEtGCDeLaRegion $SeqQuery $TailleQueryNt]
    if {$GCResults !=""} {
	AfficheLeSpectreGCDUneListe $K $GCResults \
		$XDebutQuery $XFinQuery \
		$YDebutGraph $YFinGraph $OneNtPixel
    }

    #5 Dessine les boites des differentes sources
    #############################################
    set IdDuGraph         0
    set LesFichiersBlast  ""
    set LesLogs           ""
    set LesFichiersOligo  ""
    foreach DirOligo $LesDirs {
	set DB [file tail $DirOligo]
	incr IdDuGraph

	set FileParam      "[file join $DirOligo "Design.parameters"]"
	set  BLASTDataBase [InterrogeDesignParameter $FileParam DataBase]
	if {$BLASTDataBase != ""} {
	    set BLASTDataBase [file tail $BLASTDataBase]
	    if {[regexp {\.[a-z]{3}$} $BLASTDataBase]} {regsub {\.[a-z]{3}$} $BLASTDataBase "" BLASTDataBase}
	}

	set FichierBlast "[file join $DirOligo "${PAB}.blastn"]" 
	lappend LesFichiersBlast $FichierBlast
	if {0&&[file exists $FichierBlast]} {
	    lappend LesFichiersBlast $FichierBlast
	    #4-3-3 Affichage du Id content du blast
	    set  IdResults ""
	    set  IdResults [PosEtPIdAligneeDeLaRegion $FichierBlast] 
	    if {$IdResults !=""} {
		AfficheLeSpectreIdDUneListe $K $IdResults \
			$XDebutQuery $XFinQuery \
			$YDebutGraph $YFinGraph $OneNtPixel "red" "Id$IdDuGraph"
	    }
	    #4-3-4 Affichage du nb de sequence du blast
	    set  NbSeqResults ""
	    set  NbSeqResults [PosEtNbSequenceAligneeDeLaRegion $FichierBlast] 
	    if {$NbSeqResults !=""} {
		AfficheLeSpectreNbSequenceDUneListe $K $NbSeqResults \
			$XDebutQuery $XFinQuery \
			$YDebutGraph $YFinGraph $OneNtPixel "deepskyblue2" "NbSeq$IdDuGraph"
	    }
	    UnsetLesTableauxDesGraphAvecBlast
	}
	
	#5-1 Charge le fichier pour obtenir les infos sur les oligos
	############################################################
	set FichierLog      "[file join $DirOligo "$PAB.log"]"
	set FichierACharger "[file join $DirOligo "$PAB.oligo"]"
	if {[file exists $FichierLog]}      {lappend LesLogs          $FichierLog}
	if {[file exists $FichierACharger]} {lappend LesFichiersOligo $FichierACharger}

	set  lesXDebutFinDeOligo [InterrogeInfoDeLOligo $FichierACharger LesDebutFin]
	if {$lesXDebutFinDeOligo == ""} {
	    if {      [InterrogeInfoDeLOligo $FichierACharger NotDone]=="1"} {
		set Message "No Oligo calculated for $PAB in $DirOligo"
	    } elseif {[InterrogeInfoDeLOligo $FichierACharger NoOligo]=="1"} {
		set Message "No Oligo available for $PAB in $DirOligo"
	    } else {set Message "Petit probleme"}
	    
	    $K create text $XMilieu $YDebutDeOligo \
		    -text   "$Message" \
		    -font   "$Police2" 
	    set YDebutDeOligo [expr {$YDebutDeOligo + 20}]
	    continue
	}

	#5-2 Construction des boites des Oligos
	#    Classement des bornes du Nt vers Ct (3prime vers 5prime)
	#    Creation des boites avec plus de 40 d ecart 
	#############################################################
	while {1} {
	    if {$lesXDebutFinDeOligo==""} {break}
	    set YFinDeOligo [expr {$YDebutDeOligo + $EpaisseurBoite}]
	    set OldDebut    -40
	    foreach UnDebutFinDeOligo [lsort -real -increasing -index 0 $lesXDebutFinDeOligo] {
		set NbUnrelated     ""
		set NomOligo        [InterrogeInfoDeLOligo $FichierACharger [lindex $UnDebutFinDeOligo 0] [lindex $UnDebutFinDeOligo 1] NomOligo]
		set LaSequence      [InterrogeInfoDeLOligo $FichierACharger $NomOligo Sequence]
		set TailleOligo     [InterrogeInfoDeLOligo $FichierACharger $NomOligo Taille]
		set GC              [InterrogeInfoDeLOligo $FichierACharger $NomOligo GC]
		set Tm              [InterrogeInfoDeLOligo $FichierACharger $NomOligo Tm]
		set ExpTm           [InterrogeInfoDeLOligo $FichierACharger $NomOligo ExpTm]
		set XDebutDeOligoNt [InterrogeInfoDeLOligo $FichierACharger $NomOligo Debut]
		set XFinDeOligoNt   [InterrogeInfoDeLOligo $FichierACharger $NomOligo Fin]
		set NbCandidats     [InterrogeInfoDeLOligo $FichierACharger $NomOligo NbCandidats]
		set NbContigues     [InterrogeInfoDeLOligo $FichierACharger $NomOligo NbContigues]
		set LesCandidats    [join [InterrogeInfoDeLOligo $FichierACharger $NomOligo LesCandidats] " "]
		set NbUnrelated     [regexp -all {NG[0-9]+} $LesCandidats]

		if {$TailleOligo     == ""}      {set TailleOligo [string length $LaSequence]}
		if {$TailleOligo>$TailleQueryNt} {set TailleOligo $TailleQueryNt}
		if {   $GC           == ""}      {
		    set GC [format "%.1f" [expr {(([NombreDeBaseDe $LaSequence "G|C"])/($TailleOligo*1.0))*100.0}]]
		}
		if {$Tm              == ""}      {set Tm    [CalculTmDeLaSeq $LaSequence NearestNeighbor]}
		if {$ExpTm           == ""}      {set ExpTm [CalculTmDeLaSeq $LaSequence DNADNA]}
		if {$XDebutDeOligoNt == ""}      {set XDebutDeOligoNt 1}
		if {$XFinDeOligoNt   == ""}      {set XFinDeOligoNt $TailleOligo}
		if {$NbCandidats     == ""}      {set NbCandidats "-1"}
		if {$NbContigues     == ""}      {set NbContigues "-1"}
		if {$NbUnrelated     == ""}      {set NbUnrelated "0"}

		set CouleurBoite    [CouleurDeLOligo $NbCandidats]
		
		#Les infos contenues dans l info bulle
		set    Texte [format   "%-4s%-10s"      "Id:"            "$NomOligo"]
		append Texte [format "\n%-10s%-10s"     "Directory:"     "$DB"]
		append Texte [format "\n%-10s%-10s"     "Database:"      "$BLASTDataBase"]
		append Texte [format "\n%-10s%-3d  %5s" "Length:"        "$TailleOligo"     "nt(s)"]
		append Texte [format "\n%-10s%3.1f %1s" "GC:"            "$GC"              "%"]
		append Texte [format "\n%-10s%3.1f %5s" "Tm:"            "$Tm"              "degree(s)"]
		append Texte [format "\n%-10s%3.1f %5s" "Ex-Tm:"         "$ExpTm"           "degree(s)"]
		append Texte [format "\n%-10s%3d %5s"   "Begin:"         "$XDebutDeOligoNt" "/ $TailleQueryNt nt(s)"]
		append Texte [format "\n%-10s%3d %5s"   "End:"           "$XFinDeOligoNt"   "/ $TailleQueryNt nt(s)"]
		append Texte [format "\n%-15s%2d"       "X-Detected:"    "$NbCandidats"]
		append Texte [format "\n%-15s%2d"       "Contiguous:"    "$NbContigues"]
		append Texte [format "\n%-15s%2d"       "Unrelated Acc:" "$NbUnrelated"]
		
		#Calcul positions de l oligo sur la query
		set XDebutDeOligo [expr {$XDebutQuery   + $OneNtPixel * $XDebutDeOligoNt - $OneNtPixel}]
		set XFinDeOligo   [expr {$XDebutDeOligo + $OneNtPixel * $TailleOligo     - $OneNtPixel}]

		if {[expr $XDebutDeOligo > [expr {$OldDebut+40.0}]] =="1"} {
		    #On dessine la boite de l oligo
		    $K create rectangle $XDebutDeOligo $YDebutDeOligo $XFinDeOligo $YFinDeOligo \
			    -outline "white" \
			    -fill    "$CouleurBoite" \
			    -tags    [list "" $Texte $XDebutDeOligoNt $XFinDeOligoNt $NomOligo $DirOligo Affichable Positionner Boite]
		    set OldDebut            $XFinDeOligo
		    set index               [lsearch  $lesXDebutFinDeOligo $UnDebutFinDeOligo]
		    set lesXDebutFinDeOligo [lreplace $lesXDebutFinDeOligo $index $index]
		} 
	    }
	    #On ajoute de l espace pour la deuxieme ligne
	    set YDebutDeOligo [expr {$YFinDeOligo + $Intervalle}]
	}
	set YDebutDeOligo [expr {$YDebutDeOligo + 15}]
	#Separateur entre chaque design
	$K create text $XDebutQuery [expr {$YDebutDeOligo-8}] \
		-text   "$IdDuGraph: $DB" \
		-font   {-family "arial" -size 10 -weight bold}\
		-anchor "w"
	$K create line $XDebutQuery $YDebutDeOligo $XFinQuery $YDebutDeOligo -fill "black" -width 2

	set YDebutDeOligo [expr {$YDebutDeOligo+$Intervalle}]
    }
    #6 Sauve les informations importantes du canvas
    SauveLesInfosDe $K MonFichierQuery  $FichierQuery
    SauveLesInfosDe $K MonAccessQuery   $AccessQuery
    SauveLesInfosDe $K MaTailleDeQuery  $TailleQueryNt
    SauveLesInfosDe $K MesDirs          $LesDirs
    SauveLesInfosDe $K MesBlasts        $LesFichiersBlast
    SauveLesInfosDe $K MesLogs          $LesLogs
    SauveLesInfosDe $K MesFichiersOligo $LesFichiersOligo
    SauveLesInfosDe $K MonPAB           $PAB
    
    #7 Correction pour affichage final
    #  Pour accorder la taille de la fenetre selon la taille des resultats a afficher
    if {$YDebutDeOligo < [expr {[winfo screenheight .] * 0.75}]} {$K configure -height $YDebutDeOligo}
    #  Pour accorder la scrollbar au dernier oligo affiche
    $K configure -scrollregion [list 0 0 $LargeurMax $YDebutDeOligo]
    
    #7-1 Binding sur les objets et boutons de la fenetres
    set InfosDeBase [list AvecSeqFormate AvecTaille AvecStatNuc AvecTm]
    $K bind "Query"       <1>         "SauveLesInfosDe $K Win \[AfficheVariable \[LesInfosDeBaseDeMaSequence $SeqQuery \"$InfosDeBase\"\]\]"
    $K bind "Query"       <3>         "SauveLesInfosDe $K Win \[AfficheLaQuery $K $FichierQuery\]"

    $K bind "Affichable"  <Enter>     "AfficheInfoBulle $K %x %y Enter"
    $K bind "Affichable"  <Leave>     "AfficheInfoBulle $K %x %y Leave"

    $K bind "Positionner" <1>         "AffichePositionOligo      $K %x %y Show"
    $K bind "Positionner" <2>         "SauveLesInfosDe $K Win \[AfficheLesAutresCandidats        $K $PAB\]"
    $K bind "Positionner" <3>         "SauveLesInfosDe $K Win \[AfficheLaSequenceDeLOligo        $K $PAB\]"
    $K bind "Positionner" <Shift-2>   "SauveLesInfosDe $K Win \[AfficheLesAutresCandidatsComplet $K $PAB\]"
    $K bind "Positionner" <Control-1> "SelectionneOligo          $K"
    $K bind "Positionner" <Control-2> "SelectionneOligoCroisee   $K"

    regsub ".canvas$" $K "" F

    set   B [BoutonneLaFenetre $F "Position Show / Hide\nHelp Main / Color"]
    bind $B <1>         "AffichePositionOligoPourTous $K %x %y Show"
    bind $B <2>         "EffacePositionOligoPourTous  $K %x %y"
    bind $B <Shift-1>   "AfficheHelp DuBouton"
    bind $B <Shift-2>   "SignificationDesCouleursDeLOligo"
    set    Texte [format   "%-10s" "Display the nt positions among the query"]
    append Texte [format "\n%-10s" "Help for program and coloring scheme"]
    set_balloon $B "$Texte"
    
    set   C [BoutonneLaFenetre $F "BLAST File / View\nShow Log / Oligo File\nBest Hit / Self-Comp."]
    bind $C <1>         "SauveLesInfosDe $K Win \[AfficheFichier \[ChoixParmi \[LesInfosDe $K MesBlasts\]\] AvecSegAlignement\]"
    bind $C <2>         "SauveLesInfosDe $K Win \[DrawBlast      \[ChoixParmi \[LesInfosDe $K MesBlasts\]\]\]"
    bind $C <Shift-1>   "SauveLesInfosDe $K Win \[AfficheFichier \[ChoixParmi \[LesInfosDe $K MesLogs\]\]\]"
    bind $C <Shift-2>   "SauveLesInfosDe $K Win \[AfficheFichier \[ChoixParmi \[LesInfosDe $K MesFichiersOligo\]\] t\]"
    bind $C <Control-1> "CheckForBestHitInDatabase $PAB          \[ChoixParmi \[LesInfosDe $K MesDirs\]\]"
    bind $C <Control-2> "SauveLesInfosDe $K Win \[SelfComplementarity_GUI $K    $PAB\]"
    set    Texte [format   "%-10s" "Display BLAST file and a graphical view"]
    append Texte [format "\n%-10s" "Display .log or .oligo file"]
    append Texte [format "\n%-10s" "Check best hit in database, Self-complementarity"]
    set_balloon $C "$Texte"
    
    set   D [BoutonneLaFenetre $F "Graph Panel\nGraph Add / Delete\nSearch / Clear"]
    bind $D <1>         "Graph_Panel             $K"
    bind $D <Shift-1>   "AjouteUnGraph $K $FichierQuery $XDebutQuery $XFinQuery $YDebutGraph $YFinGraph $OneNtPixel"
    bind $D <Shift-2>   "SupprimeUnGraph         $K"
    bind $D <Control-1> "TrouveDansLesInfos      $K"
    bind $D <Control-2> "DeselectionneLesTrouves $K"
    set    Texte [format   "%-10s" "Panel to change graph display (order, color)"]
    append Texte [format "\n%-10s" "Allow to add or delete graph"]
    append Texte [format "\n%-10s" "Search function in all oligo (use text in Info Bubbles)"]
    set_balloon $D "$Texte"
    
    set   E [BoutonneLaFenetre $F "Selected / Reset / Save\nX-Selected / Reset / Save\nAuto Selection / X-Selection"]
    bind $E <1>         "AllumeLaSelection             $K $PAB \[LesInfosDe $K MesDirs\]"
    bind $E <2>         "DeselectionneLesSelectionnes  $K"
    bind $E <3>         "SauveLesSelectionnes          $K $PAB \[LesInfosDe $K MesDirs\]"
    bind $E <Shift-1>   "AllumeLaXSelection            $K $PAB \[LesInfosDe $K MesDirs\]"
    bind $E <Shift-2>   "DeselectionneLesXSelectionnes $K"
    bind $E <Shift-3>   "SauveLesXSelectionnes         $K $PAB \[LesInfosDe $K MesDirs\]"
    bind $E <Control-1> "SelectionDesOligos_GUI           $PAB \[ChoixParmi \[LesInfosDe $K MesDirs\]\]"  
    bind $E <Control-2> "SelectionCroiseeDesOligos_GUI    $PAB \[LesInfosDe $K MesDirs\]"
    set    Texte [format   "%-10s" "Allow to display, hide or save Selected oligo"]
    append Texte [format "\n%-10s" "Allow to display, hide or save X-Selected oligo"]
    append Texte [format "\n%-10s" "Do Selection or X-Selection"]
    set_balloon $E "$Texte"

    set   F [BoutonneLaFenetre $F "Redraw / Erase / Reset\nSave (tab file) / Project\nUpdate another design"]
    bind $F <1>         "RestrictedParameter_Panel     $K"
    bind $F <2>         "EffaceLesBoitesDuCanvas       $K"
    bind $F <3>         "InterrogeInfoDeLOligo \[ChoixParmi \[LesInfosDe $K MesFichiersOligo\]\] Decharge"
    bind $F <Shift-1>   "UnOligoPourExcel_GUI          $K $PAB"
    bind $F <Shift-2>   "LesOligosPourExcel_GUI                \[ChoixParmi \[LesInfosDe $K MesDirs\]\]"
    bind $F <Control-1> "CompleteLaListeDesOligos_GUI  $K $PAB \[ChoixParmi \[LesInfosDe $K MesDirs\]\]"
    set    Texte [format   "%-10s" "Allow to redraw results with restricted parameters"]
    append Texte [format "\n%-10s" "Allow to save results under tab delimited file"]
    append Texte [format "\n%-10s" "Update another design with the selected oligo(s)"]
    set_balloon $F "$Texte"

    #7-2 Le visuer (Pointeur)
    bind $K <Shift-1>         "ViseurDeSeq $K %x %y $XDebutQuery $XFinQuery $YDebutGraph $YDebutDeOligo $OneNtPixel"
    bind $K <Shift-B1-Motion> "ViseurDeSeq $K %x %y $XDebutQuery $XFinQuery $YDebutGraph $YDebutDeOligo $OneNtPixel"

    #7-3 Binding pour permettre de zoomer en utilisant le 3eme bouton
    bind $K <3>               "zoomMark   $K %x %y"
    bind $K <B3-Motion>       "zoomStroke $K %x %y"
    bind $K <ButtonRelease-3> "zoomArea   $K %x %y"
    
    wm deiconify $w

    #8 Check le design pour la presence de la query dans le blast
    #  Pop up une boite de dialogue le cas echeant (Warning)
    set      FEvalue  ""
    foreach Dir $LesDirs {
	set  FEvalue [EvalueLeDesign $PAB $Dir]
	if {$FEvalue!=""} {
	    raise $FEvalue $w
	    SauveLesInfosDe $K Win $FEvalue
	}
	set  FEvalue  ""
    }
     
    return ""
}

proc LesCoordonneesExonicMap {Query} {

    Wup "Permet de donner les coordonnees des sequences alignees sur le genome"
    
    set ListeDFExon ""
    set LesRangs   [lindex [InterrogeFichierExonicMap $Query LesRangs] 0] 
    foreach Rang $LesRangs {
	set NbExon [InterrogeFichierExonicMap $Query Rang $Rang NbExonicAreas]

	for {set i  1} {$i <= $NbExon} {incr i} {
	    set Debut [InterrogeFichierExonicMap $Query Rang $Rang ExonArea $i DQ]
	    set Fin   [InterrogeFichierExonicMap $Query Rang $Rang ExonArea $i FQ]
	    lappend ListeDFExon [list $Debut $Fin]
	}
    }
    return "$ListeDFExon"
}

proc EffaceLesBoitesDuCanvas {K} {
 
    Wup "Permet d effacer tout sauf l entete du Illustreoligo cad les graphes et la query"

    set LesIds [$K find withtag {!("OligoGraph"^"Entete")}]
 
    foreach Ids $LesIds {
	$K delete $Ids
    }
    return ""
}

proc RedrawDesign {K} {

    Wup "Permet de redessiner les boites en utilisant des parametres restreint a l affichage"

    if {$K==""} {return ""}
    
    ResetZoom $K
    EffaceLesBoitesDuCanvas $K

    #Recupere les infos essentielles du canvas a redessiner
    set LesDirs          [LesInfosDe $K MesDirs]
    set LesFichiersBlast [LesInfosDe $K MesBlasts]
    set LesLogs          [LesInfosDe $K MesLogs]
    set LesFichiersOligo [LesInfosDe $K MesFichiersOligo]
    set PAB              [LesInfosDe $K MonPAB]
    set FichierQuery     [LesInfosDe $K MonFichierQuery]
    set TailleQueryNt    [LesInfosDe $K MaTailleDeQuery]
    set AccessQuery      [LesInfosDe $K MonAccessQuery]

    set Police         "Courier 12"
    set Police2         {-family "arial" -size 12 -weight bold}
    set CouleurOutline "black"
    set LargeurMax     [expr {[winfo screenwidth  .] - 200}]
    set HauteurMax     [expr {[winfo screenheight .] * 1/2}]
    set LargeurFenetre [expr {[winfo screenwidth  .] - 200}]
    set HauteurFenetre [expr {[winfo screenheight .] * 1/2}]
    set XMilieu        [expr {$LargeurMax/2}]
    set BordureX       50.0
    set BordureY       50.0
    set Intervalle     10.0
    set EpaisseurBoite 15.0

    #Position du Graph content
    set YDebutGraph    [expr {0 + $BordureY}]
    set YFinGraph      [expr {$YDebutGraph + 100}]

    #Position de la query
    set XDebutQuery    [expr {0            + $BordureX}]
    #set YDebutQuery    [expr $YFinGraph   + 10]
    set YDebutQuery    [expr {$YFinGraph   + 20}]
    set XFinQuery      [expr {$LargeurMax  - $BordureX}] 
    set YFinQuery      [expr {$YDebutQuery + $EpaisseurBoite}]
    
    #Rapport du nombre de pixel pour un nt
    set OneNtPixel     [expr {($XFinQuery-$XDebutQuery)/($TailleQueryNt-1)}] 
    #Attention on commence au premier nt

    #Debut des Oligos
    set YDebutDeOligo  [expr {$YFinQuery + $Intervalle}]
        
    #CutOff pour limiter l affichage si besoin est
    ##############################################
    set LesCutOffs [list NomOligo GC Tm ExpTm XDebutDeOligoNt XFinDeOligoNt NbCandidats NbContigues NbUnrelated LesCandidats]

    #Dessine les boites des differentes sources
    ###########################################
    set IdDuGraph 0
    foreach DirOligo $LesDirs {
	set DB [file tail $DirOligo]
	incr IdDuGraph

	set FileParam      "[file join $DirOligo Design.parameters]"
	set  BLASTDataBase [InterrogeDesignParameter $FileParam DataBase]
	if {$BLASTDataBase != ""} {
	    set BLASTDataBase [file tail $BLASTDataBase]
	    if {[regexp {\.[a-z]{3}$} $BLASTDataBase]} {regsub {\.[a-z]{3}$} $BLASTDataBase "" BLASTDataBase}
	}

	#Charge le fichier pour obtenir les infos sur les oligos
	########################################################
	set FichierACharger "[file join $DirOligo "${PAB}.oligo"]"

	set  lesXDebutFinDeOligo [InterrogeInfoDeLOligo $FichierACharger LesDebutFin]
	if {$lesXDebutFinDeOligo == ""} {
	    if {[InterrogeInfoDeLOligo $FichierACharger NotDone]=="1"} {
		set Message "No Oligo calculated for $PAB in $DirOligo"
	    } elseif {[InterrogeInfoDeLOligo $FichierACharger NoOligo]=="1"} {
		set Message "No Oligo available for $PAB in $DirOligo"
	    } else {set Message "Petit probleme"}
	    
	    $K create text $XMilieu $YDebutDeOligo \
		    -text   "$Message" \
		    -font   "$Police2" 
	    set YDebutDeOligo [expr {$YDebutDeOligo + 20}]
	    continue
	}

	#Construction des boites des Oligos
	#Classement des bornes du Nt vers Ct (3prime vers 5prime)
	#Creation des boites avec plus de 40 d ecart 
	#########################################################
	while {1} {
	    if {$lesXDebutFinDeOligo==""} {break}
	    set YFinDeOligo [expr {$YDebutDeOligo + $EpaisseurBoite}]
	    set OldDebut    -40
	    foreach UnDebutFinDeOligo [lsort -real -increasing -index 0 $lesXDebutFinDeOligo] {
		set NbUnrelated ""
		set NomOligo        [InterrogeInfoDeLOligo $FichierACharger [lindex $UnDebutFinDeOligo 0] [lindex $UnDebutFinDeOligo 1] NomOligo]
		set LaSequence      [InterrogeInfoDeLOligo $FichierACharger $NomOligo Sequence]
		set TailleOligo     [InterrogeInfoDeLOligo $FichierACharger $NomOligo Taille]
		set GC              [InterrogeInfoDeLOligo $FichierACharger $NomOligo GC]
		set Tm              [InterrogeInfoDeLOligo $FichierACharger $NomOligo Tm]
		set ExpTm           [InterrogeInfoDeLOligo $FichierACharger $NomOligo ExpTm]
		set XDebutDeOligoNt [InterrogeInfoDeLOligo $FichierACharger $NomOligo Debut]
		set XFinDeOligoNt   [InterrogeInfoDeLOligo $FichierACharger $NomOligo Fin]
		set NbCandidats     [InterrogeInfoDeLOligo $FichierACharger $NomOligo NbCandidats]
		set NbContigues     [InterrogeInfoDeLOligo $FichierACharger $NomOligo NbContigues]
		set LesCandidats    [join [InterrogeInfoDeLOligo $FichierACharger $NomOligo LesCandidats] " "]
		set NbUnrelated     [regexp -all {NG[0-9]+} $LesCandidats]
		
		if {$TailleOligo     == ""}      {set TailleOligo [string length $LaSequence]}
		if {$TailleOligo>$TailleQueryNt} {set TailleOligo $TailleQueryNt}
		if {   $GC           == ""}      {
		    set GC [format "%.1f" [expr {(([NombreDeBaseDe $LaSequence "G|C"])/($TailleOligo*1.0))*100.0}]]
		}
		if {$Tm              == ""}      {set Tm    [CalculTmDeLaSeq $LaSequence NearestNeighbor]}
		if {$ExpTm           == ""}      {set ExpTm [CalculTmDeLaSeq $LaSequence DNADNA]}
		if {$XDebutDeOligoNt == ""}      {set XDebutDeOligoNt 1}
		if {$XFinDeOligoNt   == ""}      {set XFinDeOligoNt $TailleOligo}
		if {$NbCandidats     == ""}      {set NbCandidats "-1"}
		if {$NbContigues     == ""}      {set NbContigues "-1"}
		if {$NbUnrelated     == ""}      {set NbUnrelated "0"}

		set Next   0
		set Result 1
		foreach CutOff $LesCutOffs {
		    set  Result [CheckForRestrictedParameter [set $CutOff] $CutOff]
		    if {$Result=="0"} {
			set Next 1
			break
		    }
		}
		
		if {$Next} {
		    set index               [lsearch  $lesXDebutFinDeOligo $UnDebutFinDeOligo]
		    set lesXDebutFinDeOligo [lreplace $lesXDebutFinDeOligo $index $index]
		    continue
		}
		
		set CouleurBoite    [CouleurDeLOligo $NbCandidats]
		
		#Les infos contenues dans l info bulle
		set    Texte [format   "%-4s%-10s"      "Id:"            "$NomOligo"]
		append Texte [format "\n%-10s%-10s"     "Directory:"     "$DB"]
		append Texte [format "\n%-10s%-10s"     "Database:"      "$BLASTDataBase"]
		append Texte [format "\n%-10s%-3d  %5s" "Length:"        "$TailleOligo"     "nt(s)"]
		append Texte [format "\n%-10s%3.1f %1s" "GC:"            "$GC"              "%"]
		append Texte [format "\n%-10s%3.1f %5s" "Tm:"            "$Tm"              "degree(s)"]
		append Texte [format "\n%-10s%3.1f %5s" "Ex-Tm:"         "$ExpTm"           "degree(s)"]
		append Texte [format "\n%-10s%3d %5s"   "Begin:"         "$XDebutDeOligoNt" "/ $TailleQueryNt nt(s)"]
		append Texte [format "\n%-10s%3d %5s"   "End:"           "$XFinDeOligoNt"   "/ $TailleQueryNt nt(s)"]
		append Texte [format "\n%-15s%2d"       "X-Detected:"    "$NbCandidats"]
		append Texte [format "\n%-15s%2d"       "Contiguous:"    "$NbContigues"]
		append Texte [format "\n%-15s%2d"       "Unrelated Acc:" "$NbUnrelated"]
		
		#Calcul positions de l oligo sur la query
		set XDebutDeOligo [expr {$XDebutQuery   + $OneNtPixel * $XDebutDeOligoNt - $OneNtPixel}]
		set XFinDeOligo   [expr {$XDebutDeOligo + $OneNtPixel * $TailleOligo     - $OneNtPixel}]

		if {[expr $XDebutDeOligo > [expr {$OldDebut+40.0}]] =="1"} {
		    #On dessine la boite de l oligo
		    $K create rectangle $XDebutDeOligo $YDebutDeOligo $XFinDeOligo $YFinDeOligo \
			    -outline "white" \
			    -fill    "$CouleurBoite" \
			    -tags    [list "" $Texte $XDebutDeOligoNt $XFinDeOligoNt $NomOligo $DirOligo Affichable Positionner Boite]
		    set OldDebut            $XFinDeOligo
		    set index               [lsearch  $lesXDebutFinDeOligo $UnDebutFinDeOligo]
		    set lesXDebutFinDeOligo [lreplace $lesXDebutFinDeOligo $index $index]
		} 
	    }
	    #On ajoute de l espace pour la deuxieme ligne
	    set YDebutDeOligo [expr {$YFinDeOligo + $Intervalle}]
	}
	set YDebutDeOligo [expr {$YDebutDeOligo + 15}]
	#Separateur entre chaque banque de design
	$K create text $XDebutQuery [expr {$YDebutDeOligo-8}] \
		-text   "$IdDuGraph: $DB" \
		-font   {-family "arial" -size 10 -weight bold}\
		-anchor "w"
	$K create line $XDebutQuery $YDebutDeOligo $XFinQuery $YDebutDeOligo -fill "black" -width 2

	set YDebutDeOligo [expr {$YDebutDeOligo+$Intervalle}]
    }

    if {$YDebutDeOligo < [winfo screenheight .] } {
	$K configure -height $YDebutDeOligo
    }
    #Pour accorder la scrollbar au dernier oligo affiche
    $K configure -scrollregion [list 0 0 $LargeurMax $YDebutDeOligo]
    
    return ""
}

proc RestrictedParameter {Qui {Valeur ""} {Quoi ""}} {
    
    Wup "Permet de gerer et stocker les parametres de restriction et de les redonner a toutes les procedures"

    #Positionne    Positionne le parametre a la valeur
    #Ajoute        Ajoute ou positionne le parametre si il n existe pas
    #Existe        Retourne 1 si la valeur existe ou 0 sinon
    #LaValeurMerci Retourne la valeur du parametre ou vide sinon
    #Reset         Positionne la valeur a vide
    #ResetAll      Efface tout

    global RestrictedParameters

    #Espionne "Param $Qui $Valeur $Quoi"
    if {$Qui==""} {
	return ""
    } elseif {$Qui=="all"} {
	if {[info exists RestrictedParameters]} {return [array get RestrictedParameters]}  
    } elseif {$Qui=="ResetAll"} {
	if {[info exists RestrictedParameters]} {array unset RestrictedParameters;return ""}  
    }

    if {$Quoi==""} {set Quoi "LaValeurMerci"}
    if {$Quoi=="Positionne"} {
	set RestrictedParameters($Qui) $Valeur
	return ""
    } elseif {$Quoi=="Reset"} {
	if {[info exists RestrictedParameters($Qui)]} {unset       RestrictedParameters($Qui)}
    } elseif {$Quoi=="LaValeurMerci"} {
	if {[info exists RestrictedParameters($Qui)]} {return [set RestrictedParameters($Qui)]} else {return ""}
    } elseif {$Quoi=="Ajoute"} {
	lappend          RestrictedParameters($Qui) $Valeur
    } elseif {$Quoi=="Existe"} {
	if {[info exists RestrictedParameters($Qui)]} {return 1} else {return 0}
    }
    return ""
}

proc LaCategorieDeCutOffDe {CutOff} {

    Wup "Permet de retourner la categorie du CutOff voulu"

    global CategorieDeCutOff

    if {[info exists CategorieDeCutOff($CutOff)]} {
	return  [set CategorieDeCutOff($CutOff)]
    } else {
	if {[info exists CategorieDeCutOff(EstCharge)]} {
	    return [LaCategorieDeCutOffDe Default]
	}
    }

    if {! [info exists CategorieDeCutOff($CutOff)]} {
	#On charge les infos
	set LesBoths   [list GC Tm ExpTm XDebutDeOligoNt XFinDeOligoNt NbCandidats NbContigues NbUnrelated]
	set LesEquals  [list NomOligo]
	set LesRegexps [list LesCandidats]
	set Default    [list Default]

	#Both   recherche les 2 criteres
	#Equal  recherche exactement ce critere
	#Regxep recherche la presence de tous en meme temps
	foreach Cat [list LesBoths LesEquals LesRegexps Default] {
	    switch -exact -- $Cat {
		"LesBoths"   {set TypeDeCat "Both"}
		"LesEquals"  {set TypeDeCat "Equal"}
		"LesRegexps" {set TypeDeCat "Regexp"}
		"Default"    {set TypeDeCat "Regexp"}
	    }
	    foreach Element [set $Cat] {
		set CategorieDeCutOff($Element) $TypeDeCat
	    }
	}
	set CategorieDeCutOff(EstCharge) 1
    }
    return "[LaCategorieDeCutOffDe $CutOff]"
}

proc CheckForRestrictedParameter {Value NomDuCutOff} {

    Wup "Permet de tester les parametres restreints selon la valeur donnee"
    Wup "3 classes de test Equal pour egal, Both pour 1 encadrement et Regexp pour une recherche de texte"

    if {$Value == ""} {return ""}
    if {[RestrictedParameter $NomDuCutOff "" Existe]=="0"} {return ""}

    set ValueDuCutOff [RestrictedParameter $NomDuCutOff "" LaValeurMerci]
    set Categorie     [LaCategorieDeCutOffDe $NomDuCutOff]

    #Both   recherche les 2 criteres
    #Equal  recherche exactement ce critere
    #Regxep recherche la presence de tous en meme temps
    set Keep 1
    switch -exact -- $Categorie {
	"Equal"  {if {$Value !="" && $Value != $ValueDuCutOff} {set Keep 0}}
	"Both"   {
	    set  ValueDuCutOff2 [RestrictedParameter ${NomDuCutOff}_2 "" LaValeurMerci]
	    if {$ValueDuCutOff==""} {set ValueDuCutOff 0.0} elseif {$ValueDuCutOff2==""} {set ValueDuCutOff2 10000000}
	    if {$Value < $ValueDuCutOff || $Value > $ValueDuCutOff2} {set Keep 0} else {set Keep 1}   
	}
 	"Regexp" {
	    if {$Value !="" && ![regexp -nocase "$ValueDuCutOff" $Value]} {
		foreach V $ValueDuCutOff {
		    if {![regexp -nocase "$V" $Value]} {set Keep 0;break}
		    set Keep 1
		}
	    }
	}
    }
    if {$Keep} {return "1"} else {return "0"}
}

proc RestrictedParameter_Panel {K} {
    
    Wup "Panel permettant de mettre des criteres de restriction pour l affichage des oligos"

    set w "$K.restricted"
     if {[winfo exists $w]} {
	wm deiconify $w
	raise $w
	return ""
    } else {toplevel $w}
    
    wm geometry  $w +300+100
    wm title     $w "Restricted Drawing"
    wm resizable $w true false

    set    FParameters "$w.fparameters"
    frame $FParameters -relief "groove"
    pack  $FParameters -fill both -expand no
    
    set LesVars [list NomOligo GC Tm ExpTm XDebutDeOligoNt XFinDeOligoNt NbCandidats NbContigues NbUnrelated LesCandidats]
    #On a des boutons differents pour les speciaux
    set LesSpeciaux      [list GC Tm ExpTm XDebutDeOligoNt XFinDeOligoNt NbCandidats NbContigues NbUnrelated]
    foreach Param $LesSpeciaux {set Special($Param) 1}
    
    foreach Nom $LesVars {
	switch -exact -- $Nom {
	    "NomOligo"        {set Dico($Nom) "Id-Oligo:";set Dico($Nom,Text) "Oligonucleotide identification (ex: XXX_OligoX_XXX)"}
	    "GC"              {set Dico($Nom) "GC:";set Dico($Nom,Text) "GC content"}
	    "Tm"              {set Dico($Nom) "Tm:";set Dico($Nom,Text) "Melting temperature"}
	    "ExpTm"           {set Dico($Nom) "Ex-Tm:";set Dico($Nom,Text) "DNA-DNA Tm"}
	    "XDebutDeOligoNt" {set Dico($Nom) "Begin:";set Dico($Nom,Text) "Oligo starting position within the query sequence"}
	    "XFinDeOligoNt"   {set Dico($Nom) "End:";set Dico($Nom,Text) "Oligo ending position within the query sequence"}
	    "NbCandidats"     {set Dico($Nom) "X-Detected:";set Dico($Nom,Text) "Number of potential X-detected sequence"}
	    "NbContigues"     {set Dico($Nom) "Contiguous:";set Dico($Nom,Text) "Number of X-detected sequences due to Max Contiguous identical sequence"}
	    "NbUnrelated"     {set Dico($Nom) "Unrelated Acc:";set Dico($Nom,Text) "Number of unrelated acc"}
	    "LesCandidats"    {set Dico($Nom) "X-Detected Acc:";set Dico($Nom,Text) "Oligonucleotide detecting this Accession"}
	}
    }

    foreach NomDeVar $LesVars {
	if {[info exists Special($NomDeVar)]} {set OkSpecial 1} else {set OkSpecial 0}
	set LowerNomDeVar [string tolower $NomDeVar]
	
	global $NomDeVar
	set FRestricted        "$FParameters.frestricted${LowerNomDeVar}"
	set  LRestricted       "$FRestricted.lrestricted"
	if {$OkSpecial} {
	    set  L2Restricted  "$FRestricted.l2restricted"
	    set  L3Restricted  "$FRestricted.l3restricted"
	    set  L4Restricted  "$FRestricted.l4restricted"
	    set  E2Restricted  "$FRestricted.e2restricted"
	}
	set  LRestricted       "$FRestricted.lrestricted"
	set  E1Restricted      "$FRestricted.e1restricted"
	set  BRestricted       "$FRestricted.brestricted"
	
	frame  $FRestricted -background "#808080" -relief "groove" -padx "0"
	pack   $FRestricted -side top  -fill both -expand no
	label  $LRestricted -background "#808FFF" -relief "groove" -padx "3" -text "[set Dico($NomDeVar)]" -width "15" -anchor "w"
	set    Texte [format   "%-10s" "[set Dico($NomDeVar,Text)]"]
	set_balloon $LRestricted "$Texte"
	if {$OkSpecial} {
	    label  $L2Restricted -background "#808FFF" -padx "3" -relief "groove" -text "<=" -width "2" -anchor "center"
	    label  $L3Restricted -background "#808080" -padx "3" -relief "groove" -text "X"  -width "2" -anchor "center"
	    label  $L4Restricted -background "#808FFF" -padx "3" -relief "groove" -text "<=" -width "2" -anchor "center"
	    entry  $E1Restricted -width      "5"       -textvariable $NomDeVar
	    entry  $E2Restricted -width      "5"       -textvariable "${NomDeVar}_2"
	    button $BRestricted  -background "yellow"  -padx "2"                  -text "Ok" \
		    -command "RestrictedParameter $NomDeVar \$$NomDeVar Positionne;RestrictedParameter ${NomDeVar}_2 \$${NomDeVar}_2 Positionne"
	} else {
	    entry  $E1Restricted                       -textvariable $NomDeVar
	    button $BRestricted  -background "yellow"  -padx   "2"      -text "Ok" \
		    -command "RestrictedParameter $NomDeVar \$$NomDeVar Positionne"
	}
	
	pack $LRestricted      -side left -fill both -expand no
	if {$OkSpecial} {
	    pack $E1Restricted -side left -fill both -expand yes
	    pack $L2Restricted -side left -fill both 
	    pack $L3Restricted -side left -fill both 
	    pack $L4Restricted -side left -fill both 
	    pack $E2Restricted -side left -fill both -expand yes
	} else {
	    pack $E1Restricted -side left -fill both -expand yes
	}
	pack $BRestricted      -side left -fill both 
    }
    
    set FPiedDeFrame "$w.fpieddeframe"
    set  BDismiss    "$FPiedDeFrame.bdismiss"
    set  BReset      "$FPiedDeFrame.breset"
    set  BReDraw     "$FPiedDeFrame.bredraw"

    frame $FPiedDeFrame -relief "ridge"
    button $BDismiss -background "red"    -padx "2" -text "Dismiss" -command "destroy $w"
    button $BReset   -background "orange" -padx "2" -text "Reset"   -command "RestrictedParameter ResetAll"
    button $BReDraw  -background "green"  -padx "2" -text "ReDraw"  -command "RedrawDesign $K"

    pack $FPiedDeFrame                 -expand no
    pack $BDismiss  -fill y -side left -expand no
    pack $BReset    -fill y -side left -expand no
    pack $BReDraw   -fill y -side left -expand no

    return ""
}

proc SauveLesInfosDe {K Qui Quoi} {
    
    Wup "Stock les informations liees au canvas qu on a besoin d interroger"

    global TabCanvas

    #Espionne "Construit $K $Qui $Quoi"

    if {! [info exists TabCanvas($K,$Qui)]} {
	set TabCanvas($K,$Qui) $Quoi
    } else {
	foreach Q $Quoi {
	    lappend TabCanvas($K,$Qui) $Q
	}
    }
    return ""
}

proc LesInfosDe {K Quoi} {

    Wup "Retourne les informations liees au canvas qu on a besoin"

    global TabCanvas
    
    #Espionne "Demande $K $Quoi"

    if {[info exists TabCanvas($K,$Quoi)]} {return [set TabCanvas($K,$Quoi)]} else {return ""}
}

proc DetruitLesInfosDe {K} {

    Wup "Nettoie les informations liees au canvas qu on a plus besoin"
    Wup "A faire quand le canvas va etre detruit"

    global TabCanvas

    #Espionne "On Decharge $K"

    #Decharge les infos des oligos
    if {[info exists TabCanvas($K,MesFichiersOligo)]} {
	foreach Fichier [lsort -unique [set TabCanvas($K,MesFichiersOligo)]] {
	    InterrogeInfoDeLOligo $Fichier Decharge
	}
	#unset TabCanvas($K,MesFichiersOligo)
    }

    #Decharge les infos des BestHitInDataBase
    if {[info exists TabCanvas($K,MesBlasts)]} {
	foreach Fichier [lsort -unique [set TabCanvas($K,MesBlasts)]] {
	    InterrogeBestHitInDatabase $Fichier Decharge 
	    AskBlast         unsetfile $Fichier
	}
	#unset TabCanvas($K,MesBlasts)
    }

    #Decharge les infos des logs
    if {[info exists TabCanvas($K,MesLogs)]} {
	foreach Fichier [lsort -unique [set TabCanvas($K,MesLogs)]] {
	    InterrogeInfoDuLogDeLOligo $Fichier Decharge
	}
	#unset TabCanvas($K,MesLogs)
    }
    #On supprime les fenetres crees et enregistrees pas K
    DetruitLesWinDe $K

    array unset TabCanvas "$K*"

    if {0} {
	#Old detruit les infos du canvas
	foreach Stocker [list MesDirs MesLogs MesBlasts MonPAB MonFichierQuery MaTailleDeQuery MonAccessQuery] {
	    if {[info exists TabCanvas($K,$Stocker)]} {unset TabCanvas($K,$Stocker)}
	}
    }
    return ""
}

proc DetruitLesWinDe {K} {

    Wup "Tue les autres fenetres associees a K"

    set        LesCanvas [LesInfosDe $K Win]
    #Espionne "$K mes fenetres: $LesCanvas"
    foreach C $LesCanvas {
	if {[winfo exists $C]} {
	    #Espionne "$C mon toplevel: [winfo toplevel $C]"
	    #set C [winfo toplevel $C]
	    #Espionne "Destruction de: $C"
	    destroy $C
	} else {continue}
    }
    return ""
}

proc PasDeCadre {K} {

    Wup "Avec proc UnCanvas de Raymond pour virer le cadre"

    set Id      [$K find withtag "Cadre"]
    set MesTags [$K gettags $Id]
    $K itemconfigure $Id -outline "grey"

    return ""
}

proc TrouveDansLesInfos {K} {

    Wup "Trouve des oligos en fonction de vos criteres"
    
    set  Quoi [Entre "Ex: Id: Oligo1_ACT1013"]
    if {$Quoi==""} {return ""}

    #Pour tenir compte des multiples blancs
    if {[regexp {: } $Quoi]} {regsub {: } $Quoi ": +" Quoi}

    DeselectionneLesTrouves $K
    set  LesIdsRetenus "" 
    set         LesIds [$K find withtag "Boite"]
    foreach Id $LesIds {
	set Info [lindex [$K gettags $Id] 1]
	if {[regexp -nocase "$Quoi" $Info]} {lappend LesIdsRetenus $Id} else {continue}
    }

    foreach Id $LesIdsRetenus {
	$K itemconfigure $Id -outline "darkviolet" 
	$K itemconfigure $Id -width   "3" 
	$K itemconfigure $Id -dash    {6 4 2 4}
	$K addtag "Finded" withtag $Id
    }
    return ""
}

proc DeselectionneLesTrouves {K} {

    Wup "Deselectionne tous les oligos trouve par TrouveDansLesInfos"
    
    set LesIds [$K find withtag "Finded"]

    foreach Id $LesIds {
	$K itemconfigure $Id -outline "white" 
	$K itemconfigure $Id -width   "1"
	$K itemconfigure $Id -dash {}
	$K dtag $Id "Finded"
    }
    return ""
}

proc AllumeLaSelection {K PAB LesDirs} {

    Wup "Highlight les oligos deja selectionnes (fichier .selection) en distinguant les differentes sources" 

    DeselectionneLesSelectionnes $K

    foreach Dir $LesDirs {
	set FichierSelection "[file join $Dir "${PAB}.selection"]"
	if {![file exists $FichierSelection]} {continue}
	
	set F [open "$FichierSelection"]
	while {[gets $F Ligne]>=0} {
	    if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	    if {  [string range $Ligne 0 0] != ">"}    {continue}
	    if {  [regexp -nocase "NoOligo"   $Ligne]} {break}
	    set LigneSplitee [split  $Ligne "\t"]
	    set Entete       [lindex $LigneSplitee 0]
	    set NomOligo     [lindex $Entete 1]
	    
	    set         LesId [$K find withtag "$NomOligo && Boite && $Dir"]
	    foreach Id $LesId {
		set MesTags [$K gettags $Id]
		$K itemconfigure $Id -outline "black" 
		$K itemconfigure $Id -width   "2" 
		$K itemconfigure $Id -dash    {6 4 2 4}
		$K addtag "Selected" withtag $Id
	    }
	}
	close $F
    }
    return ""
}

proc SelectionneOligo {K} {

    Wup "Selectionne graphiquement l oligo choisis"

    set Id      [$K find withtag current]
    set MesTags [$K gettags $Id]

    if {[regexp "Selected" $MesTags]} {
	$K itemconfigure $Id -outline "white" 
	$K itemconfigure $Id -width   "1"
	$K itemconfigure $Id -dash {}
	$K dtag $Id "Selected"
	return ""
    } else { 
	$K itemconfigure $Id -outline "black" 
	$K itemconfigure $Id -width   "2" 
	$K itemconfigure $Id -dash {6 4 2 4}
	$K addtag "Selected" withtag $Id
    }
    return ""
}

proc LesOligosSelectionnes {K {Option ""}} {

    Wup "Retourne la liste des Noms des oligos Selectionnes par proc SelectionneOligo"

    set LesIds ""
    set LesIds [$K find withtag "Selected"]
    if {$Option=="LesIds"} {return $LesIds}

    set LesNomsDesOligos ""
    foreach Id $LesIds {
    	set MesTags   [$K gettags $Id]
	set Nom       [lindex $MesTags 4]
	lappend LesNomsDesOligos $Nom
    }
    return $LesNomsDesOligos
}

proc DeselectionneLesSelectionnes {K} {

    Wup "Deselectionne tous les oligo choisis"
    
    set         LesIds [$K find withtag "Selected"]
    foreach Id $LesIds {
	$K itemconfigure $Id -outline "white" 
	$K itemconfigure $Id -width   "1"
	$K itemconfigure $Id -dash {}
	$K dtag $Id "Selected"
    }
    return ""
}

proc SauveLesSelectionnes {K PAB LesDirs} {

    Wup "Sauve les Oligos selectionnes en distinguant les differentes sources"
    
    foreach DirOligo $LesDirs {
	set FichierSelection "[file join $DirOligo "$PAB.selection"]"
	set DB [file tail $DirOligo]

	set  NosIds ""
	set  NosIds [$K find withtag "Selected && $DirOligo"]
	if {$NosIds==""} {
	    if {[file exists $FichierSelection]} {
		if {[OuiOuNon "A previous selection exists for $PAB in $DB.\nWould you like to delete it ?"]} {
		    file delete -force $FichierSelection
		} 
	    }
	    
	    #file delete -force $FichierSelection
	    continue
	}
	
	if {[file exists $FichierSelection]} {
	    if {[OuiOuNon "A selection has already been saved for $PAB in $DB.\nWould you like to overwrite it ?"]} {
		file delete -force $FichierSelection
	    } elseif {! [OuiOuNon "It will append it.\nYou still want to save ?"]} {continue}
	}

	foreach Id $NosIds {
	    set MesTags   [$K gettags $Id]
	    set NomOligo  [lindex $MesTags 4]
	    set Selection [join [ExtraitOligoDuTFA $PAB $NomOligo $DirOligo] "\n"]
	    AppendAuFichier $FichierSelection "$Selection"
	}
    }
    return ""
}

proc AllumeLaXSelection {K PAB LesDirs} {

    Wup "Highlight les oligos croisee deja selectionnes (fichier .croisee) en distinguant les differentes sources" 

    DeselectionneLesXSelectionnes $K

    foreach Dir $LesDirs {
	set FichierSelection "[file join $Dir "$PAB.croisee"]"
	if {![file exists $FichierSelection]} {continue}
	set F [open      "$FichierSelection"]
	while {[gets $F Ligne]>=0} {
	    if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	    if {  [string range $Ligne 0 0] != ">"}    {continue}
	    if {  [regexp -nocase "NoOligo"   $Ligne]} {break}
	    set LigneSplitee [split  $Ligne "\t"]
	    set Entete       [lindex $LigneSplitee 0]
	    set NomOligo     [lindex $Entete 1]
	    
	    set         LesId [$K find withtag "$NomOligo && Boite && $Dir"]
	    foreach Id $LesId {
		set MesTags   [$K gettags $Id]
		set MesCoords [$K coords $Id]
		set Xmin [lindex $MesCoords 0]
		set Xmax [lindex $MesCoords 2]
		set Ymin [lindex $MesCoords 1]
		set Ymax [lindex $MesCoords 3]
		#set Ymin [expr [lindex $MesCoords 1]-([lindex $MesCoords 3]-[lindex $MesCoords 1])/4]
		#set Ymax [expr [lindex $MesCoords 3]+([lindex $MesCoords 3]-[lindex $MesCoords 1])/4]
		$K create line $Xmin $Ymin $Xmax $Ymax \
			-tags [list "XSelect" "Trace" "$NomOligo"] \
			-fill "black" -width 2
		$K create line $Xmin $Ymax $Xmax $Ymin \
			-tags [list "XSelect" "Trace" "$NomOligo"] \
			-fill "black" -width 2
		$K addtag "XSelect" withtag $Id
	    }
	}
	close $F
    }
    return ""
}

proc SelectionneOligoCroisee {K} {

    Wup "Selectionne graphiquement l oligo choisis et croisee"

    set Id       [$K find withtag current]
    set MesTags  [$K gettags $Id]
    set NomOligo [lindex $MesTags 4]

    if {[regexp     "XSelect" $MesTags]} {
	$K dtag $Id "XSelect"
	set          LesIds [$K find withtag "XSelect && Trace && $NomOligo"]
	foreach Ids $LesIds {
	    $K delete $Ids
	}
	return ""
    } else { 
	set MesCoords  [$K coords $Id]
	set Xmin [lindex $MesCoords 0]
	set Xmax [lindex $MesCoords 2]
	set Ymin [lindex $MesCoords 1]
	set Ymax [lindex $MesCoords 3]
	#set Ymin [expr [lindex $MesCoords 1]-([lindex $MesCoords 3]-[lindex $MesCoords 1])/4]
	#set Ymax [expr [lindex $MesCoords 3]+([lindex $MesCoords 3]-[lindex $MesCoords 1])/4]
	$K create line $Xmin $Ymin $Xmax $Ymax \
		-tags [list "XSelect" "Trace" "$NomOligo"] \
		-fill "black" -width 2
	$K create line $Xmin $Ymax $Xmax $Ymin \
		-tags [list "XSelect" "Trace" "$NomOligo"] \
		-fill "black" -width 2
	$K addtag "XSelect" withtag $Id
    }
    return ""
}

proc DeselectionneLesXSelectionnes {K} {

    Wup "Deselectionne tous les oligo croises choisis"
    
    set         LesIds [$K find withtag "Trace"]
    foreach Id $LesIds {
	$K delete $Id
    }
    set         LesIds [$K find withtag "XSelect"]
    foreach Id $LesIds {
	$K dtag $Id "XSelect"
    }
    return ""
}

proc SauveLesXSelectionnes {K PAB LesDirs} {

    Wup "Sauve les Oligos croisee selectionnes en distinguant les differentes sources"
    
    foreach DirOligo $LesDirs {
	set FichierSelection "[file join $DirOligo "$PAB.croisee"]"

	set  NosIds ""
	set  NosIds [$K find withtag "XSelect && $DirOligo"]
	if {$NosIds==""} {
	    file delete -force $FichierSelection
	    continue
	}
	
	if {[file exists $FichierSelection]} {
	    if {[OuiOuNon "Some X-Selected Oligo have already been saved for $PAB\nWould you like to overwrite it ?"]} {
		file delete -force $FichierSelection
	    } elseif {! [OuiOuNon "It will append to it\nYou still want to save ?"]} {return ""}
	}
	foreach Id $NosIds {
	    set MesTags   [$K gettags $Id]
	    set NomOligo  [lindex $MesTags 4]
	    set Selection [join [ExtraitOligoDuTFA $PAB $NomOligo $DirOligo] "\n"]
	    AppendAuFichier $FichierSelection "$Selection"
	}
    }
    return ""
}

proc Graph_Panel {LeCanvasPere} {

    Wup "Panel de choix d affichage et de couleur des differents graphes Tm GC ID NbSeq"

    set  LesGraphes [LesGraphesDeLOligo $LeCanvasPere]
    if {$LesGraphes==""} {return ""}

    set GrapheTopLevel "$LeCanvasPere.graphepanel"
    if {[winfo exists $GrapheTopLevel]} {
	wm deiconify  $GrapheTopLevel
	raise         $GrapheTopLevel
	return ""
    } else {toplevel  $GrapheTopLevel}
    
    wm geometry  $GrapheTopLevel +300+100
    wm title     $GrapheTopLevel "Graph Selection"
    wm resizable $GrapheTopLevel false false
     
    set    FrameGraphe "$GrapheTopLevel.lesgraphesframe"
    frame $FrameGraphe -borderwidth "3" -relief "groove"
    pack  $FrameGraphe -expand   "true" -side      "top" -fill "both"
    
    set IndGraph 0
    set NbGraph  [llength $LesGraphes]

    #Trie pour avoir GC et Tm devant les autres
    if {[regexp "GC" $LesGraphes]} {set        GraphesOrdonnes "GC"}
    foreach G $LesGraphes {
	if {[regexp -nocase "Tm" $G]} {lappend GraphesOrdonnes $G} else {continue}
    }
    foreach G $LesGraphes {
	if {$G=="GC" || [regexp -nocase "Tm" $G]} {continue} else {lappend GraphesOrdonnes $G}
    }

    set  LesGraphes $GraphesOrdonnes
    unset   GraphesOrdonnes
    foreach Graphe $LesGraphes {
	incr IndGraph
	set TagOrId $Graphe
	set Graphe  [string tolower $Graphe]

	set PetiteFrameGraphe "$FrameGraphe.f$Graphe"
	set LGr         "$PetiteFrameGraphe.l$Graphe"
	set BGrOn       "$PetiteFrameGraphe.bgon$Graphe"
	set BGrOff      "$PetiteFrameGraphe.bgoff$Graphe"
	set BMoyOn      "$PetiteFrameGraphe.bmoyon$Graphe"
	set BMoyOff     "$PetiteFrameGraphe.bmoyoff$Graphe"
	set BColor      "$PetiteFrameGraphe.bcolor$Graphe"
	
	frame $PetiteFrameGraphe                               -borderwidth "0" -relief "groove" 
	label  $LGr     -text "$TagOrId"             -padx "2" -borderwidth "3" -relief "groove" -background "orange"
	button $BGrOn   -text "On"  -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Raiser $LeCanvasPere $TagOrId"
	button $BGrOff  -text "Off" -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Lower  $LeCanvasPere $TagOrId"
	button $BMoyOn  -text "On"  -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Raiser $LeCanvasPere Average${TagOrId}"
	button $BMoyOff -text "Off" -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Lower  $LeCanvasPere Average${TagOrId}"
	button $BColor  -text "C"   -anchor "center" -padx "3" -borderwidth "2" -relief "groove" -background "#808FFF" 

	#bind  $LGr    <Enter> "LePanneauDeCouleursDuGraph $LGr    $K $TagOrId"
	#bind  $BColor <Enter> "LePanneauDeCouleursDuGraph $BColor $K $TagOrId"
	bind  $BColor <1> "LePanneauDeCouleursDuGraph $BColor $LeCanvasPere $TagOrId"
	
	grid $PetiteFrameGraphe        -sticky ewns 
	grid $LGr     -row 1 -column 0 -sticky ewns 
	grid $BGrOn   -row 1 -column 1 -sticky ewns
	grid $BGrOff  -row 1 -column 2 -sticky ewns 
	grid $BMoyOn  -row 1 -column 3 -sticky ewns
	grid $BMoyOff -row 1 -column 4 -sticky ewns
	grid $BColor  -row 1 -column 5 -sticky ewns

	if {$IndGraph==1} {
	    set DismissB "$PetiteFrameGraphe.destroybutton"
	    set LDuGraph "$PetiteFrameGraphe.ldugraph"
	    set LDuMoy   "$PetiteFrameGraphe.ldumoyenne"
	    set LDeDeco  "$PetiteFrameGraphe.ldedeco"
    
	    button $DismissB -text "Exit"    -anchor "center" -padx "3" -background "red"     -borderwidth "2" -relief "groove" -activebackground "red" -overrelief "sunken" \
		    -command "destroy $GrapheTopLevel"
	    label $LDuGraph  -text "Graph"   -anchor "center" -padx "2" -background "#808FFF" -borderwidth "3" -relief "groove"
	    label $LDuMoy    -text "Average" -anchor "center" -padx "2" -background "#808FFF" -borderwidth "3" -relief "groove"
	    label $LDeDeco   -text ""        -anchor "center" -padx "2" -background "green3"  -borderwidth "3" -relief "groove"
	    grid $DismissB -row 0 -column 0 -sticky ewns 
	    grid $LDuGraph -row 0 -column 1 -sticky ewns -columnspan 2
	    grid $LDuMoy   -row 0 -column 3 -sticky ewns -columnspan 2
	    grid $LDeDeco  -row 0 -column 5 -sticky ewns 
	}

	if {$IndGraph==$NbGraph} {
	    set LAllDuGr  "$PetiteFrameGraphe.lalldugraph"
	    set BAllGrOn  "$PetiteFrameGraphe.ballgon"
	    set BAllGrOff "$PetiteFrameGraphe.ballgoff"
	    set BAllAvOn  "$PetiteFrameGraphe.ballaon"
	    set BAllAvOff "$PetiteFrameGraphe.ballaoff"
	    set LDeDeco   "$PetiteFrameGraphe.ldedeco2"
	   	    
	    label  $LAllDuGr  -text "All" -anchor "center" -padx "2" -relief "groove" -background "green3" -borderwidth "3" 
	    button $BAllGrOn  -text "On"  -anchor "center" -padx "3" -relief "groove" -background "yellow" -borderwidth "2" \
		    -command "Raiser $LeCanvasPere OligoGraph"
	    button $BAllGrOff -text "Off" -anchor "center" -padx "3" -relief "groove" -background "yellow" -borderwidth "2" \
		    -command "Lower  $LeCanvasPere OligoGraph"
	    button $BAllAvOn  -text "On"  -anchor "center" -padx "3" -relief "groove" -background "yellow" -borderwidth "2" \
		    -command "RaiseLesAverageDeLOligo $LeCanvasPere"
	    button $BAllAvOff -text "Off" -anchor "center" -padx "3" -relief "groove" -background "yellow" -borderwidth "2" \
		    -command "LowerLesAverageDeLOligo $LeCanvasPere"
	    label $LDeDeco    -text ""    -anchor "center" -padx "2" -relief "groove" -background "green3" -borderwidth "3" 

	    grid $LAllDuGr  -row 2 -column 0 -sticky ewns 
	    grid $BAllGrOn  -row 2 -column 1 -sticky ewns 
	    grid $BAllGrOff -row 2 -column 2 -sticky ewns 
	    grid $BAllAvOn  -row 2 -column 3 -sticky ewns 
	    grid $BAllAvOff -row 2 -column 4 -sticky ewns
	    grid $LDeDeco   -row 2 -column 5 -sticky ewns 
	}
	grid columnconfigure $PetiteFrameGraphe 0 -weight 1
	grid columnconfigure $PetiteFrameGraphe 1 -weight 0
	grid columnconfigure $PetiteFrameGraphe 2 -weight 0
	grid columnconfigure $PetiteFrameGraphe 3 -weight 0
	grid columnconfigure $PetiteFrameGraphe 4 -weight 0
    }
    return ""
}

proc LePanneauDeCouleursDuGraph {target LeCanvasPere tagorid} {
    
    Wup "Petit panneau de couleur pour changer la couleur des graphes Tm GC Nbseq Id"

    set w          ".choixcouleur"
    set GraphPanel "[winfo toplevel $target]"
    
    if {[winfo exists $w]} {destroy $w}
    toplevel            $w
    wm overrideredirect $w 1
    wm resizable        $w false false
    
    set    FInfo "$w.frame"
    frame $FInfo -borderwidth "1" -relief "ridge" -background "gray"
    pack  $FInfo -fill "both"     -expand "true"

    bind  $FInfo      <Leave>   "destroy $w"
    bind  $GraphPanel <Destroy> "destroy $w"
    #bind $target <Motion>  {wm geometry $w +[winfo rootx $target]+[winfo rooty $target]}
       
    #Palette pour le panneau special de choix de couleur
    set LesCouleurs ""
    set LesCouleurs [list black red green3 orange deeppink deepskyblue2 darkviolet firebrick4 palette]
    set NbCouleur   [llength $LesCouleurs]
    set MaxRow      2
    set MaxCol      [expr {$NbCouleur/$MaxRow}]
    set Row         0
    set Column      0
    foreach Couleur $LesCouleurs {
	if {$Row>=$MaxRow} {incr Column;set Row 0}
	if {$Couleur=="palette"} {
	    set    LCoul "${FInfo}.${Couleur}"
	    label $LCoul -background "yellow"  -relief "ridge" -width "2" -text "P"
	    grid  $LCoul -column     "$Column" -row    "$Row"  -padx  "2" -pady "2"
	    bind  $LCoul  <ButtonPress-1>   "ChangeLaCouleurDuGraph $LeCanvasPere $tagorid \[tk_chooseColor\]"
	    bind  $LCoul  <ButtonRelease-1> "destroy $w"
	    continue
	}
	set    LCoul "${FInfo}.${Couleur}"
	label $LCoul -background "$Couleur" -relief "sunken" -width "2"        
	grid  $LCoul -column     "$Column"  -row    "$Row"   -padx  "2" -pady "2"
	bind  $LCoul  <ButtonPress-1>   "ChangeLaCouleurDuGraph $LeCanvasPere $tagorid $Couleur"
	bind  $LCoul  <ButtonRelease-1> "destroy $w"
	incr Row
    }
    
    set x [expr {[winfo rootx $target]}]
    set y [expr {[winfo rooty $target]}]
    wm geometry $w +${x}+${y}    

    return ""
}    

proc ChangeLaCouleurDuGraph {K tagorid Couleur} {

    Wup "Change la couleur du graph tagorid"

    set LesIds [$K find withtag $tagorid]
    #Attention pour l oval creer dans TmGraph
    if {[regexp -nocase "Plot" $tagorid]} {set AvecContour 1} else {set AvecContour 0}
    
    foreach Id $LesIds {
	if {! $AvecContour} {
	    $K itemconfigure $Id -fill $Couleur
	} else {
	    $K itemconfigure $Id -fill $Couleur -outline $Couleur
	}
    }
    return ""
}

proc Raiser {K tagorid} {

    Wup "Met au premier plan le widget marque par tagorid"

    $K raise $tagorid
    return ""
}

proc Lower {K tagorid} {

    Wup "Met en arriere plan le widget marque par tagorid"

    $K lower $tagorid
    return ""
}

proc RaiseLesAverageDeLOligo {K} {

    Wup "Met au premier plan les averages des graphes"

    set            LesGraphes [LesGraphesDeLOligo $K]
    foreach Graph $LesGraphes {
	Raiser $K "Average${Graph}"
    }
    return ""
}

proc LowerLesAverageDeLOligo {K} {

    Wup "Met en arriere plan les averages des graphes"

    set            LesGraphes [LesGraphesDeLOligo $K]
    foreach Graph $LesGraphes {
	Lower $K "Average${Graph}"
    }
    return ""
}

proc LesGraphesDeLOligo {K} {

    Wup "Retourn les tag des graphes disponibles Tm, GC ..."

    set LesGraphes ""
    set         LesId [$K find withtag "OligoGraph"]
    foreach Id $LesId {
    	set MesTags        ""
        set index          ""

    	set MesTags        [$K gettags $Id]
        set index          [lsearch $MesTags "OligoGraph"]
	lappend LesGraphes [lindex  $MesTags [expr {$index+1}]]
    }
    set     LesGraphes [lsort -unique $LesGraphes]
    return $LesGraphes
}

proc SignificationDesCouleursDeLOligo {} {
    
    Wup "Signification des Couleurs des oligos"

    set    Texte "Oligonucleotide Color:\n"
    append Texte "\n"
    append Texte "White ------ O  Oligo  detected\n"
    append Texte "Green ------ 1  Oligo  detected\n"
    append Texte "Blue ------- 2  Oligos detected\n"
    append Texte "Yellow ---- 3-5 Oligos detected\n"
    append Texte "Orange ---- 6-8 Oligos detected\n"
    append Texte "Red ------- >8  Oligos detected\n"
    append Texte "Black ----- -1  Problem\n"
    after idle {   .fairelire.msg configure -wraplength 10i -background red -font "Courier 12 bold"}
    tk_dialog      .fairelire "Please press OK" "$Texte" {} 0 Acknowledge
    catch {destroy .fairelire}

    return ""
}

proc CouleurDeLOligo {NbCandidat} {
    
    Wup "Retourne une couleur de boite en fonction du nombre de candidats"

    if {$NbCandidat==""} {return "black"}

    set CouleurDeLOligo ""
    switch -exact -- $NbCandidat {
	"-1" {set CouleurDeLOligo "black"}
	"0"  {set CouleurDeLOligo "white"}
	"1"  {set CouleurDeLOligo "green3"}
	"2"  {set CouleurDeLOligo "skyblue"}
	"3"  {set CouleurDeLOligo "Yellow2"}
	"4"  {set CouleurDeLOligo "Yellow2"}
	"5"  {set CouleurDeLOligo "Yellow2"}
	"6"  {set CouleurDeLOligo "orange"}
	"7"  {set CouleurDeLOligo "orange"}
	"8"  {set CouleurDeLOligo "orange"}
    }
    if {   $CouleurDeLOligo==""} {set CouleurDeLOligo "red"}
    return $CouleurDeLOligo
}

proc AfficheHelp {{Quoi ""}} {

    Wup "Permet de donner une petite aide pour les commandes disponibles"

    set    Texte ""
    append Texte "Graph Panel:\n"
    append Texte "------------\n"
    append Texte "   Panel that allows you display or hide,\n"
    append Texte "   \tGraph itself\n"
    append Texte "   \tAverage\n"
    append Texte "   \tAll or none\n\n"
    append Texte "GC Content:\n"
    append Texte "-----------\n"
    append Texte "   Upper  Value: 100% GC bases\n"
    append Texte "   Middle Value: Average composition in GC bases\n"
    append Texte "   Lower  Value: 100% AT bases\n"
    append Texte "   Enter  Display the %GC of the current Position\n\n"
    append Texte "Tm Content:\n"
    append Texte "-----------\n"
    append Texte "   Upper  Value: maximum Tm value\n"
    append Texte "   Middle Value: Average Tm value\n"
    append Texte "   Lower  Value: minimum Tm value\n"
    append Texte "   Enter  Display the Tm of the current Position\n\n"
    append Texte "Identity Percent (Blastn):\n"
    append Texte "--------------------------\n"
    append Texte "   Upper  Value: 100% Id\n"
    append Texte "   Middle Value: Average Id percent\n"
    append Texte "   Lower  Value: 0%   Id\n"
    append Texte "   Enter  Display the %Id of the current Position\n\n"
    append Texte "Sequence Content (Blastn):\n"
    append Texte "--------------------------\n"
    append Texte "   Upper  Value: maximum number of sequence\n"
    append Texte "   Middle Value: Average number of sequence\n"
    append Texte "   Lower  Value: minimum number of sequence\n"
    append Texte "   Enter  Display the number of sequence under the current Position\n\n"
    append Texte "Query:\n"
    append Texte "------\n"
    append Texte "   Mouse 1 Basic nucleic informations (GC, length, Tm ...)\n"
    append Texte "   Mouse 3 Display Query s Sequence in fasta format\n\n"
    append Texte "Oligonucleotide (Probe):\n"
    append Texte "------------------------\n"
    append Texte "   Enter   Display Information about the current Oligo\n"
    append Texte "   Mouse 1 Display the Position regarding the Query\n"
    append Texte "   Mouse 2 Display Oligo s specificity\n"
    append Texte "   Mouse 3 Display Oligo s Sequence (fasta format)\n"
    append Texte "   Shift  -2 Display Oligo s specificity in detail\n"
    append Texte "   Control-1 Select the oligonucleotide for that database\n"
    append Texte "   Control-2 Select the oligonucleotide as X-Selection"

    if {$Quoi=="DuBouton"} {
	after idle {   .fairelire.msg configure -wraplength 10i -background red -font "Courier 12"}
	tk_dialog      .fairelire "Please press OK" "$Texte" {} 0 Acknowledge
	catch {destroy .fairelire}
	return ""
    } else {return $Texte}
}

proc AfficheLaQuery {K FichierQuery} {

    Wup "Permet d afficher la sequence tfa de la query"
    
    if {!  [file exists $FichierQuery]} {
	if {[regexp {/} $FichierQuery]} {
	    set PAB [file tail $FichierQuery]
	} else {
	    set PAB $FichierQuery
	}
	set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
	set Repnuctfa "[file join [RepertoireDuGenome] nuctfa]"
	if {      [file exists "[file join $RepmRNA $PAB]"]} {
	    set FichierQuery   "[file join $RepmRNA $PAB]"
	} elseif {[file exists "[file join $Repnuctfa $PAB]"]} {
	    set FichierQuery   "[file join $Repnuctfa $PAB]"
	} else {set FichierQuery ""}
    }
    if {$FichierQuery == ""} {return [AfficheVariable "No sequence file for $FichierQuery"]}
    set LeTFADeLaQuery [join [LesLignesDuFichier $FichierQuery] "\n"]

    return "[AfficheVariable $LeTFADeLaQuery "" "$FichierQuery"]"
}

proc AfficheLesAutresCandidatsComplet {K PAB} {

    Wup "Permet d afficher les candidats de l oligo avec leur pourcentages d identite et leur nb bases contigues"
    Wup "On ajoute les boutons pour les liens vers les banques"

    set Id       [$K find withtag current]
    set MesTags  [$K gettags $Id]
    set NomOligo [lindex $MesTags 4]
    set Dir      [lindex $MesTags 5]
    set Debut    [lindex $MesTags 2]
    set Fin      [lindex $MesTags 3]

    set  LesAccessDef [LesAccessStatCompletes "[file join $Dir "${PAB}.log"]" $Debut $Fin]
    if {$LesAccessDef==""} {return ""}
  
    #Recuperation de la banque BLAST utilisee pendant le design
    set FileParam "[file join $Dir Design.parameters]"
    set DataBase [InterrogeDesignParameter $FileParam DataBase]
        
    set Fen  [AfficheVariable [join $LesAccessDef "\n"] "" "$NomOligo from $PAB in [file tail $Dir]"]
    set FenB "$Fen.buttons"

    TouchePour $FenB NouvelleGamme
    TouchePour $FenB NouvelleCouleur "white"
    set D [TouchePour $FenB "FastaCmd" ""]
    TouchePour $FenB NouvelleCouleur "green"
    #TouchePour $FenB     "Sel" "catch {SauveLesInfosDe $K Win \[FastaCmd \[selection get\]\]}"
    TouchePour $FenB     "Sel" "catch {SauveLesInfosDe $K Win \[FastaCmd \[selection get\] \"\" 1 $DataBase\]}"
    TouchePour <2>      "/All" "catch {SauveLesInfosDe $K Win \[FastaCmd \[set PagePropre($Fen)\] \"\" 1 $DataBase\]}"
    TouchePour <3> "/Complete" "catch {SauveLesInfosDe $K Win \[FastaCmd \[selection get\] Complet 1 $DataBase\]}"
    #TouchePour $FenB     "Sel" "catch {SauveLesInfosDe $K Win \[FastaCmd \[selection get\]\]}"
    #TouchePour <2>      "/All" "catch {SauveLesInfosDe $K Win \[FastaCmd \[set PagePropre($Fen)\]\]}"
    #TouchePour <3> "/Complete" "catch {SauveLesInfosDe $K Win \[FastaCmd \[selection get\] Complet\]}"
    set    Texte [format   "%-10s" "Buttons below:"]
    append Texte [format "\n%-10s" "To retrieve sequence in fasta format from blast database"]
    append Texte [format "\n%-10s" "To retrieve sequence header (access and definition)"]
    set_balloon $D "$Texte"
    $D configure -state disabled -disabledforeground "black"

    #TouchePour $FenB NouvelleGamme
    #TouchePour $FenB NouvelleCouleur  "white"
    #set B [TouchePour $FenB "Link Uniseq To" ""]
    #TouchePour $FenB NouvelleCouleur  "green"
    #TouchePour $FenB "Unigene ID Sel" "catch {SauveLesInfosDe $K Win \[LeUnigeneDeUniseq_GUI \[selection get\]\]}" 
    #TouchePour <2>             "/All" "catch {SauveLesInfosDe $K Win \[LeUnigeneDeUniseq_GUI \[set PagePropre($Fen)\]\]}"
    
    #TouchePour $FenB NouvelleGamme
    #TouchePour $FenB NouvelleCouleur "green"
    #TouchePour $FenB "Genbank Sel" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]        uniseq genbank All\]}"
    #TouchePour <2>           "/Id" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]\]}"
    #TouchePour <3>          "/All" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[set PagePropre($Fen)\]\]}"
    
    #TouchePour $FenB "Protein Sel" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]        uniseq protein All\]}"
    #TouchePour <2>           "/Id" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]        uniseq protein\]}"
    #TouchePour <3>          "/All" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[set PagePropre($Fen)\] uniseq protein\]}"

    #set    Texte [format   "%-10s" "Buttons below:"]
    #append Texte [format "\n%-10s" "To Link Unigene to Genbank or to Protein database"]
    #append Texte [format "\n%-10s" "Allows to retrieve access only (Id and all) or complete entry (Sel.)"]
    #append Texte [format "\n%-10s" "Allows to retrieve also Unigene (Ug.xxx) id from Uniseq (HS#xxx)"]
    #set_balloon $B "$Texte"
    #$B configure -state disabled -disabledforeground "black"
    
    return $Fen
}

proc LesAccessStatCompletes {LogFile Debut Fin} {

    Wup "Permet de recuperer les candidats de l oligo avec leur pourcentages d identite et leur nb bases contigues"

    if {![file exists $LogFile]} {return ""}

    set LesCandidats ""
    set LesAccessDef ""
    set     LesCandidats [InterrogeInfoDuLogDeLOligo "$LogFile" $Debut $Fin LesCandidats]
    if {   $LesCandidats == ""} {
	set LesCandidats "No Hit found in database or in log file"
	lappend LesAccessDef $LesCandidats
    } else {
	foreach C $LesCandidats {
	    lappend LesAccessDef "[format "%-16s %6.2f" $C \
		    [InterrogeInfoDuLogDeLOligo "$LogFile" $Debut $Fin $C IDG]] \
		    [InterrogeInfoDuLogDeLOligo "$LogFile" $Debut $Fin $C NbContigue]"
	}
    }
    return $LesAccessDef
}

proc AfficheLesAutresCandidats {K PAB} {

    Wup "Permet d afficher les candidats de l oligo"
    Wup "On ajoute les boutons pour les liens vers les banques"

    set Id           [$K find withtag current]
    set MesTags      [$K gettags $Id]
    set NomOligo     [lindex $MesTags 4]
    set Dir          [lindex $MesTags 5]

    set  LesAccessDef [LesAccessDefDeLOligo $PAB $NomOligo $Dir]
    if {$LesAccessDef==""} {return ""}

    #Recuperation de la banque BLAST utilisee pendant le design
    set FileParam "[file join $Dir Design.parameters]"
    set DataBase [InterrogeDesignParameter $FileParam DataBase]
    
    if {! [regexp "No Hit found" $LesAccessDef]} {
	set LesAccessDef [join $LesAccessDef "\n"]
    }
    
    set Fen  [AfficheVariable $LesAccessDef "" "$NomOligo from $PAB in [file tail $Dir]"]
    set FenB "$Fen.buttons"

    TouchePour $FenB NouvelleGamme
    TouchePour $FenB NouvelleCouleur "white"
    set D [TouchePour $FenB "FastaCmd" ""]
    TouchePour $FenB NouvelleCouleur "green"
    #TouchePour $FenB     "Sel" "catch {SauveLesInfosDe $K Win \[FastaCmd \[selection get\]\]}"
    TouchePour $FenB     "Sel" "catch {SauveLesInfosDe $K Win \[FastaCmd \[selection get\] \"\" 1 $DataBase\]}"
    TouchePour <2>      "/All" "catch {SauveLesInfosDe $K Win \[FastaCmd \[set PagePropre($Fen)\] \"\" 1 $DataBase\]}"
    TouchePour <3> "/Complete" "catch {SauveLesInfosDe $K Win \[FastaCmd \[selection get\] Complet 1 $DataBase\]}"
    set    Texte [format   "%-10s" "Buttons below:"]
    append Texte [format "\n%-10s" "To retrieve sequence in fasta format from BLAST database"]
    append Texte [format "\n%-10s" "To retrieve sequence header (access and definition)"]
    set_balloon $D "$Texte"
    $D configure -state disabled -disabledforeground "black"

    #TouchePour $FenB NouvelleGamme
    #TouchePour $FenB NouvelleCouleur  "white"
    #set B [TouchePour $FenB "Link Uniseq To" ""]
    #TouchePour $FenB NouvelleCouleur  "green"
    #TouchePour $FenB "Unigene ID Sel" "catch {SauveLesInfosDe $K Win \[LeUnigeneDeUniseq_GUI \[selection get\]\]}" 
    #TouchePour <2>             "/All" "catch {SauveLesInfosDe $K Win \[LeUnigeneDeUniseq_GUI \[set PagePropre($Fen)\]\]}"
    
    #TouchePour $FenB NouvelleGamme
    #TouchePour $FenB NouvelleCouleur "green"
    #TouchePour $FenB "Genbank Sel" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]        uniseq genbank All\]}"
    #TouchePour <2>           "/Id" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]\]}"
    #TouchePour <3>          "/All" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[set PagePropre($Fen)\]\]}"
    
    #TouchePour $FenB "Protein Sel" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]        uniseq protein All\]}"
    #TouchePour <2>           "/Id" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[selection get\]        uniseq protein\]}"
    #TouchePour <3>          "/All" "catch {SauveLesInfosDe $K Win \[GetzUniseq_GUI $K \[set PagePropre($Fen)\] uniseq protein\]}"

    #set    Texte [format   "%-10s" "Buttons below:"]
    #append Texte [format "\n%-10s" "To Link Unigene to Genbank or to Protein database"]
    #append Texte [format "\n%-10s" "Allows to retrieve access only (Id and all) or complete entry (Sel.)"]
    #append Texte [format "\n%-10s" "Allows to retrieve also Unigene (Ug.xxx) id from Uniseq (HS#xxx)"]
    #set_balloon $B "$Texte"
    #$B configure -state disabled -disabledforeground "black"

    return $Fen
}

proc LesAccessDefDeLOligo {PAB NomOligo Dir} {

    Wup "Rend les Access des autres candidats de l oligo"

    set Fichier "[file join $Dir "${PAB}.oligo"]"
    if {! [file exists $Fichier]} {
	set Fichier "[file join $Dir $PAB]"
	if {! [file exists $Fichier]} {return ""}
    }

    set  LesCandidats [InterrogeInfoDeLOligo $Fichier $NomOligo LesCandidats]

    if {[regexp "No Hit found" $LesCandidats]} {
	return "$LesCandidats"
    }

    return [lsort -unique $LesCandidats]
}

proc OLD_ChangeDebutAccessDeDir {{Dir ""}} {
    
    #DEPRECATED PROC DO NO USE IT ANYMORE

    Wup "Permet de changer la valeur du separateur pour les differents candidats a la X-hybridation"
    Wup "Dependant des access present dans la banque utilisee"
        
    global DebutAccessDeDir

    #Attention pour differencier DbActin, Unigene, RefSeq et Human_genome
    #Positionne le separateur pour differencier les Access de la banque
    
    #Espionne "ChangeDebutAccessDeDir -- $Dir"

    if {[info exists DebutAccessDeDir] && $DebutAccessDeDir !="" } {
	if {! [OuiOuNon "Change X-Sequences separator ?\nOld one is <<$DebutAccessDeDir>>"]} {return ""} else {set DebutAccessDeDir ""}
    }
    
    if {$Dir != ""} { 
	if {      [regexp -nocase "Unigene"  $Dir]} {
	    set DebutAccessDeDir  "UG:"
	} elseif {[regexp -nocase "RefSeq"   $Dir]} {
	    set DebutAccessDeDir  "RSHUM:"
	} elseif {[regexp -nocase "HsGenome" $Dir]} {
	    set DebutAccessDeDir  "HS"
	} elseif {[regexp -nocase "DbActin"  $Dir]} {
	    set DebutAccessDeDir  "ACT"
	} elseif {[regexp -nocase "RSHUM:"   $Dir]} {
	    set DebutAccessDeDir  "RSHUM:"
	} elseif {[regexp -nocase "UG:"      $Dir]} {
	    set DebutAccessDeDir  "UG:"
	} elseif {[regexp -nocase {[a-z]{2}_} $Dir]} {
	    #RefSeq ameliore pour des access du type NM_, XM_, NG_ ...
	    set DebutAccessDeDir  "\[a-z\]{2}_"
	} else {set DebutAccessDeDir ""}
    }
    if {$DebutAccessDeDir != ""} {return ""} else {set DebutAccessDeDir ""}
    
    set        Texte "Enter a new separator for potential X-sequences\nEx: for RefSeq it is RSHUM:"
    if {$Dir != ""} {
	append Texte "\nYour Acc are: $Dir"
    }
    while {$DebutAccessDeDir==""} {
	if {[OuiOuNon $Texte]} {
	    set DebutAccessDeDir [Entre "RSHUM: or UG: for RefSeq or Unigene"]
	} else {
	    FaireLire "You do not set the Acc Separator, nothing is done"
	    set DebutAccessDeDir ""
	    break
	}
    }
    return ""
}

proc OLD_LesAccessDefDeLOligo {PAB NomOligo Dir} {

    #DEPRECATED PROC DO NO USE IT ANYMORE

    Wup "Rend les Access et les Def des autres candidats de l oligo"
    Wup "Mise a jour ajout de ChangeDebutAccessDeDir"
    Wup "Mise a jour Integration de InterrogeInfoDeOligo"

    global DebutAccessDeDir

    if {! [info exists DebutAccessDeDir]} {set DebutAccessDeDir ""}
    if {$PAB == "" || $NomOligo == "" || $Dir == ""} {return ""}

    set Fichier "$Dir/${PAB}.oligo"
    if {! [file exists $Fichier]} {
	set Fichier "$Dir/$PAB"
	if {! [file exists $Fichier]} {return ""}
    }

    set  LesCandidats [InterrogeInfoDeLOligo $Fichier $NomOligo LesCandidats]
    Espionne "$LesCandidats"

    if {$LesCandidats == ""}                             {return [list "No Hit found during Database searches"]}
    if {  [regexp -nocase "No Hit found" $LesCandidats]} {return [list "$LesCandidats"]}
    if {! [info exists DebutAccessDeDir]}                {                        ChangeDebutAccessDeDir $LesCandidats} 
    if {              $DebutAccessDeDir  == ""}          {                        ChangeDebutAccessDeDir $LesCandidats} 
    if {! [regexp     $DebutAccessDeDir  $LesCandidats]} {set DebutAccessDeDir "";ChangeDebutAccessDeDir $LesCandidats}

    if {$DebutAccessDeDir==""} {return ""}

    set LesAccessDef ""
    set MaLigne      ""
    foreach Candidat [split $LesCandidats " "] {
	if {! [regexp -nocase {[a-z0-9]+} $Candidat]} {continue}
	if {! [info exists DebutAccessDeDir]} {ChangeDebutAccessDeDir $Ligne} 
	if {  [regexp -nocase "$DebutAccessDeDir" $Candidat] || [regexp -nocase "$PAB" $Candidat]} {
	    if {$MaLigne!=""} {lappend LesAccessDef $MaLigne;set MaLigne ""}
	    set MaLigne $Candidat
	    continue
	}
	append MaLigne " $Candidat" 
    }
    #Pour le dernier de la liste sinon pas ajouter a LesAccessDef
    lappend LesAccessDef $MaLigne

    return [lsort -unique $LesAccessDef]
}

proc AfficheLaSequenceDeLOligo {K PAB} {

    Wup "Affiche la sequence tfa de l oligo du PAB voulu"
    Wup "Mise a jour pour InterrogeInfoDeOligo"
        
    set Id       [$K find withtag current]
    set MesTags  [$K gettags $Id]
    set NomOligo [lindex $MesTags 4]
    set Dir      [lindex $MesTags 5]

    set Fichier "[file join $Dir "${PAB}.oligo"]"
    if {! [file exists $Fichier]} {
	set Fichier "[file join $Dir $PAB]"
	if {! [file exists $Fichier]} {return ""}
    }
    set Entete     ">$PAB $NomOligo"
    set Sequence   [InterrogeInfoDeLOligo $Fichier $NomOligo Sequence]
    set TFADeOligo [join [list $Entete $Sequence] "\n"]
    
    return [AfficheVariable $TFADeOligo "AvecFormateAvecBlaste" "$Dir/$PAB"]
}

proc AfficheExtraitOligoDuTFA {K PAB} {

    Wup "Affiche la sequence tfa de l oligo du PAB voulu"
        
    set Id         [$K find withtag current]
    set MesTags    [$K gettags $Id]
    set NomOligo   [lindex $MesTags 4]
    set Dir        [lindex $MesTags 5]
    set TFADeOligo [ExtraitOligoDuTFA $PAB $NomOligo $Dir]

    return [AfficheVariable [join $TFADeOligo "\n"] "AvecFormateAvecBlaste" "$Dir/$PAB"]
}

proc ExtraitOligoDuTFA {PAB Oligo {Dir ""}} {

    Wup "Pour recuperer la sequence tfa de l oligo du PAB voulu"

    set Fichier "[file join $Dir "${PAB}.oligo"]"
    if {! [file exists $Fichier]} {
	set Fichier "[file join $Dir $PAB]"
	if {! [file exists $Fichier]} {return ""}
    }
    set MonTFAOligo ""
    set JeLaTiens   0
    set F [open "$Fichier"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {  [string range $Ligne 0 0] == ">"} {
	    if {!  [regexp -nocase $Oligo $Ligne]} {continue}
	    lappend MonTFAOligo $Ligne
	    set JeLaTiens 1
	    continue
	}
	if {! $JeLaTiens} {continue}
	lappend MonTFAOligo $Ligne
	break
    }
    close $F
    return $MonTFAOligo
}

proc ExtraitOligoDuTFA2 {PAB Oligo {Dir ""} {Fichier ""}} {

    Wup "Pour recuperer la sequence tfa de l oligo du PAB voulu"

    if {$Fichier ==""} {
	set Fichier "[file join $Dir "${PAB}.oligo"]"
    } else {
	set Fichier "[file join $Dir $Fichier]"
    }

    if {! [file exists $Fichier]} {
	set Fichier "[file join $Dir $PAB]"
	if {! [file exists $Fichier]} {return ""}
    }
    set MonTFAOligo ""
    set JeLaTiens   0
    set F [open "$Fichier"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {  [string range $Ligne 0 0] == ">"} {
	    if {!  [regexp -nocase $Oligo $Ligne]} {continue}
	    lappend MonTFAOligo $Ligne
	    set JeLaTiens 1
	    continue
	}
	if {! $JeLaTiens} {continue}
	lappend MonTFAOligo $Ligne
	break
    }
    close $F
    return $MonTFAOligo
}

proc AffichePositionOligoPourTous {K X Y Action} {
    
    Wup "Affiche a droite et a gauche la position de tous les oligos par rapport a la query"

    EffacePositionOligoPourTous $K $X $Y
    
    set         LesId [$K find withtag "Positionner"]
    foreach Id $LesId {
	AffichePositionOligo $K $X $Y $Action $Id
    }
    return ""
}

proc EffacePositionOligoPourTous {K X Y} {
    
    Wup "Affiche a droite et a gauche la position de tous les oligos par rapport a la query"

    global PositionExisteDeja 
    
    set Texte " "
    set         LesIds [$K find withtag "Positionner"] 
    foreach Id $LesIds {
	if {! [info exists PositionExisteDeja($K,$Id)]} {
	    set PositionExisteDeja($K,$Id) 0
	} else {
	    set PositionExisteDeja($K,$Id) 0
	}
    }
    set     Id ""
    foreach Id [$K find withtag "Position"] {$K itemconfigure $Id -text $Texte}
    return ""
}

proc AffichePositionOligo {K X Y Action {Id ""}} {
    
    Wup "Affiche a droite et a gauche la position de l oligo sur la query"

    global PositionExisteDeja 

    if {$Id == ""} {set Id [$K find withtag current]}
    
    set MesTags   [$K gettags $Id]
    set MesCoords [$K coords  $Id]
    set Nom       [lindex $MesTags 4]
    set Banque    [lindex $MesTags 5]

    if {! [info exists PositionExisteDeja($K,$Id)]} {set PositionExisteDeja($K,$Id) 0}  
    if {$Action=="Hide" || [set PositionExisteDeja($K,$Id)]=="1"} { 
	set PositionExisteDeja($K,$Id) 0
	set Texte " "
	set Id    ""
	foreach Id [$K find withtag "Position"] {
	    if {[lindex [$K gettags $Id] 0] != $Nom || [lindex [$K gettags $Id] 1] !=$Banque} {continue}
	    $K itemconfigure $Id -text $Texte
	}
	return ""
    }
    set PositionExisteDeja($K,$Id) 1

    set Xmin [lindex $MesCoords 0]
    set Xmax [lindex $MesCoords 2]
    set Ymin [lindex $MesCoords 1]
    set Ymax [lindex $MesCoords 3]
    $K create text [expr {$Xmin-3}] [expr {(($Ymax-$Ymin)/2)+$Ymin}] \
	    -text [lindex $MesTags 2] \
	    -tags [list $Nom $Banque "Position"] \
	    -anchor "e"

    $K create text [expr {$Xmax+3}] [expr {(($Ymax-$Ymin)/2)+$Ymin}] \
	    -text [lindex $MesTags 3] \
	    -tags [list $Nom $Banque "Position"] \
	    -anchor "w"
    return ""
}

proc OLD_AfficheInfoBulle {K X Y Action} {
    
    Wup "Affiche un cadre contenant les infos sur l oligo"    

    regsub ".canvas" $K ".message" M
    catch {destroy $M}
    
    if {$Action=="Leave"} {return ""}
    
    set Id      [$K find withtag current]
    set MesTags [$K gettags $Id]
    set Texte   [lindex $MesTags 1]

    message $M \
	    -borderwidth "3" \
	    -font {Courier 8} \
	    -relief "raise" \
	    -width "800" \
	    -text $Texte \
	    -background "lightyellow" \
	    -foreground "black"

    if {$X>400} {set EW "e";incr X -20} else {set EW "w";incr X 20}
    if {$Y< 50} {set EW "n$EW" }
    place $M -x $X -y [expr {$Y-30}] -anchor $EW
    return $M
}    

proc AfficheInfoBulle {K X Y Action} {

    Wup "Permet d avoir des InfoBulles pour des Id dans un canvas"    
    Wup "Ces InfosBulles sont independantes du canvas"
    
    catch {destroy .infobulle}

    if {$Action=="Leave"} {return ""}
    
    set Id      [$K find withtag current]
    set MesTags [$K gettags $Id]
    set Texte   [lindex $MesTags 1]

    #Les coordonnees de l Id a InfoBuller
    set MesCoords [$K coords $Id]
    set Xmin      [lindex $MesCoords 0]
    set Xmax      [lindex $MesCoords 2]
    set Ymin      [lindex $MesCoords 1]
    set Ymax      [lindex $MesCoords 3]

    #Cas particulier des lignes verticales on prend comme repere la position de la souris
    if {$Ymin==$Ymax} {set Xmin $X;set Xmax $X}
    #Correction pour ceux avec seulement 2 coordonnees
    if {$Xmax==""} {set Xmax $Xmin}
    if {$Ymax==""} {set Ymax $Ymin}

    #Il nous faut les mensurations de l objet a InfoBuller
    if {$Xmax>$Xmin} {
	set TagLargeur [expr {$Xmax-$Xmin}]
    } else {
	set TagLargeur [expr {$Xmin-$Xmax}]
	set Xtmp $Xmin
	set Xmin $Xmax
	set Xmax $Xtmp
    }
    if {$Ymax>$Ymin} {
	set TagHauteur [expr {$Ymax-$Ymin}]
    } else {
	set TagHauteur [expr {$Ymin-$Ymax}]
	set Ytmp $Ymin
	set Ymin $Ymax
	set Ymax $Ytmp
    }

    #Les positions initiales du canvas contenant Id
    set PosX [winfo rootx $K]
    set PosY [winfo rooty $K]

    #Correction pour le facteur de Zoom
    set Facteurx [ZoomFacteur $K X]
    #set Facteury [ZoomFacteur $K Y]
    set Xmin     [expr {$Xmin/$Facteurx}]
    set Xmax     [expr {$Xmax/$Facteurx}]

    #Creation du InfoBulle independant du canvas
    toplevel .infobulle -background "black"
    wm overrideredirect .infobulle 1
    label .infobulle.label \
            -text       "$Texte" \
	    -relief     "flat" \
	    -font       {Courier 8} \
            -background "lightyellow" \
	    -foreground "black" \
	    -padx       "2" \
	    -pady       "0" \
	    -anchor     "w" \
	    -justify    "left"

    #1 Creation
    pack .infobulle.label -side "left" -padx "1" -pady "1"
    #2 On Oublie
    wm withdraw .infobulle
    update
    if {! [winfo exists .infobulle]} {return ""} 
    #3 Recuperation des mensurations
    set MessageLargeur [winfo width  .infobulle]
    set MessageHauteur [winfo height .infobulle]
    if {$MessageLargeur=="1" && $MessageHauteur=="1"} {
	set MessageLargeur [winfo reqwidth  .infobulle]
	set MessageHauteur [winfo reqheight .infobulle] 
    }
    
    #4 Ajustement des positions de l InfoBulle
    #4 Par rapport a l objet a infobuller
    #4 Par rapport a sa position sur l ecran
    #Sur Beaufort le withdraw empeche d avoir la taille de la fenetre voir mise a jour
    #Mais le reqwidth et le reqheight contourne le pb
    set OnForceADroite 1
    set OnForceEnBas   0
    if {$MessageHauteur==1} {set OnForceADroite 0}
    if {$MessageLargeur==1} {set OnForceEnBas   1}

    set EcranHauteur [winfo screenheight .]
    set EcranLargeur [winfo screenwidth  .]
    if {$OnForceADroite && $X > [expr {$EcranLargeur/2}]} {
	#On affiche a gauche
	incr X -15
	while {$X>$Xmin} {incr X -15}
	set PosX [expr {$PosX - $MessageLargeur +$X}]
    } else {
	#On affiche a droite
	incr X 15
	while {$X<$Xmax} {incr X 15}
	set PosX [expr {$PosX + $X}]
    }
    if {$OnForceEnBas || $Y< 50} {
	#On affiche en bas
	incr Y 10 
	set PosY [expr {$PosY + $Y}]
    } else {
	#On affiche en haut
	incr Y -10 
	set PosY [expr {$PosY - $MessageHauteur +$Y}]
    }

    set PosX [format "%.0f" $PosX]
    set PosY [format "%.0f" $PosY]

    #Ajustement final des positions de l InfoBulle et positionnement
    if {$PosX<0} {
	#On sort de l ecran par la gauche
	set PosX 0
    } elseif {[expr {$PosX + $MessageLargeur}]> $EcranLargeur} {
	#On sort de l ecran par le droite
	set PosX [expr {$EcranLargeur - $MessageLargeur}]
    }
    if {$PosY<0} {
	#On sort de l ecran par le dessus
	set PosY 0
    } elseif {[expr {$PosY + $MessageHauteur}]> $EcranHauteur} {
	#On sort de l ecran par le dessous
	set PosY [expr {$EcranHauteur - $MessageHauteur}]
    }

    wm geometry .infobulle +${PosX}+${PosY}
           bind .infobulle <Leave>   "destroy .infobulle"
           bind .infobulle <1>       "destroy .infobulle"
           bind $K         <Destroy> "destroy .infobulle"
    #5 Apres avoir oublie la fenetre on se souvient
    wm deiconify .infobulle
    return ""
}

#####################################################
#   Fiche info de la sequence et PAB                #
#   Selection des zones de design preferentielles   #
#####################################################
proc LesInfosDeBaseDeMaSequence {SeqQuery {Quoi ""}} {
    
    Wup "Retourne les informations de bases de la sequence composition en ATGC, les Tms ..."

    set Fiche          ""
    set AvecSeq        0
    set AvecSeqFormate 0
    set AvecStatNuc    0
    set AvecTm         0
    set AvecTaille     0
    set LaTotale       0

    if {$Quoi==""|| $Quoi=="LaTotale"} {set LaTotale 1}
    foreach Choix $Quoi {
	switch $Choix {
	    "AvecSeq"        {set AvecSeq        1;set AvecSeqFormate 0}
	    "AvecSeqFormate" {set AvecSeqFormate 1;set AvecSeq        0}
	    "AvecTaille"     {set AvecTaille     1}
	    "AvecStatNuc"    {set AvecStatNuc    1}
	    "AvecTm"         {set AvecTm         1}
	    "default"        {set LaTotale       1}
	}
    }
    set TailleQueryNt [string length $SeqQuery]
    if {$AvecSeq || $LaTotale} {
	set     SeqRes ""
	lappend SeqRes ">Sequence:"
	lappend SeqRes "----------"
	lappend SeqRes "$SeqQuery"
	lappend Fiche  [join $SeqRes "\n"]
    }
    if {$AvecSeqFormate} {
	set     SeqRes ""
	lappend SeqRes ">Sequence:"
	lappend SeqRes "----------"
	set MaSeq $SeqQuery
	while {1} {
	    if {$MaSeq==""} {break}
	    lappend SeqRes [string range $MaSeq  0  79]
	    set      MaSeq [string range $MaSeq 80 end]
	}
	lappend Fiche  [join $SeqRes "\n"]
    }
    if {$AvecTaille || $LaTotale} {
	set     TailleRes ""
	lappend TailleRes ">Length:"
	lappend TailleRes "--------"
	lappend TailleRes "$TailleQueryNt nt(s)"
	lappend Fiche     [join $TailleRes "\n"]
    }
    if {$AvecStatNuc || $LaTotale} {
	set nA        [NombreDeBaseDe $SeqQuery "A"]
	set nT        [NombreDeBaseDe $SeqQuery "T"]
	set nG        [NombreDeBaseDe $SeqQuery "G"]
	set nC        [NombreDeBaseDe $SeqQuery "C"]
	set nN        [NombreDeBaseDe $SeqQuery "N"]
	set nAT       [expr {$nT + $nA}]
	set nGC       [expr {$nG + $nC}]
	set nPu       [expr {$nG + $nA}]
	set nPy       [expr {$nT + $nC}]
	set PercentA  [expr {($nA*1.0/$TailleQueryNt)*100.0}]
	set PercentT  [expr {($nT*1.0/$TailleQueryNt)*100.0}]
	set PercentG  [expr {($nG*1.0/$TailleQueryNt)*100.0}]
	set PercentC  [expr {($nC*1.0/$TailleQueryNt)*100.0}]
	set PercentN  [expr {($nN*1.0/$TailleQueryNt)*100.0}]
	set PercentAT [expr {$PercentA + $PercentT}]
	set PercentGC [expr {$PercentC + $PercentG}]
	set PercentPu [expr {($nPu*1.0/$TailleQueryNt)*100.0}]
	set PercentPy [expr {($nPy*1.0/$TailleQueryNt)*100.0}]
	set     StatNucRes ""
	lappend StatNucRes ">Nucleic Acid Composition:"
	lappend StatNucRes "--------------------------"
	lappend StatNucRes [format " %5s %8s %4s" "Bases" "Number" "%"]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" " A " $nA  $PercentA]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" " T " $nT  $PercentT]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" " G " $nG  $PercentG]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" " C " $nC  $PercentC]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" " N " $nN  $PercentN]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" "A+T" $nAT $PercentAT]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" "G+C" $nGC $PercentGC]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" "Pur" $nPu $PercentPu]
	lappend StatNucRes [format "  %3s  %6d    %5.1f" "Pyr" $nPy $PercentPy]
	lappend Fiche      [join $StatNucRes "\n"]
    }
    if {$AvecTm || $LaTotale} {
	set TmWallace [CalculTmDeLaSeq $SeqQuery Wallace "" 0.0]
	set TmDNADNA  [CalculTmDeLaSeq $SeqQuery DNADNA  "" 0.0]
	set TmDNARNA  [CalculTmDeLaSeq $SeqQuery DNARNA  "" 0.0]
	set TmRNARNA  [CalculTmDeLaSeq $SeqQuery RNARNA  "" 0.0]
	set TmNN      [CalculTmDeLaSeq $SeqQuery NearestNeighbor "" 0.0]
	if {$TmNN=="-1"} {set TmNN "0.0"}

	set     TmRes ""
	lappend TmRes ">Melting Temperature:"
	lappend TmRes "---------------------"
	lappend TmRes [format "%8s %9s %5s\n" "Method" "" "Result"]
	lappend TmRes [format "%-16s %-5.0f \C %-s" "Wallace"     $TmWallace "Tm = 4*(G+C)+2*(A+T)"]
	lappend TmRes [format "%-16s %-5.1f \C %-s" "DNA-DNA"      $TmDNADNA "Tm = 81.5+16.6*log M+0.41(%GC)-500/L-0.62F"]
	lappend TmRes [format "%-16s %-5.1f \C %-s" "DNA-RNA"      $TmDNARNA "Tm = 79.8+18.5*log M+0.584(%GC)+0.11(%GC)^2-820/L-0.35F"]
	lappend TmRes [format "%-16s %-5.1f \C %-s" "RNA-RNA"      $TmRNARNA "Tm = 79.8+18.5*log M+0.584(%GC)+0.11(%GC)^2-820/L-0.50F"]
	lappend TmRes [format "%-16s %-5.1f \C %-s" "Nearest Neighbor" $TmNN "Tm = 1000*DH/(DS+R*ln(\[ADN\]/4)-273.15)"]
	lappend Fiche [join $TmRes "\n"]
    }
    return [join $Fiche "\n\n"]
}

proc AfficheLesInfosDeMaSequence {PAB} {
    
    Wup "Montre les regions interessantes"

    set Indice [file tail $PAB]
    regsub -all  {\.} $Indice "" Indice
    
    set w ".lesinfosdemaseq$Indice"
    if {[winfo exists $w]} {wm deiconify $w;raise $w;return ""} else {toplevel $w}
    
    wm geometry  $w +300+100
    wm title     $w "Information for $PAB"
    wm resizable $w false false

    set    FInfo "$w.frameinfo"
    frame $FInfo -borderwidth "3" -relief "groove"
    pack  $FInfo -fill both       -expand true
    
    #Attention il faut une query, sinon pas de taille ....
    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set Repnuctfa "[file join [RepertoireDuGenome] nuctfa]"
        
    set FichierQuery $PAB
    if {[regexp {/} $PAB]} {set PAB [file tail $PAB]}

    #ATTENTION MODIF
    regsub {.[a-zA-Z]+$} $PAB "" PAB

    if {    [file exists $FichierQuery]} {
	set FichierQuery $FichierQuery
    } elseif {[file exists "[file join $RepmRNA $PAB]"]} {
	set FichierQuery   "[file join $RepmRNA $PAB]"
    } elseif {[file exists "[file join $Repnuctfa $PAB]"]} {
	set FichierQuery   "[file join $Repnuctfa $PAB]"
    } else {AjouteAuLog "No Input File for the Query ($PAB)";destroy $w;return ""}

    set Police         {-family "arial" -size 12 -weight bold}
    set CouleurOutline "black"
    set LargeurMax     [expr {[winfo screenwidth  .] * 2/3}]
    set HauteurMax     [expr {[winfo screenheight .] * 1/2}]
    set LargeurFenetre [expr {[winfo screenwidth  .] * 2/3}]
    set HauteurFenetre [expr {[winfo screenheight .] * 1/2}]
    set XMilieu        [expr {$LargeurMax/2}]
    
    set BordureX       50.0
    set BordureY       50.0
    set Intervalle     10.0
    set EpaisseurBoite 15.0

    #Position du Graph content
    set YDebutGraph    [expr {0 + $BordureY}]
    set YFinGraph      [expr {$YDebutGraph + 100}]

    #Position de la query
    set XDebutQuery    [expr {0            + $BordureX}]
    set YDebutQuery    [expr {$YFinGraph   + 10}]
    set XFinQuery      [expr {$LargeurMax  - $BordureX}] 
    set YFinQuery      [expr {$YDebutQuery + $EpaisseurBoite}]
    
    #Besoin de la taille de la query pour conversion et affichage
    set AccessQuery    [AccessDuTFADumRNA         $FichierQuery]
    set  SeqQuery      [QueLaSequenceDuFichierTFA $FichierQuery]
    if {$SeqQuery==""} {Espionne "$FichierQuery no sequence available";return ""}
    set TailleQueryNt  [string length $SeqQuery]
    #Rapport du nombre de pixel pour un nt
    set OneNtPixel     [expr {($XFinQuery-$XDebutQuery)/($TailleQueryNt - 1)}] 

    #Debut des Oligos
    set YDebutDeOligo  [expr {$YFinQuery + $Intervalle}]
        
    set     K "$FInfo.canvas"
    canvas $K    -width      $LargeurFenetre -height $HauteurFenetre 
    $K configure -height     $YDebutDeOligo
    $K configure -background "grey" 

    SauveLesInfosDe $K MonPAB $PAB
    #On modifie le comportement lors de la destruction de la fenetre principale
    #Ceci pour vider les infos stockees
    wm protocol $w WM_DELETE_WINDOW "DetruitLesInfosDe $K;after 10 destroy $w"

    #Le Masque pour les graphes
    $K create rectangle [expr {$XDebutQuery-40}] [expr {$YDebutGraph-4}] [expr {$XFinQuery+40}] [expr {$YFinGraph+4}] \
	    -fill    "gray97" \
	    -outline "gray97" \
	    -tags    [list "" "Masque"]

    #Affiche la petite aide
    $K create oval [expr {$XDebutQuery-30}] [expr {$YDebutGraph-30}] [expr {$XDebutQuery-10}] [expr {$YDebutGraph-10}] \
	    -fill    "green" \
	    -outline "white"
    $K create text [expr {$XDebutQuery-20}] [expr {$YDebutGraph-20}] \
	    -text    "?" \
	    -tags    [list "" "[AfficheHelpInfoDeMaSequence]" "Affichable"] \
	    -anchor  "center" 

    #Affichage du Tm content
    #On recupere la methode de Tm par defaut choisie et on la remet apres
    set TmDefault [LesParametresDuDesign TmMethod "LaValeurMerci"]
    LesParametresDuDesign TmMethod "NearestNeighbor" "Positionne"

    set  TmResults [PosEtTmDeLaRegion $SeqQuery $TailleQueryNt] 
    if {$TmResults !=""} {
	AfficheLeSpectreTmDUneListe $K $TmResults \
		$XDebutQuery $XFinQuery \
		$YDebutGraph $YFinGraph $OneNtPixel "orange" "TmNN"
    }
    LesParametresDuDesign TmMethod $TmDefault "Positionne"

    #Affichage du GC content
    set  GCResults [PosEtGCDeLaRegion $SeqQuery $TailleQueryNt]
    if {$GCResults !=""} {
	AfficheLeSpectreGCDUneListe $K $GCResults \
		$XDebutQuery $XFinQuery \
		$YDebutGraph $YFinGraph $OneNtPixel
    }

    #Affiche la query
    $K create rectangle $XDebutQuery $YDebutQuery $XFinQuery $YFinQuery \
	    -outline "$CouleurOutline" \
	    -fill    "red" \
	    -tags    "Query"
    $K create text [expr {$XDebutQuery-5.0}] [expr {$YDebutQuery + ($EpaisseurBoite/2.0)}] \
	    -text    "1" \
	    -font    "$Police" \
	    -anchor  "e"
    $K create text [expr {$XFinQuery  +5.0}] [expr {$YDebutQuery + ($EpaisseurBoite/2.0)}] \
	    -text    "$TailleQueryNt" \
	    -font    "$Police" \
	    -anchor  "w"
    $K create text $XMilieu [expr {1*$BordureY/3}] \
	    -text    "Sequence Informations from $PAB" \
	    -font    "$Police" \
	    -anchor  "center" \
	    -justify "center"

    #La fiche contenant les infos
    set  FicheInfo  ""
    set  Text       "$FInfo.text"
    set  ListBoxSBH "$FInfo.xscroll"
    set  ListBoxSBV "$FInfo.yscroll"

    scrollbar $ListBoxSBH -orient "horizontal" -command "$Text xview"
    scrollbar $ListBoxSBV -orient   "vertical" -command "$Text yview"
    text $Text \
	    -state          "disabled" \
	    -wrap           "word" \
	    -font           "courier 10" \
	    -xscrollcommand "$ListBoxSBH set" \
	    -yscrollcommand "$ListBoxSBV set"
    
    set FicheInfo [LesInfosDeBaseDeMaSequence $SeqQuery]
    foreach element [split $FicheInfo "\n"] {
	$Text configure -state normal
	$Text insert    "end"  "$element\n"
	$Text configure -state disabled
    }

    #Les boutons
    set ButtonFrame    "$FInfo.buttonframe"
    set  ActionButton  "$ButtonFrame.button1"
    set  DestroyButton "$ButtonFrame.button2"
    set  LCharpente1   "$ButtonFrame.lcharpente1" 
    set  LCharpente2   "$ButtonFrame.lcharpente2" 
    set  ZoneButton1   "$ButtonFrame.zbutton1"
    set  ZoneButton2   "$ButtonFrame.zbutton2"
    #set  ZoneButton3   "$ButtonFrame.zbutton3"

    frame  $ButtonFrame  -borderwidth "3"      -relief "groove"
    button $ActionButton -background  "yellow" -text "Graph Panel / Add / Delete" 
      bind $ActionButton  <1> "Graph_Panel     $K"
      bind $ActionButton  <2> "AjouteUnGraph   $K $FichierQuery $XDebutQuery $XFinQuery $YDebutGraph $YFinGraph $OneNtPixel"
      bind $ActionButton  <3> "SupprimeUnGraph $K"
    set    Texte [format     "%-10s" "Panel to change graph display (order,color)"]
    append Texte [format   "\n%-10s" "Allow to add or delete graph"]
    append Texte [format   "\n%-10s" "Allow to search items in oligo display text"]
    set_balloon $ActionButton "$Texte"

    button $DestroyButton -background "red"    -text "Close / Help"  -command "eval \[wm protocol $w WM_DELETE_WINDOW\]"
      bind $DestroyButton <2> "AfficheHelpInfoDeMaSequence DuBouton"

    button $ZoneButton1  -background "green"   -text "Area(s) Draw / Clear / Save" 
      bind $ZoneButton1   <1> "MontreLesZones $K $XDebutQuery $XFinQuery $YDebutGraph $OneNtPixel"
      bind $ZoneButton1   <2> "ClearLesZones  $K"
      bind $ZoneButton1   <3> "SauveLesZones  $K"
    set Texte   [format      "%-10s" "Allow to show (defined), clear or save designing area(s)"]
    set_balloon $ZoneButton1 "$Texte"

    button $ZoneButton2  -background "green"   -text "Reset for $PAB / all" 
      bind $ZoneButton2   <1> "ResetLesZones    $K"
      bind $ZoneButton2   <2> "ResetAllLesZones $K"
    set Texte   [format      "%-10s" "Allow to clear saved design areas"]
    set_balloon $ZoneButton2 "$Texte"

    #button $ZoneButton3  -background "yellow"  -text "Zoom In / Out / Reset"
    #  bind $ZoneButton3  <1> "zoom $K 1.25 1.25"
    #  bind $ZoneButton3  <2> "zoom $K 0.8 0.8"
    #  bind $ZoneButton3  <3> "ResetZoom $K"

    label  $LCharpente1 -background "#808FFF" -relief "groove" 
    label  $LCharpente2 -background "#808FFF" -relief "groove" 

    pack $K             -fill both -side top
    pack $ButtonFrame   -fill both -side top    -expand true
    pack $LCharpente1   -fill both -side left   -expand true
    pack $DestroyButton -fill y    -side left
    #pack $ZoneButton3   -fill y    -side left
    pack $ActionButton  -fill y    -side left
    pack $ZoneButton1   -fill y    -side left
    pack $ZoneButton2   -fill y    -side left
    pack $LCharpente2   -fill both -side left   -expand true

    pack $ListBoxSBV    -fill y    -side right 
    pack $Text          -fill both              -expand true
    pack $ListBoxSBH    -fill x    -side bottom 

    $K bind "Affichable" <Enter> "AfficheInfoBulle $K %x %y Enter"
    $K bind "Affichable" <Leave> "AfficheInfoBulle $K %x %y Leave"
			
    #$K bind "Query"      <2>     "AfficheFetch   $AccessQuery OnVeutNature"
    $K bind "Query"      <3>     "AfficheLaQuery $K $FichierQuery"

    bind $K <Control-1>       "PositionneLesZones $K %x %y $XDebutQuery $XFinQuery $YDebutGraph $OneNtPixel"
    bind $K <Shift-B1-Motion> "ViseurDeSeq        $K %x %y $XDebutQuery $XFinQuery $YDebutGraph $YFinQuery $OneNtPixel"
    bind $K <Shift-1>         "ViseurDeSeq        $K %x %y $XDebutQuery $XFinQuery $YDebutGraph $YFinQuery $OneNtPixel"
    return ""
}

proc AfficheHelpInfoDeMaSequence {{Quoi ""}} {

    Wup "Permet de donner une petite aide pour les commandes disponibles"

    set Texte ""
    append Texte "Designing Help:\n"
    append Texte "---------------\n"
    append Texte "You are able to display, select and save designing area(s).\n"
    append Texte "   Shift  -1             Show position on the query\n"
    append Texte "   Control-1             Put one Flag for area selection\n\n"
    append Texte "   Mouse 1(Green button) Show previously areas (in the Panel)\n"
    append Texte "   Mouse 2(Green button) Save areas (!!It overwrite previous!!)\n"
    append Texte "   Mouse 3(Green button) Clear areas\n"

    append Texte "Graph Panel:\n"
    append Texte "------------\n"
    append Texte "   Panel that allows you display or hide,\n"
    append Texte "   \tGraph itself\n"
    append Texte "   \tAverage\n"
    append Texte "   \tAll or none\n\n"

    append Texte "GC Content:\n"
    append Texte "-----------\n"
    append Texte "   Upper  Value: 100% GC bases\n"
    append Texte "   Middle Value: Average composition in GC bases\n"
    append Texte "   Lower  Value: 100% AT bases\n"
    append Texte "   Enter  Display the %GC of the current Position\n\n"

    append Texte "Tm Content:\n"
    append Texte "-----------\n"
    append Texte "   Upper  Value: maximum Tm value\n"
    append Texte "   Middle Value: Average Tm value\n"
    append Texte "   Lower  Value: minimum Tm value\n"
    append Texte "   Enter  Display the Tm of the current Position\n\n"
		  
    append Texte "Query:\n"
    append Texte "------\n"
    append Texte "   Mouse 1 Basic information with query sequence\n"
    append Texte "   Mouse 3 Display Query s Sequence in fasta format"
		  
    if {$Quoi=="DuBouton"} {
	after idle {.fairelire.msg configure -wraplength 10i -background red -font "Courier 12"}
	tk_dialog .fairelire "Please press OK" "$Texte" {} 0 Acknowledge
	catch {destroy .fairelire}
	return ""
    } else {return $Texte}
}

proc MontreLesZones {K XDebut XFin YDebut {OneNtPixel ""}} {
   
    Wup "Affiche les zones de design deja definies dans le Panel"
    Wup "Attention le tableau est indexee par la position en nt"
    
    global LigneZone 

    set Police    {-family "comic" -size 10 -weight bold}

    set PAB [LesInfosDe $K MonPAB]
    set  LesZones [LesParametresDuDesign Areas $PAB "LaValeurMerci"]
    if {$LesZones==""} {
	ClearLesZones $K
	return ""
    }

    set  LesZones [lsort -increasing -index 0 $LesZones]
    set IdQ       [$K find withtag "Query"]
    set YQFin     [lindex [$K coords $IdQ] 3]
    
    foreach CoupleDeZone $LesZones  {
	foreach Zone $CoupleDeZone {
	    set x [expr {$XDebut + $Zone * $OneNtPixel}]
	    if {[info exists LigneZone($K,$Zone)]} {
		catch {$K delete [set LigneZone($K,$Zone)]}
		catch {$K delete [set LigneZone($K,$Zone,zone)]}
		
		unset LigneZone($K,$Zone)
		if {[info exists LigneZone($K,$Zone,zone)]} {unset LigneZone($K,$Zone,zone)}
	    }
	    if {$x>$XFin} {
		set x    $XFin 
		set Zone [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) + 1}]
	    }    
	    set Numero ""
	    set LigneZone($K,$Zone,zone) [$K create text $x [expr {$YDebut-11}] \
		    -font   "$Police" \
		    -text   "$Numero" \
		    -fill   "red" \
		    -anchor "center" \
		    -tags   [list "" "Zonable" "Flag"]] 
	    set LigneZone($K,$Zone)      [$K create line $x $YDebut $x $YQFin \
		    -fill "#808FFF" \
		    -tags [list "" "[format "%.0f" $Zone]" "Affichable" "Zonable"]]
	    
	    CheckLesZones $K
	}
    }
    return ""
}

proc SauveLesZones {K} {

    Wup "Sauvegarde les zones de design"

    set LesZones ""
    set LesIds   [$K find withtag {"Zonable" && "Affichable"}]
    foreach Id $LesIds {
	lappend LesZones [lindex [$K gettags $Id] 1]  
    }
    
    set LesZones [lsort -integer -increasing $LesZones]
    set NbZones  [llength $LesZones]
    #Si impair
    if {[expr {$NbZones%2}]} {
	FaireLire "You need pairs of Flags"
    } else {
	set PAB [LesInfosDe $K MonPAB]
	LesParametresDuDesign Areas $PAB "Reset"
	foreach {Debut Fin} $LesZones {
	    LesParametresDuDesign Areas $PAB [list $Debut $Fin] "Ajoute"
	    AjouteAuLog "Design from $Debut to $Fin for $PAB"
	}
    }
    return ""
}

proc ResetLesZones {K} {
    
    Wup "Unset les zones de design"

    set PAB [LesInfosDe $K MonPAB]

    LesParametresDuDesign Areas $PAB "Reset"
    AjouteAuLog "Own specific designing areas reset to null for $PAB."
    return ""    
}

proc ResetAllLesZones {K} {
    
    Wup "Unset toutes les zones de design"

    LesParametresDuDesign Areas all "Reset"
    AjouteAuLog "Own specific designing areas reset to null for all."
    return ""    
}

proc ClearLesZones {K} {
    
    Wup "Efface les zones de design"

    global LigneZone

    set         LesIds ""
    set         LesIds [$K find withtag "Zonable"]
    foreach Id $LesIds {
	catch {$K delete $Id} 
    }
    if {[info exists LigneZone]} {unset LigneZone}
    return ""    
}

proc PositionneLesZones {K X Y XDebut XFin YDebut {OneNtPixel ""}} {

    Wup "Positionne une zone de design"
    Wup "Attention le tableau est indexee par la position en nt"

    global LigneZone 

    set Police {-family "Comic" -size 10 -weight bold}

    #Les positions sur le canvas
    set x [$K canvasx $X]
    set y [$K canvasy $Y]
    if {$x>$XFin || $x<$XDebut} {return ""}

    set IdQ   [$K find withtag "Query"]
    set YQFin [lindex [$K coords $IdQ] 3]
    set X     [format "%0.f" [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) + 1}]]

    set Numero ""
    if {[info exists LigneZone($K,$X)]} {
	catch {$K delete [set LigneZone($K,$X)]}
	catch {$K delete [set LigneZone($K,$X,zone)]}
	AfficheInfoBulle $K %x %y Leave
	unset LigneZone($K,$X)
	if {[info exists LigneZone($K,$X,zone)]} {unset LigneZone($K,$X,zone)}
	CheckLesZones $K
	return ""
    }

    set LigneZone($K,$X,zone) [$K create text $x [expr {$YDebut-11}] \
	    -font "$Police" \
	    -text "$Numero" \
	    -fill "yellow" \
	    -anchor center \
	    -tags [list "" "Zonable" "Flag"]] 
    set LigneZone($K,$X)      [$K create line $x $YDebut $x $YQFin \
	    -fill "green3" \
	    -tags [list "" "$X" "Affichable" "Zonable"]]

    CheckLesZones $K
    return ""
}

proc CheckLesZones {K} {

    Wup "Permet de mettre les numeros de zones dans un ordre convenable"
    
    set ListeIdLigne ""
    set         LesIds [$K find withtag {"Zonable" && "Flag"}]
    foreach Id $LesIds {
	lappend ListeIdLigne [list $Id [lindex [$K coords $Id] 0]]
    }
    set nbLigne      [llength $ListeIdLigne]
    set ListeIdLigne [lsort -increasing -real -index 1 $ListeIdLigne]

    set i 0
    foreach Ligne $ListeIdLigne {
	incr i
	if {$i >$nbLigne} {break}
	$K itemconfigure [lindex $Ligne 0] -text "$i"
    }
    return ""
}

proc ViseurDeSeq {K X Y XDebut XFin YDebut {YFin ""} {OneNtPixel ""}} {
 
    Wup "Un viseur avec la position en nt sur la query"
    Wup "Domaine de validite sur X (XDebut XFin) et valeur sur Y (YDebut YFin)"
    Wup "Modif pour tenir compte du zoom"

    global LigneX OldX

    if {$YFin==""} {set YFin [$YDebut + 1000]}
    set Police {-family "Comic" -size 10 -weight bold}

    if {[info exists LigneX($K)]} {
	catch {$K delete [set LigneX($K)]}
	catch {$K delete [set LigneX($K,zone)]}
	unset LigneX($K)
	if {[info exists LigneX($K,$X,zone)]} {unset LigneX($K,$X,zone)}
	if {$X == [set OldX($K)] } {return ""}
    }
    set OldX($K) $X 
 
    #Pour tenir compte du facteur de zoom
    set  TailleQueryNt [LesInfosDe $K MaTailleDeQuery]
    if {$TailleQueryNt!=""} {
	set Facteurx   [ZoomFacteur $K X]
	set Facteury   [ZoomFacteur $K Y]
	set OneNtPixel [expr {$OneNtPixel * $Facteurx}]
	if {$Facteurx!="1.0"} {
	    foreach F [list XDebut XFin] {
		set $F [expr [set $F]*$Facteurx]
	    }
	}
	if {$Facteury!="1.0"} {
	    foreach F [list YDebut YFin] {
		set $F [expr [set $F]*$Facteury]
	    }
	}
    }
    
    set x [$K canvasx $X]
    if {$x>$XFin || $x<$XDebut} {return ""}
    
    set Nt [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) + 1}]

    set LigneX($K)      [$K create line $x $YDebut $x $YFin] 
    set LigneX($K,zone) [$K create text $x [expr {$YDebut-11}] \
	    -font   "$Police" \
	    -text   "[format "%.0f" $Nt]" \
	    -fill   "yellow" \
	    -anchor "center"] 
    return ""
}

###############################################
#          Les Profils sur la sequence        #
#                                             #
#   Calcul du Tm Content          et graphe   #
#   Calcul du GC Content          et graphe   #
#   Calcul du Id Content du blast et graphe   #
#   Calcul du Nb Seq     du blast et graphe   #
###############################################
proc AfficheLeSpectreGCDUneListe {K UneListe XDebutQuery XFinQuery MinSpectreY MaxSpectreY {OneNtPixel ""} {ColorLine "black"} {TagDuGraph "GC"}} {
    global RepertoireDuGenome

    Wup "Affiche le spectre GC pour IllustreOligo"

    if {$OneNtPixel==""} {set OneNtPixel 1}

    set HauteurDuSpectreY [expr {$MaxSpectreY-$MinSpectreY}]
     
    set n 0
    set PremiereLigne 1
    foreach Ligne $UneListe {
	if {$PremiereLigne} {
	    scan $Ligne "%d %d" xMin yMin
	    #Mise a l echelle plus debut de la query
	    #set xMin [expr $XDebutQuery + [expr $xMin * $OneNtPixel]]
	    set VraiyMin [expr {$yMin*1.0}]
	    set yMin 0

	    set PremiereLigne 0
	    set DeuxiemeLigne 1
	    continue
	}
	if {$DeuxiemeLigne} {
	    scan $Ligne "%d %d" xMax yMax
	    #set xMax [expr $XFinQuery   + [expr $xMax * $OneNtPixel]]
	    #set xMax [expr $XDebutQuery + [expr $xMax * $OneNtPixel]]
	    set VraiyMax [expr {$yMax*1.0}]
	    set yMax 100	    
	    
	    set DeuxiemeLigne 0
	    set TroisiemeLigne 1
	    continue
	}
	if {$TroisiemeLigne} {
	    scan $Ligne "%f %f" x yMoyen
	    #set xMin   [expr $x * $OneNtPixel]
	    
	    set TroisiemeLigne 0
	    $K create text [expr {$XDebutQuery-5}] [expr {$MaxSpectreY-($HauteurDuSpectreY/4)+(($HauteurDuSpectreY/4)/2)}] \
		-text "% GC" \
		-tags [list "" "Scale from $yMin to $yMax\nMin $VraiyMin Max $VraiyMax" "Affichable" "OligoGraph" "$TagDuGraph"] \
		-fill $ColorLine -anchor e
	    
	    #Ligne 0% de GC
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMin-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMin-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "OligoGraph" "$TagDuGraph"] \
		-fill $ColorLine
	    
	    #Ligne Moyenne
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMoyen-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMoyen-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "Mean GC [format "%4.1f" $yMoyen]%" "Affichable" "OligoGraph" "$TagDuGraph" "AverageGC"] \
		-fill $ColorLine \
		-dash {6 4}
	    
	    #Ligne 100% de GC
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMax-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMax-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "OligoGraph" "$TagDuGraph"] \
		-fill $ColorLine
	    continue
	}
	scan $Ligne "%f %f" x y
	
	set X [expr {$XDebutQuery + ($x * $OneNtPixel)}]
	set Y [expr {$MaxSpectreY-((($y-$yMin)*($HauteurDuSpectreY))/($yMax-$yMin))}]
	
	incr n
	if { $n > 1 } { 
	    $K create line [set Coordonnees(LastX)] [set Coordonnees(LastY)] $X $Y \
		-tags [list "" "[format "%.1f" [expr {([set Coordonnees(VraiY)]+$y)/2}]] % GC" "Affichable" "OligoGraph" "$TagDuGraph"] \
		-fill $ColorLine
	    set n 1
	}
	set Coordonnees(VraiY) $y
	set Coordonnees(LastX) $X
	set Coordonnees(LastY) $Y
	lappend LesPoints "$X $Y $y"
    }
 
    foreach Coord $LesPoints {
	set X  [lindex $Coord 0]
	set Y  [lindex $Coord 1]
	set GC [lindex $Coord 2]
	$K create line $X $Y $X $Y \
		-fill "$ColorLine" \
		-tags [list "" "[format "%.1f" $GC] % GC" "Affichable" "OligoGraph" "$TagDuGraph"] 
    }

    if {[info exists Coordonnees]} {array unset Coordonnees}
    return ""
}

proc AfficheLeSpectreIdDUneListe {K UneListe XDebutQuery XFinQuery MinSpectreY MaxSpectreY {OneNtPixel ""} {ColorLine "black"} {TagDuGraph ""}} {
    global RepertoireDuGenome

    Wup "Affiche le spectre du Nb de sequence pour IllustreOligo"

    if {$OneNtPixel==""} {set OneNtPixel 1}

    set HauteurDuSpectreY [expr {$MaxSpectreY-$MinSpectreY}]

    set n 0
    set PremiereLigne 1
    foreach Ligne $UneListe {
	if {$PremiereLigne} {
	    scan $Ligne "%d %f" yMin VraiyMin
	    set PremiereLigne 0
	    set DeuxiemeLigne 1
	    continue
	}
	if {$DeuxiemeLigne} {
	    scan $Ligne "%d %f" yMax VraiyMax
	    set DeuxiemeLigne 0
	    set TroisiemeLigne 1
	    continue
	}
	if {$TroisiemeLigne} {
	    scan $Ligne "%d %f" yMoyen VraiyMoyen
	    	    	    
	    set TroisiemeLigne 0

	    $K create text [expr {$XDebutQuery-5}] [expr {$MaxSpectreY-(2*$HauteurDuSpectreY/4)+(($HauteurDuSpectreY/4)/2)}] \
		-text "% Id" \
		-tags [list "" "Mean $TagDuGraph\nScale from $yMin to $yMax\nMin $VraiyMin Max $VraiyMax" "Affichable" "OligoGraph" "$TagDuGraph"] \
		-fill $ColorLine -anchor e
        
	    #Ligne min de nb seq 0
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMin-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMin-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "OligoGraph" "$TagDuGraph"]
	    
	    #Ligne Moyenne
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMoyen-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMoyen-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "Mean Id [format "%4.1f" $yMoyen]%" "Affichable" "OligoGraph" "$TagDuGraph" "Average$TagDuGraph"] \
		-fill "$ColorLine" \
		-dash {6 4}

	    #Ligne de max nbseq
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMax-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMax-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "OligoGraph" "$TagDuGraph"]
	    continue
	}
	scan $Ligne "%f %f" x y
	
	set X [expr {$XDebutQuery + ($x * $OneNtPixel)}]
	set Y [expr {$MaxSpectreY-((($y-$yMin)*($HauteurDuSpectreY))/($yMax-$yMin))}]
	
	incr n
	if {$n > 1} { 
	    $K create line [set Coordonnees(LastX)] [set Coordonnees(LastY)] $X $Y \
		-tags [list "" "[format "%.1f" [expr {([set Coordonnees(VraiY)]+$y)/2}]] % Id" "Affichable" "OligoGraph" "$TagDuGraph"] \
		-fill "$ColorLine"
	    set n 1
	}
	set Coordonnees(VraiY) $y
	set Coordonnees(LastX) $X
	set Coordonnees(LastY) $Y
	lappend LesPoints "$X $Y $y"
    }
 
    foreach Coord $LesPoints {
	set X  [lindex $Coord 0]
	set Y  [lindex $Coord 1]
	set Id [lindex $Coord 2]
	$K create line $X $Y $X $Y \
	    -fill "$ColorLine" \
	    -tags [list "" "[format "%.1f" $Id] % Id" "Affichable" "OligoGraph" "$TagDuGraph"] 
    }
    if {[info exists Coordonnees]} {array unset Coordonnees}
    return ""
}

proc AfficheLeSpectreNbSequenceDUneListe {K UneListe XDebutQuery XFinQuery MinSpectreY MaxSpectreY {OneNtPixel ""} {ColorLine "black"} {TagDuGraph ""}} {
    global RepertoireDuGenome

    Wup "Affiche le spectre du Nb de sequence pour IllustreOligo"

    if {$OneNtPixel==""} {set OneNtPixel 1}

    set HauteurDuSpectreY [expr {$MaxSpectreY-$MinSpectreY}]

    set n 0
    set PremiereLigne 1
    foreach Ligne $UneListe {
	if {$PremiereLigne} {
	    scan $Ligne "%d %f" yMin VraiyMin
	    set PremiereLigne 0
	    set DeuxiemeLigne 1
	    continue
	}
	if {$DeuxiemeLigne} {
	    scan $Ligne "%d %f" yMax VraiyMax
	    set DeuxiemeLigne 0
	    set TroisiemeLigne 1
	    continue
	}
	if {$TroisiemeLigne} {
	    scan $Ligne "%d %f" yMoyen VraiyMoyen
	    	    	    
	    set TroisiemeLigne 0
	    $K create text [expr {$XDebutQuery-5}] [expr {$MaxSpectreY-(4*$HauteurDuSpectreY/4)+(($HauteurDuSpectreY/4)/2)}] \
		-text "nSeq" \
		-tags [list "" "$TagDuGraph\nScale from $yMin to $yMax\nMin $VraiyMin Max $VraiyMax" "Affichable" "OligoGraph" "$TagDuGraph"] \
		-fill $ColorLine -anchor e

	    #Ligne min de nb seq 0
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMin-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMin-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "OligoGraph" "$TagDuGraph"]
	    
	    #Ligne Moyenne
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMoyen-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMoyen-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "Mean NbSeq [format "%3.1f" $yMoyen]" "Affichable" "OligoGraph" "$TagDuGraph" "Average$TagDuGraph"] \
		-fill "$ColorLine" \
		-dash {6 4}
	    
	    #Ligne de max nbseq
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMax-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMax-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "OligoGraph" "$TagDuGraph"]
	    continue
	}
	scan $Ligne "%d %d" x y
	
	set X [expr {$XDebutQuery + ($x * $OneNtPixel)}]
	set Y [expr {$MaxSpectreY-((($y-$yMin)*($HauteurDuSpectreY))/($yMax-$yMin))}]
	
	incr n
	if {$n > 1} { 
	    $K create line [set Coordonnees(LastX)] [set Coordonnees(LastY)] $X $Y \
		-tags [list "" "[expr {([set Coordonnees(VraiY)]+$y)/2}] seq(s)" "Affichable" "OligoGraph" "$TagDuGraph"] \
		-fill "$ColorLine"
	    set n 1
	}
	set Coordonnees(VraiY) $y
	set Coordonnees(LastX) $X
	set Coordonnees(LastY) $Y
	lappend LesPoints "$X $Y $y"
    }
 
    foreach Coord $LesPoints {
	set X    [lindex $Coord 0]
	set Y    [lindex $Coord 1]
	set NbSq [lindex $Coord 2]
	$K create line $X $Y $X $Y \
		-fill "$ColorLine" \
		-tags [list "" "$NbSq seq(s)" "Affichable" "OligoGraph" "$TagDuGraph"] 
    }
    if {[info exists Coordonnees]} {array unset Coordonnees}
    return ""
}

proc AfficheLeSpectreTmDUneListe {K UneListe XDebutQuery XFinQuery MinSpectreY MaxSpectreY {OneNtPixel ""} {ColorLine "black"} {TagDuGraph ""}} {
    global RepertoireDuGenome

    Wup "Affiche le spectre du Tm pour IllustreOligo"

    if {$OneNtPixel==""} {set OneNtPixel 1}

    set HauteurDuSpectreY [expr {$MaxSpectreY-$MinSpectreY}]

    set n 0
    set PremiereLigne 1
    foreach Ligne $UneListe {
	if {$PremiereLigne} {
	    scan $Ligne "%f %f" yMin VraiyMin
	    set PremiereLigne 0
	    set DeuxiemeLigne 1
	    continue
	}
	if {$DeuxiemeLigne} {
	    scan $Ligne "%f %f" yMax VraiyMax
	    set DeuxiemeLigne 0
	    set TroisiemeLigne 1
	    continue
	}
	if {$TroisiemeLigne} {
	    scan $Ligne "%f %f" yMoyen VraiyMoyen
	    	    	    
	    set TroisiemeLigne 0
	    $K create text [expr {$XDebutQuery-5}] [expr {$MaxSpectreY-(3*$HauteurDuSpectreY/4)+(($HauteurDuSpectreY/4)/2)}] \
		-text "Tm" \
		-tags [list "" "$TagDuGraph\nMin $VraiyMin Max $VraiyMax " "Affichable" "OligoGraph" "$TagDuGraph"] \
		-fill $ColorLine -anchor e
	    
	    #Ligne min de nb seq 0
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMin-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMin-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "OligoGraph" "$TagDuGraph"]
	    
	    #Ligne Moyenne
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMoyen-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMoyen-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "Mean Tm [format "%4.1f" $yMoyen]C" "Affichable" "OligoGraph" "$TagDuGraph" "Average$TagDuGraph"] \
		-fill "$ColorLine" \
		-dash {6 4}
	    
	    #Ligne de max nbseq
	    $K create line \
		$XDebutQuery \
		[expr {$MaxSpectreY-($yMax-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		$XFinQuery \
		[expr {$MaxSpectreY-($yMax-$yMin)*($HauteurDuSpectreY)/($yMax-$yMin)}] \
		-tags [list "" "OligoGraph" "$TagDuGraph"]
	    continue
	}
	scan $Ligne "%f %f" x y
	
	set X [expr {$XDebutQuery + ($x * $OneNtPixel)}]
	set Y [expr {$MaxSpectreY-($y-$yMin)*($HauteurDuSpectreY*1.0)/($yMax-$yMin)}]
	
	incr n
	if {$n > 1} { 
	    $K create line [set Coordonnees(LastX)] [set Coordonnees(LastY)] $X $Y \
		-tags [list "" "[format "%.1f" [expr {([set Coordonnees(VraiY)]+$y)/2}]] C" "Affichable" "OligoGraph" "$TagDuGraph"] \
		-fill "$ColorLine"
	    set n 1
	}
	set Coordonnees(VraiY) $y
	set Coordonnees(LastX) $X
	set Coordonnees(LastY) $Y
	lappend LesPoints "$X $Y $y"
    }
 
    foreach Coord $LesPoints {
	set X  [lindex $Coord 0]
	set Y  [lindex $Coord 1]
	set Tm [lindex $Coord 2]
	$K create line $X $Y $X $Y \
		-fill "$ColorLine" \
		-tags [list "" "[format "%.1f" $Tm] C" "Affichable" "OligoGraph" "$TagDuGraph"] 
    }
    if {[info exists Coordonnees]} {array unset Coordonnees}
    return ""
}

proc PosEtGCDeLaRegion {LaSeq TailleQuery} {
    
    Wup "Calcul le GC pour les regions de la query"
    
    if {$LaSeq==""} {return ""}
    set  LaSeq [split $LaSeq ""]

    set GCTotal      0
    set GCMax        0
    set GCMin     9999
    set LesGCs      ""
    set DebutSeq     1
    set LengthOligo 30 
    set Fin         $LengthOligo
    set Pas         10
    #Calcul de la moyenne sur une sliding window de taille Facteur et de pas j
    for {set Debut $DebutSeq} {$Fin <= $TailleQuery} {incr Debut $Pas} {
	set Fin [expr {$Debut + $LengthOligo - 1}] 
	if {![expr $Fin <= $TailleQuery]}          {break}
	
	set Seq [join [lrange $LaSeq [expr {$Debut - 1}] [expr {$Fin - 1}]] ""]
	set  TailleSeq [string length $Seq]
	if {$TailleSeq<$LengthOligo}     {continue}
	
	set GC  [expr {([NombreDeBaseDe $Seq "G|C"]*100.0)/$TailleSeq}]
	if {$GC=="-1"} {continue} elseif {$GC==""} {return ""}	
	
	if {$Debut==1} {
	    lappend LesGCs "1 [format "%.1f" $GC]"
	} else {
	    #lappend LesGCs "$Fin [format "%.1f" $GC]"
	    #lappend LesGCs "$Debut [format "%.1f" $GC]"
	    lappend LesGCs "[expr {$Debut + ($Fin-$Debut)/2}] [format "%.1f" $GC]"
	}
	set GCTotal [expr {$GCTotal + $GC}]
	if {$GC > $GCMax} {set GCMax $GC}
	if {$GC < $GCMin} {set GCMin $GC}
    }
    set nbLesGCs [llength $LesGCs]
    set Moyenne  [expr {($GCTotal*1.0)/($nbLesGCs)}]
    set Entete   [list "0 $GCMin" "100 $GCMax" "$Moyenne $Moyenne"]

    set     LesGCs [concat $Entete $LesGCs] 
    return $LesGCs
}

proc OLD_PosEtGCDeLaRegion {{SequenceADN ""} {lWin 400}} {
 
    Wup "Calcul le pourcentage de GC" 

    global RepertoireDuGenome
    
    if {$SequenceADN==""} {return""} 
    set ADN $SequenceADN
    set l   [string length $ADN]

    set nGC 0
    for {set i 0} {$i<$lWin} {incr i} {
	set Win($i)       "A"
	set Win([incr i]) "C"
	incr nGC
    }

    set pWin  0
    set ntGC  0
    set Mi   99
    set Ma    0
    for {set i 1} {$i < $l} {incr i} {
	set c [string index $ADN $i]
	set w [set Win($pWin)]
	if { $c == "G" || $c == "C" } { incr nGC  1 ; incr ntGC}
	if { $w == "G" || $w == "C" } { incr nGC -1 }
	set Win($pWin) $c
	set pWin   [expr ($pWin+1)%$lWin]
	set NewpGC [format "%.1f" [expr ($nGC*100.0)/$lWin]]
	if {[expr $i>$lWin/2] && [expr $i%($lWin/4)] == 0} {lappend SpectreXY "[expr $i-$lWin/2] $NewpGC"}
	set pGC $NewpGC
	set Mi [Mini $Mi $NewpGC]
	set Ma [Maxi $Ma $NewpGC]
	continue
    }
    set Moyen  [format "%.1f" [expr ($ntGC*100)/[string length $ADN]]]
    set Entete [list "0 $Mi" "100 $Ma" "$Moyen $Moyen"]

    set     SpectreGCXY [concat $Entete $SpectreXY] 
    return $SpectreGCXY
}

proc PosEtPIdAligneeDeLaRegion {FichierBlast} {
    
    Wup "Calcul le pourcentage d identite du blast le long de la query"
    
    global ProfilBlast
    
    if {![info exists FichierBlast] || ![file exists $FichierBlast]} {return ""}

    set LesProfilsAligneesEtAlignables ""

    set NbSubject   [AskBlast $FichierBlast NbSubject]
    set TailleQuery [AskBlast $FichierBlast Taille]
     if {$NbSubject==0} {
	set Entete                   [list "0 0" "100 0" "0 0"]
	set LesSommeDesPIdDeLaRegion [list "1 0" "$TailleQuery 0"]
	set LesSommeDesPIdDeLaRegion [concat $Entete $LesSommeDesPIdDeLaRegion] 
	return $LesSommeDesPIdDeLaRegion
    }

    #On elimine la redondance de plusieurs HSPs identiques pour un meme subject
    #Le profil des bases alignees et le profil des bases alignables
    set LesProfilsAligneesEtAlignables [LesProfilsDeLaQuerySurToutLeBlast $FichierBlast "LesDeux"]

    set LesPositions ""
    foreach LesProfils $LesProfilsAligneesEtAlignables {
	set Position 0
	foreach LaBase [lindex $LesProfils 1] {
	    set Alignees   [lindex $LaBase 0]
	    set Alignables [lindex $LaBase 1]
	    if {![info exists ProfilBlast($Position,Alignees)]} {
		lappend LesPositions $Position
		lappend ProfilBlast(LesPositions) $Position
		set ProfilBlast($Position,Alignees)   0
		set ProfilBlast($Position,Alignables) 0
	    }
	    if {$Alignees>=1}   {incr ProfilBlast($Position,Alignees)} 
	    if {$Alignables>=1} {incr ProfilBlast($Position,Alignables)}
	    incr Position
	}
    }
    
    set LesPIdDeLaPosition {}
    foreach Pos $LesPositions {
	#Attention on fixe a 0 l identite si une sequence alignee plus marquant
	if {[set ProfilBlast($Pos,Alignables)]==0 ||[set ProfilBlast($Pos,Alignables)]==1} {lappend LesPIdDeLaPosition "$Pos 0.0";continue}
	if {[set ProfilBlast($Pos,Alignables)]==0} {lappend LesPIdDeLaPosition "$Pos 0.0";continue}
	lappend LesPIdDeLaPosition "$Pos [expr {([set ProfilBlast($Pos,Alignees)]*100.0)/[set ProfilBlast($Pos,Alignables)]}]"
    }

    set PIdTotal   0
    set Facteur   20
    set nbLesIds   0
    set PIdMax     0
    set PIdMin 99999
    set Pas       10
    #Calcul de la moyenne sur une sliding window de taille Facteur et de pas j
    for {set j 0} {$j < [llength $LesPIdDeLaPosition]} {incr j $Pas} {
	set SommeId              0
	set LaSommePIdDeLaRegion ""
	incr nbLesIds 

	set LesPIdDeLaRegion [lrange $LesPIdDeLaPosition $j [expr {$j+$Facteur-1}]]
	set nbMesPId         [llength $LesPIdDeLaRegion]
	
	foreach PId $LesPIdDeLaRegion {
	    set Id      [lindex $PId 1]
	    set SommeId [expr {$SommeId+$Id}]
	}
	set LaSommePIdDeLaRegion [expr {$SommeId/($nbMesPId*1.0)}]
	
	if {$LaSommePIdDeLaRegion>$PIdMax} {set PIdMax $LaSommePIdDeLaRegion}
	if {$LaSommePIdDeLaRegion<$PIdMin} {set PIdMin $LaSommePIdDeLaRegion}

	lappend LesSommeDesPIdDeLaRegion "[expr {$j+1}] $LaSommePIdDeLaRegion"
	
	set PIdTotal [expr {$PIdTotal + $LaSommePIdDeLaRegion}]
    }
    set Moyenne [expr {($PIdTotal*1.0)/($nbLesIds)}]
    set Entete  [list "0 $PIdMin" "100 $PIdMax" "$Moyenne $Moyenne"]

    set     LesSommeDesPIdDeLaRegion [concat $Entete $LesSommeDesPIdDeLaRegion] 
    return $LesSommeDesPIdDeLaRegion
}

proc PosEtNbSequenceAligneeDeLaRegion {FichierBlast} {
    
    Wup "Calcul le nombre de sequence detectees dans le blast pour chaque position"
  
    global ProfilBlast

    set LesPositions ""
    if {! [info exists ProfilBlast]} {   
	if {! [info exists FichierBlast] || ! [file exists $FichierBlast]} {return ""}

	set  NbSubject [AskBlast $FichierBlast NbSubject]
	if {$NbSubject!=0} {
	    set LesProfilsAlignables [LesProfilsDeLaQuerySurToutLeBlast $FichierBlast "BaseDansAlignement"]
	    foreach LeProfilAlignables $LesProfilsAlignables {
		set LesBasesAlignables [lindex $LeProfilAlignables 1]
		set Position 0
		foreach Alignables $LesBasesAlignables {
		    if {![info exists ProfilBlast($Position,Alignables)]} {
			lappend LesPositions $Position
			set ProfilBlast($Position,Alignables) 0
		    }
		    if {$Alignables>=1} {incr ProfilBlast($Position,Alignables)}
		    incr Position
		}
	    }
	}
    } else {
	set LesPositions [set ProfilBlast(LesPositions)]
    }
    
    set NbSubject   [AskBlast $FichierBlast NbSubject]
    set TailleQuery [AskBlast $FichierBlast Taille]
    if {$NbSubject==0} {
	set Entete                      [list "0 0" "100 0" "0 0"]
	set LesSommesDesNbSeqDeLaRegion [list "1 0" "$TailleQuery 0"]
	set LesSommesDesNbSeqDeLaRegion [concat $Entete $LesSommesDesNbSeqDeLaRegion] 
	return $LesSommesDesNbSeqDeLaRegion
    }

    set LesNbSeqDeLaPosition ""
    set NbAlignableMax        0
    set NbAlignableMin    99999
    foreach Pos $LesPositions {
	#if {0 && [set ProfilBlast($Pos,Alignables)]==0 ||[set ProfilBlast($Pos,Alignables)]==1} {lappend LesNbSeqDeLaPosition "$Pos 0.0";continue}
	if {[set ProfilBlast($Pos,Alignables)]==0} {lappend LesNbSeqDeLaPosition "$Pos 0.0";continue}
	lappend LesNbSeqDeLaPosition "$Pos [set ProfilBlast($Pos,Alignables)]"
	if {[set ProfilBlast($Pos,Alignables)] > $NbAlignableMax} {set NbAlignableMax [set ProfilBlast($Pos,Alignables)]}
	if {[set ProfilBlast($Pos,Alignables)] < $NbAlignableMin} {set NbAlignableMin [set ProfilBlast($Pos,Alignables)]}
    }

    set NbSeqTotal   0
    set Facteur     10
    set nbLesNbSeqs  0
    set Pas         10
    #Calcul de la moyenne sur une sliding window de taille Facteur et de pas j
    for {set j 0} {$j < [llength $LesNbSeqDeLaPosition]} {incr j $Pas} {
	set SommeNbSeq              0
	set LaSommeNbSeqDeLaRegion  ""
	incr nbLesNbSeqs 

	set LesNbSeqDeLaRegion [lrange $LesNbSeqDeLaPosition $j [expr {$j+$Facteur-1}]]
	set nbMesNbSeq         [llength $LesNbSeqDeLaRegion]
	foreach NbSeq $LesNbSeqDeLaRegion {
	    set NbSeq      [lindex $NbSeq 1]
	    set SommeNbSeq [expr {$SommeNbSeq+$NbSeq}]
	}
	set LaSommeNbSeqDeLaRegion [expr {$SommeNbSeq/($nbMesNbSeq*1.0)}]

	if {[expr $LaSommeNbSeqDeLaRegion<1.0] && [expr $LaSommeNbSeqDeLaRegion>0.0]} {set LaSommeNbSeqDeLaRegion 1.0} 
	lappend LesSommesDesNbSeqDeLaRegion "[expr {$j+1}] $LaSommeNbSeqDeLaRegion"
	
	set NbSeqTotal [expr {$NbSeqTotal + $LaSommeNbSeqDeLaRegion}]
    }
    set Moyenne [expr {($NbSeqTotal*1.0)/($nbLesNbSeqs)}]
    set Entete  [list "0 $NbAlignableMin" "$NbAlignableMax $NbAlignableMax" "$Moyenne $Moyenne"]

    set     LesSommesDesNbSeqDeLaRegion [concat $Entete $LesSommesDesNbSeqDeLaRegion] 
    return $LesSommesDesNbSeqDeLaRegion
}

proc PosEtTmDeLaRegion {LaSeq TailleQuery} {
    
    Wup "Calcul le Tm pour les regions de la query"
    
    if {$LaSeq==""} {return ""}
    set  LaSeq [split $LaSeq ""]

    set TmTotal      0
    set TmMax        0
    set TmMin     9999
    set LesTms      ""
    set DebutSeq     1

    set  LengthOligo [LesParametresDuDesign LengthOligo "LaValeurMerci"]
    if {$LengthOligo==""} {set LengthOligo 60}

    set  TmMethod    [LesParametresDuDesign TmMethod    "LaValeurMerci"] 
    if {$TmMethod   ==""} {set TmMethod    "NearestNeighbor"}

    set Fin         $LengthOligo
    set Pas         10
    #Calcul de la moyenne sur une sliding window de taille Facteur et de pas j
    for {set Debut $DebutSeq} {$Fin <= $TailleQuery} {incr Debut $Pas} {
	set Fin [expr {$Debut + $LengthOligo - 1}] 
	if {![expr $Fin <= $TailleQuery]}          {break}
		
	set Seq [join [lrange $LaSeq [expr {$Debut - 1}] [expr {$Fin - 1}]] ""]
	if {[string length $Seq]<$LengthOligo}     {continue}

	set Tm  [CalculTmDeLaSeq $Seq "$TmMethod"]
	if {$Tm=="-1"} {continue} elseif {$Tm==""} {return ""}	

	if {$Debut==1} {
	    lappend LesTms "1 [format "%.1f" $Tm]"
	} else {
	    #lappend LesTms "$Fin [format "%.1f" $Tm]"
	    #lappend LesTms "$Debut [format "%.1f" $Tm]"
	    lappend LesTms "[expr {$Debut + ($Fin-$Debut)/2}] [format "%.1f" $Tm]"
	}
	set TmTotal [expr {$TmTotal + $Tm}]
	if {$Tm > $TmMax} {set TmMax $Tm}
	if {$Tm < $TmMin} {set TmMin $Tm}
    }
    set  nbLesTms [llength $LesTms]
    if {$nbLesTms=="0"} {
	#Le Tm n a pu etre calcule on renvoi rien
	return ""
    } else {
	set Moyenne  [expr {($TmTotal*1.0)/($nbLesTms)}]
	set Entete   [list "$TmMin $TmMin" "$TmMax $TmMax" "$Moyenne $Moyenne"]
    }

    set     LesTms [concat $Entete $LesTms] 
    return $LesTms
}

proc UnsetLesTableauxDesGraphAvecBlast {} {
    
    Wup "Permet de supprimer les tableaux en memoire des profils du blast pour faire Id et NbSeq"

    global ProfilBlast
    
    if {[array exists ProfilBlast]} {array unset ProfilBlast}

    return ""
}

proc AjouteUnGraph {K FichierQuery XDebutQuery XFinQuery YDebutGraph YFinGraph OneNtPixel} {
    
    Wup "Ajoute un graph a un canvas"
    Wup "Ex Id et NbSeq lie au blastn et Tm et GC lie a une sequence donnee"
    
    set AvecTm        0
    set AvecGC        0
    set AvecId        0
    set AvecNbSeq     0
    set AvecTmNN      0
    set AvecTmWallace 0
    set AvecTmDNADNA  0
    set AvecTmDNARNA  0
    set AvecTmRNARNA  0
    set TmDefault    ""

    FaireLire "Choose between available graph to add."
    set  LesChoix [EntreDeux [list TmNN TmWallace TmDNADNA TmDNARNA TmRNARNA GC Id NbSeq]]
    if {$LesChoix == ""} {return ""}
    #On recupere la methode de Tm par defaut choisie et on la remet apres
    if {[regexp -nocase "tm" $LesChoix]} {set TmDefault [LesParametresDuDesign TmMethod "LaValeurMerci"]}

    foreach Choix $LesChoix {
	if {$Choix=="TmNN"}      {set AvecTm    1;set AvecTmNN      1}
	if {$Choix=="TmWallace"} {set AvecTm    1;set AvecTmWallace 1}
	if {$Choix=="TmDNADNA"}  {set AvecTm    1;set AvecTmDNADNA  1}
	if {$Choix=="TmDNARNA"}  {set AvecTm    1;set AvecTmDNARNA  1}
	if {$Choix=="TmRNARNA"}  {set AvecTm    1;set AvecTmRNARNA  1}
	if {$Choix=="GC"}        {set AvecGC    1}
	if {$Choix=="Id"}        {set AvecId    1}
	if {$Choix=="NbSeq"}     {set AvecNbSeq 1}
    }
    
    #On remet le zoom a la position initiale pour ne pas avoir de probleme de positionnement des graphes
    ResetZoom $K

    if {$AvecNbSeq || $AvecId} {
	set FichierBlastN ""
	set MesBlasts     "" 
	
	set PAB           "[file tail $FichierQuery]"
	regsub {.[a-zA-Z]+$} $PAB "" PAB

	set  MesBlasts [LesInfosDe $K MesBlasts]
	if {$MesBlasts != ""} { 
	    while {1} {
		if {[OuiOuNon "We need a related blastn file to build graph.\nCan I use one of these blasts ?\n$MesBlasts"]} {
		    set FichierBlastN [ChoixParmi $MesBlasts]
		    if {! [regexp -nocase $PAB $FichierBlastN]} {
			FaireLire "Choose a correct blastn file according to your query sequence"
		    } else {break}
		    if {![OuiOuNon "Do you want to continue ?"]} {break}
		} else {
		    break
		} 
	    }
	}
	if {$FichierBlastN=="" || ! [file exists $FichierBlastN]} {
	    FaireLire "Choose a blastn file to add two new graph to display."
	    while {1} {
		#Les types de fichier autorises
		set types {
		    {{Blastn Files} {.blastn}}
		    {{All Files}    *        }
		}
		
		if {[BROWSEDDIR] != ""} {
		    set FichierBlastN [tk_getOpenFile -title "Choose a blastn file" -initialdir "[BROWSEDDIR]" -filetypes $types]
		} else {
		    set FichierBlastN [tk_getOpenFile -title "Choose a blastn file" -initialdir "[HOMEDIR]"    -filetypes $types]
		}
		if {! [regexp -nocase $PAB $FichierBlastN]} {
		    FaireLire "Choose a correct blastn file according to your query sequence"
		} else {break}
		if {![OuiOuNon "Do you want to continue ?"]} {return ""}
		set BrowsedDir [file dirname [lindex $FichierBlastN end]]
		BROWSEDDIR $BrowsedDir
	    }  
	}
	if {[file exists $FichierBlastN]} {
	    if {$AvecId} {
		#Affichage du Id content du blast
		set IdDuGraph ""
		while {1} {
		    if {[$K find withtag "Id$IdDuGraph"] != ""} {
			if {$IdDuGraph == ""} {set IdDuGraph 1} else {incr IdDuGraph}
		    } else {break}
		}
		set  IdResults [PosEtPIdAligneeDeLaRegion $FichierBlastN] 
		if {$IdResults !=""} {
		    AfficheLeSpectreIdDUneListe $K $IdResults \
			    $XDebutQuery $XFinQuery \
			    $YDebutGraph $YFinGraph $OneNtPixel "red" "Id$IdDuGraph"
		}
	    }
	    if {$AvecNbSeq} {
		#Affichage du nb de sequence du blast
		set IdDuGraph ""
		while {1} {
		    if {[$K find withtag "NbSeq$IdDuGraph"] != ""} {
			if {$IdDuGraph == ""} {set IdDuGraph 1} else {incr IdDuGraph}
		    } else {break}
		}
		set  NbSeqResults [PosEtNbSequenceAligneeDeLaRegion $FichierBlastN] 
		if {$NbSeqResults !=""} {
		    AfficheLeSpectreNbSequenceDUneListe $K $NbSeqResults \
			    $XDebutQuery $XFinQuery \
			    $YDebutGraph $YFinGraph $OneNtPixel "deepskyblue2" "NbSeq$IdDuGraph"
		}
	    }
	    UnsetLesTableauxDesGraphAvecBlast
	}
    }
    if {$AvecGC || $AvecTm} {
	set SeqQuery       [QueLaSequenceDuFichierTFA $FichierQuery]
	if {$SeqQuery==""} {Espionne "$FichierQuery no sequence available";return ""}
	set TailleQueryNt  [string length $SeqQuery]
    }
    if {$AvecTm && $AvecTmNN && [$K find withtag {"OligoGraph" && "TmNN"}] == ""} {
	LesParametresDuDesign TmMethod "NearestNeighbor" "Positionne"
	#Affichage du Tm content
	set  TmResults [PosEtTmDeLaRegion $SeqQuery $TailleQueryNt] 
	if {$TmResults !=""} {
	    AfficheLeSpectreTmDUneListe $K $TmResults \
		    $XDebutQuery $XFinQuery \
		    $YDebutGraph $YFinGraph $OneNtPixel "orange" "TmNN"
	}
    }
    if {$AvecTm && $AvecTmWallace && [$K find withtag {"OligoGraph" && "TmWallace"}] == ""} {
	LesParametresDuDesign TmMethod "Wallace" "Positionne"
	#Affichage du Tm content
	set  TmResults [PosEtTmDeLaRegion $SeqQuery $TailleQueryNt] 
	if {$TmResults !=""} {
	    AfficheLeSpectreTmDUneListe $K $TmResults \
		    $XDebutQuery $XFinQuery \
		    $YDebutGraph $YFinGraph $OneNtPixel "orange1" "TmWallace"
	}
    }
    if {$AvecTm && $AvecTmDNADNA && [$K find withtag {"OligoGraph" && "TmDNADNA"}] == ""} {
	LesParametresDuDesign TmMethod "DNADNA" "Positionne"
	#Affichage du Tm content
	set  TmResults [PosEtTmDeLaRegion $SeqQuery $TailleQueryNt] 
	if {$TmResults !=""} {
	    AfficheLeSpectreTmDUneListe $K $TmResults \
		    $XDebutQuery $XFinQuery \
		    $YDebutGraph $YFinGraph $OneNtPixel "orange2" "TmDNADNA"
	}
    }
    if {$AvecTm && $AvecTmDNARNA && [$K find withtag {"OligoGraph" && "TmDNARNA"}] == ""} {
	LesParametresDuDesign TmMethod "DNARNA" "Positionne"
	#Affichage du Tm content
	set  TmResults [PosEtTmDeLaRegion $SeqQuery $TailleQueryNt] 
	if {$TmResults !=""} {
	    AfficheLeSpectreTmDUneListe $K $TmResults \
		    $XDebutQuery $XFinQuery \
		    $YDebutGraph $YFinGraph $OneNtPixel "orange3" "TmDNARNA"
	}
    }
    if {$AvecTm && $AvecTmRNARNA && [$K find withtag {"OligoGraph" && "TmRNARNA"}] == ""} {
	LesParametresDuDesign TmMethod "RNARNA" "Positionne"
	#Affichage du Tm content
	set  TmResults [PosEtTmDeLaRegion $SeqQuery $TailleQueryNt] 
	if {$TmResults !=""} {
	    AfficheLeSpectreTmDUneListe $K $TmResults \
		    $XDebutQuery $XFinQuery \
		    $YDebutGraph $YFinGraph $OneNtPixel "orange4" "TmRNARNA"
	}
    }
    if {$AvecGC && [$K find withtag {"OligoGraph" && "GC"}] == ""} {
	#Affichage du GC content
	set  GCResults [PosEtGCDeLaRegion $SeqQuery $TailleQueryNt]
	if {$GCResults !=""} {
	    AfficheLeSpectreGCDUneListe $K $GCResults \
		    $XDebutQuery $XFinQuery \
		    $YDebutGraph $YFinGraph $OneNtPixel
	}
    }
    
    #On recupere la methode de Tm par defaut choisie et on la remet apres
    LesParametresDuDesign TmMethod $TmDefault "Positionne"

    return ""
}

proc SupprimeUnGraph {K} {
    
    Wup "Ajoute un graph a un canvas ici Id et NbSeq lie au blastn"
    
    set LesGraphes [EntreDeux [LesGraphesDeLOligo $K]]
    foreach Graphe $LesGraphes {
	set LesIds [$K find withtag $Graphe]
	foreach Id $LesIds {
	    catch {$K delete $Id} 
	}
    }
    return ""
}

#################################
#   Petits outils pour le GUI   # 
#################################

proc CADO4MI_Font {} {
    
    Wup "Definit la fonte par defaut de CADO4MI"
    Wup "Reinitialise la fonte par defaut de CADO4MI"

    set WinSys ""
    if {[catch {set WinSys [tk windowingsystem]} Message]} {
	#Espionne "$Message"
	#puts "$Message"
    }
        
    if {[regexp "CADO4MI_Font" [font names]]} {
	set Exists 1
    } else {
	set Exists 0
    }

    if {$WinSys == "win32"} {
	#Pour windows la taille de la fonte est legerement plus petite
	if {$Exists != "1"} {
	    set CADO4MI_Font [font create CADO4MI_Font -family "arial" -size 8 -weight bold]
	    option add *font CADO4MI_Font startupFile
	} else {
	    font configure CADO4MI_Font -family "arial" -size  8 -weight bold -slant "roman"
	}
    } else {
	if {$Exists != "1"} {
	    set CADO4MI_Font [font create CADO4MI_Font -family "arial" -size 12 -weight bold]
	    option add *font CADO4MI_Font startupFile
	} else {
	    font configure CADO4MI_Font -family "arial" -size 12 -weight bold -slant "roman"
	}
    }
}

proc changeFont2Default {} {

    Wup "Permet de reinitialiser les parametres de la fonte CADO4MI"

    CADO4MI_Font 
    #font configure CADO4MI_Font -family "arial" -size 10 -weight bold -slant "roman"
    #font configure $font -family "arial" -size 12 -weight bold -slant "roman"
}

proc changeFontType {font NewFont} {

    Wup "Permet de modifier le type de fonte"

    font configure $font -family $NewFont
}

proc changeFontSize {font direction} {

    Wup "Permet de modifier le hauteur de la fonte"

    set size [font configure $font -size]
    if {$direction == "+"} {
	incr size 2
    } else {
	incr size -2
    }
    font configure $font -size $size
}

proc changeFontWeight {font} {

    Wup "Permet de modifier l epaisseur de la fonte"
    
    set  Weight [font configure $font -weight]
    if {$Weight == "normal"} {
	set Type "bold"
    } else {
	set Type "normal"
    }
    font configure $font -weight $Type
}

proc changeFontSlant {font} {
    
    Wup "Permet de modifier l orientation de la fonte"
    
    set  Slant [font configure $font -slant]
    if {$Slant == "roman"} {
	set Type "italic"
    } else {
	set Type "roman"
    }
    font configure $font -slant $Type
}

proc Font_Panel {} {

    Wup "Manageur des parametres de la fonte dans CADO4MI"
    Wup "Permet de changer la taille, le type et l epaisseur de la fonte CADO4MI_Font"
    
    set w ".fontmanagement"
    if {[winfo exists $w]} {wm deiconify $w;raise $w;return ""} else {toplevel $w}
    
    wm geometry  $w +300+100
    wm title     $w "Font Management"
    wm resizable $w false false
    wm protocol  $w WM_DELETE_WINDOW "destroy $w"

    #Frame Generale contenant les autres frame, le bouton dismiss et le bouton default
    set     FirstFrame    "$w.framehome"
    set     ButtonDismiss "$FirstFrame.bdismiss"
    set     ButtonDefault "$FirstFrame.bdefault"
    frame  $FirstFrame    -borderwidth "3"     -relief "groove"
    button $ButtonDismiss -background "red"    -relief "ridge" -text "Dismiss" -padx "2" -command "eval \[wm protocol $w WM_DELETE_WINDOW\]"
    button $ButtonDefault -background "orange" -relief "ridge" -text "Default" -padx "2" -command {changeFont2Default}

    #Frame pour changer le type de fonte
    set FontValues [list arial times courier couriernew helvetica]
    global SbType
    set      FType  "$FirstFrame.ftype"
    set      LType  "$FType.ltype"
    set      SbType "$FType.sbtype"
    frame   $FType  -borderwidth "3"      -relief "groove"
    label   $LType  -background "#808FFF" -relief "groove" -text "Type" -width "5" -anchor "center" -padx "2"
    spinbox $SbType -width "9" -state readonly -values $FontValues -textvariable FontType -command {
	set NewFont [$SbType get]
	changeFontType CADO4MI_Font $NewFont
    }

    #Frame pour changer le epaisseur de la fonte
    global  BWeight
    global  BSlant
    set     FWeight "$FirstFrame.fweight"
    set     BWeight "$FWeight.bweight"
    set     BSlant  "$FWeight.bslant"
    frame  $FWeight -borderwidth "3"     -relief "groove"
    button $BWeight -background "orange" -relief "ridge"  -text "Bold"  -width "5" -anchor "center" -padx "2" -command {
	changeFontWeight CADO4MI_Font
	set  TextBWeight [lindex [$BWeight configure -text] 4]
	if {$TextBWeight == "Normal"} {
	    set TextBWeight "Bold"
	} else {
	    set TextBWeight "Normal"
	}
	$BWeight configure -text $TextBWeight
    }
    button $BSlant -background "orange"  -relief "ridge"  -text "Roman" -width "5" -anchor "center" -padx "2" -command {
	changeFontSlant CADO4MI_Font
	set  TextBSlant [lindex [$BSlant configure -text] 4]
	if {$TextBSlant == "Roman"} {
	    set TextBSlant "Italic"
	} else {
	    set TextBSlant "Roman"
	}
	$BSlant configure -text $TextBSlant
    }

    #Frame pour changer la taille de la fonte
    set     FInfo "$FirstFrame.ffonte"
    set     Label "$FInfo.label"
    frame  $FInfo -borderwidth "3"      -relief "groove"
    label  $Label -background "#808FFF" -relief "groove" -text "Font Management" -width "15" -anchor "center" -padx "2"
    set Texte   "Changing font parameters such as size, type and others (bold, italic)"
    set_balloon $Label "$Texte"
    
    set     FSize  "$FirstFrame.fsize"
    set     LSize  "$FSize.lsize"
    set     BPlus  "$FSize.bplus"
    set     BMinus "$FSize.bminus"
    frame  $FSize  -borderwidth "3"         -relief "groove"
    label  $LSize  -background "#808FFF"    -relief "groove" -text "Size" -width "3" -anchor "center" -padx "2"
    button $BPlus  -background "OliveDrab1" -relief "ridge"  -text "+"    -width "1" -anchor "center" -padx "2" -command {
	changeFontSize CADO4MI_Font +
    }
    button $BMinus -background "OliveDrab1" -relief "ridge"  -text "-"    -width "1" -anchor "center" -padx "2" -command {
	changeFontSize CADO4MI_Font -
    }

    pack $FirstFrame         -fill both -expand true

    pack $FInfo   -side top  -fill both -expand true
    pack $Label   -side top  -fill both -expand true
   
    pack $FType   -side top  -fill both -expand true
    pack $LType   -side left -fill both -expand true
    pack $SbType  -side left -fill both -expand true
   
    pack $FWeight -side top  -fill both -expand true
    pack $BWeight -side left -fill both -expand true
    pack $BSlant  -side left -fill both -expand true

    pack $FSize   -side top  -fill both -expand true
    pack $LSize   -side left -fill both -expand true
    pack $BPlus   -side left -fill both -expand true
    pack $BMinus  -side left -fill both -expand true

    pack $ButtonDismiss -side left -fill both -expand true
    pack $ButtonDefault -side left -fill both -expand true

    return ""
}

proc UnCanvaDe {LargeurMaxi HauteurMaxi {LargeurVoulue 768} {HauteurVoulue 512} {Titre ""} {AvecMainLevee ""} } {
    
    Wup "Permet de cree une fenetre composee de 1 canvas, de une frame avec des boutons ..."

    set CouleurDuFondDeUnCanva "white"

    if {[regexp "MainLevee" $AvecMainLevee]} {set AvecMainLevee 1} else {set AvecMainLevee 0} 

    set w [NomDe canva]
    if {$Titre==""} {set Titre $w} 
    catch {destroy $w}
    toplevel    $w
    wm title    $w $Titre
    wm iconname $w $Titre

    set CanvaScroMinX 0
    set CanvaScroMinY 0
    set CanvaScroMaxX $LargeurMaxi
    set CanvaScroMaxY $HauteurMaxi

    set K       $w.canvas
    set ScroHor $w.hscroll
    set ScroVer $w.vscroll
    set KShSv   $w.grid
    set Boutons $w.buttons

    frame $Boutons
    pack  $Boutons -side bottom -fill x -pady 2m
    
    button $Boutons.dismiss -text "Dismiss" -background "red" -command "eval \[wm protocol $w WM_DELETE_WINDOW\]" -padx 3
      bind $Boutons.dismiss <Control-2> "MainLeveeSurUnCanva $K"
      pack $Boutons.dismiss -side left -expand 1

    button $Boutons.zoom    -text "Zoom In / Out\nZoom x In / Out\nZoom Reset all" -background "yellow" -padx 3
      bind $Boutons.zoom <1>         "zoom $K 1.25 1.25"
      bind $Boutons.zoom <2>         "zoom $K 0.8 0.8"
      bind $Boutons.zoom <Shift-1>   "zoom $K 1.25 1.0"
      bind $Boutons.zoom <Shift-2>   "zoom $K 0.8 1.0"
      bind $Boutons.zoom <Control-1> "ResetZoom $K"
      #bind $Boutons.zoom <Control-1> "zoom $K 1.0 1.25"
      #bind $Boutons.zoom <Control-2> "zoom $K 1.0 0.8"
      pack $Boutons.zoom -side left -expand 1

    set    Texte [format   "%-10s" "Zoom Menu"]
    append Texte [format "\n%-10s" "Allow to zoom in and out (X or X+Y axis) or to reset."]
    append Texte [format "\n%-10s" "You can also use Button-3 to select an area to zoom in."]
    set_balloon $Boutons.zoom "$Texte"

    frame $KShSv
    scrollbar $ScroHor -orient horiz -command "$K xview"
    scrollbar $ScroVer               -command "$K yview"
    
    canvas $K -width $LargeurVoulue -height $HauteurVoulue
    $K configure -scrollregion [list $CanvaScroMinX $CanvaScroMinY $CanvaScroMaxX $CanvaScroMaxY] \
	    -xscrollcommand "$ScroHor set" \
	    -yscrollcommand "$ScroVer set"

    grid rowconfig    $KShSv 0 -weight 1 -minsize 0
    grid columnconfig $KShSv 0 -weight 1 -minsize 0
    
    grid $K       -in $KShSv -padx 1 -pady 1 -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
    grid $ScroVer -in $KShSv -padx 1 -pady 1 -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky nsew
    grid $ScroHor -in $KShSv -padx 1 -pady 1 -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
    pack $KShSv   -expand yes -fill both -padx 1 -pady 1
    if {$AvecMainLevee} {MainLeveeSurUnCanva $K}

    wm protocol $w WM_DELETE_WINDOW "after 10 destroy $w"

    return $K
}

proc EntreDeux {Liste {ListeDeTextes {}} {Invite "Select ... and validate."}} {

    Wup "Permet une selection entre plusieurs choix"

    return [lindex [LesPEtA $Liste $ListeDeTextes $Invite] 0]
}

proc LesPEtA {Liste {ListeDeTextes {}} {Invite "Select ... and validate."}} {

    Wup "Permet une selection entre plusieurs choix"

    global ChoixMultipleRadio
    global validOk
    
    if { $ListeDeTextes  == {}} {set ListeDeTextes $Liste}
    if {[llength $Liste] == 0 } {return [list {} {}]}

    set MaxScreenX [expr {[winfo screenwidth  .]/2}]
    set MaxScreenY [expr {[winfo screenheight .]/2}]

    set n        0
    set LesLooks {}
    foreach Element $Liste {
	incr n
	set Look "a$n"
	lappend LesLooks $Look
    }

    set w [NomDe fenetre]
    catch {destroy $w}
    catch {unset validOk}
    toplevel $w
    wm geometry $w +300+100
    wm title $w "$Invite"
    #wm resizable $w false false
    
    #Creation des boutons
    #####################
    set     Boutons $w.termine
    frame  $Boutons
    button $Boutons.dismiss     -text "Dismiss"  -background "red"    -relief "ridge" -padx "2" -command {set validOk 0}
    button $Boutons.toutpresent -text "All +"    -background "yellow" -relief "ridge" -padx "2" -command { 
	foreach Look [array names ChoixMultipleRadio] {
	    set ChoixMultipleRadio($Look) p
	}
    }
    button $Boutons.toutabsent  -text "All -"    -background "orange" -relief "ridge" -padx "2" -command { 
	foreach Look [array names ChoixMultipleRadio] {
	    set ChoixMultipleRadio($Look) a
	}
    }
    button $Boutons.validate    -text "Validate" -background "green"  -relief "ridge" -padx "2" -command {set validOk 1}
    pack $Boutons -side bottom  -pady 2          -fill x 
    pack $Boutons.dismiss $Boutons.toutpresent $Boutons.toutabsent $Boutons.validate -side left -expand yes

    #Creation de la frame contenant
    #    les radio boutons 
    #    les scrollbars
    ###############################
    frame $w.frame
    set BouRad  "$w.frame.radios"
    set ScroHor "$w.frame.xscroll"
    set ScroVer "$w.frame.yscroll"
    scrollbar $ScroVer -command "$BouRad yview"
    scrollbar $ScroHor -command "$BouRad xview" -orient horizontal 
    
    set CanvaScreenX   200
    set CanvaScreenY    25
    set HauteurUneLigne 25
    canvas $BouRad -width  $CanvaScreenX -height $CanvaScreenY -xscrollcommand "$ScroHor set" -yscrollcommand "$ScroVer set"
    
    set HauteurTotale 5
    set LargeurTotale 0
    set TailleLabel   0
    set LongestLabel  0
    foreach Look $LesLooks Element $Liste Texte $ListeDeTextes {
	set          FLook "$BouRad.$Look"
	frame       $FLook
	radiobutton $FLook.radioP -relief flat -variable ChoixMultipleRadio($Look) -value p -width 0
	radiobutton $FLook.radioA -relief flat -variable ChoixMultipleRadio($Look) -value a -width 0
	label       $FLook.text   -text $Texte -anchor "w"
	pack $FLook.radioP $FLook.radioA $FLook.text -side left 

	$BouRad create window 0 $HauteurTotale -anchor "nw" -window $FLook
	incr HauteurTotale $HauteurUneLigne
	set  TailleLabel [winfo reqwidth $FLook.text]
	if {$TailleLabel > $LongestLabel} {
	    set LaPlusLongueFrame "$FLook" 
	    set LongestLabel      $TailleLabel
	}
    }
    update
    grid $w.frame.radios         -row    0   -column  0 -rowspan 1 -columnspan 1 -sticky nsew
    grid $w.frame.yscroll        -row    0   -column  1 -rowspan 1 -columnspan 1 -sticky nsew
    grid $w.frame.xscroll        -row    1   -column  0 -rowspan 1 -columnspan 1 -sticky nsew
    grid rowconfig    $w.frame 0 -weight 1   -minsize 0
    grid columnconfig $w.frame 0 -weight 1   -minsize 0
    pack $w.frame                -expand yes -fill    both 

    #Mise a jour de la scroll region et de la taille du canvas
    #update  
    set  Taille1 [winfo reqwidth $LaPlusLongueFrame]
    set  Taille2 [winfo width    $LaPlusLongueFrame]
    if {$Taille1 > $Taille2} {set Taille $Taille1} else {set Taille $Taille2}
    $BouRad configure -width        [Mini $Taille                  $MaxScreenX]
    $BouRad configure -height       [Mini [expr {$HauteurTotale+10}] $MaxScreenY]
    $BouRad configure -scrollregion [list 0 0 $Taille $HauteurTotale]

    tkwait variable validOk
    destroy $w

    if {! $validOk} {return [list {} {}]}
    foreach Look $LesLooks {
	lappend LesPAIs [set ChoixMultipleRadio($Look)]
    }	
        
    set LesPresents {}
    set LesAbsents  {}
    foreach Look $LesLooks Element $Liste PAI $LesPAIs {
	if {$PAI == "p"} {lappend LesPresents $Element}
	if {$PAI == "a"} {lappend LesAbsents  $Element}
    }
    return [list $LesPresents $LesAbsents]
}

proc FaireSavoir {{Texte ""} {Titre ""}} {

    Wup "Petite boite de dialogue qui attend sa reponse sans bloquer tout le monde"
    Wup "Permet de retourner qq chose mais ne peut etre lance plusieurs fois car vwait fait attendre le suivant"

    global NomDeRetour
    
    set w [NomDe fenetre]

    if {$Titre==""} {set Titre $w}

    toplevel     $w
    wm geometry  $w +300+100
    wm title     $w "$Titre"
    wm resizable $w false false
    wm protocol  $w WM_DELETE_WINDOW "destroy $w"

    if {[string length $Texte] > 1000} {
	set Texte "[string range $Texte 0 499]\n ... \n ... [string range $Texte end-499 end]"
    }

    set frameLabel  "$w.framelabel"
    set frameButton "$w.framebutton"
    set Label       "$frameLabel.label"
    set Button      "$frameButton.b"
    frame  $frameLabel  -borderwidth "3" -relief "flat" -pady "2" -padx "10"
    frame  $frameButton -borderwidth "3" -relief "flat" -pady "2" -padx "10"
    label  $Label       -borderwidth "1" -relief "flat" -pady "4" -padx "5" -background "red" -text "$Texte" -wraplength "10i" -justify left -font "Courier -12 bold"
    button $Button      -text "Ok"       -command "set NomDeRetour($w) $w"
#    button $Button      -text "Ok"       -command "\
#	    set w \[winfo toplevel $Button\];\
#	    set NomDeRetour(\$w) \$w;\
#	    set Retour \[set NomDeRetour(\$w)\];\
#	    unset NomDeRetour(\$w);\
#	    after 10 destroy \$w;\
#	    return 1"

    pack $frameLabel  -fill both           -expand true
    pack $frameButton -fill both           -expand true
    pack $Label       -fill x    -side top -expand true
    pack $Button                 -side top

    vwait NomDeRetour($w)
    if {[info exists NomDeRetour($w)]} {
	set Retour [set NomDeRetour($w)]
	unset NomDeRetour($w)
    } else {
	set Retour ""
    }
    destroy $w
    return $Retour
}

proc FaireSavoir_Multi {{Texte ""} {Titre ""}} {

    Wup "Petite boite de dialogue qui attend sa reponse sans bloquer tout le monde"
    Wup "Attention permet de le lancer plusieurs fois en meme temps mais pas de return sous Windows"

    global NomDeRetour
    
    set w [NomDe fenetre]

    if {$Titre==""} {set Titre $w}

    toplevel     $w
    wm geometry  $w +300+100
    wm title     $w "$Titre"
    wm resizable $w false false
    wm protocol  $w WM_DELETE_WINDOW "after 10 destroy $w"

    if {[string length $Texte] > 1000} {
	set Texte "[string range $Texte 0 499]\n ... \n ... [string range $Texte end-499 end]"
    }

    set frameLabel  "$w.framelabel"
    set frameButton "$w.framebutton"
    set Label       "$frameLabel.label"
    set Button      "$frameButton.b"
    frame  $frameLabel  -borderwidth "3" -relief "flat" -pady "2" -padx "10"
    frame  $frameButton -borderwidth "3" -relief "flat" -pady "2" -padx "10"
    label  $Label       -borderwidth "1" -relief "flat" -pady "4" -padx "5" -background "red" -text "$Texte" -wraplength "10i" -justify left -font "Courier -12 bold"
    button $Button      -text "Ok"       -command "eval \[wm protocol $w WM_DELETE_WINDOW\]"

    pack $frameLabel  -fill both           -expand true
    pack $frameButton -fill both           -expand true
    pack $Label       -fill x    -side top -expand true
    pack $Button                 -side top

    return "$w"
}

proc FaireAttendre {w ti tx} {

    Wup "Permet de faire attendre le user pendant que le prog travaille"
    Wup "Fait apparaitre une boite de dialogue pour dire ce qui se passe"

    global oldGrab
    global oldFocus
    global grabStatus
 
    catch {destroy $w}
    
    toplevel $w
    wm title $w $ti

    if {[winfo viewable [winfo toplevel [winfo parent $w]]] } {
        wm transient $w [winfo toplevel [winfo parent $w]]
    }

    label $w.label -background red -text $tx -justify left
    pack  $w.label -padx 10 -pady 10
    
    wm withdraw $w
    set x [expr {[winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2  - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2  - [winfo vrooty [winfo parent $w]]}]

    wm geom $w +$x+$y
    wm deiconify $w

    update idletasks

    set oldFocus [focus]
    set oldGrab  [grab current $w]
    if {[string compare $oldGrab ""]} {
        set grabStatus [grab status $oldGrab]
    }

    grab  $w
    focus $w

    #update idletasks
    update
    #tkwait visibility $w.label
    
    return ""
}

proc FaireLaSuite {w} {

    Wup "Permet de faire attendre le user pendant que le prog travaille"
    Wup "Fait diapparaitre la boite de dialogue et remet le focus sur la precedente fenetre"

    global oldGrab
    global oldFocus
    global grabStatus

    catch {focus $oldFocus}
    catch {destroy $w}

    if {[string compare $oldGrab ""]} {
      if {[string compare $grabStatus "global"]} {
	  catch {grab $oldGrab}
      } else {
          catch {grab -global $oldGrab}
      }
  }

  update idletasks
  
  return ""
}

#########################################################
#                                                       #
#  Zoom                                                 #
#                                                       #
#  Zoom the canvas view, based on scale factor          #
#  and centerpoint and size of new viewport.            #
#  If the center point is not provided, zoom            #
#  in/out on the current window center point.           #
#                                                       #
#  This procedure uses the canvas scale function to     #
#  change coordinates of all objects in the canvas.     #
#                                                       #
#########################################################
proc zoom { canvas factorx factory \
	{xcenter ""} {ycenter ""} \
	{winxlength ""} {winylength ""} } {
    
    #########################################################
    #  If (xcenter,ycenter) were not supplied,              #
    #  get the canvas coordinates of the center             #
    #  of the current view.  Note that canvas               #
    #  size may have changed, so ask the window             #
    #  manager for its size                                 #
    #########################################################
    if { [string equal $xcenter ""] } {
	set winxlength [winfo width $canvas]
	set winylength [winfo height $canvas]
	set xcenter [$canvas canvasx [expr {$winxlength/2.0}]]
	set ycenter [$canvas canvasy [expr {$winylength/2.0}]]
    }
    
    #########################################################
    #  Scale all objects in the canvas                      #
    #  Adjust our viewport center point                     #
    #########################################################
    $canvas scale all 0 0 $factorx $factory
    set xcenter [expr {$xcenter * $factorx}]
    set ycenter [expr {$ycenter * $factory}]

    ZoomFacteur $canvas X $factorx
    ZoomFacteur $canvas Y $factory

    #########################################################
    #  Get the size of all the items on the canvas.         #
    #                                                       #
    #  This is *really easy* using                          #
    #      $canvas bbox all                                 #
    #  but it is also wrong.  Non-scalable canvas           #
    #  items like text and windows now have a different     #
    #  relative size when compared to all the lines and     #
    #  rectangles that were uniformly scaled with the       #
    #  [$canvas scale] command.                             #
    #                                                       #
    #  It would be better to tag all scalable items,        #
    #  and make a single call to [bbox].                    #
    #  Instead, we iterate through all canvas items and     #
    #  their coordinates to compute our own bbox.           #
    #########################################################
    set x0 1.0e30; set x1 -1.0e30 ;
    set y0 1.0e30; set y1 -1.0e30 ;
    foreach item [$canvas find all] {
	switch -exact [$canvas type $item] {
	    "arc" -
	    "line" -
	    "oval" -
	    "polygon" -
	    "rectangle" {
		set coords [$canvas coords $item]
		foreach {x y} $coords {
		    if { $x < $x0 } {set x0 $x}
		    if { $x > $x1 } {set x1 $x}
		    if { $y < $y0 } {set y0 $y}
		    if { $y > $y0 } {set y1 $y}
		}
	    }
	}
    }
    
    #########################################################
    #  Now figure the size of the bounding box              #
    #########################################################
    set xlength [expr {$x1-$x0}]
    set ylength [expr {$y1-$y0}]
    
    #########################################################
    #  But ... if we set the scrollregion and xview/yview   #
    #  based on only the scalable items, then it is not     #
    #  possible to zoom in on one of the non-scalable items #
    #  that is outside of the boundary of the scalable items#
    #                                                       #
    #  So expand the [bbox] of scaled items until it is     #
    #  larger than [bbox all], but do so uniformly.         #
    #########################################################
    foreach {ax0 ay0 ax1 ay1} [$canvas bbox all] {break}
    
    while { ($ax0<$x0) || ($ay0<$y0) || ($ax1>$x1) || ($ay1>$y1) } {
	# triple the scalable area size
	set x0 [expr {$x0-$xlength}]
	set x1 [expr {$x1+$xlength}]
	set y0 [expr {$y0-$ylength}]
	set y1 [expr {$y1+$ylength}]
	set xlength [expr {$xlength*3.0}]
	set ylength [expr {$ylength*3.0}]
    }
    
    #########################################################
    #  Now that we ve finally got a region defined with     #
    #  the proper aspect ratio (of only the scalable items) #
    #  but large enough to include all items, we can compute#
    #  the xview/yview fractions and set our new viewport   #
    #  correctly.                                           #
    #########################################################
    set newxleft [expr {($xcenter-$x0-($winxlength/2.0))/$xlength}]
    set newytop  [expr {($ycenter-$y0-($winylength/2.0))/$ylength}]
    $canvas configure -scrollregion [list $x0 $y0 $x1 $y1]
    $canvas xview moveto $newxleft
    $canvas yview moveto $newytop
    
    #########################################################
    #  Change the scroll region one last time, to fit the   #
    #  items on the canvas.                                 #
    #########################################################
    $canvas configure -scrollregion [$canvas bbox all]
}

#########################################################
#                                                       #
#  zoomMark                                             #
#                                                       #
#  Mark the first (x,y) coordinate for zooming.         #
#                                                       #
#########################################################
proc zoomMark {c x y} {
    global zoomArea
    set zoomArea(x0) [$c canvasx $x]
    set zoomArea(y0) [$c canvasy $y]
    $c create rectangle $x $y $x $y -outline black -tag zoomArea
}

#########################################################
#                                                       #
#  zoomStroke                                           #
#                                                       #
#  Zoom in to the area selected by itemMark and         #
#  itemStroke.                                          #
#                                                       #
#########################################################
proc zoomStroke {c x y} {
    global zoomArea
    set zoomArea(x1) [$c canvasx $x]
    set zoomArea(y1) [$c canvasy $y]
    $c coords zoomArea $zoomArea(x0) $zoomArea(y0) $zoomArea(x1) $zoomArea(y1)
}

#########################################################
#                                                       #
#  zoomArea                                             #
#                                                       #
#  Zoom in to the area selected by itemMark and         #
#  itemStroke.                                          #
#                                                       #
#########################################################
proc zoomArea {c x y} {
    global zoomArea
    
    #########################################################
    #  Get the final coordinates.                           #
    #  Remove area selection rectangle                      #
    #########################################################
    set zoomArea(x1) [$c canvasx $x]
    set zoomArea(y1) [$c canvasy $y]
    $c delete zoomArea
    
    #########################################################
    #  Check for zero-size area                             #
    #########################################################
    if {($zoomArea(x0)==$zoomArea(x1)) || ($zoomArea(y0)==$zoomArea(y1))} {
	return
    }
    
    #########################################################
    #  Determine size and center of selected area           #
    #########################################################
    set areaxlength [expr {abs($zoomArea(x1)-$zoomArea(x0))}]
    set areaylength [expr {abs($zoomArea(y1)-$zoomArea(y0))}]
    set xcenter [expr {($zoomArea(x0)+$zoomArea(x1))/2.0}]
    set ycenter [expr {($zoomArea(y0)+$zoomArea(y1))/2.0}]
    
    #########################################################
    #  Determine size of current window view                #
    #  Rq that canvas scaling always changes the coordinates# 
    #  into pixel coordinates, so the size of the current   #
    #  viewport is always the canvas size in pixels.        #
    #  Since the canvas may have been resized, ask the      #
    #  window manager for the canvas dimensions.            #
    #########################################################
    set winxlength [winfo width $c]
    set winylength [winfo height $c]
    
    #########################################################
    #  Calculate scale factors, and choose smaller          #
    #########################################################
    set xscale [expr {$winxlength/$areaxlength}]
    set yscale [expr {$winylength/$areaylength}]
    if { $xscale > $yscale } {
	set factor $yscale
    } else {
	set factor $xscale
    }
    
    #########################################################
    #  Perform zoom operation                               #
    #########################################################
    zoom $c $factor $factor $xcenter $ycenter $winxlength $winylength
}

proc ResetZoom {K} {

    Wup "Permet de faire un reset du zoom"

    set factorx [expr {1/[ZoomFacteur $K X]}]
    set factory [expr {1/[ZoomFacteur $K Y]}]

    zoom $K $factorx $factory
    return ""
}

proc ZoomFacteur {K Quoi {Valeur ""}} {
    
    Wup "Permet de stocker les facteurs de zoom"

    global ZoomFacteur
    
    if {$Quoi   == ""} {return ""}
    if {$Valeur == ""} {
	if {[info exists ZoomFacteur($K,$Quoi)]} {
	    return "[set ZoomFacteur($K,$Quoi)]"
	} else {
	    return "1.0"
	}
    } else {
	if {! [info exists ZoomFacteur($K,$Quoi)]} {
	    set ZoomFacteur($K,$Quoi) $Valeur
	} else {
	    set ZoomFacteur($K,$Quoi) [expr {[set ZoomFacteur($K,$Quoi)] * $Valeur}]
	}
    }
    return ""
}    