#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

########################################################
#   1.Programme de determination d oligo specifiques   #
#   2.Selection des meilleurs oligos                   #
#     Selection croisee entre 2 banques                #
#   3.Gestion de la sortie des resultats               #
#     Fichier delimite par des tabulations             #
#     Utilisable dans Excel                            #
#   4.Proc pour gerer les resultats du design          #
########################################################

#Monitor time consuming
#MODIF
#set initTime [clock clicks -milliseconds]
#set EndTime  [clock clicks -milliseconds]
#Espionne     "Time: [expr $EndTime - $initTime*1.0]"
#AjouteAuLog    "Time: [expr $EndTime - $initTime*1.0]"

######################################################
#   Programme de determination d oligo specifiques   #
######################################################
proc LogoCADO4MI {} {
    
    Wup "Logo, contact et copyright de CADO4MI"

    global LogoAffiche

    if {[info exists LogoAffiche] && $LogoAffiche=="1"} {return ""}

    AjouteAuLog ""
    AjouteAuLog "                                 _                _     "
    AjouteAuLog "   ____   ___   ____  ____      / /      _     _ |_|    "
    AjouteAuLog "  /  __| /   \\ |  _ \\/  _ \\    / / _    | \\   / | _ "
    AjouteAuLog " |  /   |  ^  || | \\ | / \\ |  / /_/ /_  |  \\_/  || | "
    AjouteAuLog " |  \\__ |  _  || |_/ | \\_/ | |___  ___/ | |   | || |  " 
    AjouteAuLog "  \\____||_| |_||____/\\____/     / /     |_|   |_||_/  "
    AjouteAuLog "                                \\/                     "
    AjouteAuLog ""
    AjouteAuLog "CADO4MI version [CADO4MI_Version], a program for oligonucleotide design."
    AjouteAuLog ""
    AjouteAuLog "Copyright (C) 2006 Jean Muller"
    AjouteAuLog ""
    AjouteAuLog "Information is available through documentation (www.bips.u-strasbg.fr/CADO4MI)"
    AjouteAuLog "Please feel free to contact us for any suggestions or bug reports."
    AjouteAuLog "email: jean.muller@igbmc.u-strasbg.fr"
    AjouteAuLog ""

    set LogoAffiche 1

    return ""
}

proc CADO4MI_Main {{args ""}} {

    Wup "Oriente CADO4MI vers l interface graphique ou la ligne de commande"
  
    if {$args == ""} {
	CADO4MI_Panel
	return ""
    } elseif {$args == "-check" | $args == "-CHECK"} {
	LogoCADO4MI
	AjouteAuLog [format "%-21s  %-12s" " -q  (Query)" "Your query sequence in fasta format \[String\]"] 
	AjouteAuLog [format "%-21s  %-12s" " -l  (Length)" "Length of designed oligonucleotide (and size of sliding window) \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" " -de (Distance)" "Distance from 3' end of all queries for restricted designing area(s) \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" " -ar (Area)" "List of begin-end points in all queries for restricted designing area(s) \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" "" "ex: 1 300 500 800 will allow design only within the two ranges"]
	AjouteAuLog [format "%-21s  %-12s" " -nb (nboligo)" "Number of oligonucleotide to keep at the end of process (default 100) \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" " -t  (Tm)" "Tm threshold (ex: 92.2) \[Real\]"]  
	AjouteAuLog [format "%-21s  %-12s" " -r  (TmRange)" "Tm range (ex: 5.0) \[Real\]"]
	AjouteAuLog [format "%-21s  %-12s" " -m  (TmMethod)" "Tm calculation method (NearestNeighbor, Wallace, DNADNA, DNARNA or RNARNA) \[String\]"]
	#AjouteAuLog [format "%-21s  %-12s" " -m  (TmMethod)" "Tm calculation method \[String\]"]
	#AjouteAuLog [format "%-21s  %-12s" "" "    NearestNeighbor 1e3DH/DS+Rln(ADN/4)-273.15) (default)"]
	#AjouteAuLog [format "%-21s  %-12s" "" "    Wallace 4(nG+nC)+2(nA+nT)]"]
	#AjouteAuLog [format "%-21s  %-12s" "" "    DNADNA  81.5+16.6log(Salt)+41.0(GC)-500/S-0.62F"]
	#AjouteAuLog [format "%-21s  %-12s" "" "    DNARNA  79.8+18.5log(Salt)+58.4(GC)-820/S-0.50F+11.8(GC)^2"]
	#AjouteAuLog [format "%-21s  %-12s" "" "    RNARNA  79.8+18.5log(Salt)+58.4(GC)-820/S-0.35F+11.8(GC)^2"]
	#AjouteAuLog [format "%-21s  %-12s" "" "    Salt, F (Formamide) and S (size in bases)"]
	AjouteAuLog [format "%-21s  %-12s" " -gl (GCLower)" "The minimum %GC accepted (ex: 35.0) \[Real\]"]
	AjouteAuLog [format "%-21s  %-12s" " -gu (GCUpper)" "The maximum %GC accepted (ex: 70.0) \[Real\]"]
	AjouteAuLog [format "%-21s  %-12s" " -p  (Prohib)" "Prohibited Sequence, must be a list of sequence (ex: AAAA TTTTT) \[String\]"]
	AjouteAuLog [format "%-21s  %-12s" " -b  (Database)" "Blast Database to assess specificity (ex: .nsq) \[String\]"]
	AjouteAuLog [format "%-21s  %-12s" " -w  (WorkingDir)" "Directory where to save your results \[String\]"]
	AjouteAuLog [format "%-21s  %-12s" " -qc (QueryCheck)" "Query validation in database with %Id and %Cover (1 or 0) \[Integer\]"] 
	AjouteAuLog [format "%-21s  %-12s" "" "Default is 1 (for yes)."] 
	AjouteAuLog [format "%-21s  %-12s" " -st (Step)" "Value between each move of the sliding window (default is 10) \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" " -sp (Specificity)" "Specificity threshold to prevent X-Hybridisation (default is 70.0) \[Real\]"]
	AjouteAuLog [format "%-21s  %-12s" " -nc (Contiguous)" "Specificity threshold for max contiguous identical bases (default is 15) \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" " -ts (Salt)" "Salt concentration (default 1.0, see Tm) \[Real\]"]
	AjouteAuLog [format "%-21s  %-12s" " -tf (Formamide)" "Formamide (default 12.3 %, see Tm) \[Real\]"]
	AjouteAuLog [format "%-21s  %-12s" " -tc (DNA)" "DNA concentration (default 0.000001, see Tm) \[Real\]"]
	AjouteAuLog [format "%-21s  %-12s" " -ev (BlastExpect)" "The Blast E-value limit (default is 100) \[Real\]"]
	AjouteAuLog [format "%-21s  %-12s" " -bw (BlastWord)" "The word length for Blast searches (default is 15) \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" " -ba (BlastNbAligned)" "The Blast sequence limit to align (default is 2000) \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" " -bs (BlastStrand)" "Blast option forward (1, default), reverse (2) or both (3) strands \[Integer\]"]
	AjouteAuLog [format "%-21s  %-12s" " -bf (BlastFilter)" "Blast filter activation, T for yes (default), F for no \[String\]"]
	AjouteAuLog "\nObligate parameters are: Query, Length, Tm, TmRange, Database and WorkingDir."

	exit
	return ""
    } else {
	LogoCADO4MI

	if {[llength $args]=="1"} {
	    if {[regexp -all {\-} [lindex $args 0]]>0} {
		set args [lindex $args 0]
	    }
	}

	#Recuperation et test de tous les arguments
	foreach {Option Valeur} $args {
	    #AjouteAuLog "O $Option V $Valeur"

	    switch -regexp -- $Option {
		"-q$"  {LesParametresDuDesign Query      $Valeur "Positionne"}
		"-l$"  {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -l use only \[Integer\]."
			exit
		    }
		    if {[regexp {\.[0-9]+} $Valeur] || [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.0f" $Valeur]
		    }
		    LesParametresDuDesign LengthOligo    $Valeur "Positionne"
		}
		"-de$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -de use only \[Integer\]."
			exit
		    }
		    if {[regexp {\.[0-9]+} $Valeur] || [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.0f" $Valeur]
		    }
		    LesParametresDuDesign Areas [list $Valeur End] "Ajoute"
		}
		"-ar$" {
		    set NbZones [llength $Valeur]
		    if {[expr {$NbZones%2}]} {
			AjouteAuLog "! Warning ! for option -ar you need pairs of begin-end points."
			exit
		    } else {
			foreach {b e} $Valeur {
			    LesParametresDuDesign Areas [list $b $e] "Ajoute"
			}
		    }
		}
		"-nb$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -nb use only \[Integer\]."
			exit
		    }
		    if {[regexp {\.[0-9]+} $Valeur] || [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.0f" $Valeur]
		    }
		    LesParametresDuDesign NbOligo         $Valeur "Positionne"
		}
		"-t$"  {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -t use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign TmLimit         $Valeur "Positionne"
		}
		"-r$"  {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -r use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign TmRange         $Valeur "Positionne"
		}
		"-gl$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -gl use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign GCContentMinus  $Valeur "Positionne"
		}
		"-gu$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -gu use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign GCContentPlus   $Valeur "Positionne"
		}
		"-b$"  {
		    if {$Valeur=="none"} {
			AjouteAuLog "Database is none, that means that you already have blastn files."
		    } elseif {![file exists $Valeur]} {
			AjouteAuLog "! Warning ! for option -b $Valeur does not exists."
			exit
		    }
		    LesParametresDuDesign DataBase        $Valeur "Positionne"
		}
		"-st$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -st use only \[Integer\]."
			exit
		    }
		    if {[regexp {\.[0-9]+} $Valeur] || [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.0f" $Valeur]
		    }
		    LesParametresDuDesign Step            $Valeur "Positionne"
		}
		"-sp$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -sp use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign IdLimit         $Valeur "Positionne"
		}
		"-nc$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -nc use only \[Integer\]."
			exit
		    }
		    if {[regexp {\.[0-9]+} $Valeur] || [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.0f" $Valeur]
		    }
		    LesParametresDuDesign NbContiguous    $Valeur "Positionne"
		}
		"-m$"  {    
		    set MethodTm [list "NearestNeighbor"  "Wallace" "DNADNA"  "DNARNA"  "RNARNA"]	
		    if {[lsearch -exact $MethodTm $Valeur]=="-1"} {
			AjouteAuLog "! Warning ! for option -m use only NearestNeighbor, Wallace, DNADNA, DNARNA or RNARNA."
			exit
		    }
		    LesParametresDuDesign TmMethod        $Valeur "Positionne"
		}
		"-p$"  {
		    if {[regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -p use only \[String\]."
			exit
		    }
		    LesParametresDuDesign Prohibited  $Valeur "Positionne"
		}
		"-ts$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -ts use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign TmSalt          $Valeur "Positionne"
		}
		"-tf$" {		 
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -tf use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign TmFormamide     $Valeur "Positionne"
		}
		"-tc$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -tc use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign TmConcentration $Valeur "Positionne"
		}
		"-w$"  {
		    if {![file exists $Valeur]} {
			AjouteAuLog "! Warning ! for option -w $Valeur does not exists."
			exit
		    }
		    LesParametresDuDesign WorkingDir      $Valeur "Positionne"
		}
		"-ev$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -ev use only \[Real\]."
			exit
		    }
		    if {! [regexp {\.[0-9]+} $Valeur] || ! [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.1f" $Valeur]
		    }
		    LesParametresDuDesign BlastExpect     $Valeur "Positionne"
		}
		"-bw$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -bw use only \[Integer\]."
			exit
		    }
		    if {[regexp {\.[0-9]+} $Valeur] || [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.0f" $Valeur]
		    }
		    LesParametresDuDesign BlastWord       $Valeur "Positionne"
		}
		"-ba$" {
		    if {! [regexp {[0-9]+} $Valeur]} {
			AjouteAuLog "! Warning ! for option -ba use only \[Integer\]."
			exit
		    }
		    if {[regexp {\.[0-9]+} $Valeur] || [regexp {[0-9]+\.} $Valeur]} {
			set Valeur [format "%.0f" $Valeur]
		    }
		    LesParametresDuDesign BlastNbAligned  $Valeur "Positionne"
		}
		"-bs$" {
		    if {$Valeur != "1" && $Valeur != "2" && $Valeur != "3"} {
			AjouteAuLog "! Warning ! for option -bs use only 1, 2 or 3."
			exit
		    }
		    LesParametresDuDesign BlastStrand     $Valeur "Positionne"
		}
		"-bf$" {
		    if {$Valeur != "T" && $Valeur != "F"} {
			AjouteAuLog "! Warning ! for option -bf use only T (True) or F (False)."
			exit
		    }
		    LesParametresDuDesign BlastFilter     $Valeur "Positionne"
		}
		"-qc$" {
		    if {$Valeur != "1" && $Valeur != "0"} {
			AjouteAuLog "! Warning ! for option -qc use only 0 or 1."
			exit
		    }
		    LesParametresDuDesign QueryCheck      $Valeur "Positionne"
		}
	    }
	}
    }

    #Positionnement des parametres par defaut (si existe ne positionne pas)
    LesParametresParDefaut
    if {[BonNombreDeParametres "AvecVerif"]} {
	MontreLesParametresDuDesign
	
	set Fichier [file join [LesParametresDuDesign WorkingDir "LaValeurMerci"] "Design.parameters"]
	#SauveLesParametresDuDesign $Fichier
	MenuSauveLesParametres $Fichier

	OligoLauncher 
    } else {
	#CADO4MI_Main "-check"
	MontreLesParametresDuDesign

	AjouteAuLog "! Warning ! Missing parameters"
	AjouteAuLog "Usage: CADO4MI -q Input -l Length -t Tm -r tmrange -d BlastDatabasePATH -w SavingDirectory"

	return ""
    }
    
    #On detruit toutes les infos
    #AskBLAST aussi ??
    #AskBlast unset
    LesParametresDuDesign UnsetAll

    exit 
}

proc OligoLauncher {} {

    Wup "Procedure qui lance tout Masking, BlastN, Specificite, Selection du meilleur oligo."

    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set Repnuctfa "[file join [RepertoireDuGenome] nuctfa]"
    if {[BonNombreDeParametres]} {
	foreach  Parametre [list Query DataBase WorkingDir BlastExpect BlastWord BlastNbAligned] {
	    set $Parametre [LesParametresDuDesign $Parametre "LaValeurMerci"]
	}
    } else {
	AjouteAuLog "Missing parameters."
	return ""
    }
    if {$Query=="ActiChip1.0" || [regexp -nocase "actichip1" $Query]} {set Query [ActiChip1_0]}
    if {$Query=="ActiChip2.0" || [regexp -nocase "actichip2" $Query]} {set Query [ActiChip2_0]}
    if {$Query=="ActiChip3.0" || [regexp -nocase "actichip3" $Query]} {set Query [ActiChip3_0]}

    if {[llength $Query]=="1"} {
	set  Return [CheckTFAFile $Query]
	if {$Return!=""} {
	    LesParametresDuDesign Query           "Reset"
	    LesParametresDuDesign Query "$Return" "Positionne"
	}
	set Query [LesParametresDuDesign Query    "LaValeurMerci"]
    }

    if {[regexp {\.[a-z]{3}$} $DataBase]} {regsub {\.[a-z]{3}$} $DataBase "" DataBase}
    set           Query [split $Query " "]
    foreach  PAB $Query {
	if {$PAB!="" && [file exists "$PAB"]} {
	    set FichierQuery   "$PAB"
	} elseif {[file exists "[file join $RepmRNA $PAB]"]} {
	    set FichierQuery   "[file join $RepmRNA $PAB]"
	} elseif {[file exists "[file join $Repnuctfa $PAB]"]} {
	    set FichierQuery   "[file join $Repnuctfa $PAB]"
	} else {
	    AjouteAuLog "No File for the Query ($PAB)"
	    continue
	}
	#Rq: On a plus le vrai chemin du fichier original
	if {[regexp {\\} $PAB] || [regexp {/} $PAB]} {set PAB [file tail $PAB]} 
	if {[regexp {\.[a-zA-Z]+$} $PAB]} {regsub {\.[a-zA-Z]+$} $PAB "" PAB}
	#if {[regexp {/} "$PAB"]} {set PAB [file tail $PAB]} 
	#if {[regexp {\.masked$} $PAB]} {regsub -all {\.masked$} $PAB "" PAB}

#	#Pour le design particulier de la banque RefSeq totale
#	if {[regexp {^NM[0-9]{4,}} $PAB]} {
#	    if {[LesParametresDuDesign WorkingDir "Existe"]=="0"} {
#		set Dir        "[file join [RepertoireDuGenome] Probes RefSeq_Design]"
#		set NMDIR      [LeRepertoireDeRefSeq $PAB] 
#		set WorkingDir "[file join ${Dir} ${NMDIR}]"
#	    }
#	}

	set FichierEnCours   "[file join $WorkingDir "${PAB}.working"]"
	set FichierMasque    "[file join $WorkingDir "${PAB}.masked"]"
	set FichierBlast     "[file join $WorkingDir "${PAB}.blastn"]"
	set FichierLog       "[file join $WorkingDir "${PAB}.log"]"
	set FichierOligo     "[file join $WorkingDir "${PAB}.oligo"]"
	set FichierSelection "[file join $WorkingDir "${PAB}.selection"]"

	if {[file exists $FichierEnCours]} {
	    AjouteAuLog "$PAB in work"
	    continue
	} elseif {[file exists $FichierLog] || [file exists $FichierOligo]} {
	    AjouteAuLog "$PAB seems to be done"
	    if {[OuiOuNonMemo "Would you like to redesign ?"]} {
		set ListeFichier ""
		if {$DataBase=="none"} {
		    set ListeFichier [list $FichierMasque $FichierLog $FichierOligo $FichierSelection]
		} else {
		    set ListeFichier [list $FichierMasque $FichierBlast $FichierLog $FichierOligo $FichierSelection]
		}
		foreach File $ListeFichier {
		    if {[file exists $File]} {file delete -force $File}
		}
	    } else {continue}
	    Sauve "" dans $FichierEnCours
	} else {
	    Sauve "" dans $FichierEnCours
	}
	AjouteAuLog ">>>In charge of $PAB<<<"

	#On filtre la sequence pour le poly A
	#####################################
	if {![file exists $FichierMasque]} {
	    AjouteAuLog "Masking Step for $PAB"
	    PrepareMaSequenceALaRechercheDHomologie $FichierQuery $FichierMasque
	    #set FichierMasque [PrepareMaSequenceALaRechercheDHomologie $FichierQuery $WorkingDir]
	} else {
	    AjouteAuLog "Masking Step already done"
	}
	
	#On recherche dans les banques avec Blast
	#########################################
	if {![file exists $FichierBlast]} {
	    if {$DataBase=="none" || $DataBase==""} {
		file delete -force $FichierEnCours
		AjouteAuLog "No database specified and no blastn files for $PAB"
		continue
	    }
	    AjouteAuLog "Blast Step for $PAB"

	    #Blastn du brin sens (S) uniquement avec une taille de mots (W)
	    set BlastStrand [LesParametresDuDesign BlastStrand "LaValeurMerci"]
	    set BlastFilter [LesParametresDuDesign BlastFilter "LaValeurMerci"]

	    set CmdBlastN "[CADO4MI_PATH blastall] \
		    -S $BlastStrand \
		    -W $BlastWord \
		    -i $FichierMasque \
		    -d $DataBase \
		    -p blastn \
		    -o $FichierBlast \
		    -e $BlastExpect \
		    -v $BlastNbAligned \
		    -b $BlastNbAligned \
		    -F $BlastFilter \
		    -a 4"
	    #Espionne "$CmdBlastN"

	    if {[catch {eval exec $CmdBlastN} Message]} {
		AjouteAuLog "Error in BlastN command:\n>>>>$Message"
		continue
	    }
	} else {AjouteAuLog "Blast Step already done"}
	
	#On teste la specificite et tous les autres parametres
	######################################################
	if {![file exists $FichierLog]} {
	    AjouteAuLog     "Analysis Step for $PAB"
	    CheckForOligo   "$FichierMasque" "$FichierBlast" "$FichierLog" "$FichierOligo"
	} else {AjouteAuLog "Analysis Step already done"}
	
	#On selectionne le meilleur
	###########################
	if {![file exists $FichierSelection]} {
	    AjouteAuLog        "Best oligo selection for $PAB"
	    SelectionDesOligos $PAB $WorkingDir
	} else {AjouteAuLog    "Best oligo selection already done"}
	InterrogeInfoDeLOligo $FichierOligo Decharge
	
	#On supprime les fichiers crees
	#file delete -force $FichierMasque
	#file delete -force $FichierBlast 
	file delete -force $FichierEnCours
	AjouteAuLog "<><End of $PAB><>\n"
    }
    AjouteAuLog "The End"

    OuiOuNonMemo UnsetAllPlease
    return ""
}

#####################################################################
#   Launcher pour lancer les etapes separemment sur tout le monde   #
#####################################################################
proc Oligo_MaskingStep_Launcher {{args ""}} {
    
    Wup "Pour faire les masked tous d un seul coup"

    if {$args==""} {
	AjouteAuLog "CADO4MI usage is query, database, length, range, nboligo and optional parameters"
	AjouteAuLog [format "%-21s  %-12s" " -q  (Query)" "Your query sequence in fasta format."] 
	AjouteAuLog [format "%-21s  %-12s" " -w  (WorkingDir)" "Choose the directory where to save your files"]
	AjouteAuLog "\nYou need to define at least Query, and WorkingDir"
	return ""
    }

    foreach {Option Valeur} $args {
	#AjouteAuLog "O $Option V $Valeur"
	switch -regexp -- $Option {
	    "-q$"  {LesParametresDuDesign Query      $Valeur "Positionne"}
	    "-w$"  {LesParametresDuDesign WorkingDir $Valeur "Positionne"}
	}
    }
    Oligo_MaskingStep_PourTous
      
    return ""
}

proc Oligo_MaskingStep_PourTous {} {

    Wup "Procedure qui lance le Masking pour tous"

    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set Repnuctfa "[file join [RepertoireDuGenome] nuctfa]"

    #Test si les parametres necessaires sont disponibles
    set OK 1
    foreach  Parametre [list Query WorkingDir] {
	set $Parametre [LesParametresDuDesign $Parametre "LaValeurMerci"]
	if {[set $Parametre]==""} {set OK 0;break}
    }
    if {$OK!="1"} {
	AjouteAuLog "Missing parameters.\nMasking Step requires the following parameters: Query and Saving Directory."
	return ""
    }

    if {[llength $Query]=="1"} {
	set  Return [CheckTFAFile $Query]
	if {$Return!=""} {
	    LesParametresDuDesign Query           "Reset"
	    LesParametresDuDesign Query "$Return" "Positionne"
	}
	set Query [LesParametresDuDesign Query    "LaValeurMerci"]
    }

    set           Query [split $Query " "]
    foreach  PAB $Query {
	if {$PAB!="" && [file exists "$PAB"]} {
	    set FichierQuery   "$PAB"
	} elseif {[file exists "[file join $RepmRNA $PAB]"]} {
	    set FichierQuery   "[file join $RepmRNA $PAB]"
	} elseif {[file exists "[file join $Repnuctfa $PAB]"]} {
	    set FichierQuery   "[file join $Repnuctfa $PAB]"
	} else {
	    AjouteAuLog "No File for the Query ($PAB)"
	    continue
	}
	#Rq: On a plus le vrai chemin du fichier original
	if {[regexp {\\} $PAB] || [regexp {/} $PAB]} {set PAB [file tail $PAB]} 
	if {[regexp {\.[a-zA-Z]+$} $PAB]} {regsub {\.[a-zA-Z]+$} $PAB "" PAB}

	set FichierEnCours   "[file join $WorkingDir "${PAB}.working"]"
	set FichierMasque    "[file join $WorkingDir "${PAB}.masked"]"
	set FichierBlast     "[file join $WorkingDir "${PAB}.blastn"]"
	set FichierLog       "[file join $WorkingDir "${PAB}.log"]"
	set FichierOligo     "[file join $WorkingDir "${PAB}.oligo"]"
	set FichierSelection "[file join $WorkingDir "${PAB}.selection"]"

	if {[file exists $FichierEnCours]} {
	    AjouteAuLog "$PAB in work"
	    continue
	} elseif {[file exists $FichierMasque]} {
	    AjouteAuLog "$PAB seems to be done"
	    if {[OuiOuNonMemo "Would you like to remask ?"]} {
		file delete -force $FichierMasque
	    } else {continue}
	    Sauve "" dans $FichierEnCours
	} else {
	    Sauve "" dans $FichierEnCours
	}
	AjouteAuLog ">>>In charge of $PAB<<<"
	
	#On filtre la sequence pour le poly A
	#####################################
	if {![file exists $FichierMasque]} {
	    AjouteAuLog "Masking $PAB"
	    PrepareMaSequenceALaRechercheDHomologie $FichierQuery $FichierMasque
	    #set FichierMasque [PrepareMaSequenceALaRechercheDHomologie $FichierQuery $WorkingDir]
	} else {
	    AjouteAuLog "Masking already done"
	} 
	file delete -force $FichierEnCours
	AjouteAuLog "<><End of $PAB><>\n"
    }

    OuiOuNonMemo UnsetAllPlease

    return ""
}

proc Oligo_BlastNStep_Launcher {{args ""}} {
    
    Wup "Pour faire les blastn pour tous d un seul coup"
    Wup "Il faut au moins la query, la database et le workingdir"

    if {$args==""} {
	AjouteAuLog "To launch BlastN for all query, usage is to define Query, Database, WorkingDir, BlastExpect, BlastWord, BlastNbAligned."
	AjouteAuLog [format "%-20s  %-12s" " -q  (Query)" "Your query sequence in fasta format."] 
	AjouteAuLog [format "%-20s  %-12s" " -b  (Database)" "Database in which specificity will be tested."]
	AjouteAuLog [format "%-20s  %-12s" " -w  (WorkingDir)" "Choose the directory where to save your files"]
	AjouteAuLog [format "%-20s  %-12s" " -ev (BlastExpect)" "The Blast E-value limit."]
	AjouteAuLog [format "%-20s  %-12s" "" "Default is 100."] 
	AjouteAuLog [format "%-20s  %-12s" " -bw (BlastWord)" "The word length for Blast searches."]
	AjouteAuLog [format "%-20s  %-12s" "" "Default is 15 nt."] 
	AjouteAuLog [format "%-20s  %-12s" " -ba (BlastNbAligned)" "The Blast sequence limit to align."]
	AjouteAuLog [format "%-20s  %-12s" "" "Default is 2000."] 
	AjouteAuLog [format "%-20s  %-12s" " -bs (BlastStrand)" "The Blast option to search on forward (1), reverse (2) or both (3) strands."]
	AjouteAuLog [format "%-20s  %-12s" "" "Default is 1."] 
	AjouteAuLog "\nYou need to define at least Query, Database and WorkingDir (3 other exists as default value)"
	return ""
    }

    #On positionne les parametres par default
    LesParametresParDefaut

    foreach {Option Valeur} $args {
	#AjouteAuLog "O $Option V $Valeur"
	switch -regexp -- $Option {
	    "-q$"  {LesParametresDuDesign Query          $Valeur "Positionne"}
	    "-b$"  {LesParametresDuDesign DataBase       $Valeur "Positionne"}
	    "-w$"  {LesParametresDuDesign WorkingDir     $Valeur "Positionne"}
	    "-ev$" {LesParametresDuDesign BlastExpect    $Valeur "Positionne"}
	    "-bw$" {LesParametresDuDesign BlastWord      $Valeur "Positionne"}
	    "-ba$" {LesParametresDuDesign BlastNbAligned $Valeur "Positionne"}
	    "-bs$" {LesParametresDuDesign BlastStrand    $Valeur "Positionne"}
	}
    }
    
    Oligo_BlastNStep_PourTous
    
    return ""
}

proc Oligo_BlastNStep_PourTous {} {

    Wup "Procedure qui lance seulement l etape de BlastN."

    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set Repnuctfa "[file join [RepertoireDuGenome] nuctfa]"

    set OK 1
    foreach  Parametre [list Query DataBase WorkingDir BlastWord BlastExpect BlastNbAligned BlastStrand BlastFilter] {
	set $Parametre [LesParametresDuDesign $Parametre "LaValeurMerci"]
	if {[set $Parametre]==""} {set OK 0;break}
    }
    if {$OK!="1"} {
	AjouteAuLog "Missing parameters.\nBlast Step requires the following parameters: Query, DataBase and Saving Directory."	
	return ""
    }
    
    if {[llength $Query]=="1"} {
	set  Return [CheckTFAFile $Query]
	if {$Return!=""} {
	    LesParametresDuDesign Query           "Reset"
	    LesParametresDuDesign Query "$Return" "Positionne"
	}
	set Query [LesParametresDuDesign Query    "LaValeurMerci"]
    }

    if {[regexp {\.[a-z]{3}$} $DataBase]} {regsub {\.[a-z]{3}$} $DataBase "" DataBase}
    set           Query [split $Query " "]
    foreach  PAB $Query {
	if {$PAB!="" && [file exists "$PAB"]} {
	    set FichierQuery   "$PAB"
	} elseif {[file exists "[file join $RepmRNA $PAB]"]} {
	    set FichierQuery   "[file join $RepmRNA $PAB]"
	} elseif {[file exists "[file join $Repnuctfa $PAB]"]} {
	    set FichierQuery   "[file join $Repnuctfa $PAB]"
	} else {
	    AjouteAuLog "No File for the Query ($PAB)"
	    continue
	}
	#Rq: On a plus le vrai chemin du fichier original
	if {[regexp {\\} $PAB] || [regexp {/} $PAB]} {set PAB [file tail $PAB]} 
	if {[regexp {\.[a-zA-Z]+$} $PAB]} {regsub {\.[a-zA-Z]+$} $PAB "" PAB}

	set FichierEnCours   "[file join $WorkingDir "${PAB}.working"]"
	set FichierMasque    "[file join $WorkingDir "${PAB}.masked"]"
	set FichierBlast     "[file join $WorkingDir "${PAB}.blastn"]"
	set FichierLog       "[file join $WorkingDir "${PAB}.log"]"
	set FichierOligo     "[file join $WorkingDir "${PAB}.oligo"]"
	set FichierSelection "[file join $WorkingDir "${PAB}.selection"]"

	if {[file exists $FichierEnCours]} {
	    AjouteAuLog "$PAB in work"
	    continue
	} 

	if {! [file exists $FichierMasque]} {
	    AjouteAuLog "Masking Step has not been performed. Please do it before running this step."
	    continue
	} 
	if {[file exists $FichierBlast]} {
	    AjouteAuLog "$PAB seems to be done"
	    if {[OuiOuNonMemo "Would you like to reblast ?"]} {
		file delete -force $FichierBlast
	    } else {continue}
	    Sauve "" dans $FichierEnCours
	} else {
	    Sauve "" dans $FichierEnCours
	}
	AjouteAuLog ">>>In charge of $PAB<<<"

	if {[file exists $FichierMasque]} {
	    AjouteAuLog "Masking Step already done."
	}
	
	#On recherche dans les banques avec Blast
	#########################################
	if {![file exists $FichierBlast]} {
	    if {$DataBase=="none" || $DataBase==""} {
		file delete -force $FichierEnCours
		#AjouteAuLog "No database specified and no blastn files for $PAB"
		continue
	    }
	    AjouteAuLog "Blast for $PAB"

	    #Blastn du brin sens (S) uniquement avec une taille de mots (W) avec/sans Filtre (F)
	    set BlastStrand [LesParametresDuDesign BlastStrand "LaValeurMerci"]
	    set BlastFilter [LesParametresDuDesign BlastFilter "LaValeurMerci"]

	    set CmdBlastN "[CADO4MI_PATH blastall] \
		    -S $BlastStrand \
		    -W $BlastWord \
		    -i $FichierMasque \
		    -d $DataBase \
		    -p blastn \
		    -o $FichierBlast \
		    -e $BlastExpect \
		    -v $BlastNbAligned \
		    -b $BlastNbAligned \
		    -F $BlastFilter \
		    -a 4"
	    if {[catch {eval exec $CmdBlastN} Message]} {
		AjouteAuLog "Error in BlastN command:\n>>>>$Message"
		continue
	    }
	} else {
	    AjouteAuLog "Blast already done"
	}
	file delete -force $FichierEnCours
	AjouteAuLog "<><End of $PAB><>\n"
    }

    OuiOuNonMemo UnsetAllPlease

    return ""
}

proc Oligo_AnalysisStep_PourTous {} {

    Wup "Procedure qui lance seulement l etape de BlastN."

    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set Repnuctfa "[file join [RepertoireDuGenome] nuctfa]"
    
    set OK 1
    foreach  Parametre [list Query LengthOligo TmLimit TmRange WorkingDir] {
	set $Parametre [LesParametresDuDesign $Parametre "LaValeurMerci"]
	if {[set $Parametre]==""} {set OK 0;break}
    }
    if {$OK!="1"} { 
	AjouteAuLog "Missing parameters.\nAnalysis Step requires the following parameters: Query, Oligo Length, Tm and Saving Directory."
	return ""
    }
    
    if {[llength $Query]=="1"} {
	set  Return [CheckTFAFile $Query]
	if {$Return!=""} {
	    LesParametresDuDesign Query           "Reset"
	    LesParametresDuDesign Query "$Return" "Positionne"
	}
	set Query [LesParametresDuDesign Query    "LaValeurMerci"]
    }

    set           Query [split $Query " "]
    foreach  PAB $Query {
	if {$PAB!="" && [file exists "$PAB"]} {
	    set FichierQuery   "$PAB"
	} elseif {[file exists "[file join $RepmRNA $PAB]"]} {
	    set FichierQuery   "[file join $RepmRNA $PAB]"
	} elseif {[file exists "[file join $Repnuctfa $PAB]"]} {
	    set FichierQuery   "[file join $Repnuctfa $PAB]"
	} else {
	    AjouteAuLog "No File for the Query ($PAB)"
	    continue
	}
	#Rq: On a plus le vrai chemin du fichier original
	if {[regexp {\\} $PAB] || [regexp {/} $PAB]} {set PAB [file tail $PAB]} 
	if {[regexp {\.[a-zA-Z]+$} $PAB]} {regsub {\.[a-zA-Z]+$} $PAB "" PAB}

	set FichierEnCours   "[file join $WorkingDir "${PAB}.working"]"
	set FichierMasque    "[file join $WorkingDir "${PAB}.masked"]"
	set FichierBlast     "[file join $WorkingDir "${PAB}.blastn"]"
	set FichierLog       "[file join $WorkingDir "${PAB}.log"]"
	set FichierOligo     "[file join $WorkingDir "${PAB}.oligo"]"
	set FichierSelection "[file join $WorkingDir "${PAB}.selection"]"

	if {  [file exists $FichierEnCours]} {
	    AjouteAuLog "$PAB in work"
	    continue
	} 
	if {! [file exists $FichierMasque]} {
	    AjouteAuLog "Masking Step has not been performed. Please do it before running this step."
	    continue
	}
	if {! [file exists $FichierBlast]} { 
	    AjouteAuLog "BLAST Step has not been performed. Please do it before running this step."
	    continue
	}
	if {[file exists $FichierLog] || [file exists $FichierOligo]} {
	    AjouteAuLog "$PAB seems to be done"
	    if {[OuiOuNonMemo "Would you like to reanalyse ?"]} {
		set ListeFichier [list $FichierLog $FichierOligo]
		foreach File $ListeFichier {
		    if {[file exists $File]} {file delete -force $File}
		}
	    } else {continue}
	    Sauve "" dans $FichierEnCours
	} else {
	    Sauve "" dans $FichierEnCours
	}
	AjouteAuLog ">>>In charge of $PAB<<<"

	if {[file exists $FichierMasque]} {
	    AjouteAuLog "Masking Step already done."
	}
	if {[file exists $FichierBlast]} {
	    AjouteAuLog "Blast already done"
	}
	#On teste la specificite et tous les autres parametres
	######################################################
	if {![file exists $FichierLog]} {
	    AjouteAuLog     "Analysis Step for $PAB"
	    CheckForOligo   "$FichierMasque" "$FichierBlast" "$FichierLog" "$FichierOligo"
	} else {AjouteAuLog "Analysis Step already done"}
	
	file delete -force $FichierEnCours
	AjouteAuLog "<><End of $PAB><>\n"
    }

    OuiOuNonMemo UnsetAllPlease
    
    return ""
}

proc PrepareMaSequenceALaRechercheDHomologie {FichierTFA {Out ""}} {
    
    Wup "Permet de virer les zones poly A douteuse pour le fasta"
    
    if {$Out==""} {
	set FichierMasked "${FichierTFA}.masked"
    } else {
	set FichierMasked $Out
	#set Queue [file tail $FichierTFA]
	#set FichierMasked "$Dir/${Queue}.masked"
    }
    
    set Seq [QueLaSequenceDuFichierTFA $FichierTFA]
    #Retourne les regions a masquer
    set LesRegionsAMasquer [MasquePolyN $Seq "A" 15]
    
    #Masque les regions dans le FichierMasked
    #Modification pour mettre les chaines en majuscules
    MasqueLesRegionsDuFichierTFA $FichierTFA $FichierMasked $LesRegionsAMasquer "N"
    
    return ""
}

proc MasquePolyN {Seq nt Limite} {

    Wup "Permet de detecter des regions contigues de base"
    Wup "nt A, T, G, C"
    Wup "Limite = nombre de base minimum successives a considerer"
    
    set LaSeq      [string toupper [split $Seq ""]]
    set Debut      0
    set Tab(Last)  N
    set Longueur   [llength $LaSeq]
    set MesRegions ""

    foreach base $LaSeq {
	incr Debut
	if {! [info exists Tab(Last)]} {set Tab(Last) N}
	if {$base !=  [set Tab(Last)]} {
	    unset Tab    
	    if {[info exists LesRegions(Debut)] &&  [info exists LesRegions(Fin)]} {
		lappend MesRegions "[set LesRegions(Debut)] [set LesRegions(Fin)]"
		unset LesRegions
	    }
	}
	if {  $base!=$nt}                            {continue}
	if {! [info exists Tab(Last)]}               {set Tab(Last) N}
	if {! [info exists     Tab($base)]}          {set Tab($base) 1} else {incr Tab($base)}
	if {  [set             Tab($base)]==1}       {set LesRegions(Debut) $Debut}
	if {$base==$nt && [set Tab($base)]>=$Limite} {set LesRegions(Fin)   $Debut}
	if {$Debut==$Longueur && $base==[set Tab(Last)]} {
	    if {$base!=$nt} {continue}
	    if {[info exists LesRegions(Debut)] && [set Tab($base)]>=$Limite} {
		lappend MesRegions "[set LesRegions(Debut)] [set LesRegions(Fin)]"
		unset LesRegions
		continue
	    }
	}
	set Tab(Last) "$base"
    }
    if {[info exists Tab]} {unset Tab}
    return $MesRegions
}

proc CheckForOligo {PAB FichierBlast {LogFile ""} {Output ""}} {
    
    Wup "Analyse la sequence pour le Tm, GC, Specificite ..."
    Wup "Ajout du classement de la sortie, des zones specifiques pour un design a facon."
    Wup "Ajout du choix de la query dans la banque (AccessDeRef) et design que pour cet access"
    Wup "Ajout de la gestion de AccessDeRef si vide"
    Wup "Ajout de la gestion des design ou il y trop de sequences par candidat au moment de la sauvegarde"
    Wup "Ajout de la gestion des zones de design specifiques (liste debut et fin) et globales (distance du 3prime)"
    Wup "06/02/2006"

    if {$PAB=="" || $FichierBlast==""} {return ""}
    if {    [file exists "$PAB"]} {
	set FichierQuery "$PAB"
    } else {AjouteAuLog "No File for the Query ($PAB)";return ""} 

    if {$LogFile == ""} {set LogFile "${PAB}.log"}
    if {$Output  == ""} {set Output  "${PAB}.oligo"}
    if {[file exists $Output] || [file exists $LogFile]} {AjouteAuLog "$PAB existe deja";return ""}
 
    #Les parametres du design
    #########################
    if {[BonNombreDeParametres]} {
	foreach  Parametre [list Step DataBase LengthOligo TmMethod TmLimit GCContentMinus GCContentPlus TmRange Prohibited NbOligo NbContiguous IdLimit] {
	    set $Parametre [LesParametresDuDesign $Parametre "LaValeurMerci"]
	}
    } else {return ""}
    
    if {[regexp {\.masked$} $PAB]} {regsub -all {\.masked$} $PAB "" PAB}
    set PAB             [file tail $PAB]
    set AccessDeLaBoite [AccessDeLaBoite $PAB]
    #set AccessDeLaBoite [AccessDeLaBoite [file tail $PAB]]

    set TmLimitSup      [expr {$TmLimit + $TmRange}]
    set TmLimitInf      [expr {$TmLimit - $TmRange}]
    if {[expr $TmLimitInf<0]} {set TmLimitInf 0}
    set CaractereMasque "N"
    set Prohibited      [linsert $Prohibited 0 $CaractereMasque]
    set Pas             $Step
    set Tab(OnEnTient)  0
    #Qiagen parameters
    #set NbContiguous 20
 
    #Charge les resultats du Blast
    ##############################
    set SeuilId      "None"
    set TailleQuery  [AskBlast $FichierBlast Taille]
    set NbSubject    [AskBlast $FichierBlast NbSubject]

    set LaSeq [split [QueLaSequenceDuFichierTFA $FichierQuery] ""]
    set EnteteQuery  [   PremiereLigneDuFichier $FichierQuery]

    #Si on vient de nuctfa
    if {[regexp ":" $EnteteQuery]} {regsub -all -nocase " :" $EnteteQuery "" EnteteQuery}	
    AppendAuFichier $LogFile "Query Length : $TailleQuery"
    AppendAuFichier $LogFile "Oligo Requested: $LengthOligo nt between $TmLimitInf and $TmLimitSup with less than $NbContiguous contiguous and $IdLimit for X-hybridisation"

    #Query Database Validation
    ##########################
    if {[LesParametresDuDesign QueryCheck "LaValeurMerci"]=="1"} {
	#Test de la presence de la Query dans le blast
	##############################################
	set AccessDeRef       ""
	if {$NbSubject>0} {
	    set PIdentity     0.95
	    set PRecouvrement 0.70
	    set LesBIdsBases  ""
	    set IdGlobal      0.0
	    set PCoverQuery   0.0
	    set PCoverSubject 0.0
	    
	    set     BId           [InterrogeBestHitInDatabase $FichierBlast InitialBId]
	    set  LesBIdsBases     [InterrogeBestHitInDatabase $FichierBlast LesBIdsBases]
	    if {$LesBIdsBases==""} {
		AppendAuFichier $LogFile "Query has no equivalent sequence in DataBase: No fitting sequence in Blast searches"
	    } else {
		#Pourcentage d identite global: Aligne sur Total dans les alignements 
		set  IdGlobal     [InterrogeBestHitInDatabase $FichierBlast IdGlobal]
		#Recouvrement de Q sur S ou inversement
		#PCoverQuery   Query   par rapport au Subject
		#PCoverSubject Subject par rapport au Query
		set PCoverQuery   [InterrogeBestHitInDatabase $FichierBlast PCoverQuery]
		set PCoverSubject [InterrogeBestHitInDatabase $FichierBlast PCoverSubject]
		set AccessDeRef   $BId
		
		if {$IdGlobal<$PIdentity || $PCoverQuery<$PRecouvrement || $PCoverSubject<$PRecouvrement} {
		    AppendAuFichier $LogFile "Query has no equivalent sequence in DataBase: $BId (\%Id [format "%.2f" $IdGlobal], Cover S/Q [format "%.2f" $PCoverQuery], Cover Q/S [format "%.2f" $PCoverSubject])."
		    #On ne met pas a zero pour preserver les cas limites de nos cutoffs
		    #set AccessDeRef ""
		    if {($PCoverQuery <= 0.2 || $PCoverSubject <= 0.2) || $IdGlobal <= 0.90} {
			#On met tout de meme a zero car trop limite
			set AccessDeRef ""
		    }
		} else {
		    AppendAuFichier $LogFile "Query has an equivalent sequence in DataBase: $BId (\%Id [format "%.2f" $IdGlobal], Cover S/Q [format "%.2f" $PCoverQuery], Cover Q/S [format "%.2f" $PCoverSubject])."
		}
	    }
	} else {
	    AppendAuFichier $LogFile "Query has no equivalent sequence in DataBase: No Hit found in Blast searches"
	    set AccessDeRef ""
	}
    } else {
	AppendAuFichier $LogFile "Query has maybe an equivalent sequence in DataBase but we did not check"
	set AccessDeRef ""
    }

    #Pour commencer a un autre endroit sur la Query
    ###############################################
    #Si il existe une zone particuliere pour la query on l utilise sinon on voit si il y en a une globale
    set     LesZones   [LesParametresDuDesign Areas $PAB "LaValeurMerci"]
    if {   $LesZones==""} {
	set LesZones   [LesParametresDuDesign Areas      "LaValeurMerci"]
	set LesTmps ""
	#2 cas possibles
	if {[regexp "End" $LesZones]} {
	    #1 On a une distance a partir de la fin 3 prime
	    set LesTmps  [lindex $LesZones 0]
	    set Distance [lindex $LesTmps  0]
	    set  Begin   [expr {$TailleQuery - $Distance}]
	    if {$Begin <= "0"} {set Begin 1}
	    
	    LesParametresDuDesign Areas $PAB [list $Begin $TailleQuery] "Ajoute"
	    set LesZones [list $Begin $TailleQuery]
	} else {
	    #2 On a des zones de design
	    set LesZones_Tmp ""
	    foreach Tmp $LesZones {
		set Begin ""
		set End   ""

		set Begin [lindex $Tmp 0]
		set End   [lindex $Tmp 1]

		if {$Begin <= "0"} {set Begin 1}
		
		LesParametresDuDesign Areas $PAB [list $Begin $End] "Ajoute"
		lappend LesZones_Tmp [list $Begin $End]
	    }
	    set LesZones $LesZones_Tmp
	}
    }

    if {   $LesZones == ""} {
	set LesZones   [list 1 $TailleQuery]
    } else {
	set LesTmps    [lsort -integer -increasing -index 0 $LesZones]
	set FinAvant   ""
	set DebutAvant ""
	set LesZones   ""
	set NbZones    [llength $LesTmps]
	set nb         0
	foreach Tmp $LesTmps {
	    set EdgeBegin [lindex $Tmp 0]
	    set EdgeEnd   [lindex $Tmp 1]
	    incr nb

	    if {$FinAvant=="" && $DebutAvant==""} {
		set DebutAvant $EdgeBegin
		set FinAvant   $EdgeEnd
		if {$nb==$NbZones} {lappend LesZones [list $DebutAvant $FinAvant]}
		continue
	    }
	    if {$EdgeBegin > $FinAvant} {
		lappend LesZones [list $DebutAvant $FinAvant]
		set DebutAvant $EdgeBegin
		set FinAvant   $EdgeEnd
	    } elseif {$EdgeEnd> $FinAvant} {
		set FinAvant   $EdgeEnd
	    }
	    if {$nb==$NbZones} {lappend LesZones [list $DebutAvant $FinAvant]}
	    continue
	}
    }

    #Analyse de chaque zone
    #######################
    foreach Zones $LesZones {
	set DebutDesign 1
	set Fin         $LengthOligo
	set FinDesign   $TailleQuery
	set EdgeBegin   [lindex $Zones 0]
	set EdgeEnd     [lindex $Zones 1]
	if {[info exists EdgeBegin] && $EdgeBegin!="" && $EdgeBegin>$DebutDesign} {set DebutDesign $EdgeBegin}
	if {[info exists EdgeEnd]   && $EdgeEnd  !="" && $EdgeEnd  <$TailleQuery} {set FinDesign   $EdgeEnd}
		
	#On teste la zone de design
	###########################
	for {set Debut $DebutDesign} {$Fin <= $FinDesign} {incr Debut $Pas} {
	    set nbGarde          0
	    set nbVire           0
	    set ProhibOnContinue 0

	    set Fin [expr {$Debut  + $LengthOligo - 1}] 
	    if {!   [expr $Fin   <= $FinDesign]} {break}
	    
	    set Seq [join [lrange $LaSeq [expr {$Debut - 1}] [expr {$Fin - 1}]] ""]
	    
	    AppendAuFichier $LogFile "\nMoving on: $Debut $Fin"
	    AppendAuFichier $LogFile "Sequence: $Seq"
	    
	    #Fin de la sequence ou non
	    ##########################
	    if {[string length $Seq]<$LengthOligo} {AppendAuFichier $LogFile "Length: TOOSHORT";continue} else {AppendAuFichier $LogFile "Length: OK"}
	    
	    #Module Determination de Sequences Prohibees
	    ############################################
	    if {$Prohibited!=""} {
		foreach Prohib $Prohibited {
		    if {[regexp $Prohib $Seq]} {set ProhibOnContinue 1;break}
		}
		if {$ProhibOnContinue==1} {
		    AppendAuFichier $LogFile "Prohibited: PROHIBES"
		    continue
		} else {
		    AppendAuFichier $LogFile "Prohibited: OK"}
	    } else {AppendAuFichier $LogFile "Prohibited: OK"}
	    
	    #Module Determination du GC Content
	    ###################################
	    if {[info exists GCContentMinus] && [info exists GCContentPlus] && $GCContentMinus!="" && $GCContentPlus!=""} {
		set xGC [NombreDeBaseDe $Seq "G|C"]
		set GC  [format "%.1f" [expr {(($xGC)/($LengthOligo*1.0))*100.0}]]
		if {$GC<$GCContentMinus || $GC>$GCContentPlus} {
		    AppendAuFichier $LogFile "GC: $GC Not OK"
		    continue
		} else {
		    AppendAuFichier $LogFile "GC: $GC OK"
		}
	    }
	    
	    #Module Calcul du Tm
	    ####################
	    set  Tm [CalculTmDeLaSeq $Seq $TmMethod]
	    if {$Tm ==""} {
		AppendAuFichier $LogFile "Tm: No Tm calculation"
		continue
	    } else {
		set Tm [format "%.1f" $Tm]
	    }

	    if {$Tm=="-1"} {
		AppendAuFichier $LogFile "Tm: Illegal caracter for Tm calculation"
		continue
	    } elseif {$Tm>$TmLimitSup} {
		AppendAuFichier $LogFile "Tm: $Tm > $TmLimitSup"
		continue
	    } elseif {$Tm<$TmLimitInf} {
		AppendAuFichier $LogFile "Tm: $Tm < $TmLimitInf"
		continue
	    } else {
		AppendAuFichier $LogFile "Tm: $Tm OK"
	    }
	
	    #Traitements cas particuliers
	    #############################
	    #1-Pas de hit du tout dans le blast
	    #  On ne calcul donc pas de specificite mais au moins le reste
	    if {$NbSubject==0 && $nbGarde>=0} {
		if {$nbGarde==0} {set Tab($Debut,$Fin,candidat)   "No Hit found during DataBase searches"}
		if {   ! [info exists Tab($Debut,$Fin,contigue)]} {set Tab($Debut,$Fin,contigue) 0}
		set Tab($Debut,$Fin,oligo)    $Seq
		set Tab($Debut,$Fin,Tm)       $Tm
		#set Tab($Debut,$Fin,GC)       $GC
		set Tab(OnEnTient)            1
		set Tab($Debut,$Fin,nbGarde)  $nbGarde
		lappend Tab($nbGarde)         "$Debut $Fin"
		continue
	    }
	    
	    #Module Calcul de la Specificite
	    ################################
	    set  LesInfos          ""
	    set  LesPetitesRegions [list [list $Debut $Fin]]
	    set  LesInfos          [LesInfosDesRegionsDansLeBlast $FichierBlast $LesPetitesRegions All $SeuilId]
	    if {$LesInfos!=""} {
		#On a des hits dans cette zone
		#Cette zone est sans l AccessDeRef donc pas la peine de la traiter 
		#Si AccessDeRef est vide on passe quand meme 
		if {$AccessDeRef==""} {
		    #On calcul quand meme
		} elseif {! [regexp -nocase "$AccessDeRef" $LesInfos]} {
		    AppendAuFichier $LogFile "No query $AccessDeRef in this part $Debut $Fin"
		    continue
		}
		
		foreach Ligne $LesInfos {
		    #Description des informations rendues par LesInfosDesRegionsDansLeBlast
		    #1  60 3       1     NM001100 {Plus/Plus} +    67 126 60     60        60
		    #DQ FQ NAliTot NArea BId      Orient      Sens DS FS  NDsAli NDsAliTot NCont
		    #Pour Pallier au probleme de plusieurs Access pour un seul Id
		    #1 60 77 2 RSHUM:CD97 {Plus / Minus} - {147 147} {157 157} 14 22 3

		    set Facteur      [llength [lindex $Ligne 8]]
		    set NbBIdentique ""
		    set NbBAlignee   ""
		    set NbContigue   ""
		    set NbBIdentique [lindex $Ligne  9]
		    set NbBAlignee   [lindex $Ligne 10]
		    set NbContigue   [lindex $Ligne 11]
		    set IdGlobal     [format "%.2f" [expr {((1.0*$NbBIdentique)/($LengthOligo*$Facteur))*100.0}]]

		    #Petit test de la query
		    #On teste si la query est suffisemment presente dans cette region
		    set  LeCandidat   [lindex $Ligne 4]
		    if {$LeCandidat=="$AccessDeRef"} {
			if {$IdGlobal<70.0} {continue}
		    }
		    
		    #On est pas assez identique
		    if {$IdGlobal<=$IdLimit} {
			#Mais on a trop de bases identiques successives
			if {$NbContigue >= $NbContiguous} {
			    AppendAuFichier $LogFile "Spec: We keep TOOMUCHCONTIGUE    $Ligne"
			    AppendAuFichier $LogFile "\tID G $IdGlobal"
			    
			    incr nbGarde
			    #On compte le nombre de sequences de 15 nts contigues
			    if {![info exists Tab($Debut,$Fin,contigue)]} {
				set Tab($Debut,$Fin,contigue) 1
			    } else {incr Tab($Debut,$Fin,contigue)}
			    
			    lappend Tab($Debut,$Fin,candidat) [lindex $Ligne 4]
			    continue
			}
			AppendAuFichier $LogFile "Spec: We left NOTENOUGHIDENTICAL $Ligne"
			incr nbVire
			continue
		    }
		    #On compte le nombre de sequences de 15 nts contigues meme pour ceux gardes avec IdGlobal important
		    if {$NbContigue >= $NbContiguous} {
			if {![info exists Tab($Debut,$Fin,contigue)]} {
			    set Tab($Debut,$Fin,contigue) 1
			} else {incr Tab($Debut,$Fin,contigue)}
		    }
		    AppendAuFichier $LogFile "Spec: We keep TOOMUCHIDENTICAL   $Ligne"
		    AppendAuFichier $LogFile "\tID G $IdGlobal"	
		    incr nbGarde
		    
		    lappend Tab($Debut,$Fin,candidat) [lindex $Ligne 4]
		}
		AppendAuFichier $LogFile ">>>\tWe keep $nbGarde\tWe left $nbVire"
	    }

	    if {$nbGarde>=0} {
		#Cas particulier pour un subject ayant plusieurs hits au sein de la meme sequence
		if {[info exists Tab($Debut,$Fin,candidat)]} {
		    set NBALTER [llength [lsort -unique [set Tab($Debut,$Fin,candidat)]]]
		    if {$nbGarde != $NBALTER} {
			set nbGarde $NBALTER
			set Tab($Debut,$Fin,candidat) [lsort -unique [set Tab($Debut,$Fin,candidat)]]
		    }
		}

		#Tous les cas particuliers
		if {$LesInfos==""} {
		    #1 - On n a pas de hits dans le blast dans cette zone
		    #    On le garde quand meme
		    AppendAuFichier $LogFile      "$Debut $Fin NoHitFound"
		    set Tab($Debut,$Fin,candidat) "No Hit found during DataBase searches"
		} else {
		    #2 - On a des hits dans le blast dans cette zone
		    if {$nbGarde==0} {
			#2 - 1 - On a des hits mais aucun suffisamment proche pour le retenir
			#        On le garde
			AppendAuFichier $LogFile      "$Debut $Fin NoHitWithEnoughIdentity"
			set Tab($Debut,$Fin,candidat) "No Hit found or retained during DataBase searches"
		    } else {
			#Equivalent a nbGarde > 0
			#2 - 2 - On a des hits et au moins un suffisamment proche pour le(s) retenir
			if {$AccessDeRef==""} {
			    #2 - 2 - 1 - On le garde quand meme car il n y a pas de query
			    AppendAuFichier $LogFile "$Debut $Fin NoHitWithEnoughIdentity"
			} elseif {! [regexp -nocase "$AccessDeRef" [set Tab($Debut,$Fin,candidat)]]} {
			    #2 - 2 - 2 - Cette zone ne contient pas (ou pas suffisamment) notre AccessDeRef
			    #On ne le garde pas
			    AppendAuFichier $LogFile "No query $AccessDeRef in this Candidat $Debut $Fin"
			    continue   
			}
		    }
		}
		if {! [info exists Tab($Debut,$Fin,contigue)]} {set Tab($Debut,$Fin,contigue) 0}
		set Tab($Debut,$Fin,oligo)   $Seq
		set Tab($Debut,$Fin,Tm)      $Tm
		#set Tab($Debut,$Fin,GC)      $GC
		set Tab(OnEnTient)           1
		set Tab($Debut,$Fin,nbGarde) $nbGarde
		lappend Tab($nbGarde)        "$Debut $Fin"
	    }
	}
    }
    
    set FirstTime 1
    #On va classer et sauvegarder les oligos si on en a trouve
    ##########################################################
    if {[set Tab(OnEnTient)] == 0} {
	#Pour les sequences sans oligos
	###############################
	scan $EnteteQuery "%s" PAB
	regsub {\>} $PAB "" PAB

	regsub ">$PAB" $EnteteQuery ">$PAB NoOligo Available" EnteteQuery
	AppendAuFichier "$Output" $EnteteQuery
	AppendAuFichier "$Output" [join $LaSeq ""]
    } else {
	#Pour les sequences avec oligos
	###############################
	while {1} {
	    if {$FirstTime} {
		set FirstTime 0
		#On enregistre dans le fichier de sortie les oligos avec moins de 30 candidats (MaxToStore) de specificite
		#On garde tout ceux qui ont 0, 1 ou 2 candidat(s) (MinimumAGarder), et pour ceux au dela jusqu a NbOligo
		set MinimumAGarder 2
		set CompteurDOligo 0
		set NumeroDeLOligo 1
		set OnAFini        0
		set OnEnASauverUn  0
		set MaxToStore     30
		#j nombre de candidats maximum a garder par oligo
		#CompteurDOligo et nbOligo nombre de d oligos a garder
	    } else {
		#On est au dessus de 30 candidats de specificite par oligo (MaxToStore)
		#Mais on a des candidats donc on recupere les categories de nb candidats (le j)
		set LesNbGarder       ""
		set LesEntetesNbGarde [array get Tab "*,*,nbGarde"]
		foreach {E NbGarder} $LesEntetesNbGarde {
		    lappend LesNbGarder $NbGarder
		}
		set LesNbGarder [lsort -unique -increasing -integer $LesNbGarder]

		#On va enregistrer dans le fichier de sortie les oligos en desespoir les 5 moins bon
		set NbOligo        5
		set MinimumAGarder 2
		set CompteurDOligo 0
		set NumeroDeLOligo 1
		set OnAFini        0
		set OnEnASauverUn  0
		set MaxToStore     [lindex $LesNbGarder end]
		#j nombre de candidats maximum a garder par oligo
		#CompteurDOligo et nbOligo nombre de d oligos a garder
	    }

	    for {set j 0} {$j <= $MaxToStore} {incr j} {
		#On recherche toutes les positions avec j candidats pour cet oligo
		if {$OnAFini=="1"} {break}
		if {[info exists Tab($j)]} {
		    set LesPositions ""
		    set MesPositions ""
		    set MesPositions [set Tab($j)]
		    
		    foreach PosDeJ $MesPositions {
			set MonDebut   [lindex $PosDeJ 0]
			set MaFin      [lindex $PosDeJ 1]
			set NbContigue [set Tab($MonDebut,$MaFin,contigue)]
			lappend LesPositions "$PosDeJ $NbContigue"
		    }
		    #On trie selon le nombre de sequences contigues
		    set LesPositions [lsort -integer -increasing -index 2 $LesPositions]
		    
		    foreach UnePosition $LesPositions {
			if {$j>$MinimumAGarder} {
			    incr CompteurDOligo
			    if {$CompteurDOligo>$NbOligo} {
				set OnAFini 1
				break
			    }
			}
			set  EnteteDeLOligo ""
			set  NomDeLOligo    "${PAB}_Oligo${NumeroDeLOligo}_${AccessDeLaBoite}"
			#set  NomDeLOligo    "Oligo${NumeroDeLOligo}_${AccessDeLaBoite}"
			incr NumeroDeLOligo
			set  UnDebut        [lindex $UnePosition 0]
			set  UneFin         [lindex $UnePosition 1]
			set  Contigue       [lindex $UnePosition 2]

			#Format: EnteteQuery Taille Tm debut_fin candidats_nbcontigue candidats
			#        Separation par des tabulations, relecture par InterrogeInfoDeLOligo
			lappend EnteteDeLOligo [linsert $EnteteQuery 1 "$NomDeLOligo"]
			lappend EnteteDeLOligo $LengthOligo
			lappend EnteteDeLOligo "[set Tab($UnDebut,$UneFin,Tm)]"
			#lappend EnteteDeLOligo "[set Tab($UnDebut,$UneFin,GC)]"
			lappend EnteteDeLOligo "$UnDebut $UneFin"
			lappend EnteteDeLOligo "[set Tab($UnDebut,$UneFin,nbGarde)] $Contigue"
			lappend EnteteDeLOligo [join [set Tab($UnDebut,$UneFin,candidat)] " "]
			set     EnteteDeLOligo [join $EnteteDeLOligo "\t"]
			
			AppendAuFichier $Output $EnteteDeLOligo
			AppendAuFichier $Output "[set Tab($UnDebut,$UneFin,oligo)]"
			
			set OnEnASauverUn 1
			continue
		    }
		} else {continue}
	    }
	    if {$OnEnASauverUn} {break}
	}
	if {[info exists Tab]}  {unset Tab}
    }
    AskBlast         unsetfile $FichierBlast
    InterrogeBestHitInDatabase $FichierBlast Decharge 
    if {[info exists Tab]}      {unset Tab}
    if {[info exists LesInfos]} {unset LesInfos}
    
    AppendAuFichier $LogFile "FIN"
    return ""
}

proc SelfComplementarity_GUI {K PAB} {
    
    Wup "Test le repliement contre elle meme pour l interface graphique"

    set Answer        ""
    set LesNomsOligos ""
    while {1} {
	set Texte  "Please choose with Control+1 the oligo(s) to test:\nThen click on Ok."
	set Answer [FaireSavoir $Texte "Self Complementary - Waiting your selection."]

	set  LesNomsOligos [LesOligosSelectionnes $K LesIds]
	if {$LesNomsOligos==""} {
	    if {! [OuiOuNon "No Oligo selected !\nWould you like to retry ?"]} {break} else {continue}
	}
	break
    }
    
    set LesFen ""
    foreach Id $LesNomsOligos { 
	set MesTags  [$K gettags $Id]
	set NomOligo [lindex $MesTags 4]
	set Dir      [lindex $MesTags 5]
	
	set Fichier "[file join $Dir "${PAB}.oligo"]"
	if {! [file exists $Fichier]} {
	    set Fichier "[file join $Dir $PAB]"
	    if {! [file exists $Fichier]} {return ""}
	}
	set Seq [InterrogeInfoDeLOligo $Fichier $NomOligo Sequence]
	set Fen [SelfComplementarity $Seq "" "$PAB $NomOligo"]

	lappend LesFen $Fen
    }
    return "$LesFen"
}

proc SelfComplementarity {Query {AvecAffichage ""} {Titre ""}} {

    Wup "Test le repliement de la sequence query sur elle meme"

    #On cherche a detecter une sequence complementaire dans notre sequence
    #Il faut donc avec (blast ou fasta ou) regexp detecter une sequence identique dans le complementaire
    #On prend le reverse pour respecter le sens des sequence 5--3 et 3--5 et donc de l appariement
    #Position Q        5      14             30     37 
    #Normal     5-D----ATGCCCCG--------------CGGGGCAT-----F-3
    #                  ||||||||              ||||||||
    #Complement 3-D----TACGGGGC--------------GCCCCGTA-----F-5
    #                  ||||||||              ||||||||
    #Reverse    5-F----ATGCCCCG--------------CGGGGCAT-----D-3
    #Position Q        37     30             14     5
    #
    #On fasta notre sequence contre elle meme
    #le -i permet d avoir notre query en reverse and complement
    #    ____  
    #   /    \ 
    #  |      |
    #  |      |
    #   \    /
    #    |--|
    #    |--|  Hairpin
    #    |--|
    #    |--|
    #    D  F
    #    |  |
    #    5  3

    if {$AvecAffichage=="" || $AvecAffichage=="1"} {set AvecAffichage 1} else {set AvecAffichage 0}
    if {! [file exists $Query]} {
	set LaSeq [join $Query ""]
	set NomDuFichier "N/A"
    } else {
	set LaSeq        [join [QueLaSequenceDuFichierTFA $Query] ""]
	set NomDuFichier [file tail $Query]
    }
    if {$Titre == ""} {
	if {$NomDuFichier != ""} {set Titre $NomDuFichier} else {set Titre "Self Complementary Test"}
    }

    set LaSeq       [string toupper $LaSeq]
    set TailleQuery [string length  $LaSeq]
    set LaSeqComp   [NucToReverseAndComplementNuc $LaSeq]

    set MaxContigue 6
    #Qiagen stem length in loop is 9
    set Fin [expr {1 + $MaxContigue - 1}]
    for {set Debut 1} {$Fin <= $TailleQuery} {incr Debut 1} {
	set HairPin  0
	set NewDebut ""
	set OldFin   ""
	set Fin [expr {$Debut  + $MaxContigue - 1}] 
	if {!   [expr $Fin   <= $TailleQuery]} {break}
	#Une recherche avec le nombre de fois trouve et les dernieres coordonnees
	if {1} {
	    #On recherche a etendre au maximum notre sequence complementaire
	    while {1} {
		set tmp       ""
		set NbMatches ""
		set TmpSeq    ""
		if {! [expr $Fin <= $TailleQuery]} {
		    set TmpSeq    [string range $LaSeq [expr {$Debut - 1}] [expr {$Fin - 1}]]
		    set NbMatches [regexp -indices -all "$TmpSeq" $LaSeqComp tmp]
		}

		#On verifie si la sequence n est pas un palindrome
		set          TailleSeq [string length $TmpSeq]
		if {! [expr {$TailleSeq % 2}]} {
		    #Si pas impair donc pair
		    set FirstHalfSeq  [string range $TmpSeq                   0 [expr {($TailleSeq/2) -1}]]
		    set SecondHalfSeq [string range $TmpSeq [expr {$TailleSeq/2}]                      end]
		    set  FirstComp    [NucToReverseAndComplementNuc $FirstHalfSeq]
		    if {$FirstComp==$SecondHalfSeq} {set NbMatches 0}
		}
		if {$NbMatches =="0"} {
		    if {$OldFin== ""} {
			#Espionne "meme fin"
			break
		    }
		    set Fin       $OldFin
		    set  NewDebut [expr {$Fin - $MaxContigue + 1}]
		    if {$NewDebut<$Debut} {set NewDebut ""}
		    break
		}
		set OldFin $Fin
		incr        Fin 
	    }
	}
	set tmp       ""
	set NbMatches ""
	set TmpSeq    ""
	#On recherche les coordonnees du plus long
	set TmpSeq    [string range $LaSeq [expr {$Debut - 1}] [expr {$Fin - 1}]]
	set NbMatches [regexp -indices -all "$TmpSeq" $LaSeqComp tmp]

	set AfterFirstTime 0
	if {$NbMatches> 1} {set Multiple 1} else {set Multiple 0}
	if {$NbMatches>=1} {
	    while {$NbMatches > 0} {
		foreach {BorneGComp BorneDComp} $tmp {
		    #Coordonnees liste (0 a 59) en reverse et passage en normal (1 a 60)
		    set FComp [expr {$TailleQuery - $BorneGComp - 1 + 1}]
		    set DComp [expr {$TailleQuery - $BorneDComp - 1 + 1}]
		    #Espionne "Attention $TmpSeq en double $Debut $Fin $tmp $DComp $FComp"
		    if {$Fin<$DComp} {set HairPin 1} else {if {$FComp<$Debut} {set HairPin 1}}
		}
		if {$HairPin} {
		    set MonComp [string range $LaSeq [expr {$DComp - 1}] [expr {$FComp - 1}]]
		    #Espionne "$TailleQuery Attention $TmpSeq/$MonComp seq complementaire $Debut/$Fin $DComp/$FComp"
		    if {$Multiple=="1" || (! [info exists TabDesHairPin($Debut,$Fin)] && ! [info exists TabDesHairPin($DComp,$FComp)])} {
			#On ne stock que les sequences presentes dans un sens
			#Ex: 25/30 matche 55/60 est stocke mais 55/60 qui matche aussi 25/30 ne sera pas stocke
			#On stock les coordonnees reelles sur la sequences de 1 a 60
			set TabDesHairPin($Debut,$Fin)   1
			set TabDesHairPin($DComp,$FComp) 1
			set TabDesHairPin($Fin)          1
			set TabDesHairPin($Debut)        $Fin
			lappend TabDesHairPin($Debut,$Fin,Complementaire) [list $DComp $FComp]
			lappend TabDesHairPin(LesPosHairPin)              [list $Debut $Fin]
		    } else {
			#Espionne "On passe pas par le stockage $Debut/$Fin $DComp/$FComp"
		    }
		} else {
		    #Espionne "Overlapping region not retained"
		}
		if {$Multiple=="1"} {
		    #Plusieurs matches donc on oublie la seq deja faite et on continue
		    set LaNewSeqComp [string range $LaSeqComp 0 [lindex $tmp 0]]
		    set NbMatches    [regexp -indices -all $TmpSeq $LaNewSeqComp tmp]
		} else {
		    incr NbMatches -1
		}
		#Espionne "Fin Nb $NbMatches"
	    }
	}
	if {$NewDebut !=""} {set Debut $NewDebut}
    }

    #Tableau de bord contenant un resume de ce qui a ete trouve
    if {! [info exists TabDesHairPin(LesPosHairPin)]} {
	set     Entete [list "Potential Complementary Sequence Searches"]
	lappend Entete ""
	lappend Entete "File: $NomDuFichier"
	lappend Entete "Sequence: $LaSeq"
	lappend Entete "Length (Query): $TailleQuery"
	lappend Entete "Stem Length: $MaxContigue"
	lappend Entete ""
	lappend Entete "No Complementary Sequence found"
	set Fen [AfficheVariable [join $Entete "\n"] "" $Titre]
	return "$Fen"
    } else {
	set LesDFHairPin [lsort -unique [set TabDesHairPin(LesPosHairPin)]]
	set LesDFHairPin [lsort -integer -increasing -index 0 $LesDFHairPin]
    }
    
    #Affichage des appariements
    if {$AvecAffichage} {
	foreach DF $LesDFHairPin {
	    set Query     ""
	    set QReverse  ""
	    set NbBlancNR ""
	    set NbBlancNN ""
	    set NucNor    [string tolower $LaSeq]
	    set NucRev    [string tolower [NucToReverseNuc $NucNor]]
	    set D         [lindex $DF 0]
	    set F         [lindex $DF 1]
	    set Taille    [expr {$F-$D+1}]

	    if {! [info exists TabComptage($Taille)]} {
		set TabComptage($Taille) 0    		
		lappend TabComptage(LesTailles) $Taille
	    }
	    incr TabComptage($Taille)       
    
	    set             LesComp [set TabDesHairPin($D,$F,Complementaire)]
	    foreach DFComp $LesComp {
		set Query    ""
		set QReverse ""
		set LesX     ""
		#Coordonnes en normal pour reverse seulement et changement N->Liste contre
		set DComp  [expr $TailleQuery - [lindex $DFComp 1] +1]
		set FComp  [expr $TailleQuery - [lindex $DFComp 0] +1]

		set NucNor [string toupper $NucNor [expr {$D -1}]                 [expr {$F -1}]]
		set NucNor [string toupper $NucNor [expr [lindex $DFComp 0] -1] [expr [lindex $DFComp 1] -1]]
		set NucRev [string toupper $NucRev [expr {$DComp -1}]             [expr {$FComp -1}]]
		set NucRev [string toupper $NucRev [expr {$TailleQuery -$F}]      [expr {$TailleQuery -$D}]]
		
		lappend ToShow ">$D/$F -- [lindex $DFComp 0]/[lindex $DFComp 1] Stem $Taille"
		
		if {$D > $DComp} {
		    set NbBlancNR [expr {$D - $DComp}] 
		    for {set i 1} {$i <= $NbBlancNR} {incr i} {append QReverse " "}
		    for {set i 1} {$i < $D}          {incr i} {append LesX     " "}
		} elseif {$D < $DComp} {
		    set NbBlancNN [expr {$DComp - $D}] 
		    for {set i 1} {$i <= $NbBlancNN} {incr i} {append Query    " "}
		    for {set i 1} {$i < $DComp}      {incr i} {append LesX     " "}
		}
		set TailleAligne [expr {$F - $D + 1}]
		for {set i 1} {$i <= $TailleAligne} {incr i} {append LesX "X"}
		
		append Query    $NucNor
		append QReverse $NucRev
		lappend ToShow $Query
		lappend ToShow $LesX
		lappend ToShow $QReverse
	    }
	}
	set     Entete [list "Potential Complementary Sequence Searches"]
	lappend Entete ""
	lappend Entete "File: $NomDuFichier"
	lappend Entete "Sequence: $LaSeq"
	lappend Entete "Length (Query): $TailleQuery"
	lappend Entete "Stem Length: $MaxContigue"
	lappend Entete ""
	lappend Entete [format "%6s %7s" "Length" "Count"]
	lappend Entete "---------------"
	set LesTailles [lsort -increasing -integer [set TabComptage(LesTailles)]]
	foreach Taille $LesTailles {
	    lappend Entete [format "%4d    %4d" $Taille [set TabComptage($Taille)]]
	}
	lappend Entete ""
	set ToShow [concat $Entete $ToShow]
	
	return "[AfficheVariable [join $ToShow "\n"] "" $Titre]"
    } else {
	
	set ListeFinale ""
	foreach DF $LesDFHairPin {
	    set D       [lindex $DF 0]
	    set F       [lindex $DF 1]
	    set Segment [list   $D $F]
	    
	    set             LesComp [set TabDesHairPin($D,$F,Complementaire)]
	    foreach DFComp $LesComp {
		lappend Segment [list [lindex $DFComp 0] [lindex $DFComp 1]]
	    }
	    lappend ListeFinale $Segment
	}
	return "$ListeFinale"
    }
}

##################################################################
#      Verifie si la banque blastee contient notre sequence      #
#                                                                #
#        GUI          - CheckForBestHitInDatabase                #
#        Warning      - EvalueLeDesign                           #
#        Excel        - ReturnBestHitFromLogFile                 #
#                       ReturnDefinitionFromBestHitFromLogFile   #
#        Pour les Acc - ReturnBestHitInDatabase                  #
#                       InterrogeBestHitInDatabase               #
#                                                                #
##################################################################
proc InterrogeBestHitInDatabase args {

    Wup "Permet de questionner les infos de BestHitInDataBase"
    Wup "Voir aussi decharge et charge"
    
    global InfoDatabase
    
    #Arguments:
    ###########
    #$FichierBlast  AccessDeRef        - Meilleur access du blast selon les criteres appliques
    # FichierBlast  BId                - Access en reserve si n a pas passe les tests (vide si trop limite)
    # FichierBlast  IdGlobal           - Pourcentage d identite global: Aligne sur Total dans les alignements
    # FichierBlast  PCoverQuery        - Recouvrement de Query   par rapport au Subject
    # FichierBlast  PCoverSubject      - Recouvrement de Subject par rapport au Query
    # FichierBlast  InitialBId         - Premier BId trouve pour pas le perdre
    # FichierBlast $InitialBId Aligned - Bases Alignees du premier BId trouve

    if {$args == "unset"} {
	if {[info exists InfoDatabase]}    {array unset InfoDatabase}
	return ""
    } 
    
    set Quoi [join $args ","]
    if {[info exists InfoDatabase($Quoi)]} {return [set InfoDatabase($Quoi)]}
    
    set FichierACharger [lindex $args 0]
    if {! [info exists InfoDatabase($FichierACharger,EstCharge)]} {set InfoDatabase($FichierACharger,EstCharge) 0}
    if {$Quoi=="$FichierACharger,Decharge"} {
	#Espionne "Decharge $FichierACharger"
	DechargeBestHitInDatabase $FichierACharger
	return ""
    }
    if {  [info exists InfoDatabase($FichierACharger,EstVide)]}   {return ""}
    if {  [info exists InfoDatabase($Quoi)]}                      {return [set InfoDatabase($Quoi)]}

    ###################CHARGE le Fichier avec les Oligos#########################
    if {! [info exists InfoDatabase($FichierACharger,EstCharge)] || [set InfoDatabase($FichierACharger,EstCharge)]=="0"} {
	set InfoDatabase($FichierACharger,EstCharge) 1
	#Espionne "On charge $FichierACharger"

	set  ResultatCharge [ChargeLesBestHitInDatabase InfoDatabase $FichierACharger]
	if {$ResultatCharge != ""} {
	    set InfoDatabase($FichierACharger,EstVide) 1
	    return ""
	}
    }
    if {![info exists InfoDatabase($Quoi)]} {return ""}
    return [set InfoDatabase($Quoi)] 
}

proc ChargeLesBestHitInDatabase {aInfoDatabase FichierBlast} {

    Wup "Permet de verifier si la query a un access dans le blast et de le rendre"
    Wup "Retourne l access et le BId si c un peu juste"

    upvar $aInfoDatabase InfoDatabase

    if {! [file exists $FichierBlast]} {set InfoDatabase($FichierBlast,NotDone) 1;return "-2"}

    set BId          ""
    set AccessDeRef  ""

    #Charge les resultats du Blast
    ##############################
    set SeuilId      "None"
    set TailleQuery  [AskBlast $FichierBlast Taille]
    set NbSubject    [AskBlast $FichierBlast NbSubject]

    set InfoDatabase($FichierBlast,LesBIdsBases) ""
    #Test de la presence de la Query dans le blast
    ##############################################
    if {$NbSubject>0} {
	set LesBIdsBases ""

	#Faut il mettre une limite au nombre de hit utilises pour la recherche du meilleur hit ?
	set  NbSubjectLimit 150
	if {$NbSubject>$NbSubjectLimit} {set NbSubject $NbSubjectLimit}

	for {set i 1} {$i <=$NbSubject} {incr i} {
	    set BId [AskBlast $FichierBlast Subject $i]
	    #On se debarrasse des NG car pas bon dans NombreDeBasesEnCommun et PourcentageDIdentiteGlobal 
	    #il reconstruit le contig en entier
	    if {[regexp {NG[0-9]+} $BId]} {continue}

	    set NbBasesCommuns   [NombreDeBasesEnCommun $FichierBlast $BId "Query"]
	    
	    lappend LesBIdsBases [list $BId $NbBasesCommuns]
	}
	set InfoDatabase($FichierBlast,LesBIdsBases) $LesBIdsBases
	if {$LesBIdsBases ==""} {
	    set BId         ""
	    set AccessDeRef ""
	} else {
 	    set LesBIdsBases  [lsort -integer -index 1 -decreasing $LesBIdsBases]
	    set BId           [lindex [lindex $LesBIdsBases 0] 0]
	    set PIdentity     0.95
	    set PRecouvrement 0.70
	
	    set InfoDatabase($FichierBlast,InitialBId)   $BId
	    set InfoDatabase($FichierBlast,$BId,Aligned) [lindex [lindex $LesBIdsBases 0] 1]
	
	    #Pourcentage d identite global: Aligne sur Total dans les alignements 
	    set IdGlobal      [PourcentageDIdentiteGlobal $FichierBlast $BId]
	    #Recouvrement de Q sur S ou inversement
	    #PCoverQuery   Query   par rapport au Subject
	    #PCoverSubject Subject par rapport au Query
	    set PCoverQuery   [PourcentageDeRecouvrement $FichierBlast $BId   "Query"]
	    set PCoverSubject [PourcentageDeRecouvrement $FichierBlast $BId "Subject"]

	    set InfoDatabase($FichierBlast,IdGlobal)      $IdGlobal
	    set InfoDatabase($FichierBlast,PCoverQuery)   $PCoverQuery
	    set InfoDatabase($FichierBlast,PCoverSubject) $PCoverSubject
	    
	    if {$IdGlobal <= $PIdentity || $PCoverQuery<$PRecouvrement || $PCoverSubject<$PRecouvrement} {
		set AccessDeRef ""
		if {($PCoverQuery <= 0.20 || $PCoverSubject <= 0.20) || $IdGlobal <= 0.90} {
		    #On met tout de meme a zero car trop limite
		    set BId     ""
		}
		#On conserve BId pour les cas litigieux
	    } else {
		set AccessDeRef $BId
	    }
	}
    } else {
	set AccessDeRef ""
    }

    set InfoDatabase($FichierBlast,AccessDeRef) $AccessDeRef
    set InfoDatabase($FichierBlast,BId)         $BId        
    return ""
}

proc DechargeBestHitInDatabase {FichierBlast} {
    
    Wup "Decharge les infos chargees avec ChargeBestHitInDatabase"
    Wup "Decharge les infos du blast associees"

    global InfoDatabase

    array    unset InfoDatabase "$FichierBlast*"
    AskBlast unsetfile           $FichierBlast
    
    return ""
}

proc CheckForBestHitInDatabase {PAB Rep} {

    Wup "Permet de verifier si la query a un access dans le blast et de le rendre"
    Wup "Utilise InterrogeBestHitInDatabase qui va faire le calcul"
    Wup "Utiliser dans GUI ou voir aussi ReturnBestHitInDatabase"

    if {$PAB =="" || $Rep==""} {return ""}

    set FichierBlast "[file join $Rep "${PAB}.blastn"]"
    
    if {![file exists $FichierBlast]} {Espionne "$FichierBlast do not exists";return ""}

    set Texte             ""
    set TailleQuery       [AskBlast                   $FichierBlast  Taille]
    set NbSubject         [AskBlast                   $FichierBlast  NbSubject]
    set AccessDeRef       [InterrogeBestHitInDatabase $FichierBlast  AccessDeRef]
    set BId               [InterrogeBestHitInDatabase $FichierBlast  BId]
    set LesBIdsBases      [InterrogeBestHitInDatabase $FichierBlast  LesBIdsBases]
    set InitialBId        [InterrogeBestHitInDatabase $FichierBlast  InitialBId]
    set InitialBIdAligned [InterrogeBestHitInDatabase $FichierBlast $InitialBId Aligned]

    #Test de la presence de la Query dans le blast
    ##############################################
    if {$NbSubject>0} {
	lappend Texte "Checking for $PAB with $TailleQuery nt(s)."
	if {$LesBIdsBases==""} {
	    lappend Texte "Seems not to have any equivalent in database."
	    lappend Texte "Be Careful with Design."
	} else {
 	    set BId           [InterrogeBestHitInDatabase $FichierBlast BId]
	    set PIdentity     0.95
	    set PRecouvrement 0.70
	    
	    lappend Texte "Best Id choosen : $InitialBId with $InitialBIdAligned nt(s) aligned."
	    
	    #Pourcentage d identite global: Aligne sur Total dans les alignements 
	    set IdGlobal      [InterrogeBestHitInDatabase $FichierBlast IdGlobal]
	    #Recouvrement de Q sur S ou inversement
	    #PCoverQuery   Query   par rapport au Subject
	    #PCoverSubject Subject par rapport au Query
	    set PCoverQuery   [InterrogeBestHitInDatabase $FichierBlast PCoverQuery]
	    set PCoverSubject [InterrogeBestHitInDatabase $FichierBlast PCoverSubject]
	    
	    if {$IdGlobal <= $PIdentity || $PCoverQuery<$PRecouvrement || $PCoverSubject<$PRecouvrement} {
		if {($PCoverQuery <= 0.2 || $PCoverSubject <= 0.2) || $IdGlobal <= 0.90} {
		    #On met tout de meme a zero car trop limite
		    lappend Texte "Is not equivalent to our query (\%Id [format "%.2f" $IdGlobal], Cover Q/S [format "%.2f" $PCoverQuery], Cover S/Q [format "%.2f" $PCoverSubject])."
		} else {
		    lappend Texte "Is weakly similar to our query (\%Id [format "%.2f" $IdGlobal], Cover Q/S [format "%.2f" $PCoverQuery], Cover S/Q [format "%.2f" $PCoverSubject])."
		}
		lappend Texte "Be Careful with this design."
	    } else {
		lappend Texte "Seems to be equivalent (\%Id [format "%.2f" $IdGlobal], Cover Q/S [format "%.2f" $PCoverQuery], Cover S/Q [format "%.2f" $PCoverSubject])."
		lappend Texte "Go on confident with this design."
	    }
	}
    } else {
	lappend Texte "Seems not to have any equivalent in database."
	lappend Texte "Be Careful with this design."
    }
    
    after idle {.fairelire.msg configure -wraplength 10i -background red -font "Courier 10 bold"}
    tk_dialog .fairelire "Please press OK" "[join $Texte "\n"]" {} 0 Acknowledge
    catch {destroy .fairelire}
    return ""
}

proc ReturnBestHitInDatabase {PAB Rep} {

    Wup "Permet de verifier si la query a un access dans le blast et de le rendre"
    Wup "Retourne l access et le BId si c un peu juste"

    if {$PAB =="" || $Rep==""} {return ""}

    set FichierBlast "[file join $Rep "${PAB}.blastn"]"
    set BId          ""
    set AccessDeRef  ""
    
    set AccessDeRef [InterrogeBestHitInDatabase $FichierBlast AccessDeRef]
    set BId         [InterrogeBestHitInDatabase $FichierBlast BId]
 
    return "[list $AccessDeRef $BId]"
}


proc EvalueLeDesign {PAB Dir} {

    Wup "Petit test rapide pour savoir si le design c bien passe en utilisant le fichier .log"
    Wup "Utiliser dans IllustreOligo et GUI pour mettre un warning si necessaire au design"
    Wup "Attention on ne recalcule rien sinon voir ReturnBestHitInDatabase ou CheckForBestHitInDatabase"

    set FichierLog   "[file join $Dir "${PAB}.log"]"
    set NomDeFenetre ""
    set NoAccess     0
    set i            0
    if {[file exists $FichierLog]} {
	set F [open "$FichierLog"]
	while {[gets $F Ligne]>=0} {
	    if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	    incr i
	    if {[regexp -nocase "Query has no equivalent sequence" $Ligne]} {set NoAccess 1;break}
	    if {$i=="6"} {break}
	}
	close $F
	
	if {$NoAccess} {
	    set  TailleTotale  [expr {35 + [string length $Dir] + [string length $PAB]}]
	    if {$TailleTotale < 65} {
		set TailleSurDeux [expr {(65 -9)/2}]
	    } elseif {$TailleTotale > 134} {
		set TailleTotale 134
		set TailleSurDeux [format "%.0f" [expr {$TailleTotale/2 -4.5}]]
	    } else {
		set TailleSurDeux [format "%.0f" [expr {$TailleTotale/2 -4.5}]]
	    }
	    set Texte "\
		    [string repeat "-" $TailleSurDeux] Warning [string repeat "-" $TailleSurDeux]\n\
		    Be careful with the design of $PAB in $Dir.\n\
		    No reliable access found in blast searches.\n\
		    You should check yourself the Blast results or use Check Best Hit.\n\
		    If it is verified, you should only trust the white (oligo) boxes."
	    set NomDeFenetre [FaireSavoir_Multi $Texte "Warning"]
	}
    }
    return "$NomDeFenetre"
}

proc ReturnBestHitFromLogFile {PAB Rep {Quoi "Bid"}} {

    Wup "Permet de connaitre le meilleur Access pour un design en utilisant le fichier .log"
    Wup "Utiliser dans IllustreOligo et GUI pour mettre un warning si necessaire au design"
    Wup "Attention on ne recalcule rien sinon voir ReturnBestHitInDatabase ou CheckForBestHitInDatabase"

    global TabBestHit

    set FichierLog "[file join $Rep "${PAB}.log"]"
    
    if {[info exists TabBestHit] && [info exists TabBestHit($FichierLog,$Quoi)]} {return [set TabBestHit($FichierLog,$Quoi)]}

    set Bid    ""
    set Id     ""
    set CoverS ""
    set CoverQ ""
    set i   0
    if {[file exists $FichierLog]} {
	set F [open "$FichierLog"]
	while {[gets $F Ligne]>=0} {
	    if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	    incr i
	    if {        [regexp -nocase -indices "has no equivalent sequence in DataBase:" $Ligne Tmp]} {
		if {    [regexp -nocase          "No Hit found in Blast searches"          $Ligne] || \
			[regexp -nocase          "No fitting sequence in Blast searches"   $Ligne]} {
		    set Bid ""
		    break
		} else {
		    #Query has no equivalent sequence in DataBase: UG:Hs#S3803333 (%Id 0.86, Cover S/Q 0.64, Cover Q/S 0.63).
		    set NewLigne [split [string trim [string range $Ligne [expr {[lindex $Tmp 1]+1}] end]] " "]
		    set Bid      "Warning [lindex $NewLigne 0] [lrange $NewLigne 1 end]"
		    regexp { \(\%Id ([0-9\.]+), Cover S/Q ([0-9\.]+), Cover Q/S ([0-9\.]+)\)} $NewLigne tmp Id CoverS CoverQ
		    break
		}
	    }
	    if {[regexp -nocase -indices "has an equivalent sequence in DataBase:" $Ligne Tmp]} {
		#Query has an equivalent sequence in DataBase: RSHUM:NM_001100 (%Id 1.00, Cover S/Q 0.98, Cover Q/S 0.98).
		set NewLigne [split [string trim [string range $Ligne [expr {[lindex $Tmp 1]+1}] end]] " "]
		set Bid "[lindex $NewLigne 0]"
		regexp { \(\%Id ([0-9\.]+), Cover S/Q ([0-9\.]+), Cover Q/S ([0-9\.]+)\)} $NewLigne tmp Id CoverS CoverQ
		break
	    }
	    if {$i=="6"} {break}
	}
	close $F
    }   
    set TabBestHit($FichierLog,Bid)    $Bid
    set TabBestHit($FichierLog,Id)     $Id
    set TabBestHit($FichierLog,CoverS) $CoverS
    set TabBestHit($FichierLog,CoverQ) $CoverQ

    return "[set TabBestHit($FichierLog,$Quoi)]"
    return "$Bid"
}
proc OLD_ReturnBestHitFromLogFile {PAB Rep} {

    Wup "Permet de connaitre le meilleur Access pour un design en utilisant le fichier .log"
    Wup "Utiliser dans IllustreOligo et GUI pour mettre un warning si necessaire au design"
    Wup "Attention on ne recalcule rien sinon voir ReturnBestHitInDatabase ou CheckForBestHitInDatabase"

    set FichierLog "[file join $Rep "${PAB}.log"]"
    
    set Bid ""
    set i   0
    if {[file exists $FichierLog]} {
	set F [open "$FichierLog"]
	while {[gets $F Ligne]>=0} {
	    if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	    incr i
	    if {        [regexp -nocase -indices "has no equivalent sequence in DataBase:" $Ligne Tmp]} {
		if {    [regexp -nocase          "No Hit found in Blast searches"          $Ligne] || \
			[regexp -nocase          "No fitting sequence in Blast searches"   $Ligne]} {
		    set Bid ""
		    break
		} else {
		    #Query has no equivalent sequence in DataBase: UG:Hs#S3803333 (%Id 0.86, Cover S/Q 0.64, Cover Q/S 0.63).
		    set NewLigne [split [string trim [string range $Ligne [expr [lindex $Tmp 1]+1] end]] " "]
		    set Bid      "Warning [lindex $NewLigne 0] [lrange $NewLigne 1 end]"
		    break
		}
	    }
	    if {[regexp -nocase -indices "has an equivalent sequence in DataBase:" $Ligne Tmp]} {
		#Query has an equivalent sequence in DataBase: RSHUM:NM_001100 (%Id 1.00, Cover S/Q 0.98, Cover Q/S 0.98).
		set NewLigne [split [string trim [string range $Ligne [expr [lindex $Tmp 1]+1] end]] " "]
		set Bid "[lindex $NewLigne 0]"
		break
	    }
	    if {$i=="6"} {break}
	}
	close $F
    }   
    return "$Bid"
}

proc ReturnDefinitionFromBestHitFromLogFile {Id {Banque ""}} {
   
    Wup "Permet de connaitre la definition du meilleur Access pour un design"
    Wup "On passe par FastaCmd marche bien pour RefSeq et Unigene"
    
    set OnaWarning 0
    if {$Id==""} {
	set Id  "Warning no best Id for the design"
	set Def "No Definition"
    } else {
	if {[regexp -nocase "Warning" $Id]} {
	    set OldId $Id
	    set Id [lindex $Id 1]
	    set OnaWarning 1
	}
	if {$Banque != ""} {
	    #MODIF 18102006 pour probleme d utilisation abusive des listes et virgule apres accolade

	    #set Def [join [FastaCmd $Id "" 0] ""]
	    set Def [FastaCmd $Id "" 0 $Banque]
	    if {! [regexp "No Result " $Def]} {
		regsub -nocase "^$Id " $Def "" Def
		#set Def [lrange $Def 1 end]
	    }
	} elseif {      [regexp "UG:Hs"  $Id]} {
	    #set Def [join [FastaCmd $Id "" 0] ""]
	    set Def [FastaCmd $Id "" 0]
	    if {! [regexp "No Result " $Def]} {
		set Def [lrange $Def 1 end]
	    }
	} elseif {[regexp "RSHUM:" $Id]} {
	    #set Def [join [FastaCmd $Id "" 0] ""]
	    set Def [FastaCmd $Id "" 0]
	    if {! [regexp "No Result " $Def]} {
		set Def [lrange $Def 2 end]
	    }
	} else {
	    set Def "No Definition"
	}
	if {$OnaWarning==1} {
	    set Id $OldId
	}
    }
    return "$Def"
}

#########################################
#   Selection des meilleurs oligos      #
#   Selection croisee entre 2 banques   #
#########################################
proc SelectionDesOligosPourTous {{LesPABs ""} {Dir ""}} {
  
    Wup "Selectionne de maniere automatique le meilleur oligo pour une banque"
    Wup "On prend l Oligo le plus a droite (3 prime, lie a la technique d extraction) et avec le meilleur score"
    Wup "On retiendra 2 oligos si un des 2 a 0 candidat et l autre plus de 1 candidat"
    Wup "Attention pose la question si pas d access retenu"
   
    if {$Dir == ""}       {
	FaireLire "Please choose directory where your design is saved."
	if {[BROWSEDDIR] != ""} {
	    set Dir [tk_chooseDirectory -title "Choose result directory." -initialdir "[BROWSEDDIR]"]
	} else {
	    set Dir [tk_chooseDirectory -title "Choose result directory." -initialdir "[HOMEDIR]"]
	}
	if {$Dir == "" } {
	    return ""
	} else {
	    BROWSEDDIR $Dir
	}
    }
    
    if {$LesPABs==""} {
	set ListeAcc ""
	foreach Acc [glob -nocomplain -directory $Dir "*.masked"] {
	    regsub ".masked" $Acc "" Acc
	    lappend ListeAcc [file tail $Acc]
	}
	set    LesPABs [EntreDeux $ListeAcc]
	if {  $LesPABs == ""}    {return ""}
    } 
    
    FaireAttendre .waitapply "Waiting" "Please wait ...\nSelecting best oligonucleotide for all Querys ..."
    foreach MonPAB $LesPABs { 
	set FichierOligo "[file join $Dir "${MonPAB}.oligo"]"
	set FichierBlast "[file join $Dir "${MonPAB}.blastn"]"
	SelectionDesOligos         $MonPAB $Dir DeGUI
	InterrogeBestHitInDatabase $FichierOligo Decharge
	InterrogeBestHitInDatabase $FichierBlast Decharge
    }
    FaireLaSuite  .waitapply

    OuiOuNonMemo UnsetAllPlease

    return ""
}

proc SelectionDesOligos_GUI {PAB DirOligo} {

    Wup "Selectionne de maniere automatique le meilleur oligo pour une banque"
    Wup "On prend l Oligo le plus a droite (3 prime,lie a la technique d extraction) et avec le meilleur score"
    Wup "On retiendra 2 oligos si un des 2 a 0 candidat et l autre plus de 1 candidat"
    Wup "Attention pose la question si pas d access retenu"
    
    if {$PAB=="" || $DirOligo==""} {return ""}

    set               FichierSelection "[file join $DirOligo "$PAB.selection"]"
    if {[file exists $FichierSelection]} {
	if {! [OuiOuNon "Selection already done.\nShould we proceed again (lost previous) ?"]} {
	    return ""
	}
	file delete -force $FichierSelection
    }
    SelectionDesOligos $PAB $DirOligo DeGUI
    
    OuiOuNonMemo UnsetAllPlease
    return ""
}

proc SelectionDesOligos {PAB DirOligo {DeGUI ""}} {

    Wup "Selectionne de maniere automatique le meilleur oligo pour une banque"
    Wup "On prend l Oligo le plus a droite (3 prime,lie a la technique d extraction) et avec le meilleur score"
    Wup "Attention si aucun Accessderef on ne choisit que parmi la categorie 0 candidat"
    Wup "On retiendra 2 oligos si un des 2 a 0 candidat et l autre plus de 1 candidat"
    Wup "Attention pose la question si pas d access retenu si on vient de GUI"
    
    set FichierOligo     "[file join $DirOligo "$PAB.oligo"]"
    set FichierSelection "[file join $DirOligo "$PAB.selection"]"
    set FichierBlastn    "[file join $DirOligo "$PAB.blastn"]"
    set AccessDeRef      ""
    set BId              ""
    set LesCategories    ""
    set Categorie        ""
    set SelOligo         ""
    set OligoAlternatif  ""
    set Selection        ""
    
    if {[file exists $FichierSelection]} {
	if {[OuiOuNonMemo "Selection already done.\nWould You like to keep it" ]} {
	    return ""
	} else {
	    file delete -force $FichierSelection
	}
    }

    #Test de la presence de la query dans le blast
    ##############################################
    #On ne veut pas forcement tenir compte de la query
    if {[LesParametresDuDesign QueryCheck "LaValeurMerci"]=="1"} {
	set RefBId      [ReturnBestHitInDatabase $PAB $DirOligo]
	set AccessDeRef [lindex $RefBId 0]
	set BId         [lindex $RefBId 1]
    } else {
	set RefBId      ""
	set AccessDeRef ""
	set BId         ""
    }
    set  LesOligos [InterrogeInfoDeLOligo $FichierOligo LesNoms]
    if {$LesOligos == ""} {
	if {       [InterrogeInfoDeLOligo $FichierOligo NotDone]=="1"} {
	    #Oligo non calcules
	    AjouteAuLog "No Oligo yet designed for $PAB"
	    return ""
	} elseif { [InterrogeInfoDeLOligo $FichierOligo NoOligo]=="1"} {
	    #Oligo non trouves
	    AjouteAuLog "No Oligo found for $PAB"
	    if {[file exists     "$FichierOligo"]} {
		file copy -force "$FichierOligo" $FichierSelection
	    } 
	    return ""
	} else {Espionne "Rien a faire ...";return ""}
    }
    
    foreach Oligo $LesOligos {
	set NbCandidats [InterrogeInfoDeLOligo $FichierOligo $Oligo NbCandidats]
	if {   $Categorie!=$NbCandidats} {
	    set Categorie  $NbCandidats
	    lappend LesCategories $Categorie
	}
	lappend TabCroisee($Categorie,LesOligos) [list $Oligo [InterrogeInfoDeLOligo $FichierOligo $Oligo Debut]]
    }
    set LesCategories [lsort -integer -increasing $LesCategories]

    #Si on pas de ref dans la banque on pose la question si on doit quand meme choisir
    #Sinon prend un access de categorie 0 si y en a
    if {$AccessDeRef==""} {
	if {$DeGUI!=""} {
	    if {! [OuiOuNonMemo "It seems that no accession has passed cutoffs for Best Hit check !\nYou might all the same choose an Oligo ?"]} {
		AjouteAuLog "No Oligo to select for $PAB due to no relevant reference in blastn"
		return ""
	    }
	    set AccessDeRef $BId
	} else {
	    AjouteAuLog "No Oligo to select for $PAB due to no relevant reference in blastn"
	    return ""
	}
    }
    if {$AccessDeRef==""} {
	if {[info exists TabCroisee(0,LesOligos)]} {set LesCategories "0"}
    }

    #Espionne "LesCategories $LesCategories"
    foreach Categorie $LesCategories {
	set            LesOligos [lsort -integer -index 1 -decreasing [set TabCroisee($Categorie,LesOligos)]]
	foreach Oligo $LesOligos {
	    set NomOligo [lindex $Oligo 0]
	    
	    #Espionne "OligoAlternatif $OligoAlternatif SelOligo $SelOligo"
	
	    set LesCandidats [join [InterrogeInfoDeLOligo $FichierOligo $NomOligo LesCandidats] " "]
	    if {($AccessDeRef!="" && [regexp "No Hit" $LesCandidats]) || ($AccessDeRef!="" && ! [regexp -nocase "$AccessDeRef" $LesCandidats]) || ($AccessDeRef=="" && ! [regexp "No Hit" $LesCandidats])} {
		#Espionne "$PAB No query $AccessDeRef for $NomOligo we skip this one"
		#Espionne "$NomOligo $LesCandidats"
		continue
	    }
	    if {    $Categorie=="0"} {
		set OligoAlternatif $NomOligo
		break
	    } else {
		set SelOligo        $NomOligo
		if {$Categorie=="1"} {set OligoAlternatif ""}
		break
	    }
	}
	if {$SelOligo!=""} {break} else {continue}
    }
    #Si au final on a rien du tout on prend OligoAlternatif
    if {$SelOligo==""} {set SelOligo $OligoAlternatif;set OligoAlternatif ""}

    #Espionne "OligoAlternatif $OligoAlternatif SelOligo $SelOligo"
    if {$SelOligo!=""} {
	set    Selection [join [ExtraitOligoDuTFA $PAB $SelOligo        $DirOligo] "\n"]
	Sauve $Selection dans $FichierSelection
    } else {
	if {$DeGUI==""} {
	    AjouteAuLog "Try to Select Oligo for $PAB but it fails"
	} else {
	    FaireLire   "Try to Select Oligo for $PAB but it fails"
	}
    }
    #Espionne "Selection $Selection"

    if {$OligoAlternatif!=""} {
	set    Selection [join [ExtraitOligoDuTFA $PAB $OligoAlternatif $DirOligo] "\n"]
	AppendAuFichier $FichierSelection $Selection
    }
    if {[info exist TabCroisee]} {unset TabCroisee}
    return ""
}

proc SelectionCroiseeDesOligosPourTous {{LesPABs ""} {DirOligo1 ""} {DirOligo2 ""}} {
   
    Wup "Selectionne de maniere automatique le meilleur oligo entre 2 banques"
    Wup "On prend l Oligo le plus a droite (3 prime, lie a la technique d extraction) et avec le meilleur score"
    Wup "Si le meilleur est avec 0 candidat, on retient aussi un oligo avec plus que 0 si il existe"

    if {$DirOligo1=="" || $DirOligo2==""} {
	FaireLire "You need 2 directories containing 2 design for your sequence."
	set i 0
	while {1} {
	    incr i
	    if {$i=="1"} {
		set Texte "Choose directory N${i} (the one where the file (.croisee) is also saved)"
	    } else {set Texte "Choose directory N${i}"}
	    FaireLire "$Texte"

	    if {[BROWSEDDIR] != ""} {
		set Dir [tk_chooseDirectory -title "Choose a directory where to save." -initialdir "[BROWSEDDIR]"]
	    } else {
		set Dir [tk_chooseDirectory -title "Choose a directory where to save." -initialdir "[HOMEDIR]"]
	    }
	    if {$Dir == ""} {
		set i [expr {$i-1}]
		if {! [OuiOuNon "Would you like to continue ?"]} {return ""}
	    } else {
		set DirOligo${i} $Dir
		BROWSEDDIR       $Dir
	    }
	    if {$i =="2"} {break}
	}
    }
    if {$LesPABs==""} {
	set ListeAcc ""
	foreach Acc [glob -nocomplain -directory $DirOligo1 "*.masked"] {
	    regsub ".masked" $Acc "" Acc
	    lappend ListeAcc [file tail $Acc]
	}
	set    LesPABs [EntreDeux [lsort $ListeAcc]]
	if {  $LesPABs == ""} {return ""}
    }

    FaireAttendre .waitapply "Waiting" "Please wait ...\nX-Selecting best oligonucleotide for all Querys ..."
    foreach MonPAB $LesPABs {
	set FichierWorking1 "[file join $DirOligo1 "${MonPAB}.working"]"
	set FichierWorking2 "[file join $DirOligo2 "${MonPAB}.working"]"
	if {[file exists $FichierWorking1] || [file exists $FichierWorking2]} {
	    AjouteAuLog "$MonPAB is currently being processed by another application. I skip it."
	    continue
	} else {
	    Sauve "" dans $FichierWorking1
	    Sauve "" dans $FichierWorking2
	}

  	SelectionCroiseeDesOligos $MonPAB $DirOligo1 $DirOligo2 DeGui
	
	file delete -force $FichierWorking1
	file delete -force $FichierWorking2

	set FichierOligo1 "[file join $DirOligo1 "${MonPAB}.oligo"]"
	set FichierOligo2 "[file join $DirOligo2 "${MonPAB}.oligo"]"
	set FichierBlast1 "[file join $DirOligo1 "${MonPAB}.blastn"]"
	set FichierBlast2 "[file join $DirOligo2 "${MonPAB}.blastn"]"

	#Attention
	#InterrogeInfoDeLOligo      "$DirOligo1/$MonPAB" Decharge
	#InterrogeInfoDeLOligo      "$DirOligo2/$MonPAB" Decharge
	#InterrogeBestHitInDataBase "$DirOligo1/$MonPAB" Decharge
	#InterrogeBestHitInDataBase "$DirOligo2/$MonPAB" Decharge

	InterrogeInfoDeLOligo      $FichierOligo1 Decharge
	InterrogeInfoDeLOligo      $FichierOligo2 Decharge
	InterrogeBestHitInDatabase $FichierBlast1 Decharge
	InterrogeBestHitInDatabase $FichierBlast2 Decharge
    }
    FaireLaSuite  .waitapply

    OuiOuNonMemo UnsetAllPlease

    return ""
}

proc SelectionCroiseeDesOligos_GUI {PAB {DirOligo ""}} {

    Wup "Selectionne de maniere automatique le meilleur oligo entre 2 banques"
    Wup "On prend l Oligo le plus a droite (3 prime, lie a la technique d extraction) et avec le meilleur score"
    Wup "Si le meilleur est avec 0 candidat, on retient aussi un oligo avec plus que 0 si il existe"
    
    if {$PAB==""} {return ""}
    
    set i 0
    foreach D $DirOligo {
	incr i
	set DirVar "DirOligo$i"
	set [set DirVar] $D
    }

    FaireLire "You need 2 directories containing 2 design for your sequence."
    set i 0
    while {1} {
	incr i
	if {$i=="1"} {
	    set Texte "Choose directory N${i} (the one where the file is also saved)"
	} else {
	    set Texte "Choose directory N${i}"
	}
	FaireLire "$Texte"
	
	set Essai "DirOligo${i}"
	if {[info exists $Essai] && [set $Essai] != "" && [OuiOuNon "Shall we use [set $Essai] ?"] } {
	    set Dir "[set $Essai]"
	} else {
	    if {[BROWSEDDIR] != ""} {
		set Dir [tk_chooseDirectory -title "Choose a directory where to save." -initialdir "[BROWSEDDIR]"]
	    } else {
		set Dir [tk_chooseDirectory -title "Choose a directory where to save." -initialdir "[HOMEDIR]"]
	    }
	}
	if {$Dir == ""} {
	    set i [expr {$i-1}]
	    if {! [OuiOuNon "Would you like to continue ?"]} {return ""}
	} else {
	    set DirOligo${i} $Dir
	    BROWSEDDIR       $Dir
	}
	if {$i =="2"} {break}
    }
    set FichierCroisee "[file join $DirOligo1 "${PAB}.croisee"]"
    if {[file exists $FichierCroisee]} {
	if {![OuiOuNon "X-Selection already done.\nShould we proceed again (lost previous) ?"]} {return ""}
	file delete -force $FichierCroisee
    }

    SelectionCroiseeDesOligos $PAB $DirOligo1 $DirOligo2 DeGUI
    
    OuiOuNonMemo UnsetAllPlease
    return ""
}

proc SelectionCroiseeDesOligos {PAB DirOligo1 DirOligo2 {DeGUI ""}} {

    Wup "Selectionne de maniere automatique le meilleur oligo entre 2 banques"
    Wup "On prend l Oligo le plus a droite (3 prime,lie a la technique d extraction) et avec le meilleur score"
    Wup "Si le meilleur est avec 0 candidat, on retient aussi un oligo avec plus que 0 si il existe"
    
    set FichierCroisee  "[file join $DirOligo1 "${PAB}.croisee"]"
    set FichierOligo    "[file join $DirOligo1 "${PAB}.oligo"]"
    set FichierBlastn   "[file join $DirOligo1 "${PAB}.blastn"]"
    set RefBId          ""
    set AccessDeRef     ""
    set BId             ""
    set AccessDeRef2    ""
    set Categorie       ""
    set LesCategories   ""
    set SelOligo        ""
    set OligoAlternatif ""
    set Selection       ""

    if {[file exists $FichierCroisee]} {
	if {[OuiOuNonMemo "X-Selection already done.\nWould You like to keep it" ]} {
	    return ""
	} else {
	    file delete -force FichierCroisee
	}
    }
    
    #Test de la presence de la query dans le blast
    ##############################################
    set RefBId               [ReturnBestHitInDatabase $PAB $DirOligo1]
    set AccessDeRef  [lindex $RefBId 0]
    set BId          [lindex $RefBId 1]
    set AccessDeRef2 [lindex [ReturnBestHitInDatabase $PAB $DirOligo2] 0]

    set  LesOligosEnCommuns  [LesOligosIdentiques $PAB $DirOligo1 $DirOligo2]
    if {$LesOligosEnCommuns==""} {
	if {$DeGUI==""} {
	    AjouteAuLog "Try to X-Select Oligo for $PAB but no shared oligo"
	} else {
	    FaireLire   "Try to X-Select Oligo for $PAB but no shared oligo"
	}
	return ""
    }
    foreach OligoEnCommun $LesOligosEnCommuns {
	if {! [regexp -nocase {[a-z0-9]+} $OligoEnCommun]} {continue}
	set NbCandidatsTotal [lindex $OligoEnCommun 3]

	if {   $Categorie!=$NbCandidatsTotal} {
	    set Categorie  $NbCandidatsTotal
	    lappend LesCategories $Categorie
	}
	lappend TabCroisee($Categorie,LesOligos) $OligoEnCommun
    }

    #Si on pas de ref dans la banque on pose la question si on doit quand meme choisir
    #Sinon prend un access de categorie 0 si y en a
    if {$AccessDeRef==""} {
	if {$DeGUI!=""} {
	    if {! [OuiOuNonMemo "It seems that no accession has passed cutoffs for Best Hit check !\nYou might all the same choose an Oligo ?"]} {
		AjouteAuLog "No Oligo to select for $PAB due to no relevant reference in blastn"
		return ""
	    }
	    set AccessDeRef $BId
	} else {
	    AjouteAuLog "No Oligo to select for $PAB due to no relevant reference in blastn"
	    return ""
	}
    }
    #Cas particulier ou les 2 designs sont problematiques
    if {$AccessDeRef == "" && $AccessDeRef2 == ""} {
	if {[info exists TabCroisee(0,LesOligos)]} {set LesCategories "0"}
    }

    set                LesCategories [lsort -integer -increasing $LesCategories]
    #Espionne "LesCategories $LesCategories"
    foreach Categorie $LesCategories {
	set            LesOligos [lsort -integer -index 1 -decreasing [set TabCroisee($Categorie,LesOligos)]]
	foreach Oligo $LesOligos {
	    set NomOligo     [lindex $Oligo 0]
	    set NbCandidats1 [lindex $Oligo 4]
	    set NbCandidats2 [lindex $Oligo 5]
	    #Espionne "CAT: $Categorie - Oligo $NomOligo - OligoAlternatif $OligoAlternatif - SelOligo $SelOligo - $NbCandidats1 - $NbCandidats2"
	    
	    #Modif les candidats
	    ####################
	    set LesCandidats [join [InterrogeInfoDeLOligo $FichierOligo $NomOligo LesCandidats] " "]
	    if {($AccessDeRef!="" && [regexp "No Hit" $LesCandidats]) || ($AccessDeRef!="" && ! [regexp -nocase "$AccessDeRef" $LesCandidats]) || ($AccessDeRef=="" && ! [regexp "No Hit" $LesCandidats])} {
		#Espionne "$PAB No query $AccessDeRef for $NomOligo we skip this one"
		#Espionne "$NomOligo $LesCandidats"
		continue
	    }
	    
	    #Le Meilleur a garde en alternatif
	    if {$Categorie=="0" || $Categorie=="1"} {
		#Si il y a ces categories c que un des deux design a eu des regions sans la query
		set OligoAlternatif $NomOligo
		break
	    }
	    #Dans une categorie on prefere garder ceux sans 0 candidats
	    if {$NbCandidats1=="0" || $NbCandidats2=="0"} {
		#Si on a pas encore de roue de secours
		if {   $OligoAlternatif == ""} {
		    set OligoAlternatif $NomOligo
		} else {continue}
	    } else {
		#On tient quelque chose
		set SelOligo $NomOligo
		if {$NbCandidats1>1} {
		    #On va ici car il n est pas specifique
		    #On met donc l oligo alternatif a la place 
		    #Espionne "On passe par la $Categorie $NbCandidats1 $NbCandidats2"
		    if {$OligoAlternatif!=""} {
			#Espionne "On change $SelOligo pour $OligoAlternatif"
			set SelOligo $OligoAlternatif;set OligoAlternatif ""
		    }
		}

		#Cas ou on a 2 candidats au total et en a 1 seul dans les 2 Design
		#mais il faut mettre la restriction si on a un des design vide 
		#Oligo specifique pour les 2 TOP
		if {$Categorie == "2"} { 
		    if {$NbCandidats1 == "1"} {
			if {$AccessDeRef2 == ""} {
			    if {$OligoAlternatif != ""} {
				#Espionne "On change $SelOligo pour $OligoAlternatif"
				set SelOligo $OligoAlternatif;set OligoAlternatif ""
				break
			    }
			} else {
			    #Espionne "$NomOligo gagne"
			    set SelOligo $NomOligo;set OligoAlternatif ""
			}
		    }
		}
	    		
		if {0} {
		    #Cas ou on a 2 candidats au total et en a 1 seul dans les 2 Design 
		    #Oligo specifique pour les 2 TOP
		    if {$Categorie == "2" && $NbCandidats1 == "1"} {
			set SelOligo $NomOligo;set OligoAlternatif ""
		    }
		}
		
		#Cas ou on a plus de 2 candidats au total mais un des 2 design n en a qu 1
		#On doit cependant bien tester si ceci a lieu pour des design correct (AccessDeRef non nul !!)
		if {($AccessDeRef != "" && $NbCandidats1 == "1")||($AccessDeRef2 != "" && $NbCandidats2 == "1")} {
		    set SelOligo $NomOligo;set OligoAlternatif ""
		}
		break
	    }
	}
	#Espionne ">> On a $SelOligo et $OligoAlternatif"
	#On en a un et on quitte ou alors on continue
	if {$SelOligo!=""} {break} else {continue}
    }
    #Espionne "OligoAlternatif $OligoAlternatif SelOligo $SelOligo"
    if {$SelOligo=="" || $AccessDeRef==""} {
	#Si au final on a rien du tout on prend OligoAlternatif
	if {$OligoAlternatif!=""} {
	    set SelOligo $OligoAlternatif;set OligoAlternatif ""
	}
    }
    #Espionne "OligoAlternatif $OligoAlternatif SelOligo $SelOligo"
    if {$SelOligo!=""} {
	#Si au final on a rien du tout on ne sauve pas
	set    Selection [join [ExtraitOligoDuTFA $PAB $SelOligo        $DirOligo1] "\n"]
	Sauve $Selection dans $FichierCroisee
    } else {
	if {$DeGUI==""} {
	    AjouteAuLog "Try to X-Select Oligo for $PAB but it fails"
	} else {
	    FaireLire   "Try to X-Select Oligo for $PAB but it fails"
	}
    }
    #Espionne "Selection $Selection"

    if {$OligoAlternatif!=""} {
	set    Selection [join [ExtraitOligoDuTFA $PAB $OligoAlternatif $DirOligo1] "\n"]
	AppendAuFichier $FichierCroisee $Selection
    }
    return ""
}

proc NomOligoEquivalent {PAB NomOligo DirOligo1 DirOligo2} {

    Wup "Permet de donner le nom de l oligo equivalent a NomOligo dans une autre recherche"

    set FichierOligo1 "[file join $DirOligo1 "${PAB}.oligo"]"
    set Debut [InterrogeInfoDeLOligo $FichierOligo1 $NomOligo Debut]
    set Fin   [InterrogeInfoDeLOligo $FichierOligo1 $NomOligo Fin]

    set FichierOligo2 "[file join $DirOligo2 "${PAB}.oligo"]"
    set            LesOligos [InterrogeInfoDeLOligo $FichierOligo2 LesNoms]
    foreach Oligo $LesOligos {
	if {! [regexp -nocase {[a-z0-9]+} $LesOligos]} {continue}
	
	set  AutreNomOligo    $Oligo
	set  AutreDebut       [InterrogeInfoDeLOligo $FichierOligo2 $Oligo Debut]
	if {$AutreDebut != $Debut} {continue}
	set  AutreFin         [InterrogeInfoDeLOligo $FichierOligo2 $Oligo Fin]
	set  AutreNbCandidats [InterrogeInfoDeLOligo $FichierOligo2 $Oligo NbCandidats]

	return $AutreNomOligo
    }
    return ""
}

proc LesOligosIdentiques {PAB DirOligo1 DirOligo2} {

    Wup "Permet de rendre la liste de oligos en commun entre deux recherche differentes"
 
    set LesOligosEnCommun ""

    #Stock les oligos du premier
    set FichierOligo1 "[file join $DirOligo1 "$PAB.oligo"]"
    set LesOligo1s    [InterrogeInfoDeLOligo $FichierOligo1 LesNoms]
    foreach Oligo1 $LesOligo1s {
	set Debut     [InterrogeInfoDeLOligo $FichierOligo1 $Oligo1 Debut]
	set Fin       [InterrogeInfoDeLOligo $FichierOligo1 $Oligo1 Fin]
		 
	set TabOligo(Oligo,$Debut,$Fin)    1
	set TabOligo(NomOligo,$Debut,$Fin) $Oligo1
    }

    #Compare les oligos du deuxieme au premier
    #Stock les oligos communs aux deux banques avec le nom de la 1ere banque et Debut Fin
    set FichierOligo2 "[file join $DirOligo2 "$PAB.oligo"]"
    set LesOligo2s    [InterrogeInfoDeLOligo $FichierOligo2 LesNoms]
    foreach Oligo2 $LesOligo2s {
	set Debut     [InterrogeInfoDeLOligo $FichierOligo2 $Oligo2 Debut]
	set Fin       [InterrogeInfoDeLOligo $FichierOligo2 $Oligo2 Fin]

	if {[info exists TabOligo(Oligo,$Debut,$Fin)]} {
	    set VraiNom          [set TabOligo(NomOligo,$Debut,$Fin)]
	    set NbCandidatsTotal [expr {[InterrogeInfoDeLOligo $FichierOligo1 $VraiNom NbCandidats]+[InterrogeInfoDeLOligo $FichierOligo2 $Oligo2 NbCandidats]}]
	    
	    lappend LesOligosEnCommun [list $VraiNom $Debut $Fin $NbCandidatsTotal [InterrogeInfoDeLOligo $FichierOligo1 $VraiNom NbCandidats] [InterrogeInfoDeLOligo $FichierOligo2 $Oligo2 NbCandidats]]
	}
    }
    set     LesOligosEnCommun [lsort -increasing -index 3 $LesOligosEnCommun]
    return $LesOligosEnCommun
}

proc CompleteLaListeDesOligos_GUI {K {PAB ""} {DirOligo1 ""}} {
    
    Wup "Ajoute un oligo a la liste de ceux deja designe de dir1 a dir2"
    Wup "Cas particulier pour un redesign et completer la liste"
    Wup "Version graphique avec selection visuelle des oligos"

    if {$PAB=="" || $DirOligo1==""} {return ""}

    set FichierACharger1 "[file join $DirOligo1 "${PAB}.oligo"]"

    while {1} {
	set Texte "Choose a directory to be completed by $PAB oligos"
	FaireLire "$Texte"
	if {[BROWSEDDIR] != ""} {
	    set Dir [tk_chooseDirectory -title "Choose the directory to be completed." -initialdir "[BROWSEDDIR]"]
	} else {
	    set Dir [tk_chooseDirectory -title "Choose the directory to be completed." -initialdir "[HOMEDIR]"]
	}
	if {   $Dir == ""} {
	    if {! [OuiOuNon "Would you like to continue ?"]} {return ""}
	} else {
	    set DirOligo2 $Dir
	    BROWSEDDIR    $Dir
	    break
	}
    }
        
    set FichierACharger2 "[file join $DirOligo2 "${PAB}.oligo"]"
    set Answer           ""
    set LesNomsOligos    ""
    while {1} {
	set Texte  "Please choose with Control+1 the oligo(s) to add:\nfrom $DirOligo1\nin $DirOligo2\nThen click on Ok."
	set Answer [FaireSavoir $Texte "Adding Results - Waiting your selection."]

	set  LesNomsOligos [LesOligosSelectionnes $K]
	if {$LesNomsOligos==""} {
	    if {! [OuiOuNon "No Oligo selected !\nWould you like to retry ?"]} {break} else {continue}
	}
	break
    }
    
    set LogFile1 "[file join $DirOligo1 "${PAB}.log"]"
    set LogFile2 "[file join $DirOligo2 "${PAB}.log"]"

    #On test si les resultats etaient deja charges ou non
    set  EstCharge  ""
    set  EstCharge [InterrogeInfoDeLOligo $FichierACharger2 EstCharge]
    if {$EstCharge==""} {set EstCharge 0}

    if {! [OuiOuNon "Are you sure to add the selected Oligo(s) to another ?"]} {return ""}
    foreach LeNomOligo $LesNomsOligos {
	set Debut [InterrogeInfoDeLOligo $FichierACharger1 $LeNomOligo Debut]
	set Fin   [InterrogeInfoDeLOligo $FichierACharger1 $LeNomOligo Fin]
	
	AjouteUnOligoPourCeuxDejaFait $PAB $PAB $DirOligo1 $DirOligo2 "$Debut $Fin"
	#Les infos sont modifies dans le deuxieme lot
	#InterrogeInfoDeLOligo $FichierACharger1 Decharge
	if {$EstCharge=="0"} {
	    #On ne decharge que si il ne l etais pas avant
	    InterrogeInfoDeLOligo $FichierACharger2 Decharge
	}	
	CompleteLeLogFileDeLOligo "$Debut $Fin" $LogFile1 $LogFile2
    }
    return ""
}

proc AjouteUnOligoPourCeuxDejaFait {PAB1 PAB2 Dir1 Dir2 Quoi} {
    
    Wup "Ajoute un oligo a la liste de ceux deja designe de dir1 a dir2"
    Wup "Cas particulier pour un redesign et completer la liste ou faire un update de l oligo"
    Wup "Quoi peut etre ref de oligo ou coordonnees debut fin"
    Wup "Rq: on teste si un oligo existe deja si oui on propose de changer de nom ou pas (update ou pas)"

    set FichierACharger1 "[file join $Dir1 "${PAB1}.oligo"]"
    set FichierACharger2 "[file join $Dir2 "${PAB2}.oligo"]"
    set NomAncienOligo   ""
    
    #Espionne ">>>>Try to complete $PAB1 with $Quoi<<<<"
    
    #Liste des oligos a partir de laquelle on va en rajouter un
    set  LesOligos1 [InterrogeInfoDeLOligo $FichierACharger1 LesNoms]
    if {$LesOligos1 == ""} {
	if {[InterrogeInfoDeLOligo $FichierACharger1 NotDone]=="1"} {
	    #Oligo non calcules
	    AjouteAuLog "Try to complete $PAB1 but no Oligo yet designed in $Dir1"
	    return ""
	} elseif {[InterrogeInfoDeLOligo $FichierACharger1 NoOligo]=="1"} {
	    #Oligo non trouves
	    AjouteAuLog "Try to complete $PAB1 but no Oligo found"
	    return ""
	} else {Espionne "Petit probleme";return ""}
    }

    #Liste des oligos auquel on veut en rajouter un
    set  LesOligos2 [InterrogeInfoDeLOligo $FichierACharger2 LesNoms]
    if {$LesOligos2 == ""} {
	if {[InterrogeInfoDeLOligo $FichierACharger2 NotDone]=="1"} {
	    #Oligo non calcules
	    AjouteAuLog "Try to complete $PAB2 but no Oligo yet designed in $Dir2"
	    return ""
	} elseif {[InterrogeInfoDeLOligo $FichierACharger2 NoOligo]=="1"} {
	    #Oligo non trouves
	    AjouteAuLog "Try to complete $PAB2 but no Oligo found"
	    return ""
	} else {Espionne "Petit probleme";return ""}
    }

    #Le nom de l oligo a ajouter
    set     LeNomOligo ""
    if {[llength $Quoi]>1} {
	set LeNomOligo [InterrogeInfoDeLOligo $FichierACharger1 [lindex $Quoi 0] [lindex $Quoi 1] NomOligo]
    } else {
	set LeNomOligo $Quoi
    }

    if {          $LeNomOligo  == ""}        {return ""}
    if {![regexp "$LeNomOligo" $LesOligos1]} {AjouteAuLog "No $LeNomOligo in $FichierACharger1";return ""}

    #On recupere les infos de l oligo a ajouter
    set Debut     [InterrogeInfoDeLOligo $FichierACharger1 $LeNomOligo Debut]
    set Fin       [InterrogeInfoDeLOligo $FichierACharger1 $LeNomOligo Fin]
    set Selection [join [ExtraitOligoDuTFA $PAB1 $LeNomOligo $Dir1] "\n"]

    #On teste dans la vieille liste si il existe deja
    if {[InterrogeInfoDeLOligo $FichierACharger2 $Debut $Fin NomOligo]!=""} {
	if {! [OuiOuNon "The Oligo for $Debut $Fin already exists\nDo I replace it ?"]} {return ""}
	set NomAncienOligo [InterrogeInfoDeLOligo $FichierACharger2 $Debut $Fin NomOligo]
	
	#On peut decider de faire un update simplement en gardant le nom de l oligo
	if {! [OuiOuNon "Should I keep the previous oligo reference (name) ?\n(Yes just for properties update)"]} {set NomAncienOligo ""}
	SupprimeUnOligoPourCeuxDejaFait $FichierACharger2 [InterrogeInfoDeLOligo $FichierACharger2 $Debut $Fin NomOligo]
	#SupprimeLeLogFileDeLOligo "$Debut $Fin" "$Dir2/${PAB2}.log"
    }

    #On change ou pas le nom de l oligo a ajouter (ou updater)
    set OldNom $LeNomOligo
    if {$NomAncienOligo!=""} {
	#On conserve le nom de l ancien oligo (juste un update)
	set LeNomOligo $NomAncienOligo
    } else {
	#On change le nom si il existe deja on incremente
	while {1} {
	    if {[regexp $LeNomOligo $LesOligos2]} {	
		regexp {Oligo([0-9]+)_} $LeNomOligo tmp Nb
		set OldNb $Nb
		incr Nb
		regsub "${OldNb}_" $LeNomOligo "${Nb}_" LeNomOligo
	    } else {break} 
	}
    }
    #Espionne "Ancien $OldNom -- Nouveau $LeNomOligo"
    regsub "$OldNom" $Selection "$LeNomOligo" Selection

    AppendAuFichier $FichierACharger2 $Selection
    #Espionne ">>>>Adding<<<<" 
    #Espionne "$Selection"
    #Espionne ">>>>>>To<<<<<<" 
    #Espionne "$FichierACharger2\n"
    return ""
}

proc AjouteUnOligoPourCeuxDejaFaitAuFichier {FichierACharger1 FichierACharger2 Quoi} {
    
    Wup "WARNING PROC NON TESTEES"

    Wup "Ajoute un oligo a la liste de ceux deja designe de dir1 a dir2"
    Wup "Cas particulier pour un redesign et completer la liste"
    Wup "Quoi peut etre ref de oligo ou coordonnees debut fin"

    #set FichierACharger1 "$Dir1/${PAB1}.oligo"
    #set FichierACharger2 "$Dir2/${PAB2}.oligo"

    set PAB1 [file tail $FichierACharger1]
    if {[regexp {\.[a-z]{3,}$} $PAB1]} {regsub {\.[a-z]{3,}$} $PAB1 "" PAB1}
    set Dir1 [file dirname $FichierACharger1]

    Espionne "$FichierACharger1 $PAB1 $Dir1"
    return ""
    Espionne ">>>>Try to complete $PAB1 with $Quoi<<<<"
    
    set  LesOligos1 [InterrogeInfoDeLOligo $FichierACharger1 LesNoms]
    if {$LesOligos1 == ""} {
	if {[InterrogeInfoDeLOligo $FichierACharger1 NotDone]=="1"} {
	    #Oligo non calcules
	    Espionne "Try to complete $PAB1 but no Oligo yet designed in $Dir1"
	    return ""
	} elseif {[InterrogeInfoDeLOligo $FichierACharger1 NoOligo]=="1"} {
	    #Oligo non trouves
	    Espionne "Try to complete $PAB1 but no Oligo found"
	    return ""
	} else {Espionne "Petit probleme";return ""}
    }

    set  LesOligos2 [InterrogeInfoDeLOligo $FichierACharger2 LesNoms]
    if {$LesOligos2 == ""} {
	if {[InterrogeInfoDeLOligo $FichierACharger2 NotDone]=="1"} {
	    #Oligo non calcules
	    Espionne "Try to complete $PAB2 but no Oligo yet designed in $Dir2"
	    return ""
	} elseif {[InterrogeInfoDeLOligo $FichierACharger2 NoOligo]=="1"} {
	    #Oligo non trouves
	    Espionne "Try to complete $PAB2 but no Oligo found"
	    return ""
	} else {Espionne "Petit probleme";return ""}
    }

    set     LeNomOligo ""
    if {[llength $Quoi]>1} {
	set LeNomOligo [InterrogeInfoDeLOligo $FichierACharger1 [lindex $Quoi 0] [lindex $Quoi 1] NomOligo]
    } else {
	set LeNomOligo $Quoi
    }

    #Espionne "Quoi $Quoi - NomOligo $LeNomOligo"

    if {          $LeNomOligo  == ""}        {return ""}
    if {![regexp "$LeNomOligo" $LesOligos1]} {Espionne "No $LeNomOligo in $FichierACharger1";return ""}

    set Debut     [InterrogeInfoDeLOligo $FichierACharger1 $LeNomOligo Debut]
    set Fin       [InterrogeInfoDeLOligo $FichierACharger1 $LeNomOligo Fin]
    #set Selection [join [ExtraitOligoDuTFA $PAB1 $LeNomOligo $Dir1] "\n"]
    set Selection [join [ExtraitOligoDuTFA2 $PAB $LeNomOligo $Dir1 [file tail $FichierACharger1]] "\n"]

    if {[InterrogeInfoDeLOligo $FichierACharger2 $Debut $Fin NomOligo]!=""} {
	if {! [OuiOuNon "Oligo for $Debut $Fin already exists\nDo I replace it ?"]} {return ""}
	set NomAncienOligo [InterrogeInfoDeLOligo $FichierACharger2 $Debut $Fin NomOligo]
	SupprimeUnOligoPourCeuxDejaFait $FichierACharger2 $NomAncienOligo
	#SupprimeLeLogFileDeLOligo "$Debut $Fin" "$Dir2/${PAB2}.log"
    }
    
    set OldNom $LeNomOligo
    #Espionne "NomOligo $LeNomOligo - OldNom $LeNomOligo"
    while {1} {
	if {[regexp $LeNomOligo $LesOligos2]} {	
	    #Espionne "NOmOligo $LeNomOligo - $LesOligos2"

	    regexp {Oligo([0-9]+)_} $LeNomOligo tmp Nb
	    #Espionne "NOmOligo $LeNomOligo - $tmp - $Nb"
	    set OldNb $Nb
	    incr Nb
	    regsub "${OldNb}_" $LeNomOligo "${Nb}_" LeNomOligo
	} else {break} 
    }
    regsub "$OldNom" $Selection "$LeNomOligo" Selection

    Espionne "Ajoute a $FichierACharger2 $Selection"
    #AppendAuFichier $FichierACharger2 $Selection
    
    #Espionne ">>>>Adding<<<<" 
    #Espionne "$Selection"
    #Espionne ">>>>>>To<<<<<<" 
    #Espionne "$FichierACharger2\n"

    Espionne "SupprimeUnOligoPourCeuxDejaFait $FichierACharger1 $OldNom"
    Espionne "Ajoute a $FichierACharger1 $Selection"

    #SupprimeUnOligoPourCeuxDejaFait $FichierACharger1 $OldNom 
    #AppendAuFichier $FichierACharger1 $Selection
    
    return ""
}

proc SupprimeUnOligoPourCeuxDejaFait {Fichier Quoi} {
     
    Wup "Supprime un oligo a la liste de ceux deja designe"
    Wup "Cas particulier pour un redesign et completer la liste"
    Wup "Quoi peut etre ref de oligo ou coordonnees debut fin"

    set  FichierNew "${Fichier}.new"
    set  LesOligos1 [InterrogeInfoDeLOligo $Fichier LesNoms]
    if {$LesOligos1 == ""} {
	if {[InterrogeInfoDeLOligo $Fichier NotDone]=="1"} {
	    AjouteAuLog "Try to delete $Quoi in $Fichier but no Oligo yet designed for $Fichier"
	    return ""
	} elseif {[InterrogeInfoDeLOligo $Fichier NoOligo]=="1"} {
	    AjouteAuLog "Try to delete $Quoi in $Fichier but no Oligo found for $Fichier"
	    return ""
	} else {Espionne "Petit probleme";return ""}
    }

    set     LeNomOligo ""
    if {[llength $Quoi]>1} {
	set LeNomOligo [InterrogeInfoDeLOligo $Fichier [lindex $Quoi 0] [lindex $Quoi 1] NomOligo]
    } else {set LeNomOligo $Quoi}
    if {           $LeNomOligo  == ""}        {return ""}
    if {! [regexp "$LeNomOligo" $LesOligos1]} {AjouteAuLog "No $LeNomOligo in $Fichier";return ""}
    
    set OnLeTient 0
    set F [open "$Fichier"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {  [string range $Ligne 0 0] != ">"} {
	    if {$OnLeTient} {set OnLeTient 0;continue}
	    AppendAuFichier $FichierNew $Ligne
	    continue
	}
	if {[regexp "$LeNomOligo" $Ligne]} {set OnLeTient 1;continue}
	AppendAuFichier $FichierNew $Ligne
    }
    close $F

    file delete -force $Fichier
    file rename -force $FichierNew $Fichier 
    return ""
}

proc CompleteLeLogFileDeLOligo {DF Log1 Log2} {

    Wup "proc AjouteUnOligoPourCeuxDejaFait --> Ajoute un oligo a la liste de ceux deja designe de Log1 a Log2"
    Wup "Ici on complete le log file associe si l oligo n existait pas"
    Wup "Cas particulier pour un redesign et completer la liste"

    set Fichier1    $Log1
    set Fichier2    $Log2
    set FichierNew "$Log2.tmp"
    if {![file exists $Fichier1] || ![file exists $Fichier2] || $DF == ""} {return ""}
    
    set Debut [lindex $DF 0]
    set Fin   [lindex $DF 1]

    #Parcours du premier log et extraction de la zone a inseree dans le deuxieme log
    set OnLeTient 0
    set NewLines ""
    set F [open "$Fichier1"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	#Moving on: 41 100
	if {$OnLeTient} {
	    if {[regexp -nocase {Moving on:} $Ligne]} {
		set OnLeTient 0
		break
	    }
	    lappend NewLines $Ligne
	}
	if {! [regexp -nocase "Moving on: $Debut $Fin" $Ligne]} {continue}
	set OnLeTient 1

	lappend NewLines $Ligne
    }
    close $F
    if {$NewLines == ""} {return ""}
    set  NewLines [join $NewLines "\n"]
    
    #Recopiage du deuxieme log et insertion au bon endroit de la zone extraite precedemment
    #On sauve les lignes toujours apres donc on peut inserer au bon endroit
    set OnAPasCommence 0
    set OkOnASauve     0
    set OnEfface       0
    set OnSkip         0
    set LesLignes      ""
    set Ligne          ""
    set F [open "$Fichier2"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {$OnSkip} {
	} elseif {$OkOnASauve} {
	    AppendAuFichier $FichierNew $Ligne
	    continue
	}
	if {$OnAPasCommence=="0" && ![regexp -nocase {Moving on:} $Ligne]} {
	    AppendAuFichier $FichierNew $Ligne
	    continue
	}
	if {[regexp -nocase {Moving on:} $Ligne]} {
	    set OnAPasCommence 1
	    set   MaLigne      ""
	    regsub "^Moving on: " $Ligne "" MaLigne
	    scan $MaLigne "%s %s" ND NF
	    
	    #On stocke les precedents
	    if {$LesLignes!=""} {
		set LesLignes [join $LesLignes "\n"]
		AppendAuFichier $FichierNew $LesLignes
		set LesLignes ""
	    }
	    #On voulait laisser tomber un resultat et maintenant on arrete
	    if {   $OnSkip == "1"} {
		set OnSkip 0
		AppendAuFichier $FichierNew $Ligne
		continue
	    }
	    if {$Debut==$ND} {
		if {! [OuiOuNon "The information in the Log File for $Debut $Fin already exists\nDo I also update them ?"]} {
		    close $F    
		    if {[file exists $FichierNew]} {file delete -force $FichierNew}
		    return ""
		}
		set OnSkip     1
		set OkOnASauve 1
		AppendAuFichier $FichierNew $NewLines
		continue
	    }
	    if {$Debut<$ND} { 
		set OkOnASauve 1
		AppendAuFichier $FichierNew $NewLines
		AppendAuFichier $FichierNew $Ligne
		continue
	    }
	    lappend LesLignes $Ligne
	    continue
	}
	if {$OnSkip==0} {lappend LesLignes $Ligne}
    }
    if {$OkOnASauve=="0"} {AppendAuFichier $FichierNew $NewLines}
    close $F

    if {$OnAPasCommence=="0"} {
	file delete -force $FichierNew
    } else {
	file delete -force $Fichier2
	file rename -force $FichierNew $Fichier2 
    }
    return ""
}

proc SupprimeLeLogFileDeLOligo {DF Log1} {

    Wup "proc AjouteUnOligoPourCeuxDejaFait --> Ajoute un oligo a la liste de ceux deja designe de dir1 a dir2"
    Wup "Ici on supprime le log file associe si l oligo existait pas"
    Wup "Cas particulier pour un redesign et completer la liste"

    set Fichier1    $Log1
    set FichierNew "$Log1.tmp"
    if {![file exists $Fichier1] || $DF == ""} {return ""}
    
    set Debut [lindex $DF 0]
    set Fin   [lindex $DF 1]

    #Recopiage du deuxieme log et insertion au bon endroit de la zone extraite precedemment
    set OnAPasCommence 0
    set OnEfface       0
    set OnSkip         0
    set LesLignes      ""
    set Ligne          ""
    set F [open "$Fichier1"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {$OnAPasCommence=="0" && ![regexp -nocase {Moving on:} $Ligne]} {
	    AppendAuFichier $FichierNew $Ligne
	    continue
	}
	if {[regexp -nocase {Moving on:} $Ligne]} {
	    set OnAPasCommence 1
	    set   MaLigne      ""
	    regsub "^Moving on: " $Ligne "" MaLigne
	    scan $MaLigne "%s %s" ND NF
	    
	    #On voulait laisser tomber un resultat et maintenant on arrete
	    if {   $OnSkip == "1"} {
		set OnSkip 0
		AppendAuFichier $FichierNew $Ligne
		continue
	    }
	    if {$Debut==$ND} {
		set OnSkip 1
		continue
	    }
	    AppendAuFichier $FichierNew $Ligne
	    continue
	}
	if {$OnSkip==0} {AppendAuFichier $FichierNew $Ligne}
    }
    close $F

    if {$OnAPasCommence=="0"} {
	file delete -force $FichierNew
    } else {
	file delete -force $Fichier1
	file rename -force $FichierNew $Fichier1
    }
    return ""
}

############################################
#   Gestion de la sortie des resultats     #
#   Fichier delimite par des tabulations   #
#   Utilisable dans Excel                  #
############################################
proc LesOligosPourExcel_GUI {{DirOligo1 ""}} {

    Wup "Permet de sauver les oligos selectionnes dans le format excel"
    Wup "Utiliser dans GUI"
    Wup "Rq les oligos sont ceux choisis automatiquement (.selection et .croisee) ou modifies par l utilisateur"
    
    if {$DirOligo1 == ""} {
	while {1} {
	    set Texte "Please select the directory containing the design results."
	    FaireLire "$Texte"	
	    if {[BROWSEDDIR] != ""} {
		set DirOligo1 [tk_chooseDirectory -title "Choose directory" -initialdir "[BROWSEDDIR]"]
	    } else {
		set DirOligo1 [tk_chooseDirectory -title "Choose directory" -initialdir "[HOMEDIR]"]
	    }
	    if {   $DirOligo1 == ""} {
		if {! [OuiOuNon "Would you like to reselect ?"]} {return ""}
		continue
	    }
	    BROWSEDDIR $DirOligo1
	    break
	}
    }
    
    while {1} {
	set DirOligo2 ""
	set Texte     "Your database is $DirOligo1\
		\nIf cross results exists, can you choose the second directory ?"
	if {[OuiOuNon "$Texte"]} {
	    while {1} {
		set T "Choose the other directory"	   
		if {[BROWSEDDIR] != ""} {
		    set DirOligo2 [tk_chooseDirectory -title $T -initialdir "[BROWSEDDIR]"]
		} else {
		    set DirOligo2 [tk_chooseDirectory -title $T -initialdir "[HOMEDIR]"]
		}
		if {   $DirOligo2 == ""} {
		    if {! [OuiOuNon "Would you like to reselect second directory ?"]} {return ""}
		    continue
		}
		BROWSEDDIR $DirOligo2
		break
	    }
	    break
	} else {break}
    }
    
    #Choix du fichier de sauvegarde
    while {1} {
	set Texte "Choose a file name to save your results."
	FaireLire "$Texte"	
	if {[BROWSEDDIR] != ""} {
	    set FichierSave [tk_getSaveFile -title "Choose a file name to save" -initialdir "[BROWSEDDIR]"]
	} else {
	    set FichierSave [tk_getSaveFile -title "Choose a file name to save" -initialdir "[HOMEDIR]"]
	}
	if {   $FichierSave == ""} {
	    if {! [OuiOuNon "Would you like to continue ?"]} {return ""}
	    continue
	} else {
	    if {[file exists $FichierSave]} {
		file delete -force $FichierSave
	    } 
	    set BrowsedDir [file dirname [lindex $FichierSave end]]
	    BROWSEDDIR $BrowsedDir
	    break
	}
    }

    set ListePossible ""
    foreach Acc [glob -nocomplain -directory $DirOligo1 "*.masked"] {
	regsub ".masked$" $Acc "" Acc
	lappend ListePossible [file tail $Acc]
    }
    if {$ListePossible!=""} {
	set LesQuery ""
	while {1} {
	    set Texte "Save the oligo informations for all sequence designed ?\nIf no, choose a sample."
	    if {! [OuiOuNon "$Texte"]} {
		while {1} {
		    #set LesQuery [MorceauxChoisis $ListePossible]
		    set  LesQuery [lsort [EntreDeux $ListePossible]]
		    if {$LesQuery == ""} {
			if {! [OuiOuNon "Would you like to choose again ?"]} {return ""}
			continue
		    }
		    break
		} 
		break
	    } else {
		set LesQuery $ListePossible
		break
	    }
	}    
	AjouteAuLog "Saving results in a tab delimited file..."
	AjouteAuLog "..."
	FaireAttendre .waitapply "Waiting" "Please wait ...\nSaving results ..."
	LesOligosPourExcel $LesQuery $DirOligo1 $DirOligo2 "" $FichierSave
	FaireLaSuite  .waitapply
	AjouteAuLog "Results saved in $FichierSave"
    }

    OuiOuNonMemo UnsetAllPlease
    return ""
}

proc UnOligoPourExcel_GUI {K PAB} {

    Wup "Permet de sauver les oligos selectionnes dans le format excel"
    Wup "Rq les oligos sont ceux choisis dans l interface et pour le seul PAB affiche"
    
    #Choix du fichier de sauvegarde
    while {1} {
	set Texte "Choose a file name to save your selection"
	FaireLire "$Texte"	
	if {[BROWSEDDIR] != ""} {
	    set FichierSave [tk_getSaveFile -title "Choose a file name to save" -initialdir "[BROWSEDDIR]"]
	} else {
	    set FichierSave [tk_getSaveFile -title "Choose a file name to save" -initialdir "[HOMEDIR]"]
	}
	if {   $FichierSave == ""} {
	    if {! [OuiOuNon "Would you like to continue ?"]} {return ""}
	    continue
	} else {
	    if {[file exists $FichierSave]} {
		file delete -force $FichierSave
	    } 
	    set BrowsedDir [file dirname [lindex $FichierSave end]]
	    BROWSEDDIR $BrowsedDir
	    break
	}
    }
    
    #Choix des Oligos a sauver
    set Answer        ""
    set LesNomsOligos ""
    while {1} {
	set Texte  "Please choose with Control+1 the oligo(s) to save under file:\nThen click on Ok."
	set Answer [FaireSavoir $Texte "Saving File - Waiting your selection."]
	
	set  LesNomsOligos [LesOligosSelectionnes $K LesIds]
	if {$LesNomsOligos==""} {
	    if {! [OuiOuNon "No Oligo selected !\nWould you like to retry ?"]} {return ""} else {continue}
	} else {break}
    }
    
    set AvecEntete "AvecEntete"
    if {[llength $LesNomsOligos]>1} {
	if {[OuiOuNon "Would you like to save the header one time in your file ?"]} {
	    set     AvecEntete "AvecEntete"
	} else {set AvecEntete "SansEntete"}
    }

    set Dir       ""
    set Dir2      ""
    set FirstTime 0

    foreach Id $LesNomsOligos { 
	set MesTags  [$K gettags $Id]
	set NomOligo [lindex $MesTags 4]
	set Dir      [lindex $MesTags 5]
	set Dir2     ""

	set     Fichier "[file join $Dir "${PAB}.oligo"]"
	if {    ! [file exists $Fichier]} {
	    set Fichier "[file join $Dir $PAB]"
	    if {! [file exists $Fichier]} {return ""}
	}

	#Deuxieme resultat de design si il y a eu une selection croisee
	if {[file exists "[file join $Dir "$PAB.croisee"]"]} {
	    if { [info exists TabDirCroiseeDe($Dir)]} {
		set Dir2 [set TabDirCroiseeDe($Dir)]
	    } 

	    set  Essai "[lindex [LesInfosDe $K MesDirs] end]"
	    if {$Dir2==""} {
		while {1} {
		    set Texte "Some crossed results exists for your sequence between\
			    \nthis design ($Dir) and another!\
			    \nWould you like to use it and give me this directory ?"
		    if {[OuiOuNonMemo "$Texte"]} {
			while {1} {
			    if {$Essai!="" && [OuiOuNon "Shall we use $Essai ?"]} {
				set Dir2 "$Essai"
			    } else {
				set T "Choose the other directory"
				if {[BROWSEDDIR] != ""} {
				    set Dir2 [tk_chooseDirectory -title $T -initialdir "[BROWSEDDIR]"]	
				} else {
				    set Dir2 [tk_chooseDirectory -title $T -initialdir "[HOMEDIR]"]	
				}
			    }
			    
			    if {$Dir2 == ""} {
				if {! [OuiOuNon "Would you like to reselect?"]} {break}
				continue
			    }
			    BROWSEDDIR $Dir2
			    break
			}
			break
		    } else {break}
		}
	    }
	}
	if {! [info exists TabDirCroiseeDe($Dir)] && $Dir2!=""} {set TabDirCroiseeDe($Dir) $Dir2}

	set D [InterrogeInfoDeLOligo $Fichier $NomOligo Debut]
	set F [InterrogeInfoDeLOligo $Fichier $NomOligo Fin]
	
	incr FirstTime
	if {$FirstTime!="1"} {set AvecEntete "SansEntete"}
	LesOligosPourExcel $PAB $Dir $Dir2 "$D $F" $FichierSave $AvecEntete
    }

    if {[info exists TabDirCroiseeDe]} {unset TabDirCroiseeDe}
    OuiOuNonMemo UnsetAllPlease
    return ""
}

proc LesOligosPourExcel {LesQuery DirOligo {DirOligo2 ""} {Quoi ""} {FichierSave ""} {AvecEntete ""}} {

    Wup "Sauve les oligos dans un format utilisable par Excel"
    Wup "Rq si DirOligo2 est renseigne on utilise le fichier .croisee si ce dernier existe aussi"
    Wup "03/09/03 Mise a jour pour definition"
    Wup "10/09/03 Mise a jour pour tenir compte des oligos croisee"
    Wup "18/03/04 Mise a jour pour tenir compte des access unigene"
    Wup "10/04/04 Mise a jour pour integrer des liens vers SRS et NCBI"
    Wup "03/02/05 Mise a jour du format"
    Wup "03/02/06 Mise a jour du format avec ajout de nouvelles colonnes (PID, coverage)"
    Wup "15/04/06 Mise a jour complete du format et de l algo"

    #Quoi peut etre ref de oligo ou coordonnees
    #LesQuery peuvent etre:
    #ActiChip1_0  - pour la liste des oligos de la premiere  version de ActiChip
    #ActiChip2_0  - pour la liste des oligos de la deuxieme  version de ActiChip
    #ActiChip3_0  - pour la liste des oligos de la troisieme version de ActiChip

    #Choix du fichier de sauvegarde
    if {$FichierSave == ""} {
	while {1} {
	    set Texte "Choose a file name to save your selection"
	    FaireLire "$Texte"
	    if {[BROWSEDDIR] != ""} {
		set FichierSave [tk_getSaveFile -title "Choose a file name to save" -initialdir "[BROWSEDDIR]"]
	    } else {
		set FichierSave [tk_getSaveFile -title "Choose a file name to save" -initialdir "[HOMEDIR]"]
	    }
	    if {   $FichierSave == ""} {
		if {! [OuiOuNon "Would you like to continue ?"]} {return ""}
		continue
	    } else {
		set BrowsedDir [file dirname [lindex $FichierSave end]]
		BROWSEDDIR $BrowsedDir
		break
	    }
	}
    }	

    if {$AvecEntete == "" || $AvecEntete == "AvecEntete" || $AvecEntete == "1"} {set AvecEntete 1} else {set AvecEntete 0} 

    #Pour utilier des references comme Debut Fin ou la ref de l oligo
    if {$Quoi==""} {set PasDeQuoi 1} else {set PasDeQuoi 0}

    set OnAUnigeneDir1 0
    set OnAUnigeneDir2 0

    #0   Creation de l entete (modulable en fonction des disponibilites)
    #0-1 Premier design entete de base
    set Entete [list Query_Access Protein_Access Design_Access Definition Query_Size Reference Database Best_ID PId PCoverQ PCoverS]

    #0-1-1 Pour le cas ou on a un access unigene on essaie de rajouter l ID
    if {[regexp -nocase "unigene" $DirOligo]} {
	set OnAUnigeneDir1 1
	set Entete [concat $Entete [list Unigene_ID]]
    }

    set Entete [concat $Entete [list Best_Definition Begin End Tm Size Sequence Nb_Detected_ID Detected_ID Unrelated_ID]]
    
    #0-2 Si on a un deuxieme design
    if {$DirOligo2!=""} {
	set Entete [concat $Entete [list Equivalent_Oligo Database2 Best_ID2 PId2 PCoverQ2 PCoverS2]]

	#0-2-1 Pour le cas ou on a un access unigene on essaie de rajouter l ID
	if {[regexp -nocase "unigene" $DirOligo2]} {
	    set OnAUnigeneDir2 1
	    set Entete [concat $Entete [list Unigene_ID2]]
	}

	set Entete [concat $Entete [list Best_Definition2 Nb_Detected_ID2 Detected_ID2]]
    }

    if {$AvecEntete} {AppendAuFichier $FichierSave [join $Entete "\t"]}
	
    if {$LesQuery=="ActiChip1_0" || [regexp -nocase "actichip1" $LesQuery]} {set LesQuery [ActiChip1_0]}
    if {$LesQuery=="ActiChip2_0" || [regexp -nocase "actichip2" $LesQuery]} {set LesQuery [ActiChip2_0]}
    if {$LesQuery=="ActiChip3_0" || [regexp -nocase "actichip3" $LesQuery]} {set LesQuery [ActiChip3_0]}

    set Database   [file tail $DirOligo]
    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set RepNuctfa "[file join [RepertoireDuGenome] nuctfa]"
    foreach Query_Access $LesQuery {
	set OnTraiteDeCroisee  0
	set Info              ""
	set LesInfos          ""
	set LesInfosDeLOligo  ""
	set Protein_Access    ""
	set Design_Access     ""
	set Equivalent_Oligo  ""
	set Best_ID           ""
	set PId               ""
	set PCoverS           ""
	set PCoverQ           ""
	set Unigene_ID        ""
	set Best_Definition   ""
	set Database2         ""
	set Nb_Detected_ID2   ""
	set Detected_ID2      ""
	set Best_ID2          ""
	set PId2              ""
	set PCoverS2          ""
	set PCoverQ2          ""
	set Best_Definition2  ""
	set Unigene_ID2       ""
	set Unrelated_ID      ""

	#1 Fichier de design du premier repertoire
	set FichierMasque    "[file join $DirOligo "${Query_Access}.masked"]"
	set FichierOligo     "[file join $DirOligo "${Query_Access}.oligo"]"
	set FichierBlastn    "[file join $DirOligo "${Query_Access}.blastn"]"
	set FichierLog       "[file join $DirOligo "${Query_Access}.log"]"
	set FichierSelection "[file join $DirOligo "${Query_Access}.selection"]"
	set FichierCroisee   "[file join $DirOligo "${Query_Access}.croisee"]"
	set FileParam1       "[file join $DirOligo "Design.parameters"]"

	if {$PasDeQuoi} {
	    set FichierACharger $FichierSelection
	} else {
	    set FichierACharger $FichierOligo
	}

	#2 Partie Information de la query
	#################################
	#2-1 Access utilise lors du design
	#    un mRNA ou sequence originale (nuctfa)
	set FichierQuery     "[file join $RepmRNA $Query_Access]"
	if {[file exists $FichierQuery]} {
	    set         Design_Access [AccessDeLaBoite $Query_Access mRNA]
	} else {
	    set FichierQuery "[file join $RepNuctfa $Query_Access]"
	    if {[file exists $FichierQuery]} {
		set     Design_Access [AccessDuTFADumRNA $FichierQuery]
	    } else {set Design_Access "$Query_Access"}
	}
	
	#2-2 Access et definition de la proteine correspondante
	set Protein_Access [AccessDeLaBoite $Query_Access protein]
	set Definition     [AccessDeLaBoite $Query_Access protein Definition]
	
	#2-3 Definition de secours (mRNA ou access)
	if {       $Definition == "" && $Design_Access != ""} {
	    set     Definition [AccessDeLaBoite $Query_Access mRNA Definition]
	    if {   $Definition == ""} {
		set Definition [lrange [DefDeAcc $Design_Access genbankfull Def] 1 end]
	    }
	} 
	##if {![info exists Tab($Design_Access)]} {set Tab($Design_Access) 1} else {Espionne "$Query_Access $Design_Access already seen";continue}

	#2-4 Best Id from blast database
	#    Percent identity coverage in both directions
	set Best_ID         [ReturnBestHitFromLogFile $Query_Access $DirOligo]
	set PId             [ReturnBestHitFromLogFile $Query_Access $DirOligo Id]
	set PCoverS         [ReturnBestHitFromLogFile $Query_Access $DirOligo CoverS]
	set PCoverQ         [ReturnBestHitFromLogFile $Query_Access $DirOligo CoverQ]
	
	set BLASTDB1        [InterrogeDesignParameter $FileParam1 DataBase]
	set Best_Definition [ReturnDefinitionFromBestHitFromLogFile $Best_ID $BLASTDB1]
	
	set Protein_Access "=LIEN_HYPERTEXTE(\"http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?-newId+-e+(\[protein-id:'$Protein_Access'\]|\[protein-acc:'$Protein_Access'\])\"; \"$Protein_Access\")"
	set Design_Access "=LIEN_HYPERTEXTE(\"http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?-newId+-e+(\[genbank:'$Design_Access'\]|\[genbank-acc:'$Design_Access'\])\"; \"$Design_Access\")"


	#1-1 Pas de design pour la query
	if {![file exists $FichierMasque]} {
	    #Creation du fichier oligo pour excel $Query_Access oligo non calcules 

	    set Warning "No Oligo yet calculated"
	    foreach Info [list Query_Access Protein_Access Design_Access Definition Warning] {
		if {[info exists $Info]} {
		    lappend LesInfos [set $Info]
		} else {
		    lappend LesInfos ""
		}
	    }
	    AppendAuFichier $FichierSave [join $LesInfos "\t"]
	    continue
	}

	#1-2 Pas de selection pour le design de la query
	if {![file exists $FichierSelection] && ![file exists $FichierCroisee]} {
	    #Creation du fichier oligo pour excel $Query_Access oligo non trouves
	    set Best_ID ""
	    if {[file exists $FichierBlastn] && [file exists $FichierLog]} {
		set Best_ID [ReturnBestHitFromLogFile $Query_Access $DirOligo]
	    }
	    set Warning "No Oligo found due to not enough identity"
	    foreach Info [list Query_Access Protein_Access Design_Access Definition Warning Best_ID] {
		if {[info exists $Info]} {
		    lappend LesInfos [set $Info]
		} else {
		    lappend LesInfos ""
		}
	    }
	    AppendAuFichier $FichierSave [join $LesInfos "\t"]
	    continue
	}
	
	#1-3 Design croisee 
	#    On peut prendre le fichier croisee
	if {$DirOligo2!=""} {
	    set FichierOligo2 "[file join $DirOligo2 "${Query_Access}.oligo"]"
	    set FileParam2    "[file join $DirOligo2 "Design.parameters"]"

	    set  EstCharge2  ""
	    set  EstCharge2  [InterrogeInfoDeLOligo $FichierOligo2 EstCharge]
	    if {$EstCharge2==""} {set EstCharge2 0}

	    if {[file exists $FichierCroisee] && [file exists "$FichierOligo2"]} {
		set OnTraiteDeCroisee 1
		if {$PasDeQuoi} {
		    set FichierACharger $FichierCroisee 
		} else {
		    set FichierACharger $FichierOligo
		}
		set Database2 [file tail $DirOligo2]
	    }
	}

	#Test si on avait deja charge le fichier auparavant
	set  EstCharge1  ""
	set  EstCharge1  [InterrogeInfoDeLOligo $FichierACharger EstCharge]
	if {$EstCharge1==""} {set EstCharge1 0}
	
	set Query_Size [string length [QueLaSequenceDuFichierTFA $FichierMasque]]

	#3 Les donnees des Oligos
	#########################
	#3-1 Cas problematiques (pas de design pas d oligo...)
	set  LesOligos [InterrogeInfoDeLOligo $FichierACharger LesNoms]
	if {$LesOligos == ""} {
	    if {       [InterrogeInfoDeLOligo $FichierACharger NotDone]=="1"} {
		#Creation du fichier oligo pour excel $Query_Access oligo non calcules 
		
		set Warning "No Oligo yet calculated"
		foreach Info [list Query_Access Protein_Access Design_Access Definition Warning] {
		    if {[info exists $Info]} {
			lappend LesInfos [set $Info]
		    } else {
			lappend LesInfos ""
		    }
		}
		AppendAuFichier $FichierSave [join $LesInfos "\t"] 
		continue
	    } elseif { [InterrogeInfoDeLOligo $FichierACharger NoOligo]=="1"} {
		#Creation du fichier oligo pour excel $Query_Access oligo non trouves

		set Best_ID ""
		set Warning "No Oligo found"
		if {[file exists $FichierBlastn] && [file exists $FichierLog]} {
		    set Best_ID [ReturnBestHitFromLogFile $Query_Access $DirOligo]
		    if {[regexp -nocase "warning" $Best_ID]} {
			set Warning "No Oligo found due to not enough identity"
		    }
		}
		foreach Info [list Query_Access Protein_Access Design_Access Definition Warning Best_ID] {
		    if {[info exists $Info]} {
			lappend LesInfos [set $Info]
		    } else {
			lappend LesInfos ""
		    }
		}
		AppendAuFichier $FichierSave [join $LesInfos "\t"]
		continue
	    } else {Espionne "$Query_Access\tPetit probleme";continue}
	}
	
	#3-2 Pour utilier des references comme Debut Fin ou la ref de l oligo
	if {$PasDeQuoi=="0"} {
	    set MonOligo ""
	    if {[llength $Quoi]>1} {
		#On a pas un access mais des debut fin
		set MonOligo [InterrogeInfoDeLOligo $FichierACharger [lindex $Quoi 0] [lindex $Quoi 1] NomOligo]
	    } else {
		set MonOligo $Quoi
	    }
	    if {          $MonOligo==""}          {continue}
	    if {![regexp "$MonOligo" $LesOligos]} {Espionne "No $MonOligo in $FichierACharger";continue}
	}   

	#3-3 Recuperation des infos pour chacun des oligos
	foreach Reference $LesOligos {
	    #Pour utilier des references comme Begin Fin ou la ref de l oligo
	    if {$PasDeQuoi=="0" && ![regexp $MonOligo $Reference]} {continue}

	    set Size           [InterrogeInfoDeLOligo $FichierACharger $Reference Taille]
	    set Tm             [InterrogeInfoDeLOligo $FichierACharger $Reference Tm]
	    set ExpTm          [InterrogeInfoDeLOligo $FichierACharger $Reference ExpTm]
	    set Begin          [InterrogeInfoDeLOligo $FichierACharger $Reference Debut]
	    set End            [InterrogeInfoDeLOligo $FichierACharger $Reference Fin]
	    set Nb_Detected_ID [InterrogeInfoDeLOligo $FichierACharger $Reference NbCandidats]
	    set NbContigues    [InterrogeInfoDeLOligo $FichierACharger $Reference NbContigues]
	    set Detected_ID    [join [InterrogeInfoDeLOligo $FichierACharger $Reference LesCandidats] " "]
	    set Sequence       [InterrogeInfoDeLOligo $FichierACharger $Reference Sequence]
	    set Unrelated_ID   [regexp -all {NG[0-9]+} $Detected_ID]
	    if {! [regexp -nocase "warning" $Best_ID]} {
		if {[regexp -nocase {[a-z]{2}_[0-9]{1,}} $Best_ID] || [regexp "^RSHUM:" $Best_ID]} {
		    if {[regexp "^RSHUM:" $Best_ID]} {
			regsub -nocase {RSHUM:} $Best_ID "" Best_ID
		    }
		    set Best_ID "=LIEN_HYPERTEXTE(\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Nucleotide&cmd=search&term=$Best_ID\"; \"$Best_ID\")"
		}	    
	    }
	    
	    if {$OnAUnigeneDir1} {
		#set Unigene_ID [LeUGIdUN_Getz $Best_ID]
		set Unigene_ID $Best_ID
		if {[regexp -nocase "warning" $Best_ID]} {
			set Unigene_ID [lindex [split $Best_ID " "] 1]
		}
		set  Unigene_ID [LeUnigeneDeUniseq $Unigene_ID $BLASTDB]
		if {$Unigene_ID!=""} {
		    set Unigene_ID "=LIEN_HYPERTEXTE(\"http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?-newId+-e+\[unigene:'$Unigene_ID'\]\"; \"$Unigene_ID\")"	
		} else {set Unigene_ID "No Unigene Id associated"}
	    } 

	    #3-4 Deuxieme repertoire de design
	    #    On recupere les infos de cet oligo dans le deuxieme design (oligo equivalent)
	    #    plus les infos sur le meilleur ID dans cette banque
	    if {$OnTraiteDeCroisee} {
		set Equivalent_Oligo [NomOligoEquivalent $Query_Access $Reference $DirOligo $DirOligo2]
		set Nb_Detected_ID2  [InterrogeInfoDeLOligo $FichierOligo2 $Equivalent_Oligo NbCandidats]
		set Detected_ID2     [join [InterrogeInfoDeLOligo $FichierOligo2 $Equivalent_Oligo LesCandidats] " "]
		set Best_ID2         [ReturnBestHitFromLogFile $Query_Access $DirOligo2]
		set PId2             [ReturnBestHitFromLogFile $Query_Access $DirOligo2 Id]
		set PCoverS2         [ReturnBestHitFromLogFile $Query_Access $DirOligo2 CoverS]
		set PCoverQ2         [ReturnBestHitFromLogFile $Query_Access $DirOligo2 CoverQ]

		set BLASTDB2         [InterrogeDesignParameter $FileParam2 DataBase]
		set Best_Definition2 [ReturnDefinitionFromBestHitFromLogFile $Best_ID2 $BLASTDB2]

		if {! [regexp -nocase "warning" $Best_ID2]} {
		    if {[regexp -nocase {[a-z]{2}_[0-9]{1,}} $Best_ID2] || [regexp "^RSHUM:" $Best_ID2]} {
			if {[regexp "^RSHUM:" $Best_ID2]} {
			    regsub -nocase {RSHUM:} $Best_ID2 "" Best_ID2
			}
			set Best_ID2 "=LIEN_HYPERTEXTE(\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Nucleotide&cmd=search&term=$Best_ID2\"; \"$Best_ID2\")"
		    }	    
		}
			
		if {$OnAUnigeneDir2} {
		    #set Unigene_ID2 [LeUGIdUN_Getz $Best_ID2]
		    set Unigene_ID2 $Best_ID2
		    if {[regexp -nocase "warning" $Best_ID2]} {
			set Unigene_ID2 [lindex [split $Best_ID2 " "] 1]
		    }
		    set  Unigene_ID2 [LeUnigeneDeUniseq $Unigene_ID2 $BLASTDB2]
		    if {$Unigene_ID2!=""} {
			set Unigene_ID2 "=LIEN_HYPERTEXTE(\"http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?-newId+-e+\[unigene:'$Unigene_ID2'\]\"; \"$Unigene_ID2\")"	
		    } else {set Unigene_ID2 "No Unigene Id associated"}
		}		
	    }
	    
	    set LesInfosDeLOligo  ""
	    foreach Info $Entete {
		if {[info exists $Info]} {
		    lappend LesInfosDeLOligo [set $Info]
		} else {
		    lappend LesInfosDeLOligo ""
		}
	    }

	    AppendAuFichier $FichierSave [join $LesInfosDeLOligo "\t"]
	    continue
	}
	if {    $EstCharge1=="0"} {InterrogeInfoDeLOligo $FichierACharger Decharge}	
	if {$OnTraiteDeCroisee} {
	    if {$EstCharge2=="0"} {InterrogeInfoDeLOligo $FichierOligo2   Decharge}	
	}
    }
    return ""
}

proc OLD_LesOligosPourExcel {LesQuery DirOligo {DirOligo2 ""} {Quoi ""} {FichierSave ""} {AvecEntete ""}} {

    Wup "Sauve les oligos dans un format utilisable par Excel"
    Wup "Rq si DirOligo2 est renseigne on utilise le fichier .croisee si ce dernier existe aussi"
    Wup "03/09/03 Mise a jour pour definition"
    Wup "10/09/03 Mise a jour pour tenir compte des oligos croisee"
    Wup "18/03/04 Mise a jour pour tenir compte des access unigene"
    Wup "10/04/04 Mise a jour pour integrer des liens vers SRS et NCBI"
    Wup "03/02/05 Mise a jour du format"
    Wup "03/02/06 Mise a jour du format avec ajout de nouvelles colonnes (PID, coverage)"

    #Quoi peut etre ref de oligo ou coordonnees
    #LesQuery peuvent etre:
    #LaTotale     - pour avoir la liste des PABs
    #ActiChip1_0  - pour la liste des oligos de la premiere  version de ActiChip
    #ActiChip2_0  - pour la liste des oligos de la deuxieme  version de ActiChip
    #ActiChip3_0  - pour la liste des oligos de la troisieme version de ActiChip

    #Choix du fichier de sauvegarde
    if {$FichierSave == ""} {
	while {1} {
	    set Texte "Choose a file name to save your selection"
	    FaireLire "$Texte"
	    if {[BROWSEDDIR] != ""} {
		set FichierSave [tk_getSaveFile -title "Choose a file name to save" -initialdir "[BROWSEDDIR]"]
	    } else {
		set FichierSave [tk_getSaveFile -title "Choose a file name to save" -initialdir "[HOMEDIR]"]
	    }
	    if {   $FichierSave == ""} {
		if {! [OuiOuNon "Would you like to continue ?"]} {return ""}
		continue
	    } else {
		set BrowsedDir [file dirname [lindex $FichierSave end]]
		BROWSEDDIR $BrowsedDir
		break
	    }
	}
    }	

    if {$AvecEntete == "" || $AvecEntete == "AvecEntete" || $AvecEntete == "1"} {set AvecEntete 1} else {set AvecEntete 0} 

    #Pour utilier des references comme Debut Fin ou la ref de l oligo
    if {$Quoi==""} {set PasDeQuoi 1} else {set PasDeQuoi 0}

    set OnAUnigeneDir1 0
    set OnAUnigeneDir2 0

    #0 Creation de l entete (modulable en fonction des disponibilites)
    set Entete "Query Access\
            \tProtein Access\
	    \tDesign Access\
	    \tDefinition\
	    \tQuery Size\
	    \tReference\
	    \tDatabase\
	    \tBest_Database\
            \t%Id\
            \t%CoverQ\
            \t%CoverS"
    if {[regexp -nocase "unigene" $DirOligo]} {
	set OnAUnigeneDir1 1
	append Entete "\tBest_Database_Ug"
    }
    append Entete "\tDefinition\
	    \tBegin\
	    \tEnd\
	    \tTm\
	    \tSize\
	    \tSequence\
	    \tDetected Acc Count\
	    \tDetected Acc\
	    \tUnrelated Acc"

    if {$DirOligo2!=""} {
	append Entete "\tEquivalent Oligo\
		\tDatabase2\
		\tBest_Database2\
                \t%Id\
                \t%CoverQ\
                \t%CoverS"
	
	if {[regexp -nocase "unigene" $DirOligo2]} {
	    set OnAUnigeneDir2 1
	    append Entete "\tBest_Database_Ug"
	}
	append Entete "\tDefinition\
		\tDetected Acc Count2\
		\tDetected Acc"
    }
    if {$AvecEntete} {AppendAuFichier $FichierSave $Entete}
    
    if {$LesQuery=="ActiChip1_0" || [regexp -nocase "actichip1" $LesQuery]} {set LesQuery [ActiChip1_0]}
    if {$LesQuery=="ActiChip2_0" || [regexp -nocase "actichip2" $LesQuery]} {set LesQuery [ActiChip2_0]}
    if {$LesQuery=="ActiChip3_0" || [regexp -nocase "actichip3" $LesQuery]} {set LesQuery [ActiChip3_0]}

    set Banque    [file tail $DirOligo]
    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set RepNuctfa "[file join [RepertoireDuGenome] nuctfa]"
    foreach PAB $LesQuery {
	set OnTraiteDeCroisee   0
	set ProteinAccess      ""
	set NucleicAccess      ""
	set OligoEquivalent    ""
	set BestBanque1        ""
	set BestBanque1_Id     ""
	set BestBanque1_CoverS ""
	set BestBanque1_CoverQ ""
	set BestBanqueUg       ""
	set Definition1        ""
	set Banque2            ""
	set NombreDeCandidat2  ""
	set Candidat2          ""
	set BestBanque2        ""
	set BestBanque2_Id     ""
	set BestBanque2_CoverS ""
	set BestBanque2_CoverQ ""
	set Definition2        ""
	set BestBanqueUg2      ""
	set UnrelatedAcc       ""

	#1 Fichier de design du premier repertoire
	set FichierMasque    "[file join $DirOligo "${PAB}.masked"]"
	set FichierOligo     "[file join $DirOligo "${PAB}.oligo"]"
	set FichierBlastn    "[file join $DirOligo "${PAB}.blastn"]"
	set FichierLog       "[file join $DirOligo "${PAB}.log"]"
	set FichierSelection "[file join $DirOligo "${PAB}.selection"]"
	set FichierCroisee   "[file join $DirOligo "${PAB}.croisee"]"
	set FileParam1       "[file join $DirOligo "Design.parameters"]"

	if {$PasDeQuoi} {
	    set FichierACharger $FichierSelection
	} else {
	    set FichierACharger $FichierOligo
	}

	#1-1 Pas de design pour la query
	if {![file exists $FichierMasque]} {
	    #Creation du fichier oligo pour excel $PAB oligo non calcules
	    set LesInfos "$PAB\tNo Oligo yet calculated"
	    AppendAuFichier $FichierSave $LesInfos
	    continue
	}
	#1-2 Pas de selection pour le design de la query
	if {![file exists $FichierSelection] && ![file exists $FichierCroisee]} {
	    #Creation du fichier oligo pour excel $PAB oligo non trouves
	    set BestBanque1 ""
	    if {[file exists $FichierBlastn] && [file exists $FichierLog]} {
		set BestBanque1 [ReturnBestHitFromLogFile $PAB $DirOligo]
	    }
	    set LesInfos "$PAB\tNo Oligo found due to not enough identity\t$BestBanque1"
	    AppendAuFichier $FichierSave $LesInfos
	    continue
	}
	
	set TailleQuery [string length [QueLaSequenceDuFichierTFA $FichierMasque]]

	#1-3 Design croisee 
	#    On peut prendre le fichier croisee
	if {$DirOligo2!=""} {
	    set FichierOligo2 "[file join $DirOligo2 "${PAB}.oligo"]"
	    set FileParam2    "[file join $DirOligo2 "Design.parameters"]"

	    set  EstCharge2  ""
	    set  EstCharge2  [InterrogeInfoDeLOligo $FichierOligo2 EstCharge]
	    if {$EstCharge2==""} {set EstCharge2 0}

	    if {[file exists $FichierCroisee] && [file exists "$FichierOligo2"]} {
		set OnTraiteDeCroisee 1
		if {$PasDeQuoi} {
		    set FichierACharger $FichierCroisee 
		} else {
		    set FichierACharger $FichierOligo
		}
		set Banque2 [file tail $DirOligo2]
	    }
	}
	#Test si on avait deja charge le fichier auparavant
	set  EstCharge1  ""
	set  EstCharge1  [InterrogeInfoDeLOligo $FichierACharger EstCharge]
	if {$EstCharge1==""} {set EstCharge1 0}
	
	#2 Partie Information de la query
	#################################

	#2-1 Access utilise lors du design
	#    un mRNA ou sequence originale (nuctfa)
	set Query     "[file join $RepmRNA $PAB]"
	if {[file exists $Query]} {
	    set         NucleicAccess [AccessDeLaBoite $PAB mRNA]
	} else {
	    set Query "[file join $RepNuctfa $PAB]"
	    if {[file exists $Query]} {
		set     NucleicAccess [AccessDuTFADumRNA $Query]
	    } else {set NucleicAccess "$PAB"}
	}
	
	#2-2 Access et definition de la proteine correspondante
	set ProteinAccess [AccessDeLaBoite $PAB protein]
	set Definition    [AccessDeLaBoite $PAB protein Definition]
	
	#2-3 Definition de secours (mRNA ou access)
	if {       $Definition == "" && $NucleicAccess != ""} {
	    set     Definition [AccessDeLaBoite $PAB mRNA Definition]
	    if {   $Definition == ""} {
		set Definition [lrange [DefDeAcc $NucleicAccess genbankfull Def] 1 end]
	    }
	} 
	if {![info exists Tab($NucleicAccess)]} {set Tab($NucleicAccess) 1} else {Espionne "$PAB $NucleicAccess deja vu";continue}

	#2-4 Best Id from blast database
	#    Percent identity coverage in both directions
	set BestBanque1        [ReturnBestHitFromLogFile $PAB $DirOligo]
	set BestBanque1_Id     [ReturnBestHitFromLogFile $PAB $DirOligo Id]
	set BestBanque1_CoverS [ReturnBestHitFromLogFile $PAB $DirOligo CoverS]
	set BestBanque1_CoverQ [ReturnBestHitFromLogFile $PAB $DirOligo CoverQ]
	
	set DataBase1   [InterrogeDesignParameter $FileParam1 DataBase]
	set Definition1 [ReturnDefinitionFromBestHitFromLogFile $BestBanque1 $DataBase1]
	
	set ProteinAccess "=LIEN_HYPERTEXTE(\"http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?-newId+-e+(\[protein-id:'$ProteinAccess'\]|\[protein-acc:'$ProteinAccess'\])\"; \"$ProteinAccess\")"
	set NucleicAccess "=LIEN_HYPERTEXTE(\"http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?-newId+-e+(\[genbank:'$NucleicAccess'\]|\[genbank-acc:'$NucleicAccess'\])\"; \"$NucleicAccess\")"

	#3 Les donnees des Oligos
	#########################
	#3-1 Cas problematiques (pas de design pas d oligo...)
	set  LesOligos [InterrogeInfoDeLOligo $FichierACharger LesNoms]
	if {$LesOligos == ""} {
	    if {       [InterrogeInfoDeLOligo $FichierACharger NotDone]=="1"} {
		#Espionne       "Excel File creation for $PAB : no oligo yet designed"
		set LesInfos    "$PAB\tNo Oligo yet calculated"
		AppendAuFichier $FichierSave $LesInfos
	    } elseif { [InterrogeInfoDeLOligo $FichierACharger NoOligo]=="1"} {
		#Espionne        "Excel File creation for $PAB : no oligo found"
		set LesInfos    "$PAB\tNo Oligo found"
		set BestBanque1 ""
		if {[file exists $FichierBlastn] && [file exists $FichierLog]} {
		    set BestBanque1 [ReturnBestHitFromLogFile $PAB $DirOligo]
		    if {[regexp -nocase "warning" $BestBanque1]} {
			set LesInfos "$PAB\tNo Oligo found due to not enough identity\t$BestBanque1"
		    }
		}
		AppendAuFichier $FichierSave $LesInfos
		continue
	    } else {Espionne "$PAB\tPetit probleme";continue}
	}
	
	#3-2 Pour utilier des references comme Debut Fin ou la ref de l oligo
	if {$PasDeQuoi=="0"} {
	    set MonOligo ""
	    if {[llength $Quoi]>1} {
		#On a pas un access mais des debut fin
		set MonOligo [InterrogeInfoDeLOligo $FichierACharger [lindex $Quoi 0] [lindex $Quoi 1] NomOligo]
	    } else {
		set MonOligo $Quoi
	    }
	    if {          $MonOligo==""}          {continue}
	    if {![regexp "$MonOligo" $LesOligos]} {Espionne "No $MonOligo dans $FichierACharger";continue}
	}   

	#3-3 Recuperation des infos pour chacun des oligos
	foreach NomOligo $LesOligos {
	    #Pour utilier des references comme Debut Fin ou la ref de l oligo
	    if {$PasDeQuoi=="0" && ![regexp $MonOligo $NomOligo]} {continue}

	    set TailleOligo  [InterrogeInfoDeLOligo $FichierACharger $NomOligo Taille]
	    set OligoTm      [InterrogeInfoDeLOligo $FichierACharger $NomOligo Tm]
	    set ExpTm        [InterrogeInfoDeLOligo $FichierACharger $NomOligo ExpTm]
	    set Debut        [InterrogeInfoDeLOligo $FichierACharger $NomOligo Debut]
	    set Fin          [InterrogeInfoDeLOligo $FichierACharger $NomOligo Fin]
	    set NbCandidats  [InterrogeInfoDeLOligo $FichierACharger $NomOligo NbCandidats]
	    set NbContigues  [InterrogeInfoDeLOligo $FichierACharger $NomOligo NbContigues]
	    set LesCandidats [join [InterrogeInfoDeLOligo $FichierACharger $NomOligo LesCandidats] " "]
	    set LaSequence   [InterrogeInfoDeLOligo $FichierACharger $NomOligo Sequence]
	    set UnrelatedAcc [regexp -all {NG[0-9]+} $LesCandidats]
	    if {! [regexp -nocase "warning" $BestBanque1]} {
		if {[regexp -nocase {[a-z]{2}_[0-9]{1,}} $BestBanque1] || [regexp "^RSHUM:" $BestBanque1]} {
		    if {[regexp "^RSHUM:" $BestBanque1]} {
			regsub -nocase {RSHUM:} $BestBanque1 "" BestBanque1
		    }
		    set BestBanque1 "=LIEN_HYPERTEXTE(\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Nucleotide&cmd=search&term=$BestBanque1\"; \"$BestBanque1\")"
		}	    
	    }
	    set LesInfosDeLOligo "$PAB\
		    \t$ProteinAccess\
		    \t$NucleicAccess\
		    \t\"$Definition\"\
		    \t$TailleQuery\
		    \t$NomOligo\
		    \t$Banque\
		    \t$BestBanque1\
		    \t$BestBanque1_Id\
		    \t$BestBanque1_CoverS\
		    \t$BestBanque1_CoverQ"
	    
	    if {$OnAUnigeneDir1} {
		#set BestBanqueUg [LeUGIdUN_Getz $BestBanque1]
		set  BestBanqueUg [LeUnigeneDeUniseq $BestBanque1] 
		if {$BestBanqueUg!=""} { 
		    if {! [regexp -nocase "warning" $BestBanqueUg]} {
			set BestBanqueUg "=LIEN_HYPERTEXTE(\"http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?-newId+-e+\[unigene:'$BestBanqueUg'\]\"; \"$BestBanqueUg\")"
		    }
		} else {set BestBanqueUg "No Unigene Id associated"}
		append LesInfosDeLOligo "\t$BestBanqueUg"
	    } 

	    append LesInfosDeLOligo "\t$Definition1\
		    \t$Debut\
		    \t$Fin\
		    \t$OligoTm\
		    \t$TailleOligo\
		    \t$LaSequence\
		    \t$NbCandidats\
		    \t$LesCandidats\
		    \t$UnrelatedAcc"

	    #3-4 Deuxieme repertoire de design
	    #    On recupere les infos de cet oligo dans le deuxieme design (oligo equivalent)
	    if {$OnTraiteDeCroisee} {
		set OligoEquivalent    [NomOligoEquivalent $PAB $NomOligo $DirOligo $DirOligo2]
		set NombreDeCandidat2  [InterrogeInfoDeLOligo $FichierOligo2 $OligoEquivalent NbCandidats]
		set Candidat2          [join [InterrogeInfoDeLOligo $FichierOligo2 $OligoEquivalent LesCandidats] " "]
		set BestBanque2        [ReturnBestHitFromLogFile $PAB $DirOligo2]
		set BestBanque2_Id     [ReturnBestHitFromLogFile $PAB $DirOligo2 Id]
		set BestBanque2_CoverS [ReturnBestHitFromLogFile $PAB $DirOligo2 CoverS]
		set BestBanque2_CoverQ [ReturnBestHitFromLogFile $PAB $DirOligo2 CoverQ]

		set DataBase2          [InterrogeDesignParameter $FileParam2 DataBase]
		set Definition2        [ReturnDefinitionFromBestHitFromLogFile $BestBanque2 $DataBase2]

		if {! [regexp -nocase "warning" $BestBanque2]} {
		    if {[regexp -nocase {[a-z]{2}_[0-9]{1,}} $BestBanque2] || [regexp "^RSHUM:" $BestBanque2]} {
			if {[regexp "^RSHUM:" $BestBanque2]} {
			    regsub -nocase {RSHUM:} $BestBanque2 "" BestBanque2
			}
			set BestBanque2 "=LIEN_HYPERTEXTE(\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Nucleotide&cmd=search&term=$BestBanque2\"; \"$BestBanque2\")"
		    }	    
		}
		append LesInfosDeLOligo "\t$OligoEquivalent\
			\t$Banque2\
			\t$BestBanque2\
             	        \t$BestBanque2_Id\
		        \t$BestBanque2_CoverS\
		        \t$BestBanque2_CoverQ"
		
		if {$OnAUnigeneDir2} {
		    #set BestBanqueUg2 [LeUGIdUN_Getz $BestBanque2]
		    set  BestBanqueUg2 [LeUnigeneDeUniseq $BestBanque2] 
		    if {$BestBanqueUg2!=""} {
			set BestBanqueUg2 "=LIEN_HYPERTEXTE(\"http://igbmc.u-strasbg.fr/srs71bin/cgi-bin/wgetz?-newId+-e+\[unigene:'$BestBanqueUg2'\]\"; \"$BestBanqueUg2\")"	
		    } else {set BestBanqueUg2 "No Unigene Id associated"}

		    append LesInfosDeLOligo "\t$BestBanqueUg2"
		}		
		append LesInfosDeLOligo "\t$Definition2\
			\t$NombreDeCandidat2\
			\t$Candidat2"
	    }

	    AppendAuFichier $FichierSave $LesInfosDeLOligo
	    continue
	}
	if {    $EstCharge1=="0"} {InterrogeInfoDeLOligo $FichierACharger Decharge}	
	if {$OnTraiteDeCroisee} {
	    if {$EstCharge2=="0"} {InterrogeInfoDeLOligo $FichierOligo2   Decharge}	
	}
    }
    return ""
}

proc InterrogeLesOligosPourExcel args {

    Wup "Permet de questionner les infos des oligos contenues dans le fichier pour excel"
    Wup "Voir aussi decharge et charge"
    
    global InfoPourExcel
    
    #Arguments:
    ###########
    #$FichierOligosPourExcel LesAccess       - LesAccess de references
    # FichierOligosPourExcel LesAccessGScope - LesAccess de gscope

    if {$args == "unset"} {
	if {[info exists InfoPourExcel]} {array unset InfoPourExcel}
	return ""
    } 
    
    set Quoi [join $args ","]
    if {[info exists InfoPourExcel($Quoi)]} {return [set InfoPourExcel($Quoi)]}
    
    set FichierACharger [lindex $args 0]
    if {! [info exists InfoPourExcel($FichierACharger,EstCharge)]} {set InfoPourExcel($FichierACharger,EstCharge) 0}
    if {$Quoi=="$FichierACharger,Decharge"} {
	#Espionne "Decharge $FichierACharger"
	DechargeInfoDeOligo $FichierACharger
	return ""
    }
    if {  [info exists InfoPourExcel($FichierACharger,EstVide)]}   {return ""}
    if {  [info exists InfoPourExcel($Quoi)]}                      {return [set InfoPourExcel($Quoi)]}

    ###################CHARGE le Fichier avec les Oligos#########################
    if {! [info exists InfoPourExcel($FichierACharger,EstCharge)] || [set InfoPourExcel($FichierACharger,EstCharge)]=="0"} {
	set InfoPourExcel($FichierACharger,EstCharge) 1
	#Espionne "On charge $FichierACharger"

	set  ResultatCharge [ChargeLesOligosPourExcel InfoPourExcel $FichierACharger]
	if {$ResultatCharge != ""} {
	    set InfoPourExcel($FichierACharger,EstVide) 1
	    return ""
	}
    }

    if {![info exists InfoPourExcel($Quoi)]} {return ""}
    return [set InfoPourExcel($Quoi)] 
}

proc ChargeLesOligosPourExcel {aInfoPourExcel FichierPourExcel} {

    Wup "Permet de charger les infos des oligos contenues dans le fichier pour excel"
    Wup "Voir aussi decharge et charge"

    #GScope Access 
    #Protein Access 
    #Design Access 
    #Definition 
    #Taille 
    #Reference
    #Banque     
    #Best_Banque
    #Definition
                          
    upvar $aInfoPourExcel InfoPourExcel 
        
    if {! [file exists $FichierPourExcel]} {set InfoPourExcel($FichierPourExcel,NotDone) 1;return "-2"}

    set InfoPourExcel($FichierPourExcel,LesAccessGScope) ""
    set InfoPourExcel($FichierPourExcel,LesAccess)       ""

    set F [open "$FichierPourExcel"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {  [regexp -nocase "GScope Access"    $Ligne]} {continue}
	set PAB              ""
	set AccessDeRef      ""
	set LesLignesSplitee ""

	set LesLignesSplitee [split $Ligne "\t"]

	set PAB         [string trim [lindex $LesLignesSplitee 0]]
	if {! [info exists InfoPourExcel($FichierPourExcel,$PAB)]} {
	    set     InfoPourExcel($FichierPourExcel,$PAB)            1
	    lappend InfoPourExcel($FichierPourExcel,LesAccessGScope) $PAB
	} else {continue}
	
	set Definition  [lindex $LesLignesSplitee 3]

	set AccessDeRef [lindex $LesLignesSplitee 7]
	#Warning UG:Hs#S1728810 (%Id 0.99, Cover S/Q 0.89, Cover Q/S 0.48)
	if {[regexp "Warning" $AccessDeRef]} {
	    set AccessDeRef "[lindex $AccessDeRef 1]"
	}
	#Mise au format pour les access RefSeq
	if {[regexp {RSHUM:} $AccessDeRef]} {
	     regsub {RSHUM:} $AccessDeRef "" AccessDeRef
	}
	if {!   [regexp {NM_} $AccessDeRef]} {
	    if {[regexp {NM}  $AccessDeRef]} {
		 regsub {NM}  $AccessDeRef "NM_" AccessDeRef
	    }
	}
	set InfoPourExcel($FichierPourExcel,$PAB,AccessDeRef) $AccessDeRef
	if {[info exists InfoPourExcel($FichierPourExcel,$AccessDeRef,PAB)]} {
	    lappend InfoPourExcel($FichierPourExcel,$AccessDeRef,PAB) $PAB
	} else {
	    set InfoPourExcel($FichierPourExcel,$AccessDeRef,PAB) $PAB
	}
	
	set InfoPourExcel($FichierPourExcel,$AccessDeRef,Definition) $Definition
	if {! [info exists InfoPourExcel($FichierPourExcel,$AccessDeRef)]} {
	    set     InfoPourExcel($FichierPourExcel,$AccessDeRef) 1
	    lappend InfoPourExcel($FichierPourExcel,LesAccess)    $AccessDeRef
	} else {continue}
    }
    close $F
    
    return ""
}

###########################################################
#         Proc pour gerer les resultats du design         #
###########################################################
#   1. Interroge charge et decharge les fichiers .oligo   #
#      .oligo contient la taille,                         #
#                         Tm, ExpTm,                      #
#                         GC content,                     #
#                         positions                       #
#                         candidats (nb aussi)            #
#                         nb contigues                    #
#                         Les Noms des oligos             #
#                         LesDebutFin                     #
#                                                         #
#   2. Interroge charge et decharge les fichiers .log     #
#      .log contient en plus pour chaque access           #
#                         % Identite                      #
#                         nb bases contigues alignees     #
#                         ...                             #
###########################################################
proc InterrogeInfoDeLOligo args {

    Wup "Permet de questionner les infos des oligos"
    Wup "Voir aussi decharge et charge"
    
    global InfoOligo
    
    #Arguments:
    ###########
    #$FichierOligo $NomOligo   Taille       - la Taille de l oligo
    # FichierOligo  NomOligo   Tm           - le Tm 
    # FichierOligo  NomOligo   ExpTm        - le Tm avec une autre methode
    # FichierOligo  NomOligo   GC           - le GC content
    # FichierOligo  NomOligo   Debut        - le debut sur la query        
    # FichierOligo  NomOligo   Fin          - la fin   sur la query        
    # FichierOligo  NomOligo   NbCandidats  - le nombre de candidats pouvant hybrider l oligo
    # FichierOligo  NomOligo   NbContigues  - le nombre de fois ou on a eu 15 nt successifs
    # FichierOligo  NomOligo   LesCandidats - la liste des access des candidats
    # FichierOligo  NomOligo   Sequence     - la sequence
    # FichierOligo $Debut $Fin NomOligo     - le nom de l oligo
    # FichierOligo LesNoms                  - la liste des noms des oligos
    # FichierOligo LesDebutFin              - la liste de listes des coordonnees des oligos

    if {$args == "unset"} {
	if {[info exists InfoOligo]} {array unset InfoOligo}
	return ""
    } 
    
    set Quoi [join $args ","]
    if {[info exists InfoOligo($Quoi)]} {return [set InfoOligo($Quoi)]}
    
    set FichierACharger [lindex $args 0]
    if {! [info exists InfoOligo($FichierACharger,EstCharge)]} {set InfoOligo($FichierACharger,EstCharge) 0}
    if {$Quoi=="$FichierACharger,Decharge"} {
	#Espionne "Decharge $FichierACharger"
	DechargeInfoDeOligo $FichierACharger
	return ""
    }
    if {  [info exists InfoOligo($FichierACharger,EstVide)]}   {return ""}
    if {  [info exists InfoOligo($Quoi)]}                      {return [set InfoOligo($Quoi)]}

    ###################CHARGE le Fichier avec les Oligos#########################
    if {! [info exists InfoOligo($FichierACharger,EstCharge)] || [set InfoOligo($FichierACharger,EstCharge)]=="0"} {
	set InfoOligo($FichierACharger,EstCharge) 1
	#Espionne "On charge $FichierACharger"

	set  ResultatCharge [ChargeLesInfoDeLOligo InfoOligo $FichierACharger]
	if {$ResultatCharge != ""} {
	    set InfoOligo($FichierACharger,EstVide) 1
	    return ""
	}
    }
    if {![info exists InfoOligo($Quoi)]} {return ""}
    return [set InfoOligo($Quoi)] 
}

proc ChargeLesInfoDeLOligo {aInfoOligo FichierOligo} {

    Wup "Charge les infos du fichier avec les oligos"
    Wup "Voir InterrogeInfoOligo pour le questionner"
    Wup "MAJ 01/12/03 pour l absence des infos dans le fasta"
    Wup "MAJ 01/12/03 pour des seq sur plusieurs lignes et plusieurs seq par fichier"
    Wup "MAJ 15/12/03 parametre par defaut si ils ne sont calcules"
    Wup "MAJ 17/03/06 Candidats n est plus un string mais une liste pour mieux continuer (plus besoin de proc AccssDebutDeDir)"

    upvar $aInfoOligo InfoOligo

    if {! [file exists $FichierOligo]} {set InfoOligo($FichierOligo,NotDone) 1;return "-2"}
    set LesOligos      ""
    set OnLeTient       0
    set SequenceStockee 0
    set i               0
    set DefautDebut     0

    set F [open "$FichierOligo"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {  [regexp -nocase "NoOligo"   $Ligne]} {set InfoOligo($FichierOligo,NoOligo) 1;return "-1"}
	#On accumule la sequence si pas sur une ligne
	if {[string range $Ligne 0 0] != ">"} {
	    if {! $OnLeTient} {continue}
	    lappend MaSequence $Ligne
	    set SequenceStockee 1
	    continue
	}
	#On a l entete une 2eme fois, il est temps de stocker la sequence
	if {$SequenceStockee=="1"} {
	    set MaSequence [join $MaSequence ""]
	    if {$TailleOligo == ""} {
		set TailleOligo [string length $MaSequence]
		set InfoOligo($FichierOligo,$NomOligo,Taille) $TailleOligo
	    }
	    if {$Fin == ""} {set Fin $TailleOligo}
	    if {$GC  == ""} {
		set GC [format "%.1f" [expr {(([NombreDeBaseDe $MaSequence "G|C"])/($TailleOligo*1.0))*100.0}]]
		set InfoOligo($FichierOligo,$NomOligo,GC)   $GC
	    }
	    
	    set ExpTm ""
	    set ExpTm [CalculTmDeLaSeq $MaSequence DNADNA]
	    set InfoOligo($FichierOligo,$NomOligo,ExpTm)    $ExpTm
	    set InfoOligo($FichierOligo,$NomOligo,Sequence) $MaSequence
	    set OnLeTient       0
	    set SequenceStockee 0
	    set MaSequence     ""
	} 
	#On a l entete et donc les infos, suivra la sequence
	incr i
	set OnLeTient    1
	set LigneSplitee ""
	set NomOligo     ""
	set TailleOligo  ""
	set OligoTm      ""
	set DebFin       ""
	set Debut        ""
	set Fin          ""
	set NbCandidats  ""
	set NbContigues  ""
	set GC           ""

	set LigneSplitee [split  $Ligne "\t"]
	set Entete       [lindex $LigneSplitee 0]
	#set LesCandidats [join [lrange $LigneSplitee 5 end] " "] 
	set LesCandidats [split [lindex $LigneSplitee 5] " "]
	set NomOligo     [lindex $Entete       1]
	set TailleOligo  [lindex $LigneSplitee 1]
	set OligoTm      [string trim [lindex $LigneSplitee 2]]
	set DebFin       [lindex $LigneSplitee 3]
	set Debut        [lindex $DebFin       0]
	set Fin          [lindex $DebFin       1]
	set CandCont     [lindex $LigneSplitee 4]
	set NbCandidats  [lindex $CandCont     0]
	set NbContigues  [lindex $CandCont     1]
	#set GC           [lindex $LigneSplitee 1]

	if {$NomOligo == ""} {set NomOligo "MaSeq${i}"}
	if {$Debut    == ""} {incr DefautDebut;set Debut $DefautDebut}

	set InfoOligo($FichierOligo,$NomOligo,Taille)       $TailleOligo
	set InfoOligo($FichierOligo,$NomOligo,GC)           $GC
	set InfoOligo($FichierOligo,$NomOligo,Tm)           $OligoTm
	set InfoOligo($FichierOligo,$NomOligo,Debut)        $Debut
	set InfoOligo($FichierOligo,$NomOligo,Fin)          $Fin
	set InfoOligo($FichierOligo,$NomOligo,NbCandidats)  $NbCandidats
	set InfoOligo($FichierOligo,$NomOligo,NbContigues)  $NbContigues
	set InfoOligo($FichierOligo,$NomOligo,LesCandidats) $LesCandidats
	set InfoOligo($FichierOligo,$Debut,$Fin,NomOligo)   $NomOligo
	lappend InfoOligo($FichierOligo,LesDebutFin)        [list $Debut $Fin]
	lappend InfoOligo($FichierOligo,LesNoms)            $NomOligo
    }
    close $F

    #On a fini de parcourir le fichier on stocke la sequence si elle ne l a pas encore ete
    if {$SequenceStockee} {
	set MaSequence [join $MaSequence ""]
	if {$TailleOligo == ""} {
	    set TailleOligo [string length $MaSequence]
	    set InfoOligo($FichierOligo,$NomOligo,Taille) $TailleOligo
	}
	if {$Fin == ""} {set Fin $TailleOligo}
	if {$GC  == ""} {
	    set GC [format "%.1f" [expr {(([NombreDeBaseDe $MaSequence "G|C"])/($TailleOligo*1.0))*100.0}]]
	    set InfoOligo($FichierOligo,$NomOligo,GC)   $GC
	}
	set ExpTm ""
	set ExpTm [CalculTmDeLaSeq $MaSequence DNADNA]
	set InfoOligo($FichierOligo,$NomOligo,ExpTm)    $ExpTm
	set InfoOligo($FichierOligo,$NomOligo,Sequence) $MaSequence
	set OnLeTient       0
	set SequenceStockee 0
	set MaSequence     ""
    } 
    return ""
}

proc DechargeInfoDeOligo {FichierOligo} {
    
    Wup "Decharge les infos chargees avec ChargeInfoDeOligo"

    global InfoOligo

    array unset InfoOligo "$FichierOligo*"
    if {[info exists InfoOligo($FichierOligo,NotDone)]}   {unset InfoOligo($FichierOligo,NotDone)}
    if {[info exists InfoOligo($FichierOligo,NoOligo)]}   {unset InfoOligo($FichierOligo,NoOligo)}
    if {[info exists InfoOligo($FichierOligo,EstVide)]}   {unset InfoOligo($FichierOligo,EstVide)}
    if {[info exists InfoOligo($FichierOligo,EstCharge)]} {unset InfoOligo($FichierOligo,EstCharge)}

    return ""
}

proc InterrogeInfoDuLogDeLOligo args {

    Wup "Permet de questionner les infos contenues dans le log des oligos"
    Wup "Voir aussi decharge et charge"
    
    global InfoDuLogDeLOligo
    
    #Indexation sur le nom du fichier mais egalement sur les positions debut fin pour ne pas tout charger
    #Arguments:
    ###########
    #$Fichierlog $Debut $Fin LesCandidats           - Les candidats detectes par l oligo
    # Fichierlog  Debut  Fin $Candidat IDG          - Les pourcentages d idendite du candidat
    # Fichierlog  Debut  Fin $Candidat NbContigue   - Le nombre de bases contigues identique 
    # Fichierlog  Debut  Fin NbContigue   - 

    if {$args == "unset"} {
	if {[info exists InfoDuLogDeLOligo]} {array unset InfoDuLogDeLOligo}
	return ""
    } 
    
    set Quoi [join $args ","]
    if {[info exists InfoDuLogDeLOligo($Quoi)]} {return [set InfoDuLogDeLOligo($Quoi)]}
    
    set FichierACharger [lindex $args 0]
    set Debut           [lindex $args 1]
    set Fin             [lindex $args 2]
    if {! [info exists InfoDuLogDeLOligo($FichierACharger,$Debut,$Fin,EstCharge)]} {set InfoDuLogDeLOligo($FichierACharger,$Debut,$Fin,EstCharge) 0}

    if {$Quoi=="$FichierACharger,Decharge"} {
	#Espionne "Decharge $FichierACharger"
	DechargeLesInfosDuFichierLog $FichierACharger
	return ""
    }
    if {  [info exists InfoDuLogDeLOligo($FichierACharger,$Debut,$Fin,EstVide)]}   {return ""}
    if {  [info exists InfoDuLogDeLOligo($Quoi)]}                                  {return [set InfoDuLogDeLOligo($Quoi)]}

    ###################CHARGE le Fichier log avec les infos complementaires des Oligos###################
    if {! [info exists InfoDuLogDeLOligo($FichierACharger,$Debut,$Fin,EstCharge)] || [set InfoDuLogDeLOligo($FichierACharger,$Debut,$Fin,EstCharge)]=="0"} {
	set InfoDuLogDeLOligo($FichierACharger,$Debut,$Fin,EstCharge) 1
	set  ResultatCharge [ChargeLesInfosDuFichierLog InfoDuLogDeLOligo $FichierACharger $Debut $Fin]
	if {$ResultatCharge != ""} {
	    set InfoDuLogDeLOligo($FichierACharger,$Debut,$Fin,EstVide) 1
	    return ""
	}
    }
    if {![info exists InfoDuLogDeLOligo($Quoi)]} {return ""}
    return [set InfoDuLogDeLOligo($Quoi)] 
}

proc ChargeLesInfosDuFichierLog {aInfoDuLogDeLOligo FichierLog Debut Fin} {

    Wup "Charge les infos du fichier log qui sont complementaires de celles de InterrogeInfoDeLOligo"
    Wup "Peut a priori tout recuperer mais pour l instant"
    Wup "MAJ 20/04/04 champs Spec avec % ID, nb Contigue, les candidats"
    Wup "Voir InterrogeLesInfosDuFichierLog pour le questionner"

    upvar $aInfoDuLogDeLOligo InfoDuLogDeLOligo

    if {! [file exists $FichierLog]} {set InfoDuLogDeLOligo($FichierLog,$Debut,$Fin,NotDone) 1;return "-2"}

    set OnLeTient 0
    set LesLignes ""
    set Ligne     ""
    set F [open "$FichierLog"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	if {[regexp {^Moving on:} $Ligne]} {
	    set   MaLigne ""
	    regsub "^Moving on: " $Ligne "" MaLigne
	    scan $MaLigne "%s %s" ND NF
	    if {$OnLeTient} {break}
	    if {$Debut==$ND && $Fin==$NF} {
		set OnLeTient 1
		continue
	    }
	    continue
	}
	if {$OnLeTient} {
	    scan $Ligne "%s" PremierMot
	    set Facteur      1
	    set NbBIdentique ""
	    set NbBAlignee   ""
	    set NbContigue   ""
	    switch -exact -- $PremierMot {
		"Spec:"       {	
		    if {[regexp "We left NOTENOUGHIDENTICAL" $Ligne]} {
			continue
		    } else {
			regsub {Spec: We keep TOOMUCH[A-Z ]{9,10}} $Ligne "" Ligne
		    }
		    set LengthOligo  [expr {[lindex $Ligne 1] - [lindex $Ligne 0] + 1}]
		    set Facteur      [llength [lindex $Ligne 8]]
		    set NbBIdentique [lindex $Ligne  9]
		    set NbBAlignee   [lindex $Ligne 10]
		    set NbContigue   [lindex $Ligne 11]
		    set IdGlobal     [format "%.2f" [expr {((1.0*$NbBIdentique)/($LengthOligo*$Facteur))*100.0}]]
		    lappend InfoDuLogDeLOligo($FichierLog,$Debut,$Fin,LesCandidats)                 [lindex $Ligne 4]
		    set     InfoDuLogDeLOligo($FichierLog,$Debut,$Fin,[lindex $Ligne 4],IDG)        $IdGlobal
		    set     InfoDuLogDeLOligo($FichierLog,$Debut,$Fin,[lindex $Ligne 4],NbContigue) $NbContigue
		}
		"Length:"     {continue}
		"Sequence:"   {continue}
		"Prohibited:" {continue}
		"GC:"         {continue}
		"Tm:"         {continue}
		"ID:"         {continue}
		"default"     {continue}
	    }
	}
    }
    close $F
    if {! [info exists InfoDuLogDeLOligo($FichierLog,$Debut,$Fin,LesCandidats)]} {
	 set           InfoDuLogDeLOligo($FichierLog,$Debut,$Fin,LesCandidats) ""
    }
    return ""
}

proc DechargeLesInfosDuFichierLog {FichierLog} {
    
    Wup "Decharge les infos chargees avec ChargeInfoDeOligo"

    global InfoDuLogDeLOligo

    array unset InfoDuLogDeLOligo "$FichierLog*"
    return ""
}
